/*****************************************************************************
 * Ports.h:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#ifndef _PORTS_H
#define _PORTS_H

/* This must be the first include in the .c files */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#ifdef RTL_PRO
#include <rtl_pthread.h>
#include <rtl_semaphore.h>
#else
#include <pthread.h>
#include <semaphore.h>
#include <sched.h>
#endif

#ifdef RTL_PRO

#else

#define rtl_pthread_t                    pthread_t
#define rtl_pthread_attr_t 	             pthread_attr_t
#define rtl_pthread_attr_init            pthread_attr_init
#define rtl_pthread_attr_setschedparam   pthread_attr_setschedparam
#define rtl_pthread_create               pthread_create
#define rtl_pthread_cancel               pthread_cancel
#define rtl_pthread_join                 pthread_join
            
#define rtl_pthread_mutex_t              pthread_mutex_t
#define rtl_pthread_mutex_lock           pthread_mutex_lock
#define rtl_pthread_mutex_unlock         pthread_mutex_unlock
#define rtl_pthread_mutex_init           pthread_mutex_init
#define rtl_pthread_mutex_destroy        pthread_mutex_destroy
#define rtl_pthread_attr_setdetachstate  pthread_attr_setdetachstate
            
#define rtl_pthread_cond_t               pthread_cond_t
#define rtl_pthread_cond_wait            pthread_cond_wait
#define rtl_pthread_cond_signal          pthread_cond_signal
#define rtl_pthread_cond_broadcast       pthread_cond_broadcast
#define rtl_pthread_cond_init            pthread_cond_init
#define rtl_pthread_cond_destroy         pthread_cond_destroy

//#define rtl_pthread_mutexattr_t          pthread_mutexattr_t
//#define rtl_pthread_condattr_init        pthread_condattr_init
//#define rtl_pthread_condattr_destroy     pthread_condattr_destroy
//#define rtl_pthread_mutexattr_setpshared pthread_mutexattr_setpshared
//#define RTL_PTHREAD_PROCESS_SHARED       PTHREAD_PROCESS_SHARED


#define rtl_sem_t                        sem_t
#define rtl_sem_wait                     sem_wait
#define rtl_sem_trywait                  sem_trywait
#define rtl_sem_timedwait                sem_timedwait
#define rtl_sem_post                     sem_post
#define rtl_sem_init                     sem_init
#define rtl_sem_destroy                  sem_destroy

#define RTL_TIMER_ADVANCE	0

#define rtl_clock_nanosleep              clock_nanosleep
#define rtl_clock_gettime                clock_gettime

#define RTL_CLOCK_REALTIME               CLOCK_REALTIME
#define rtl_timespec                     timespec
#define rtl_sched_get_priority_max       sched_get_priority_max
#define rtl_sched_get_priority_min       sched_get_priority_min
#define rtl_sched_param                  sched_param
#define rtl_pthread_attr_setstacksize    pthread_attr_setstacksize
#define rtl_pthread_attr_setstackaddr    pthread_attr_setstackaddr
#define RTL_SCHED_FIFO                   SCHED_FIFO

#define rtl_timespec_add_ns              timespec_add_ns
#define rtl_timespec_from_ns             timespec_from_ns
#define RTL_TIMER_ABSTIME                TIMER_ABSTIME

#define rtl_shm_open                     shm_open
#define rtl_mmap                         mmap
#define rtl_munmap                       munmap
#define rtl_close                        close
#define rtl_shm_unlink                   shm_unlink

#define RTL_PROT_READ                    PROT_READ
#define RTL_O_CREAT                      O_CREAT
#define RTL_O_RDWR                       O_RDWR
#define RTL_O_RDONLY                     O_RDONLY
#define RTL_O_PHYS                       0
#define RTL_PROT_WRITE                   PROT_WRITE
#define RTL_MAP_SHARED                   MAP_SHARED
#define RTL_MAP_FAILED                   MAP_FAILED

#endif

#ifdef RTL_PRO
#define ERROR_MSG(F, S...)     {rtl_printf("\n[%s] %s() " F , __FILE__, __FUNCTION__, ## S);}
#define PRINT_MSG(F, S...)     {rtl_printf("\n" F , ## S);}
#define PRINTF(F, S...)        {rtl_printf( F , ## S);}
#else
#define ERROR_MSG(F, S...)     {fprintf(stderr,"\n[%s] %s() " F , __FILE__, __FUNCTION__, ## S);fflush(stdout);}
#define PRINT_MSG(F, S...)     {fprintf(stdout,"\n" F , ## S);fflush(stdout);}
#define PRINTF(F, S...)        {fprintf(stdout, F , ## S);fflush(stdout);}

static inline void timespec_add_ns(struct timespec *a, unsigned long ns)
{
#define NSEC_PER_SEC	1000000000L
	ns += a->tv_nsec;
	while(ns >= NSEC_PER_SEC) {
		ns -= NSEC_PER_SEC;
		a->tv_sec++;
	}
	a->tv_nsec = ns;
}

static inline struct timespec timespec_from_ns(unsigned long ns)
{
	struct timespec x={0,0};
	timespec_add_ns( &x, ns);
	return x;
}

#endif

#ifdef RTL_PRO
//#if __BYTE_ORDER == __BIG_ENDIAN
//cannot use this MACRO. It is not provided by rtl-gcc
inline static unsigned short ntohs (unsigned short nValue){
	return (((nValue>> 8)) | (nValue << 8));
}

inline static unsigned short htons (unsigned short nValue){
	return (((nValue>> 8)) | (nValue << 8));
}

inline static unsigned int ntohi (unsigned int nValue){
	return ((nValue>>24) | ((nValue>>8)&0x0000FF00) | ((nValue<<8)&0x00FF0000) | (nValue<<24));
}

inline static unsigned int htoni (unsigned int nValue){
	return ((nValue>>24) | ((nValue>>8)&0x0000FF00) | ((nValue<<8)&0x00FF0000) | (nValue<<24));
}
#else
#include <netinet/in.h>
#define ntohi ntohl
#define htoni htonl
#endif

typedef struct{
	rtl_sem_t semaphore;
	unsigned char is_destroyed;
}FTT_SEMAPHORE;

static inline signed char FTT_SEMAPHORE_wait( FTT_SEMAPHORE *sem )
{
	signed char ret;
	ret = rtl_sem_wait(&(sem->semaphore));
	if (ret<0) return ret;
	if (sem->is_destroyed) return -10;
	return 0;
}

static inline void FTT_SEMAPHORE_post( FTT_SEMAPHORE *sem )
{
	rtl_sem_post(&(sem->semaphore));
}

static inline signed char FTT_SEMAPHORE_init( FTT_SEMAPHORE *sem, int init_state )
{
	sem->is_destroyed = 0;
	return rtl_sem_init( &(sem->semaphore), 0, init_state);
}

static inline signed char FTT_SEMAPHORE_destroy( FTT_SEMAPHORE *sem )
{
	sem->is_destroyed = 1;
	rtl_sem_post(&(sem->semaphore)); // wake it
	
	/* and make sure to clean up the semaphore */
	rtl_sem_destroy(&(sem->semaphore));
	
	return 0;
}
#ifdef RTL_PRO
/* It seams that the  rtl_sem_trywait doesn't work properly under RTL
 * Maybe because it's being called from within the IRQ
 * If I call it right after the sem_init it works.
 * The workaround checks directly the sem value */
static inline signed char FTT_SEMAPHORE_trywait( FTT_SEMAPHORE *sem )
{
	if (sem->semaphore.value){
		sem->semaphore.value --;
		return 0;
	}else{
		return -1;
	}
}
#else
static inline signed char FTT_SEMAPHORE_trywait( FTT_SEMAPHORE *sem )
{
	if ( rtl_sem_trywait(&(sem->semaphore)) < 0 )
		return -1;
	return 0;
}
#endif


#endif

