
include $(PATH_TO_RTL)/lnet/lnet.mk

ifneq ($(wildcard $(RTL_DIR)/.config),)
include $(RTL_DIR)/.config
endif

ORIG_PATH	:= $(PATH)
PATH	:= $(DEVKIT_DIR)bin:$(PATH)
LOCAL_SYMBOLS	:= -Lmain -L__rtcore_app

#CFLAGS comes from rtl.mk
CFLAGS := $(filter-out -msoft-float,$(CFLAGS)) -mhard-float -fshort-enums
CFLAGS += -DRTL_PRO

%.rtl: %.o
	@echo "Linking \033[;33m$@"; tput sgr0 #Yellow
	$(CC) $(CFLAGS_RTCORE_NAMES) $(CFLAGS) -DKMOD=RTL_MODULE\($(shell basename $@|sed -e s/\\\.rtl//)\) \
		-c $(RTL_DIR)/mainhelper/main_crt0.c -o $@.crt0.o
	$(LD) -r -o $<.mainmod $^ $(RTL_LIBS_DIR)/rtl_crt0.o $@.crt0.o $(LDFLAGS_$@) $(LDFLAGS) 
	$(OBJCOPY) $(LOCAL_SYMBOLS) $<.mainmod $<.mainmod
	cp $< $*.o.debug
	$(OBJCOPY) --strip-debug $<.mainmod
ifeq ($(CONFIG_RTL_LINUX26),y)
	$(CC) -c -D__KERNEL__ -DMODULE $(CFLAGS) -DKBUILD_MODNAME=$(shell basename $@|sed -e s/\\\.rtl//) $(RTL_DIR)/mainhelper/module_stub.c -o $<.stub.o
	@mv -f $<.mainmod $<.elf
	$(LD) $(LDFLAGS) -r -o $<.mainmod $<.elf $<.stub.o
endif
	@rm -f $<.mod $@.crt0.o $<.elf
	$(OBJCOPY) $(OBJCOPY_ARGS) \
		--add-section=.image=$<.mainmod \
		$(RTL_LIBS_DIR)/user.o $@.user.o
	@rm -f $<.mainmod
	$(LD) -r $(RTL_DIR)/mainhelper/ld.user $@.user.o -o $@.final.o
	$(CC) -o $@ $@.final.o
	@rm -f $@.final.o $@.user.o
	$(STRIP) $@

%.ko: %.o
	@echo "Linking \033[;33m$@"; tput sgr0 #Yellow
	$(CC) -c -D__KERNEL__ -DMODULE $(CFLAGS) -DKBUILD_MODNAME=$* $(RTL_DIR)/mainhelper/module_stub.c -o $<.stub.o
	$(LD) $(LDFLAGS) -r -o $@ $^ $<.stub.o $(LDFLAGS_$@)

%.o: %.c
	@echo "Compiling \033[;34m$@"; tput sgr0 #Blue
	$(CC) $(CFLAGS) $(CFLAGS_$@) -o $@ -c $<

%.rtl.o: %.c
	@echo "Compiling \033[;34m$@"; tput sgr0 #Blue
	$(CC) $(CFLAGS) $(CFLAGS_$@) -o $@ -c $<

%.rtl.a:
	@echo -n "Packing \033[4m$@\033[0m  " #underline
#	@echo -n " Deps: $^"
	@rm -rf tmp ; mkdir tmp;
	@cd tmp; for lib in $(filter %.a,  $^); do $(AR) x ../$$lib ; done; cd ..
	@x=`find ./tmp -name "*.o*"` ; $(AR) cr $@ $$x $(filter %.o,  $^)
	@ranlib $@
	@rm -rf tmp
	@echo "  \033[;32m OK"; tput sgr0 #Green

#
# Don't allow .o's to be deleted if they were created so we could
# build a .ko file.  This allows us to just target .ko and get the
# .o built for us -- Cort <cort@fsmlabs.com>
#		
.PRECIOUS: %.o

.d.%.c: %.c
	@set -e; $(CC) -MM $(CFLAGS) $(CFLAGS_$@) $< \
		| sed 's/\($*\)\.o[ :]*/\1.o $@ : /g' > $@; \
		[ -s $@ ] || rm -f $@

