/*****************************************************************************
 * ftt_master_run.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#ifdef RTL_PRO
#include <stdio.h>
#include <stdlib.h>
#include <linux/moduleparam.h>

#elif defined L26
#include <signal.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#elif defined __QNXNTO__
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#else
#error "Would you mind selecting a target architecture"
#endif


/*
 * Startup Parameters: 
 * ./ftt_master_run.rtl <mac_address> [dev_num]
 * insmod ftt_master_run.ko <mac_address> [dev_num]
 * ./ftt_master_run <mac_address> [dev_num]
 *
 * <dev_num> - LNET device number || ethX number
 * <mac_address> - The MAC address the device must take
 */

#include "ftt-master.h"

extern signed char my_app_init(unsigned char app_id);
extern void my_app_close(unsigned char app_id);

#ifdef RTL_PRO
#ifdef MASTER_RTL_OUTPUT


int main(int argc, char **argv)
{
	unsigned char dev=0;
	unsigned int ec=0;

	if ((argc!=2)&&(argc!=3)){
		rtl_printf("Error Invalid parameters number %d\n",argc );
		rtl_printf("%s [dev_num] <EC(ms)>\n",argv[0] );
		return -1;
	}

	dev = atoi(argv[1]);
	if (dev>4) {
		rtl_printf("Device number is too high\n",dev);
		return -1;
	}
	ec=1000; //1ms
	if (argc==3) ec = atoi(argv[2])*1000;

	rtl_printf(" LNET device: %d \n", dev);
	rtl_printf(" EC time: %d \n", ec);
	
	ftt_master_start_up(NULL,dev, NULL,ec);

	rtl_main_wait();
	ftt_master_close_up();
	return 0;
}

#else // .ko

static char mac[] = "FF:FF:FF:FF:FF:FF";
static int dev_no = -1;
//module_param(dev_no, int, S_IRUGO);
module_param(dev_no, int, 0);
//MODULE_PARM_DESC(dev_no, "The Lnet network device no");
//module_param(mac, charp, S_IRUGO);
module_param(mac, charp, 0);
//MODULE_PARM_DESC(mac, "The MAC address to be used");

int init_module(void){
	unsigned char temp;
	if ( dev_no < 0 )
		return -1;
	if (mac[0] == 'F')
		return -2;
	return ftt_master_start_up(NULL,dev_no, NULL,1000);
}

void cleanup_module(void){
	ftt_master_close_up();
	return;
}
#endif

#else

static unsigned char appID;

void sigfun(int dummy) {
	if ( appID ) my_app_close(appID);

	ftt_master_close_up();
	printf("\nCtrl-C pressed\n");
	(void) signal(SIGINT, SIG_DFL);
}


int main(int argc, char **argv)
{
	unsigned char dev=0;
	unsigned int ec=0;
	int ret;
	
	(void) signal(SIGINT, sigfun);

	
	if ((argc!=3)&&(argc!=4)){
		printf("Error Invalid parameters number %d\n",argc );
		printf("%s <dev_num|ifname> <EC(ms)> [app_no]\n",argv[0] );
		printf("[app_no] is optional but if set, will run any of the slaves' my_app! \n");
		return -1;
	}

	ec=1000; //1ms
	ec = atoi(argv[2])*1000;
	printf(" EC time: %d \n", ec);
	
	appID = 0;
	if (argc == 4){
		appID = atoi(argv[3]);
		if ( !appID ){
			printf("Error in parameters: app_no must be non-zero");
			return -2;
		}
	}
	
	if (strlen(argv[1])<2){
		/*dev_num*/
		if ( (dev = atoi(argv[1])) > 4 ) {
			printf("Device number is too high: %d\n",dev);
			return -1;
		}
		printf(" Connecting to Linux eth%d \n", dev);
		ret=ftt_master_start_up(NULL,dev, NULL,ec);
	}else {
		/* Device name */
		printf(" Connecting to Linux %s \n", argv[1] );
		ret=ftt_master_start_up(NULL,99, argv[1],ec);
	}
	if (ret<0) return -1;
	
	if ( appID ) my_app_init(appID);
	
	pause();
	
	return 0;
}
#endif



