/*****************************************************************************
 * ftt_slave_run.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/


#ifdef RTL_PRO
#include <stdio.h>
#include <stdlib.h>
#include <linux/moduleparam.h>

#elif defined L26
#include <signal.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#elif defined __QNXNTO__
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#else
#error "Would you mind selecting a target architecture"
#endif

#include "ftt-slave.h"

//#define MY_APP /*Makefile controled*/

extern signed char my_app_init(unsigned char app_id);
extern void my_app_close(unsigned char app_id);

#ifdef RTL_PRO

#ifdef SLAVE_RTL_OUTPUT
int main(int argc, char **argv)
{
	//unsigned char temp=0; //default dev_num
	unsigned char temp1;
	unsigned char temp2;

	if (argc!=3){
		printf("Error Invalid parameters number \n");
		printf("%s <dev_num> <app_no>\n",argv[0] );
		return -1;
	}

	temp1 = atoi(argv[1]);
	temp2 = atoi(argv[2]);

	rtl_printf(" LNET device: %d \n", temp1);
	rtl_printf(" appID: %d \n", temp2);

	ftt_slave_start_up(NULL,temp1, NULL, 0 /*No app_nodeID*/ );
#ifdef MY_APP
	if ( my_app_init(temp2) < 0 )
		return -3;
#endif



	rtl_main_wait();
#ifdef MY_APP
	my_app_close(temp2);
#endif
	ftt_slave_close_up();

	return 0;
}
#else

static char mac[] = "FF:FF:FF:FF:FF:FF    ";
static int dev_no = 0;
static int app_no = 0;
//module_param(dev_no, int, S_IRUGO);
module_param(dev_no, int, 0);
module_param(app_no, int, 0);
//module_param(mac, charp, S_IRUGO);
module_param(mac, charp, 0);

int init_module(void)
{
	if ( ftt_slave_start_up( NULL, dev_no, NULL, 0 /*No app_nodeID*/ ) < 0 )
//	if ( ftt_slave_start_up( mac, dev_no, 0 /*No app_nodeID*/ ) < 0 )
		return -1;

	#ifdef MY_APP
	my_app_init(app_no);
	#endif

	return 0;
}
void cleanup_module(void)
{
#ifdef MY_APP
	my_app_close();
#endif
	ftt_slave_close_up();
	return;
}
#endif

#else

static unsigned char appID;


void sigfun(int dummy) {
#ifdef MY_APP
	my_app_close(appID);
#endif
	printf("Calling ftt_slave_close_up \n");
	ftt_slave_close_up();
	printf("\nCtrl-C pressed\n");
	(void) signal(SIGINT, SIG_DFL);
}

int main(int argc, char **argv)
{
	unsigned char dev;
	int ret;

	(void) signal(SIGINT, sigfun);
	
	if (argc!=3){
		printf("Error Invalid parameters number \n");
		printf("%s <dev_num|ifname> <app_no>\n",argv[0] );
		return -1;
	}

	appID = atoi(argv[2]);
	printf(" appID: %d \n", appID);

	
	if (strlen(argv[1])<2){
		/*dev_num*/
		if ( (dev = atoi(argv[1])) > 4 ) {
			printf("Device number is too high:%d\n",dev);
			return -1;
		}
		printf(" Connecting to Linux eth%d \n", dev);
		ret=ftt_slave_start_up(NULL,dev, NULL, 0 /*No app_nodeID*/ ); /* 1 ms for scheduling - but 100ms for tx */
	}else {
		/* Device name */
		printf(" Connecting to Linux %s \n", argv[1] );
		ret=ftt_slave_start_up(NULL,99, argv[1], 0 /*No app_nodeID*/ ); /* 1 ms for scheduling - but 100ms for tx */
	}
	if (ret<0) return -1;

#ifdef MY_APP
	my_app_init(appID);
#endif
	
	pause();
	return 0;
}
#endif



