/*****************************************************************************
 * my_app.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#ifdef MY_APP
#include "../misc/arch-ports/Ports.h"
#include "ftt-slave.h"

#define ASYNC_TEST

//#define TEST_CHANGING_ONLINE_CHANNEL_PROPERTIES



static rtl_pthread_t my_app_th;

#define VAR_PERIOD 10
#define VAR_SIZE 30000
#define VAR_MAX_SIZE 31000


#ifdef ASYNC_TEST
#define MEM_SLOT_NO	10
#else
#define MEM_SLOT_NO	3
#endif

//#define MEM_MAX_SIZE ((VAR_MAX_SIZE*MEM_SLOT_NO)+200)
#define MEM_MAX_SIZE ((VAR_MAX_SIZE*(MEM_SLOT_NO+1))+200)  //+1 'cause we admit two possible source nodes for this var.
static unsigned char buf_t[MEM_MAX_SIZE];

void my_app1_close(void)
{
	return;
}

void *my_app1(void *t)
{
	int ret;
	M_INT_APP_MSG_ID app_msg_id;

	S_FTT_INTERFACE_L_init_block(NULL);
	PRINT_MSG("Application 111111111111111111111111111111111111111111 Load");

#ifndef ASYNC_TEST
	app_msg_id.type = M_INT_MSG_SYNCH;
#else
	app_msg_id.type = M_INT_MSG_ASYNCH_S;
#endif
	app_msg_id.id = 9;

	if ( (ret = S_FTT_INTERFACE_L_var_add_load ( 
		app_msg_id,
		VAR_SIZE, /* size */
		VAR_MAX_SIZE, /* max_size */
		2, /*Max number source nodes*/
		1450, /* MTU */
		VAR_PERIOD, /* period */
		0 /* offset */
		)) < 0 ){
		ERROR_MSG("S_FTT_INTERFACE_L_var_add_load ret: %d", ret);
		return NULL;
	}
	
#ifdef TEST_CHANGING_ONLINE_CHANNEL_PROPERTIES
	/* 1 sec later update the var details */
	usleep(1000000); //1sec

	if ( (ret = S_FTT_INTERFACE_L_var_add_load ( 
		app_msg_id,
		VAR_SIZE+1, /* size */
		VAR_MAX_SIZE, /* max_size */
		2, /*Max number source nodes*/
		1450, /* MTU */
		VAR_PERIOD, /* period */
		0 /* offset */
			)) < 0 ){
		ERROR_MSG("S_FTT_INTERFACE_L_var_add_load ret: %d", ret);
		return NULL;
	}
	
	/* 15 sec later update the var details */
	usleep(15000000); //15sec
	
	if ( (ret = S_FTT_INTERFACE_L_var_add_load ( 
		app_msg_id,
		VAR_SIZE, /* size */
		VAR_MAX_SIZE, /* max_size */
		2, /*Max number source nodes*/
		1450, /* MTU */
		VAR_PERIOD, /* period */
		0 /* offset */
		)) < 0 ){
		ERROR_MSG("S_FTT_INTERFACE_L_var_add_load ret: %d", ret);
		return NULL;
	}
	
	/* 15 sec later update the var details */
	usleep(15000000); //15sec
	
	if ( (ret = S_FTT_INTERFACE_L_var_rm_load ( app_msg_id )) < 0 ){
		ERROR_MSG("S_FTT_INTERFACE_L_var_rm_load ret: %d", ret);
		return NULL;
	}
	
#endif
	
	
	return NULL;
}



void my_app2_close(void)
{
	M_INT_APP_MSG_ID app_msg_id;
	int ret;
	
#ifndef ASYNC_TEST
	app_msg_id.type = M_INT_MSG_SYNCH;
#else
	app_msg_id.type = M_INT_MSG_ASYNCH_S;
#endif
	app_msg_id.id = 9;
	
	ret = S_FTT_INTERFACE_L_unbind( app_msg_id );
	PRINT_MSG("S_FTT_INTERFACE_L_unbind ret: %d", ret);
	
	return;
}

void *my_app2(void *t)
{
	S_FTT_INTERFACE_L_init_block(NULL);
	PRINT_MSG("Application 222222222222222222222222222222222222222222 tx");
	unsigned char seq_no=0;
	
	{
		M_INT_APP_MSG_ID app_msg_id;
		
		int ret;
		unsigned int actual_var_size;
		unsigned short actual_var_period;
		unsigned char sending_tag;
		int counter;
		
		unsigned int app_id = (unsigned int)t;
		
		void *prod_p;
		
#ifndef ASYNC_TEST
		app_msg_id.type = M_INT_MSG_SYNCH;
#else
		app_msg_id.type = M_INT_MSG_ASYNCH_S;
#endif
		app_msg_id.id = 9;
		
/////////////////////
#if 0
		if ( (ret = S_FTT_INTERFACE_L_var_add_load ( 
													app_msg_id,
													VAR_SIZE, /* size */
													VAR_MAX_SIZE, /* max_size */
													2, /*Max number source nodes*/
													1450, /* MTU */
													VAR_PERIOD, /* period */
													0 /* offset */
													)) < 0 ){
			ERROR_MSG("S_FTT_INTERFACE_L_var_add_load ret: %d", ret);
			return NULL;
		}
#endif
///////////////////////
		
		
	rebind_app2:
		PRINT_MSG("Attaching %d", app_msg_id.id );

		if ( (ret = S_FTT_INTERFACE_L_attach_tx( app_msg_id )) < 0 ){
			ERROR_MSG("FATAL: S_FTT_INTERFACE_L_attach_tx ret: %d", ret);
			return NULL;
		}
		
		PRINT_MSG("Binding %d", app_msg_id.id );
		
		if ( (ret = S_FTT_INTERFACE_L_bind( app_msg_id, buf_t, MEM_MAX_SIZE, MEM_SLOT_NO)) < 0 )
			ERROR_MSG("S_FTT_INTERFACE_L_bind_tx ret: %d", ret);
			
		counter=100000;
		
	retag_app2:
		if ( (ret = S_FTT_INTERFACE_L_var_details( app_msg_id, &actual_var_size, &actual_var_period )) < 0 ){
			ERROR_MSG("FATAL: S_FTT_INTERFACE_L_var_details ret: %d", ret);
			return NULL;
		}
		sending_tag = ret;
		
		PRINT_MSG("Now sending with tag:%d  period: %d   size: %d\n", sending_tag, actual_var_period, actual_var_size);

		while(counter--){

			if ( (ret = S_FTT_INTERFACE_L_pre_send( app_msg_id, 1 /*queue-block*/, &prod_p)) < 0 ){
				if (ret == -4){
					/* CAB/FIFO was destroyed while waiting */
					S_FTT_INTERFACE_L_unbind( app_msg_id );
					//ERROR_MSG("Rebinding");
					goto rebind_app2;
				}
				ERROR_MSG("S_FTT_INTERFACE_L_pre_send failed");
				return NULL;
			}
			PRINTF("s");
			//PRINT_MSG("%d",counter);
			
			//PRINTF(" %d", seq_no );
			
			( (unsigned char *)prod_p )[0] = seq_no;
			( (unsigned char *)prod_p )[1] = (unsigned char) app_id;
					
			
			if ( (ret = S_FTT_INTERFACE_L_send( app_msg_id, sending_tag, NULL, 1 /*queue-block*/, 1/*non-block*/)) < 0 ){
				switch (ret) {
					case -4:
						S_FTT_INTERFACE_L_pre_send_abort( app_msg_id );
						S_FTT_INTERFACE_L_unbind( app_msg_id );
						goto rebind_app2;
						break;
					case -2:
						S_FTT_INTERFACE_L_pre_send_abort( app_msg_id );
						goto retag_app2;
						break;
					case -3:
						/* FIFO full */
						break;
					default:
						ERROR_MSG("S_FTT_INTERFACE_L_send failed");
						return NULL;
				}
			}
			seq_no++;
		}
		
		PRINT_MSG("Dettaching %d", app_msg_id.id );
		
		S_FTT_INTERFACE_L_dettach( app_msg_id );
		goto rebind_app2;

		PRINT_MSG("Finished");
	}
	return NULL;
}


void my_app3_close(void)
{
	
	M_INT_APP_MSG_ID app_msg_id;
	int ret;
	
#ifndef ASYNC_TEST
	app_msg_id.type = M_INT_MSG_SYNCH;
#else
	app_msg_id.type = M_INT_MSG_ASYNCH_S;
#endif
	app_msg_id.id = 9;
	
	ret = S_FTT_INTERFACE_L_unbind( app_msg_id );
	PRINT_MSG("S_FTT_INTERFACE_L_unbind ret: %d", ret);
	
	return;
}

void *my_app3(void *t)
{

	S_FTT_INTERFACE_L_init_block(NULL);
	PRINT_MSG("Application 333333333333333333333333333333333333333333 rx");
{
	int ret;
	M_INT_APP_MSG_ID app_msg_id;
	unsigned int actual_var_size;
	unsigned short actual_var_period;
	void *cons_p;

#ifndef ASYNC_TEST
	app_msg_id.type = M_INT_MSG_SYNCH;
#else
	app_msg_id.type = M_INT_MSG_ASYNCH_S;
#endif
	app_msg_id.id = 9;

rebind_app3:
	PRINT_MSG("Attaching RX");
	if ( (ret = S_FTT_INTERFACE_L_attach_rx( app_msg_id )) < 0 ){
		ERROR_MSG("FATAL: S_FTT_INTERFACE_L_attatch_rx ret: %d", ret);
		return NULL;
	}

	if ( (ret = S_FTT_INTERFACE_L_var_details( app_msg_id, &actual_var_size, &actual_var_period )) < 0 ){
		ERROR_MSG("FATAL: S_FTT_INTERFACE_L_var_details ret: %d", ret);
		return NULL;
	}

	PRINT_MSG("\nIn 33333333333333333 The message attatched has tag:%d  period: %d   size: %d", ret, actual_var_period, actual_var_size);

	if ( (ret = S_FTT_INTERFACE_L_bind( app_msg_id, buf_t, MEM_MAX_SIZE, MEM_SLOT_NO)) < 0 ){
		ERROR_MSG("FATAL: S_FTT_INTERFACE_L_bind_rx ret: %d", ret);
		return NULL;
	}
	
	PRINT_MSG("Bound");

	while (1){
		if ( (ret = S_FTT_INTERFACE_L_pre_receive( app_msg_id, NULL, &cons_p, RX_BLOCK_NEW_RX)) < 0 ){
			S_FTT_INTERFACE_L_unbind( app_msg_id );
			PRINT_MSG("Unbinded1");
			goto rebind_app3;
			ERROR_MSG("S_FTT_INTERFACE_L_pre_receive failed ret: %d", ret);
			return NULL;
		}

		//PRINTF(".");
		PRINTF("%d", ( (unsigned char *)cons_p )[1] );
		
//		for ( ret=0; ret<20; ret++)
//			PRINTF(" %x", ( (unsigned char *)cons_p )[ret]);
//		PRINT_MSG("");

		if ( S_FTT_INTERFACE_L_receive( app_msg_id, NULL, NULL, 0) < 0 ){
			S_FTT_INTERFACE_L_unbind( app_msg_id );
			PRINT_MSG("Unbinded2");
			goto rebind_app3;
			ERROR_MSG("S_FTT_INTERFACE_L_receive failed");
			return NULL;
		}
	}
}

return NULL;
}

signed char my_app_init(unsigned char app_id)
{

	rtl_pthread_attr_t attr;
	struct rtl_sched_param sched_param;
	int ret;


	/* Initialize the thread that will wait for the asynch channels */
	rtl_pthread_attr_init( &attr );
	sched_param.sched_priority = rtl_sched_get_priority_min(RTL_SCHED_FIFO);
	rtl_pthread_attr_setschedparam( &attr, &sched_param );
#ifdef RTL_PRO	
	rtl_pthread_attr_setfp_np(&attr, 1);
#endif
	switch(app_id){
		case 1:
			if ( (ret=rtl_pthread_create( &my_app_th, &attr, my_app1, (void *)((int)app_id) )) ){
				ERROR_MSG("Init Fatal Error: pthread_create returned %d", ret);
				return -1;
			}
			break;
		case 2:
		case 4:
			if ( (ret=rtl_pthread_create( &my_app_th, &attr, my_app2, (void *)((int)app_id) )) ){
				ERROR_MSG("Init Fatal Error: pthread_create returned %d", ret);
				return -1;
			}
			break;
		case 3:
			if ( (ret=rtl_pthread_create( &my_app_th, &attr, my_app3, (void *)((int)app_id) )) ){
				ERROR_MSG("Init Fatal Error: pthread_create returned %d", ret);
				return -1;
			}
			break;

		default:
			PRINT_MSG("This application ID has no application test available");
	}
	return 0;
}

void my_app_close(unsigned char app_id)
{
	
	switch(app_id){
		case 1:
			my_app1_close();
			break;
		case 2:
		case 4:
			my_app2_close();
			break;
		case 3:
			my_app3_close();
			break;
		default:
			PRINT_MSG("This application ID has no application test available");
	}
	
	PRINT_MSG("Going for my_app join ");
	
	/* cancel the my_app thread */
	rtl_pthread_cancel( my_app_th );
	rtl_pthread_join( my_app_th, NULL );

	PRINT_MSG("Joined ");

	return ;
}

#endif
