/*****************************************************************************
 * M_debug.module.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/


#include "Ports.h"
#include <unistd.h>
#include <fcntl.h> //for FIFOs

#include <sys/stat.h>

static char fifo_path[40]="/tmp/ftt_master_debug";
static int fifo_fd;
#define DEBUG_FIFO_SIZE 50

static rtl_pthread_t debug_periodic_thread_th;


static signed char Read_debug_fifo( unsigned char *to ) //non_blocking
{
	if ( to == NULL ) return -1;

	/* return 1 if there's a command returned by 'to' pointer */
	return (read( fifo_fd, to, sizeof(unsigned char) ) == sizeof(unsigned char) );
}

static void *debug_periodic_thread(void *t)
{
	struct rtl_timespec next_activation; //used to time activations	
//	int ret;
	unsigned char get;

	/* get the current time and setup for the first tick */
	rtl_clock_gettime( RTL_CLOCK_REALTIME, &next_activation );

	while (1){
		rtl_timespec_add_ns( &next_activation, 1000000*500 ); //500ms
		/* sleep */
		rtl_clock_nanosleep( RTL_CLOCK_REALTIME, RTL_TIMER_ABSTIME, &next_activation, NULL);
		//PRINT_MSG("Reading from DEBUG FIFO");
		if ( Read_debug_fifo( &get ) ){
			PRINT_MSG("Got a command: %c", get);
			if (get == '1'){	/*echo -n "1" > /tmp/ftt_master*/
				PRINT_MSG("Printing var table");
				M_FTT_CORE_L_print_db();
			}
			if (get == '2'){	/*echo -n "2" > /tmp/ftt_master*/
				PRINT_MSG("Printing Management table");
				M_FTT_MANAGEMENT_L_print(1);
			}
			if (get == '3'){	/*echo -n "3" > /tmp/ftt_master*/
				PRINT_MSG("Printing APPs table");
				M_int_nodes_db_PrintTable();
			}
			if (get == '4'){	/*echo -n "4" > /tmp/ftt_master*/
				PRINT_MSG("Printing SRDB chcklist");
				M_SRDB_system_checklist_print();
			}

		}

		
	}
}

signed char M_DEBUG_MODULE_init( void )
{
	rtl_pthread_attr_t attr;
	struct rtl_sched_param sched_param;
	int ret;

	PRINT_MSG("Initializing layer");

	/******FIFO******/
	if ( mkfifo( fifo_path, 0600) < 0 ){
		ERROR_MSG("Error executing mkfifo(%s, 0600)", fifo_path );
		return -3;
	}

	fifo_fd = open( fifo_path, O_NONBLOCK );
	ftruncate( fifo_fd, DEBUG_FIFO_SIZE);


	/* Periodic thread */
	rtl_pthread_attr_init( &attr );
	sched_param.sched_priority = rtl_sched_get_priority_min(RTL_SCHED_FIFO);
	rtl_pthread_attr_setschedparam( &attr, &sched_param );
#ifdef RTL_PRO	
	rtl_pthread_attr_setfp_np(&attr, 1);
#endif
	if ( (ret=rtl_pthread_create( &debug_periodic_thread_th, &attr, debug_periodic_thread, (void *)0 )) ){
		ERROR_MSG("Init Fatal Error: pthread_create returned %d", ret);
		return -1;
	}
	return 0;
}

signed char M_DEBUG_MODULE_close(void)
{
	/* Periodic thread */
	rtl_pthread_cancel( debug_periodic_thread_th );
	rtl_pthread_join( debug_periodic_thread_th, NULL );


	/******FIFO******/
	close( fifo_fd );
	unlink( fifo_path );

	return 0;
}

