/*****************************************************************************
 * ethernet.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *         Augusto Born  (QNX port)
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#include "Ports.h"

#include "ethernet.layer.h"

#ifdef RTL_PRO
	#include <lnet_eth.h>
#elif defined L26
	#include <sys/socket.h>
	#include <stdlib.h>   //malloc
	#include <linux/if_ether.h> //ETH_P_ALL
	#include <linux/if.h>
	#include <net/ethernet.h>
	#include <sys/ioctl.h>
	#include <netpacket/packet.h>
	#define ETH_FTT_TYPE 	0x8FF0
#elif defined __QNXNTO__
	#include <sys/socket.h>
	#include <stdlib.h>   //malloc
	#include <sys/ioctl.h>
	#include <errno.h>
	#define ETH_FTT_TYPE 	0x8FF0

	#include <sys/neutrino.h>

	#include <sys/types.h>
	#include <sys/socket.h>
	#include <stdio.h>
	#include <errno.h>
	#include <stdlib.h>
	#include <net/if_ether.h>
	#include <net/if_arp.h>
	#include <net/if_dl.h>
	#include <net/if_types.h>
	#include <net/if.h>
	#include <arpa/inet.h>
	#include <string.h>
	#include <time.h>
	#include <sys/time.h>
	#include <netdb.h>
	#include <netinet/in.h>
	#include <netinet/ip.h>
	#include <netinet/ip_icmp.h>
	#include <unistd.h>
	#include <sys/resource.h>	/* setpriority() and getpriority() */
	#include <sys/neutrino.h>
	#include <net/bpf.h>
	#include <sys/types.h>
	#include <sys/stat.h>
	#include <fcntl.h>
	#include <sys/ioctl.h>
	#include <unistd.h>
	#include <net/pfvar.h>
	#include <sched.h>			/* sched_setscheduler() */
	#include <ifaddrs.h>
#else
	#error "Choose any architecture"
#endif


/* Ethernet pool */
#define POOL_SIZE	500

typedef struct{
	/* Ethernet transmission Buffer  */
#ifdef RTL_PRO
	struct rtl_msghdr Tx_eth_msg;
#else
	unsigned char *Tx_eth_msg_l;
#endif
	unsigned int size;
} ETH_MSG_IDX;

/* Linked Listed queues */

typedef struct link_item_idx{
	ETH_MSG_IDX	msg;
	struct link_item_idx *link;
}LINK_ITEM_IDX;

LINK_ITEM_IDX eth_pool[POOL_SIZE];

typedef struct{
	LINK_ITEM_IDX *q;
	rtl_pthread_mutex_t q_mutex;
}MESSAGES_QUEUE;

MESSAGES_QUEUE free_queue;
MESSAGES_QUEUE tx_hold_queue;
MESSAGES_QUEUE tx_queue;
MESSAGES_QUEUE reservation_queue; //typically NULL||single element

static signed char queue_init(MESSAGES_QUEUE *queue_p)
{
	if (rtl_pthread_mutex_init(&(queue_p->q_mutex), 0) < 0){
		ERROR_MSG("Initializing queue mutex");
		return -1;
	}
	queue_p->q = NULL;
	return 0;
}
static signed char queue_close(MESSAGES_QUEUE *queue_p)
{
	if (rtl_pthread_mutex_destroy(&(queue_p->q_mutex)) < 0){
		ERROR_MSG("Destroying queue mutex");
		return -1;
	}

	queue_p->q = NULL;
	return 0;
}

static void queue_add_LIFOway(MESSAGES_QUEUE *queue_p, LINK_ITEM_IDX *p)
{
	rtl_pthread_mutex_lock(&(queue_p->q_mutex));
	
	/* insert in the list top */
	p->link = queue_p->q;
	queue_p->q = p;
	
	rtl_pthread_mutex_unlock(&(queue_p->q_mutex));
}

static void queue_add_FIFOway(MESSAGES_QUEUE *queue_p, LINK_ITEM_IDX *p)
{
	LINK_ITEM_IDX **idx_temp = &(queue_p->q);

	rtl_pthread_mutex_lock(&(queue_p->q_mutex));
	
	/* insert in the list end */
	while ( *idx_temp != NULL ){
		/* Increment */
		idx_temp = &((*idx_temp)->link);
	}

	*idx_temp = p;
	p->link = NULL;
	
	rtl_pthread_mutex_unlock(&(queue_p->q_mutex));
}

#define queue_get_FIFOway queue_get_LIFOway

static signed char queue_get_LIFOway(MESSAGES_QUEUE *queue_p, LINK_ITEM_IDX **ret_p)
{
	rtl_pthread_mutex_lock(&(queue_p->q_mutex));

	if ((queue_p->q) == NULL){
		rtl_pthread_mutex_unlock(&(queue_p->q_mutex));
		return -1;
	}

	*ret_p = (queue_p->q);
	(queue_p->q) = (queue_p->q)->link;
	
	rtl_pthread_mutex_unlock(&(queue_p->q_mutex));

	return 0;
}
//NOT USED
//static void queue_print( MESSAGES_QUEUE *queue_p )
//{
//	LINK_ITEM_IDX **idx_temp = &(queue_p->q);
//
//	rtl_pthread_mutex_lock(&(queue_p->q_mutex));
//	PRINTF("\n Start=");
//	while (1){
//		if (*idx_temp == NULL){
//			PRINTF(" NULL");
//			break;
//		}
//		PRINTF(" %p ->", *idx_temp);
//
//		/* Increment */
//		idx_temp = &((*idx_temp)->link);
//	}
//	rtl_pthread_mutex_unlock(&(queue_p->q_mutex));
//	return ;
//}
//static void queue_print_size( MESSAGES_QUEUE *queue_p )
//{
//	LINK_ITEM_IDX **idx_temp = &(queue_p->q);
//	unsigned short counter=0;
//
//	rtl_pthread_mutex_lock(&(queue_p->q_mutex));
//	while (1){
//		if (*idx_temp == NULL){
//			break;
//		}
//		
//		counter ++;
//
//		/* Increment */
//		idx_temp = &((*idx_temp)->link);
//	}
//	PRINTF("\n queue size =%d", counter);
//	rtl_pthread_mutex_unlock(&(queue_p->q_mutex));
//	return ;
//}

static unsigned char queue_isEmpty( MESSAGES_QUEUE *queue_p) 
{
	unsigned char ret;

	rtl_pthread_mutex_lock(&(queue_p->q_mutex));
	ret = (queue_p->q == NULL);
	rtl_pthread_mutex_unlock(&(queue_p->q_mutex));
	
	return (ret);
}

static signed char queues_init(void)
{
	int i;

	queue_init(&free_queue);
	queue_init(&tx_hold_queue);
	queue_init(&tx_queue);
	queue_init(&reservation_queue);

	for (i=0; i<POOL_SIZE; i++){
#ifdef RTL_PRO		
		/* Alloc DMAable memory */
		if(!( eth_pool[i].msg.Tx_eth_msg.msg_iov[0].iov_base = lnet_alloc(LNET_ETH_MAXTU) )){
			ERROR_MSG("Net memory alloc failure!");
			return -1;
		}
#else
		if(!( eth_pool[i].msg.Tx_eth_msg_l = malloc(1514) )){
			ERROR_MSG("Net memory alloc failure!");
			return -1;
		}
#endif
		/* insert in the free queue */
		queue_add_LIFOway(&free_queue, &(eth_pool[i]) );
	}

	return 0;
}

static signed char queues_close(void)
{
	int i;
	
	queue_close(&free_queue);
	queue_close(&tx_hold_queue);
	queue_close(&tx_queue);
	queue_close(&reservation_queue);

	for (i=0; i<POOL_SIZE; i++){
#ifdef RTL_PRO		
		lnet_free(eth_pool[i].msg.Tx_eth_msg.msg_iov[0].iov_base);
#else
		free(eth_pool[i].msg.Tx_eth_msg_l);
#endif
	}
	return 0;
}

static FTT_SEMAPHORE sem_TXtask_handler;
static rtl_pthread_t TXtask_handler_th;

static rtl_pthread_mutex_t tx_mutex;

/* Socket descriptor */
static int sd; //both for LNET and Linux


static void *TXtask_handler(void *t)
{
	LINK_ITEM_IDX *p_temp;

	while (1){
		/* Wait for a message tx event */
		if ( FTT_SEMAPHORE_wait(&sem_TXtask_handler) < 0 )
			return NULL;
//		PRINT_MSG("Txhandler starting");

		rtl_pthread_mutex_lock(&tx_mutex);

		/* Remove the message from the tx_queue and put it in the free */
		if ( queue_get_FIFOway( &tx_queue, &p_temp ) < 0 ) {
			ERROR_MSG("Abnormal situation");
			continue;
		}
		queue_add_LIFOway( &free_queue, p_temp );

		rtl_pthread_mutex_unlock(&tx_mutex);
	
		/* a tx_event means that some LNET resources have been free
		 * let's try to xmit any hold message */
		if ( queue_get_FIFOway( &tx_hold_queue, &p_temp ) == 0 ) {
			if ( 1 /*send_success*/ ){
				/* Add in transmisssion queue */
				queue_add_FIFOway( &tx_queue, p_temp );
			}else{
				/* Add in transmisssion hold queue */
				queue_add_FIFOway( &tx_hold_queue, p_temp );
			}
		}
	}
	return NULL;
}

static void (*rx_handler)(unsigned char *base_add, unsigned short size, struct rtl_timespec rx_timestamp);

/* Where goes the architecture dependent code LNET/Linux */
#ifdef RTL_PRO
static void tx_event(void *priv, struct rtl_msghdr *msg, unsigned long status)
{
//	FTT_PRINTF("E");
	FTT_SEMAPHORE_post(&sem_TXtask_handler);
}

static void rx_event(void *priv)
{
	struct rtl_timespec rx_timestamp; /* Mesg Rx timestamp */
	int ret;
	struct rtl_msghdr rmsg;

	/* Timestamp the message */
	rtl_clock_gettime( RTL_CLOCK_REALTIME, &rx_timestamp );
rx_event_lopp:
	if ( (ret=rtl_recvmsg(sd, &rmsg, 0)) <= 0 ) {
		//ERROR_MSG("Rx_event Error rtl_recvmsg() returned %d", ret);
		return;
	}

	/* ret is the packet size */

	if (rx_handler != NULL)
		rx_handler( (unsigned char *)rmsg.msg_iov[0].iov_base, LNET_ETH_MAXTU, rx_timestamp );

	if ( (ret=rtl_fcntl(sd, LNET_FCNTL_RETURN_MSG, &rmsg)) ) {
		ERROR_MSG("Rx_event Error: rtl_fcntl(RETURN_MSG) returned %d", ret);
		return;
	}

	/* One event might hold more than one packet */
	goto rx_event_lopp;
}

static inline signed char ethernet_driver_init(unsigned char dev_num, char *dev_name, unsigned char *ret_mac)
{
	int ret; //temp
	char device[6];
	struct rtl_sockaddr socket_addr;


	/* open a LNet Ethernet socket with protocol=DEVICE */
	ret = rtl_socket(LNET_DOMAIN_ETH, LNET_TYPE_RT, LNET_PROTOCOL_DEVICE);
	if (ret < 0){
		ERROR_MSG("Init Fatal Error: rtl_socket() returned %d", ret);
		return -2;
	}
	sd = ret;

	sprintf(device, "%s%d", LNET_SA_DATA_DSTR_ETHERNET, dev_num);
	socket_addr.sa_data = device;
	socket_addr.sa_family = LNET_FAMILY_ETH;
	ret = rtl_bind(sd, &socket_addr, sizeof(socket_addr));
	if (ret)
	{
		rtl_printf("\n[Dispatcher.c] Init Fatal Error: rtl_bind() returned %d", ret);
		return -3;
	}

	/* Get the MAC address */
	if (ret_mac!=NULL){
		rtl_fcntl(sd,LNET_FCNTL_GUID, ret_mac);
//		PRINT_MSG("Host MAC address:"); for (ret=0; ret<6; ret++) PRINTF(" %2x",ret_mac[ret]);
	}

	/* assign a Tx event handler to this socket */
	if ( (ret = rtl_fcntl(sd, LNET_FCNTL_TX_NOTIFY, tx_event, NULL)) ){
		ERROR_MSG("Init Fatal Error: rtl_fcntl(TX_NOTIFY) returned %d", ret);
		return -4;
	}
	
	if ((ret = rtl_fcntl(sd, LNET_FCNTL_PROMISC, LNET_PROMISC_ON)))
	{
		ERROR_MSG("Configuring Promiscuous...%d",ret);
		return -5;
	}

	/* Assign a Rx event handler to this socket */
	if ((ret = rtl_fcntl(sd, LNET_FCNTL_RX_NOTIFY, rx_event, NULL)))
	{
		ERROR_MSG("Init: Rx event handler not created...%d",ret);
		return -6;
	}
	return 0;
}

static inline signed char ethernet_driver_close(void)
{
	if (sd == -1)
		return-1; /* No socket has beeen created */
	
	rtl_shutdown(sd, 0);
	sd = -1;
	return 0;
}

static signed char ethernet_driver_sendmessage(LINK_ITEM_IDX *p)
{
	struct rtl_msghdr *temp_msg = &(p->msg.Tx_eth_msg);
	unsigned short t_size = p->msg.size;
	int ret; //temp

	temp_msg->msg_iovlen = 1;
	temp_msg->msg_namelen = LNET_ETH_GUID_LEN;
	temp_msg->msg_iov[0].iov_len = t_size;


	if( (ret = rtl_sendmsg(sd, temp_msg, LNET_SENDFLAG_COOKED)) < 0){
	//if( (ret = rtl_sendmsg(sd, temp_msg, LNET_SENDFLAG_COOKED)) < 0){
		ERROR_MSG("In %d rtl_sendmsg() returned: %d", 0, ret);
		return -1;
	}
//	FTT_PRINTF("S");
	return 0;
}

#elif defined(__QNXNTO__) // qnx version (uses bsd packet filter)

static rtl_pthread_t rx_event_th;
static unsigned char buf_rx[ETHER_MAX_LEN];

static unsigned char * bpf_buf_rx;
static int bpf_buf_len;

static unsigned char src_mac[6];

void *rx_event(void *t){ // qnx: done
	struct rtl_timespec rx_timestamp; /* Mesg Rx timestamp */
	int BytesReceived = -1;
	
	ERROR_MSG("qnx: before rx_event loop");
	
	while (1) {
		
		// Flushes the buffer of incoming packets and resets the statistics that are returned by BIOCGSTATS.
		
		// qnx: Pedro says this is necessary before every read 
		
		/*
		 
		 if( ioctl( sd, BIOCFLUSH ) == -1) {
		 ERROR_MSG( "BIOCFLUSH failed" );
		 return(EXIT_FAILURE);
		 
		 }
		 
		 */
		memset(bpf_buf_rx, 0, bpf_buf_len); // qnx: needed for trailing bytes from different sized bufs
		
		BytesReceived = read(sd, bpf_buf_rx, bpf_buf_len);
		if (BytesReceived == -1) {
			ERROR_MSG("read()");
			return NULL;
		}
		
		// qnx: handle more than 1 packet in a bpf buffer
		
		/* Timestamp the message */
		
		struct bpf_hdr *bpf_ptr = (struct bpf_hdr *) bpf_buf_rx;
		unsigned char * eth_ptr;
		
		int pcount = 0;
        
		while((unsigned char *) bpf_ptr < bpf_buf_rx + BytesReceived) {
			
			eth_ptr = (unsigned char *) bpf_ptr + bpf_ptr->bh_hdrlen;
			
			memset(buf_rx, 0, ETHER_MAX_LEN);
			memcpy(buf_rx, eth_ptr, bpf_ptr->bh_caplen); // qnx: try to kill this TODO
			rtl_clock_gettime( CLOCK_REALTIME, &rx_timestamp ); // qnx: one per packet or one per read? bpf has bh_tstamp on each bpf_hdr, investigate!
			
			//PRINT_MSG("will handle a msg!! curr bpf buf: %d curr pkt number %d curr caplen %d", BytesReceived, pcount, bpf_ptr->bh_caplen);
		    if (rx_handler != NULL)
				rx_handler( (unsigned char *)buf_rx, ETHER_MAX_LEN, rx_timestamp );
			//PRINT_MSG("after handing a msg!! curr bpf buf: %d curr pkt number %d curr caplen %d", BytesReceived, pcount, bpf_ptr->bh_caplen);
			
			pcount++;
			bpf_ptr = (void *) ((unsigned char *) bpf_ptr + BPF_WORDALIGN(bpf_ptr->bh_hdrlen + bpf_ptr->bh_caplen));
			//PRINT_MSG("after handing a msg!! curr bpf ptr: %d, curr end ptr: %d", bpf_ptr, bpf_buf_rx + BytesReceived);
			
		}
		
		
	}
	
}

static inline signed char ethernet_driver_init(unsigned char dev_num, char *dev_name, unsigned char *ret_mac)
{
	
	
	ERROR_MSG("qnx: before ethernet_driver_init");
    
	// adjust clock in QNX to faster than default (def: 1ms)
	struct _clockperiod clkper; // to control the clock period 
	clkper.nsec = 50000; 
	clkper.fract = 0;   
	ClockPeriod(CLOCK_REALTIME,&clkper,NULL,0); 
	
	
	/* ################# START DEFINITIONS ######################*/
	int bpf = -1;
	char buf[11] = {0}; // qnx: for dev name string
	int i = 0, buf_len = 1;
	
	char interface[6];
	int found = 0;
	char * iface_prefixes[] = {"wm", "emac", "en"}; // qnx: iface name depends on driver, save some headache by brute forcing here

	if (dev_name != NULL){
#warning Ricardo: Augusto, now you're able to pass the device name string, chack the dev_name parameter. Dind't touch the code. Hope the headache is gone !	
		ERROR_MSG("Ricardo: Augusto, now you're able to pass the device name string, chack the dev_name parameter. Dind't touch the code. Hope the headache is gone");
		return -100;
	}
	
	int j = 0;
	struct ifaddrs		*ifaddrs, *ifap;
	struct sockaddr_dl *sdl = NULL;
	
	if (getifaddrs(&ifaddrs) != 0) {
		ERROR_MSG("getifaddrs");
		return(EXIT_FAILURE);
	}
	
	while(found == 0 && j < (sizeof iface_prefixes / sizeof (char*))) {
		sprintf( interface, "%s%i", iface_prefixes[j], dev_num );
		
		for (ifap = ifaddrs; ifap != NULL; ifap = ifap->ifa_next) {
			if(strcmp(ifap->ifa_name, interface) == 0) {
				found = 1;
				break;
			}
		}
		
		j++;
	}
	
	printf("qnx: using %s\n", interface);
	
	sdl = (struct sockaddr_dl *)ifap->ifa_addr;
	memcpy(src_mac, LLADDR(sdl), sdl->sdl_alen);
	
	freeifaddrs(ifaddrs);
	
	printf("Host MAC address: %02X:%02X:%02X:%02X:%02X:%02X\n", 
		   src_mac[0],src_mac[1],src_mac[2],src_mac[3],src_mac[4],src_mac[5]);
	
	
	if(ret_mac != NULL) {
		int mac_i;
		for(mac_i = 0; mac_i < 6; mac_i++) {
			ret_mac[mac_i] = src_mac[mac_i];
		}
	}
	
	
	
#define ON 1
#define OFF !ON
	
	/* Try to open the next available BPF interface */
	for (i=0;i<99;i++) {
		sprintf( buf, "/dev/bpf%i", i );
		
		/* Try to open the /dev/bpfi device */
		bpf = open( buf, O_RDWR );
		if( bpf != -1 )
			break;
	}
	
	sd = bpf;
	
	if (bpf == -1) {
		ERROR_MSG("Couldn't open BPF interface (More than 99?)");
		return(EXIT_FAILURE);
	}
	
	/*
	 // request buffer length
	 if( ioctl( bpf, BIOCGBLEN, &buf_len ) == -1 ){
	 ERROR_MSG( "BIOCGBLEN failed" );
	 return(EXIT_FAILURE);
	 }
	 
	 
	 int bufsize = 170;
	 if( ioctl( bpf, BIOCSBLEN, &bufsize ) == -1 ){
	 perror( "BIOCSBLEN failed" );
	 return(EXIT_FAILURE);
	 }*/
	
	// request buffer length
	if( ioctl( bpf, BIOCGBLEN, &buf_len ) == -1 ){
		ERROR_MSG( "BIOCGBLEN failed" );
		return(EXIT_FAILURE);
	}
	
	bpf_buf_len = buf_len;
	
	PRINT_MSG("buffer: %d", bpf_buf_len);
	bpf_buf_rx = (unsigned char *) malloc(buf_len);
	
	// Associate the BPF device with the network interface
	struct ifreq bound_if;
	strcpy( bound_if.ifr_name, interface );
	if( (ioctl( bpf, BIOCSETIF, &bound_if )) < 0 ) {
		ERROR_MSG( "BIOCSETIF failed");
		return(EXIT_FAILURE);
	}
	
	// Activate immediate mode
	int immed_mode = ON;
	if( ioctl( bpf, BIOCIMMEDIATE, &immed_mode ) == -1 ) {
		ERROR_MSG( "BIOCIMMEDIATE failed" );
		return(EXIT_FAILURE);
	}
	
	
	// Hide what I send
	int seesent = OFF;
	if( ioctl( bpf, BIOCSSEESENT, &seesent ) == -1 ) {
		ERROR_MSG( "BIOSSEESENT failed" );
		return(EXIT_FAILURE);
	}
	
	
	// We want to add our own Header (source and destination MAC address and Ethernet Type)
	int manual_header = ON;	// ON = Manual header insertion | OFF = Automatic kernel header insertion
	if( ioctl( bpf, BIOCSHDRCMPLT, &manual_header ) == -1) {
		ERROR_MSG( "BIOCSHDRCMPLT failed" );
		return(EXIT_FAILURE);
	}
	
	
	/* The rx_handler */
	{
		rtl_pthread_attr_t attr;
		struct sched_param sched_param;
		
		rtl_pthread_attr_init( &attr );
		sched_param.sched_priority = sched_get_priority_max(SCHED_FIFO); // QNX: do equivalent --- already works?
		rtl_pthread_attr_setschedparam( &attr, &sched_param );
		
		/*from pedro:
		 struct sched_param schedParameters;
		 schedParameters.sched_priority = 99;
		 id_t pid = getpid();
		 if ( sched_setscheduler(pid, SCHED_FIFO, &schedParameters) == -1) {
		 perror("sched_setscheduler()");
		 return EXIT_FAILURE;
		 }*/
		
		
		if ( rtl_pthread_create( & rx_event_th, &attr, rx_event, (void *)0 ) ){
			ERROR_MSG("Fatal Error: pthread_create");
			return -2;
		}
#ifdef PRINT_THREAD_ID
		
		ERROR_MSG("New thread id: %llu", rx_event_th);
#endif
		
	}
	
	ERROR_MSG("qnx: after ethernet_driver_init");
	
	return 0;
}

static inline signed char ethernet_driver_close(void) // qnx: done
{
    ERROR_MSG("qnx: before ethernet_driver_close");
	
    //	struct ifreq ifr;
	
    /* cancel the ftt_dispatcher thread */
    rtl_pthread_cancel( rx_event_th );
    rtl_pthread_join( rx_event_th, NULL );
	
    if (sd == -1)
        return-1; /* No socket has beeen created */
	
    //	strncpy(ifr.ifr_name, ETH_DEVICE, IFNAMSIZ);
    //	ioctl(sd, SIOCGIFFLAGS, &ifr);
    //	ifr.ifr_flags &= ~IFF_PROMISC;
    //	ioctl(sd, SIOCSIFFLAGS, &ifr);
	
	
    close(sd);
	
    free(bpf_buf_rx);
	
    ERROR_MSG("qnx: after ethernet_driver_close");
	
    return 0;
}

static signed char ethernet_driver_sendmessage(LINK_ITEM_IDX *p) // qnx: done
{
    /*Send packet*/
    //ERROR_MSG("qnx: before ethernet_driver_sendmessage");
    //ERROR_MSG("qnx: msgsize %d", p->msg.size);
	
	
    //printf("qnx: outgoing MAC address before: %02X:%02X:%02X:%02X:%02X:%02X\n", 
    //(p->msg.Tx_eth_msg_l+6)[0],(p->msg.Tx_eth_msg_l+6)[1],(p->msg.Tx_eth_msg_l+6)[2],(p->msg.Tx_eth_msg_l+6)[3],(p->msg.Tx_eth_msg_l+6)[4],(p->msg.Tx_eth_msg_l+6)[5]);
	
    /* qnx:
	 int i;
	 for (i = 0; i < 6; i++)
	 (p->msg.Tx_eth_msg_l+6)[i]=src_mac[i];
	 */
	
    //printf("qnx: outgoing MAC address after: %02X:%02X:%02X:%02X:%02X:%02X\n", 
    //(p->msg.Tx_eth_msg_l+6)[0],(p->msg.Tx_eth_msg_l+6)[1],(p->msg.Tx_eth_msg_l+6)[2],(p->msg.Tx_eth_msg_l+6)[3],(p->msg.Tx_eth_msg_l+6)[4],(p->msg.Tx_eth_msg_l+6)[5]);
	
	
    if( write(sd, p->msg.Tx_eth_msg_l, p->msg.size) == -1) {
        ERROR_MSG("Could not send the message");
		
        printf( "QNX: Error writing to BPF: %s\n", strerror( errno ) );
        //returning zero because I don't know the error origin
        return -1;
    }
	
    FTT_SEMAPHORE_post( &sem_TXtask_handler);
	
    //ERROR_MSG("qnx: after ethernet_driver_sendmessage");
	
    return 0;
}


#else // linux code L26

static rtl_pthread_t rx_event_th;
static unsigned char buf_rx[ETHER_MAX_LEN];


void *rx_event(void *t){
	struct rtl_timespec rx_timestamp; /* Mesg Rx timestamp */
	int ret;

	while (1) {
		if ( (ret = recv(sd, buf_rx, ETHER_MAX_LEN, 0)) == -1) {
			ERROR_MSG("recv() ret: %d\n", ret);
			exit(1);
		}

		/* Timestamp the message */
		//rtl_clock_gettime( CLOCK_REALTIME, &rx_timestamp );
		ioctl(sd, SIOCGSTAMPNS, &rx_timestamp);

		if (rx_handler != NULL) rx_handler( (unsigned char *)buf_rx, ETHER_MAX_LEN, rx_timestamp );

	}

}

static inline signed char ethernet_driver_init(unsigned char dev_num, char *dev_name, unsigned char *ret_mac)
{
	int ret; //temp
	char device[6];
	struct ifreq ifr;
	int ifindex = 0;	/*Ethernet Interface index*/
	struct  sockaddr_ll socket_address;
	unsigned char src_mac[6];	/*Local host NIC MAC address*/
	
	
	ret = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
	if (ret == -1){
		ERROR_MSG("socket() returned: %d         try as root", ret);
		return -1;
	}
	sd = ret;

//	PRINT_MSG("Successfully opened socket: %i", sd);
	
	/* Let's get the ethernet interface index --- "linux/if.h"*/
	
	if (dev_name == NULL)
		sprintf(device, "eth%d", dev_num);
	else
		strcpy(device, dev_name);
//	PRINT_MSG("Openning ethernet device %s", device);

	
	strncpy(ifr.ifr_name, device, IFNAMSIZ);
	if (ioctl(sd, SIOCGIFINDEX, &ifr) == -1) {
		ERROR_MSG("SIOCGIFINDEX - %s not present", device);
		return -2;
	}
	ifindex = ifr.ifr_ifindex;
	//PRINT_MSG("Successfully got interface index: %i", ifindex);

	/* Let's get the MAC Address of the device */
	if (ioctl(sd, SIOCGIFHWADDR, &ifr) == -1) {
		ERROR_MSG("SIOCGIFHWADDR");
		return -3;
	}
	
	/* Set promiscuous */
	ifr.ifr_flags |= IFF_PROMISC;
//	ifr.ifr_flags |= IFF_NOARP;
//	ifr.ifr_flags |= IFF_POINTOPOINT;
	ioctl(sd, SIOCSIFFLAGS, &ifr);

	memcpy(src_mac, ifr.ifr_hwaddr.sa_data, 6);
	if (ret_mac!=NULL) memcpy(ret_mac, ifr.ifr_hwaddr.sa_data, 6);
//	for (i = 0; i < 6; i++) {
//		src_mac[i] = ifr.ifr_hwaddr.sa_data[i];
//		if (ret_mac!=NULL) ret_mac[i] = src_mac[i];
//	}
	
//	PRINT_MSG("Got MAC address: %02X:%02X:%02X:%02X:%02X:%02X", 
//		src_mac[0],src_mac[1],src_mac[2],src_mac[3],src_mac[4],src_mac[5]);
	
	/* Finaly Let's bind the socket namespace to the device --- man 7 packet */
	socket_address.sll_family = PF_PACKET;
	socket_address.sll_protocol = htons(ETH_P_ALL);
//	socket_address.sll_protocol = htons(ETH_FTT_TYPE);
	socket_address.sll_ifindex = ifindex;
	//socket_address.sll_hatype = PF_PACKET;
	socket_address.sll_pkttype = PACKET_BROADCAST;	//Reception use see man 7 packet
	socket_address.sll_halen = ETH_ALEN;
	memcpy(socket_address.sll_addr, src_mac, 6);

	bind (sd, (struct sockaddr*)&socket_address, sizeof(socket_address));
	
	/* The rx_handler */
	{
		rtl_pthread_attr_t attr;
		struct sched_param sched_param;
	
		rtl_pthread_attr_init( &attr );
		sched_param.sched_priority = sched_get_priority_max(SCHED_FIFO);
		rtl_pthread_attr_setschedparam( &attr, &sched_param );

		if ( rtl_pthread_create( & rx_event_th, &attr, rx_event, (void *)0 ) ){
			ERROR_MSG("Fatal Error: pthread_create");
			return -2;
		}
#ifdef PRINT_THREAD_ID
		ERROR_MSG("New thread id: %lu", rx_event_th);
#endif

	}

	return 0;
}

static inline signed char ethernet_driver_close(void)
{
//	struct ifreq ifr;

	/* cancel the ftt_dispatcher thread */
	rtl_pthread_cancel( rx_event_th );
	rtl_pthread_join( rx_event_th, NULL );
	
	if (sd == -1)
		return-1; /* No socket has beeen created */

//	strncpy(ifr.ifr_name, ETH_DEVICE, IFNAMSIZ);
//	ioctl(sd, SIOCGIFFLAGS, &ifr);
//	ifr.ifr_flags &= ~IFF_PROMISC;
//	ioctl(sd, SIOCSIFFLAGS, &ifr);
	
	shutdown(sd,0);
	return 0;
}

static signed char ethernet_driver_sendmessage(LINK_ITEM_IDX *p)
{
//	char i;
//	for (i = 0; i < 6; i++)
//		(p->msg.Tx_eth_msg_l+6)[i]=src_mac[i];

	/*Send packet*/
	if ( send(sd, p->msg.Tx_eth_msg_l , p->msg.size, 0) == -1) {
		ERROR_MSG("Could not send the message");
		//returning zero because I don't know the error origin
		return -1;
	}
	
	FTT_SEMAPHORE_post(&sem_TXtask_handler);
	return 0;
}
#endif

signed char ETH_init(unsigned char dev_num, char *dev_name, unsigned char *ret_mac)
{
	int ret; //temp
	rtl_pthread_attr_t attr;
	struct rtl_sched_param sched_param;

	if (queues_init() < 0)	return -1;
	
	/* Create tx handling function */
	if ( FTT_SEMAPHORE_init(&sem_TXtask_handler, 0) < 0 ){	//Initialization locked
		ERROR_MSG("Couldn't initialize semaphore");
		return -2;
	}
	

	/* Create the conditional var to decide whether a
	 * sent msg goes for the transmission queue or tx_hold */
	rtl_pthread_mutex_init(&tx_mutex, 0);

	rtl_pthread_attr_init( &attr );
	sched_param.sched_priority = rtl_sched_get_priority_max(RTL_SCHED_FIFO);
	rtl_pthread_attr_setschedparam( &attr, &sched_param );
#ifdef RTL_PRO
	rtl_pthread_attr_setfp_np(&attr, 1);
#endif
	if ( (ret = rtl_pthread_create( &TXtask_handler_th, &attr, TXtask_handler, 0)) != 0 ){
		ERROR_MSG("Init Fatal Error: pthread_create returned %d", ret);
		FTT_SEMAPHORE_destroy(&sem_TXtask_handler);
		queues_close();
		return -3;
	}
#ifdef PRINT_THREAD_ID
#ifdef RTL_PRO
	ERROR_MSG("New thread id: %d", TXtask_handler_th);
#else
	ERROR_MSG("New thread id: %lu", TXtask_handler_th);
#endif
#endif


	/* make sure the socket descriptor is off */
	sd = -1;
	rx_handler = NULL;

	if ( (ret = ethernet_driver_init(dev_num, dev_name, ret_mac)) < 0 )
		return ret;

//	PRINT_MSG("Net init: Ok");
	
	return 0;
}

signed char ETH_close(void)
{
	rx_handler = NULL;

	ethernet_driver_close();

	/* cancel the ftt_dispatcher thread */
	rtl_pthread_cancel( TXtask_handler_th );
	rtl_pthread_join( TXtask_handler_th, NULL );

	/* Clean up the conditional var */
	rtl_pthread_mutex_destroy(&tx_mutex);

	/* Clean up the semaphore */
	FTT_SEMAPHORE_destroy(&sem_TXtask_handler);
	
	if (queues_close() < 0) return -1;
	
//	PRINT_MSG("Net close: Ok");

	return 0;
}

void ETH_rx_assignement(void *fun)
{
	rx_handler = fun;
	return;
}
void ETH_rx_deassign (void)
{
	rx_handler = NULL;
	return;
}

signed char ETH_pre_reserve_buffer( unsigned char **ret_buf_p )
{
	LINK_ITEM_IDX *p_temp;

	if ( queue_get_LIFOway( &free_queue, &p_temp) < 0)
		return -1;

	queue_add_FIFOway( &reservation_queue, p_temp );
#ifdef RTL_PRO	
	*ret_buf_p = p_temp->msg.Tx_eth_msg.msg_iov[0].iov_base;
#else
	*ret_buf_p = p_temp->msg.Tx_eth_msg_l;
#endif
	return 0;
}

signed char ETH_un_reserve_buffer(void)
{
	LINK_ITEM_IDX *p_temp;

	queue_get_FIFOway( &reservation_queue, &p_temp );
	queue_add_LIFOway( &free_queue, p_temp );

	return 0;
}

signed char ETH_Send_MessageBuffer(unsigned char *buf, unsigned short m_size)
{
	LINK_ITEM_IDX *p_temp;
	unsigned char *dest_p;
	unsigned short i;

	
	if ( queue_isEmpty(&reservation_queue) != 0 ){
		/* True */
		if (buf==NULL){
			ERROR_MSG("Buffer was NULL and no pre-reserved buffer");
			return -1;
		}else{
			/* We have to download from the buf pointer */
			if ( queue_get_LIFOway( &free_queue, &p_temp) < 0)
				return -2;
#ifdef RTL_PRO
			dest_p = p_temp->msg.Tx_eth_msg.msg_iov[0].iov_base;
#else
			dest_p = p_temp->msg.Tx_eth_msg_l;
#endif
			for (i=0; i<m_size; i++) dest_p[i]=buf[i];
		}	
	
	}else{
		/* False - There is at least one pre reservation */
		queue_get_FIFOway( &reservation_queue, &p_temp );
	}

	/* Assign in the structure the message size */
	p_temp->msg.size = m_size;

	rtl_pthread_mutex_lock(&tx_mutex);
	if ( ethernet_driver_sendmessage(p_temp) == 0 ){
		/* Add in transmisssion queue */
		queue_add_FIFOway( &tx_queue, p_temp );
	}else{
		ERROR_MSG("Got error in the low level tx");
		/* Add in transmisssion hold queue */
		queue_add_FIFOway( &tx_hold_queue, p_temp );
	}
	rtl_pthread_mutex_unlock(&tx_mutex);
	return 0;
}

