/*****************************************************************************
 * ethernet.layer.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#define ETHERNET_LAYER_C

#include "Ports.h"
#include "ethernet.h"

static unsigned char ETH_L_init_flag=0;

signed char ETH_L_init(unsigned char dev_num, char *dev_name, unsigned char *ret_mac)
{
	signed char ret;
#ifdef POLITE_LAYER
	PRINT_MSG("Initializing the Ethernet layer");

#endif
	if ( (ret=ETH_init(dev_num, dev_name, ret_mac))==0 )
		ETH_L_init_flag=1;
	else
		ETH_L_init_flag=0;
	return ret;
}

signed char ETH_L_close(void)
{
#ifdef POLITE_LAYER
	PRINT_MSG("Closing the Ethernet layer");
#endif
	ETH_L_init_flag=0;
	return ETH_close();
}

signed char ETH_L_Init_check(void)
{
	if (ETH_L_init_flag == 0)
		return -1;
	return 0;
}
void ETH_L_rx_fun_register(void *fun)
{
	ETH_rx_assignement(fun);
	return ;
}

void ETH_L_rx_fun_unregister(void)
{
	ETH_rx_deassign();
	return ;
}

signed char ETH_L_tx_reserve_buffer(unsigned char **ret_buf_p)
{
	return ETH_pre_reserve_buffer( ret_buf_p );
}

signed char ETH_L_tx_un_reserve_buffer( void )
{
	return ETH_un_reserve_buffer( );
}

signed char ETH_L_tx_send_messagebuffer(unsigned char *buf, unsigned short m_size)
{
	return ETH_Send_MessageBuffer(buf, m_size);
}

void ETH_L_Str2Addr(unsigned char *add,  unsigned char *ds)
{
	unsigned int t[6];
	int i;
	sscanf(add, "%x:%x:%x:%x:%x:%x", &t[0], &t[1], &t[2], &t[3], &t[4], &t[5]);
	
	for (i=0; i<6; i++) ds[i]=(unsigned char)(t[i]);
	
	return;
/*	
	int i,j;
	unsigned char c;

	i = 0;
	for(j = 0; j < 6; j++) {
		ds[j] = 0;
			while((add[i] != ':') && (add[i] != 0)) {
				c = add[i++];
				if (c <= '9') c = c - '0';
				else c = c - 'A' + 10;
				ds[j] = ds[j] * 16 + c;
			}
		i++;
	}
	return;
 */
}

void ETH_L_CopyMAC_to_from(unsigned char *dest, unsigned char *src)
{
	unsigned char t;
	for (t=0; t<6; t++)
		dest[t]=src[t];
	return ;
}

unsigned char ETH_L_CompareMAC(unsigned char *first, unsigned char *second)
{
	unsigned char t;
	for (t=0; t<6; t++)
		if (first[t]!=second[t])
			return 0;
	return 1;
}

