/*****************************************************************************
 * ftt-global.h:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#ifndef  FTT_GLOBAL_H
#define  FTT_GLOBAL_H

///////////Linux compiling slowing factor
#define L26_SLOWING 1  // Using a factor smaller than 100*1ms is dangerous in L26 
                         //remember that the L26 resolution is 10 ms


#define MST_MAX_SM_VARS	100
#define MST_MAX_AM_VARS 100

#define STA_MAX_SM_VARS	10
#define STA_MAX_AM_VARS 10	//influencia o tamanho sa status ackowledge poll


/** @brief Maximum # of messages in 1 EC (Synchronous and Asynchronous) */
#define EC_MAX_SM_VARS	50	/* isto influencia o tamanho máximo da TM*/
#define EC_MAX_AM_VARS	300
#define EC_MAX_VARS	(EC_MAX_SM_VARS + EC_MAX_AM_VARS)
#define EC_MAX_SIG_MISSING_REQUESTS 50

/** @brief Maximum valid ID of messages */
//#define MAX_ID 		 0x07FF  
#define MAX_ID 		 (MESGID_MASK)

/** @brief Maximum number of messages that a Station can handle */
#define STA_MAX_VAR	 50

#ifdef SW_UNICAST
#define SW_MAX_PORTS	10
#endif

/** @brief Maximum load allowed due to the scheduler (in %) */
#define SCHED_LOAD_BOUND 80   	 

/** @brief Size (in # of messages) of each buffer associated with asynch mesgs transmitted or recieved locally by the node */
#define STA_AM_BUFF_SIZE  1
#define STA_SM_BUFF_SIZE  3

/** @brief Worst case len  of TM: Header(ID+Flags+nmesgs sinc and asinc + ID/Txi of sync. mesgs + ID of asynch mesgs + Prop delay */
#define TM_LEN_bits ((long)( FTT_PKT_TM_REALSIZE_B_from_no( EC_MAX_SM_VARS, EC_MAX_AM_VARS, EC_MAX_SIG_MISSING_REQUESTS ) )*8+FTT_IFS_bits)

#ifdef NODE_ASYNC_REPLY
#define POLL_STATUS_MAX_PAYLOAD ( sizeof(FTT_ASM_HEADER) + STA_MAX_AM_VARS*sizeof(FTT_ASM_MSG_QUEUE_INDEX) )
#define POLL_STATUS_LEN_bits ((long)(FRAME_REALSIZE_B(POLL_STATUS_MAX_PAYLOAD))*8+FTT_IFS_bits)
#endif

/** @brief Bit time in us (for 100Mb/s Ethernet) */
//#define BIT_TIME_US   0.1
#define BIT_TIME_US   ((float)0.01)

/** @brief EC length in bits */
#define EC_LEN_bits(_ec)    (long)(_ec/BIT_TIME_US)

/** @brief Percentage of the EC reserved to the Synch Phase */
//#define SWIN_SHARE	0.10
//#define SWIN_SHARE	0.85
#define SWIN_SHARE	0.50

/** @brief Percentage of the EC reserved to the Asynch Phase */
#define AWIN_SHARE      (float)(1-SWIN_SHARE) /* Percentage of the EC reserved to the Asynch Phase */
                      /* Note : These values should no be used in the code! */
		      /* AWIN_SHARE + SWIN_SHARE cant be 100%, since some space is */
                      /*        used by the trigger message (and other control messages?)*/
                      /*  Use in the code EC_MAX_bits_SP/AP that have already included the overheads */

/** @brief Size of the scanning window (in us) after Trig mesg.\n
This time is intended to allow that all stations decode the TM and set the Tx of the first data message
Human rights equality for everyone... */
#define SCAN_WINDOW 0 /* us */
//#define SCAN_WINDOW_BITS (long)(SCAN_WINDOW/BIT_TIME_US) /* bits */


#define REAL_IDLE_TIME	200 /* us - Time between the the TM and the transmission of the Synchs */
#if (REAL_IDLE_TIME > SCAN_WINDOW)
	#define DEAD_TIME (REAL_IDLE_TIME)
#else
	#define DEAD_TIME (SCAN_WINDOW)
#endif

#define DEAD_TIME_BITS (long)(DEAD_TIME/BIT_TIME_US) /* bits */

#define EC_EFF_LEN_bits(__ec) (long)(EC_LEN_bits(__ec)-DEAD_TIME_BITS-TM_LEN_bits)

/** @brief EC Maximum Synchronous Phase duration measured in bits */
#define EC_MAX_bits_SP(___ec)	(long)( (EC_EFF_LEN_bits(___ec))*SWIN_SHARE)

/** @brief EC Maximum asynchronous Phase duration measured in bits */
#define EC_MAX_bits_AP(____ec)  (long)( (EC_EFF_LEN_bits(____ec)) - EC_MAX_bits_SP(____ec) )

/** @brief EC Maximum Synchronous Phase duration (in microsecs) */
#define EC_MAX_us_SP(____ec)	(long)(EC_MAX_bits_SP(____ec)*BIT_TIME_US) 

/** @brief EC Maximum Asynchronous Phase duration (in microsecs) */
#define EC_MAX_us_AP(_____ec)    (long)(EC_MAX_bits_AP(_____ec)*BIT_TIME_US) 

#ifdef NODE_ASYNC_REPLY
#define EC_MAX_us_POLL	((long)((DEAD_TIME_BITS + TM_LEN_bits)*BIT_TIME_US))
#endif





/** @defgroup Ethernet_related_constants
 * Ethernet related constants
 * @{
 */

#define ETH_MAC_PREAMBLE_B	8	/* bytes */
#define ETH_MAC_DEST_B		6	/* bytes */
#define ETH_MAC_SOURCE_B	6	/* bytes */
#define ETH_MAC_TYPE_B		2	/* bytes */
#define ETH_MINIMUM_PAYLOAD	46	/* bytes */
#define ETH_MAXIMUM_PAYLOAD	1500/* bytes */
#define ETH_CRC_B			4	/* bytes */
#define ETH_INTERFRAMEGAP_B	12	/* bytes */


#define ETH_DATALEN_MIN	(ETH_MINIMUM_PAYLOAD)
#define ETH_DATALEN_MAX (ETH_MAXIMUM_PAYLOAD)


/** @brief Overhead of an Ethernet Frame:\n
 Dest. Addr (6) + Src. Addr (6) + Type (2)\n
 Note : from a logical point of view, Prelude and FCS not included */
#define ETH_LOGICAL_OVERHEAD	(ETH_MAC_DEST_B + ETH_MAC_SOURCE_B + ETH_MAC_TYPE_B)
#define ETH_OVERHEAD_BYTES		( ETH_MAC_PREAMBLE_B + ETH_LOGICAL_OVERHEAD + ETH_CRC_B + ETH_INTERFRAMEGAP_B )
#define ETH_OVERHEAD_bits ( ETH_OVERHEAD_BYTES*8 )

#define FRAME_LOGICALSIZE_B(_a)	( (_a) + ETH_LOGICAL_OVERHEAD )
#define FRAME_REALSIZE_B(_a)	( (_a) + ETH_OVERHEAD_BYTES )

#define ETH_FRAME_HIDDEN_OVERHEAD ( ETH_MAC_PREAMBLE_B + ETH_CRC_B + ETH_INTERFRAMEGAP_B )
/** @} */




/** @brief Ethernet Frame type ID */
#define ETH_FTT_TYPE 	0x8FF0	

/** @brief Worst-case gap that must be forced between short FTT Data frames, due to non-simultaneous\n 
reception of the TM mesg in all nodes and software jitter (on reception of TM / txmit of data frame */
#define FTT_IFS_bits	        500

/** @brief Minimum allowed size for a data frame in FTT Ethernet\n
(to support bus and software jitter still without collisions) */
#define FTT_MINLEN_bits ((ETH_OVERHEAD_BYTES + ETH_DATALEN_MIN)*8 + FTT_IFS_bits)

/** @brief Value of the Slot time used as a count unit for message tx time indication in the trig. mesg */
#define SLOTV_US    10

#ifdef QNX_TAGGING
typedef unsigned int QNX_TAGGING_TYPE;
#endif

/* FTT Generic */
typedef struct{
	unsigned char eth_dest[ETH_MAC_DEST_B];
	unsigned char eth_src[ETH_MAC_DEST_B];
	unsigned short eth_type;
}__attribute__ ((packed)) FTT_ETH_HEADER;

typedef enum{
	FTT_MST_MSG = 0,
	FTT_SDATA_MSG,
	FTT_ADATA_MSG,
	FTT_ASTATUS_MSG,
	FTT_IDLE,
	FTT_PLUGnPLAY,
	FTT_NOTUSED = 0xFFFF /* only to force the enum size to 2B */
}__attribute__ ((packed)) FTT_PKT_TYPE;

typedef struct{
	FTT_PKT_TYPE type;
}__attribute__ ((packed)) FTT_PKT_HEADER;


typedef struct{
	FTT_ETH_HEADER eth_header;
	FTT_PKT_HEADER pkt_header;
}__attribute__ ((packed)) FTT_PKT_GENERIC_HEADERS;

#define FTT_PKT_GENERIC_max_payload ( ETH_DATALEN_MAX - sizeof(FTT_PKT_HEADER) )
#define FTT_PKT_GENERIC_min_payload ( ETH_DATALEN_MIN - sizeof(FTT_PKT_HEADER) )

/* TM */
typedef struct{
	unsigned char  seq_no;
	unsigned char  ec_time_ms;
	unsigned short nsm;        ///< Number of synch. mesgs
	unsigned short nam;        ///< Number of asynch. mesgs
	unsigned short nsa;        ///< Number of signaling !ack
} __attribute__ ((packed)) FTT_PKT_TM_HEADER;

#define TM_SEQN_MASK     0x00FF

typedef struct{
	FTT_ETH_HEADER eth_header;
	FTT_PKT_HEADER pkt_header;
	FTT_PKT_TM_HEADER tm_header;
	unsigned char data[];
} __attribute__ ((packed)) FTT_PKT_TM_HEADERS;

typedef struct{
	unsigned short SMesgId;	 ///< ID
	unsigned short SMesg_frag_no;
	unsigned char  source_nodeID;
} __attribute__ ((packed)) TM_SYNC_MSG_INDEX;

typedef struct{
	unsigned short AMesgId;	 ///< ID
	unsigned short AMesg_frag_no;
	unsigned char  source_nodeID;	
} __attribute__ ((packed)) TM_ASYNC_MSG_INDEX;

typedef struct{
	unsigned short nodeID;	 ///< ID
	unsigned char sig_seqno;
} __attribute__ ((packed)) TM_SIG_ACK_INDEX;

#define FTT_PKT_TM_DATALEN_MAX ( FTT_PKT_GENERIC_max_payload - sizeof(FTT_PKT_TM_HEADER))
#define FTT_PKT_TM_DATALEN_MIN ( FTT_PKT_GENERIC_min_payload - sizeof(FTT_PKT_TM_HEADER))
#define FTT_PKT_TM_LOGICALSIZE_B_from_no( _no_SM, _no_AM, _no_Sig_ACK ) ( _no_Sig_ACK*sizeof(TM_SIG_ACK_INDEX) + _no_SM*sizeof(TM_SYNC_MSG_INDEX) + _no_AM*sizeof(TM_ASYNC_MSG_INDEX) + sizeof(FTT_PKT_TM_HEADERS) )
#define FTT_PKT_TM_REALSIZE_B_from_no( _no_SM, _no_AM, _no_Sig_ACK ) ( _no_Sig_ACK*sizeof(TM_SIG_ACK_INDEX) + _no_SM*sizeof(TM_SYNC_MSG_INDEX) + _no_AM*sizeof(TM_ASYNC_MSG_INDEX) + sizeof(FTT_PKT_TM_HEADERS) + ETH_FRAME_HIDDEN_OVERHEAD)

/** @brief Mask for the 9th bit of identifier (flags) */
#define TXNEWDAT_MASK 	0x01

/* SDM */
typedef struct{
	unsigned short id;	       ///< Frame ID
	unsigned char seq_no;
	unsigned char flag_s;
	unsigned short frag_seqno; ///< Sequence number within the fragmentation (could be char)
	unsigned char  source_nodeID;	
#ifdef QNX_TAGGING
	QNX_TAGGING_TYPE tagging;
#endif
} __attribute__ ((packed)) FTT_PKT_SDM_HEADER;

typedef struct{
	FTT_ETH_HEADER eth_header;
	FTT_PKT_HEADER pkt_header;
	FTT_PKT_SDM_HEADER sdm_header;
	unsigned char data[];
} __attribute__ ((packed)) FTT_PKT_SDM_HEADERS;

#define FTT_PKT_SDM_DATALEN_MAX ( FTT_PKT_GENERIC_max_payload - sizeof(FTT_PKT_SDM_HEADER))
#define FTT_PKT_SDM_DATALEN_MIN ( FTT_PKT_GENERIC_min_payload - sizeof(FTT_PKT_SDM_HEADER))
#define FTT_PKT_SDM_LOGICALSIZE_B( _load ) ( _load + sizeof(FTT_PKT_SDM_HEADERS) )
#define FTT_PKT_SDM_REALSIZE_B( _load ) ( _load + sizeof(FTT_PKT_SDM_HEADERS) + ETH_FRAME_HIDDEN_OVERHEAD)
#define FTT_PKT_SDM_REALSIZE_B_reverse( _load ) ( _load - sizeof(FTT_PKT_SDM_HEADERS) - ETH_FRAME_HIDDEN_OVERHEAD)

/* ADM */
typedef struct{
	unsigned short id;	       ///< Frame ID
	unsigned char seq_no;
	unsigned char flag_s;
	unsigned short frag_seqno;  ///< Sequence number within the fragmentation
	unsigned char  source_nodeID;
#ifdef QNX_TAGGING
	QNX_TAGGING_TYPE tagging;
#endif
} __attribute__ ((packed)) FTT_PKT_ADM_HEADER;

typedef struct{
	FTT_ETH_HEADER eth_header;
	FTT_PKT_HEADER pkt_header;
	FTT_PKT_ADM_HEADER adm_header;
	unsigned char data[];
} __attribute__ ((packed)) FTT_PKT_ADM_HEADERS;

#define FTT_PKT_ADM_DATALEN_MAX ( FTT_PKT_GENERIC_max_payload - sizeof(FTT_PKT_ADM_HEADER))
#define FTT_PKT_ADM_DATALEN_MIN ( FTT_PKT_GENERIC_min_payload - sizeof(FTT_PKT_ADM_HEADER))
#define FTT_PKT_ADM_LOGICALSIZE_B( _load ) ( _load + sizeof(FTT_PKT_ADM_HEADERS) )
#define FTT_PKT_ADM_REALSIZE_B( _load ) ( _load + sizeof(FTT_PKT_ADM_HEADERS) + ETH_FRAME_HIDDEN_OVERHEAD)
#define FTT_PKT_ADM_REALSIZE_B_reverse( _load ) ( _load - sizeof(FTT_PKT_ADM_HEADERS) - ETH_FRAME_HIDDEN_OVERHEAD)

/* ASM */
typedef struct{
	unsigned char seq_no;
	unsigned char nodeID;
	unsigned short nam;
} __attribute__ ((packed)) FTT_PKT_ASM_HEADER;

typedef struct{
	FTT_ETH_HEADER eth_header;
	FTT_PKT_HEADER pkt_header;
	FTT_PKT_ASM_HEADER asm_header;
	unsigned char data[];
} __attribute__ ((packed)) FTT_PKT_ASM_HEADERS;

typedef struct{
	unsigned short AMesgId;	 ///< ID
	unsigned short  AMesgQueueLen; ///< Len of the queue in the asynchronous producer
} __attribute__ ((packed)) ASM_MSG_QUEUE_INDEX;


#define FTT_PKT_ASM_LOGICALSIZE_B_from_no( _no ) ( _no*sizeof(ASM_MSG_QUEUE_INDEX) + sizeof(FTT_PKT_ASM_HEADERS) )
#define FTT_PKT_ASM_REALSIZE_B_from_no( _no ) ( _no*sizeof(ASM_MSG_QUEUE_INDEX) + sizeof(FTT_PKT_ASM_HEADERS) + ETH_FRAME_HIDDEN_OVERHEAD)

/* IDLE */
typedef struct{
	unsigned short id;	       ///< Frame ID
} __attribute__ ((packed)) FTT_PKT_IDLE_HEADER;

typedef struct{
	FTT_ETH_HEADER eth_header;
	FTT_PKT_HEADER pkt_header;
	FTT_PKT_IDLE_HEADER idle_header;
} __attribute__ ((packed)) FTT_PKT_IDLE_HEADERS;


//////////////////////////////////////////////////////////////////////////////////

#define FTT_SM (0)
#define FTT_AM (1)

typedef unsigned short FTT_VAR_ID;   /* 1-type 3-tag 12-ID */

#define FTT_VAR_ID_TYPE_MASK (0x8000)
#define FTT_VAR_ID_TAG_MASK  (0x7000)
#define FTT_VAR_ID_ID_MASK   (0xFFF)

#define FTT_VAR_ID_TYPE_SHIFT (15)
#define FTT_VAR_ID_TAG_SHIFT (12)
#define FTT_VAR_ID_ID_SHIFT (0)

#define FTT_VAR_ID_TAG_BASEMASK  (0x7)

#define FTT_VAR_ID_GET_ID( _id ) ( ( (FTT_VAR_ID)_id & FTT_VAR_ID_ID_MASK ) >> FTT_VAR_ID_ID_SHIFT )
#define FTT_VAR_ID_GET_TAG( _id ) ( ( (FTT_VAR_ID)_id & FTT_VAR_ID_TAG_MASK ) >> FTT_VAR_ID_TAG_SHIFT )
#define FTT_VAR_ID_GET_TYPE( _id ) ( ( (FTT_VAR_ID)_id & FTT_VAR_ID_TYPE_MASK ) >> FTT_VAR_ID_TYPE_SHIFT )

#define FTT_VAR_ID_SET_ID( _idout, _idin ) { _idout = (( ((FTT_VAR_ID)_idin) << FTT_VAR_ID_ID_SHIFT) & FTT_VAR_ID_ID_MASK) | ( ((FTT_VAR_ID)_idout) & (~FTT_VAR_ID_ID_MASK) ); }
#define FTT_VAR_ID_SET_TAG( _idout, _idin ) { _idout = (( ((FTT_VAR_ID)_idin) << FTT_VAR_ID_TAG_SHIFT) & FTT_VAR_ID_TAG_MASK) | ( ((FTT_VAR_ID)_idout) & (~FTT_VAR_ID_TAG_MASK) ); }
#define FTT_VAR_ID_SET_TYPE( _idout, _idin ) { _idout = (( ((FTT_VAR_ID)_idin) << FTT_VAR_ID_TYPE_SHIFT) & FTT_VAR_ID_TYPE_MASK) | ( ((FTT_VAR_ID)_idout) & (~FTT_VAR_ID_TYPE_MASK) ); }

#define FTT_VAR_ID_SET_TYPE_SYNC( _id )    ( FTT_VAR_ID_SET_TYPE( _id, FTT_SM ) )
#define FTT_VAR_ID_SET_TYPE_ASYNC( _id )    ( FTT_VAR_ID_SET_TYPE( _id, FTT_AM ) )

#define FTT_VAR_ID_IS_ID_SYNC( __id )       ( FTT_VAR_ID_GET_TYPE( __id ) == FTT_SM )
#define FTT_VAR_ID_IS_ID_ASYNC( __id )       ( FTT_VAR_ID_GET_TYPE( __id ) == FTT_AM )

#define FTT_VAR_ID_CMP_ID( _id1, _id2)          ( ((FTT_VAR_ID)_id1 & FTT_VAR_ID_ID_MASK) == ((FTT_VAR_ID)_id2 & FTT_VAR_ID_ID_MASK) )
#define FTT_VAR_ID_CMP_TAG( _id1, _id2)          ( ((FTT_VAR_ID)_id1 & FTT_VAR_ID_TAG_MASK) == ((FTT_VAR_ID)_id2 & FTT_VAR_ID_TAG_MASK) )
#define FTT_VAR_ID_CMP_TYPE( _id1, _id2)          ( ((FTT_VAR_ID)_id1 & FTT_VAR_ID_TYPE_MASK) == ((FTT_VAR_ID)_id2 & FTT_VAR_ID_TYPE_MASK) )

#define FTT_VAR_ID_CMP_ID_TYPE( _id1, _id2)       ( ((FTT_VAR_ID)_id1 & (FTT_VAR_ID_ID_MASK|FTT_VAR_ID_TYPE_MASK)) == ((FTT_VAR_ID)_id2 & (FTT_VAR_ID_ID_MASK|FTT_VAR_ID_TYPE_MASK)) )
#define FTT_VAR_ID_CMP_ID_TYPE_TAG( _id1, _id2)   ( (FTT_VAR_ID)_id1 == (FTT_VAR_ID)_id2 )


/* Asynchronous communication channel - Used for Dbs synch */

#define MASTER_BROADCAST_A_CH_ID	1
#define MASTER_BROADCAST_A_CH_SIZE	120
#define MASTER_BROADCAST_A_CH_MTU	1200

typedef enum
{
	FTT_CMD_SET_NODEID=0,
	FTT_CMD_SET_APP_PNP_IDS,
	FTT_CMD_ADD_MSG,
	FTT_CMD_DEL_MSG,
	FTT_CMD_DEL_OLD_TAG,
	FTT_CMD_DUMMY
}FTT_COMMANDS;

/** \struct MASTER_ASYNCH 
\brief Header data carried in the Trigger Message
*/
typedef struct{
	unsigned short COMMAND_id;	 ///< Command id
	unsigned short flags;		 ///< Sequence number + command hold
} __attribute__ ((packed)) A_MST_CHANNEL_HEADER;

typedef struct{
	A_MST_CHANNEL_HEADER header;
	unsigned short nodeID;
	unsigned char mac[6];
} __attribute__ ((packed)) A_MST_SET_NODEID;

typedef struct{
	A_MST_CHANNEL_HEADER header;
	unsigned char adding_type; //0-change if possible   1-force as new
	unsigned char mesg_type;
	unsigned short mesg_id;
	unsigned char mesg_id_tag;
	unsigned int mesg_size;
	unsigned int mesg_max_size;
	unsigned short mesg_MTU;
	unsigned int mesg_period;
	unsigned char mesg_production_mac[6];
	unsigned char mesg_producerID[40];  //nodeID
	unsigned char mesg_producers_no;
	unsigned char mesg_producers_max_no;
	unsigned char mesg_consumers_no;	//if 0 broadcast
	unsigned char mesg_consumerID[40];
} __attribute__ ((packed)) A_MST_ADD;

typedef struct{
	A_MST_CHANNEL_HEADER header;
	unsigned char mesg_type;
	unsigned short mesg_id;
} __attribute__ ((packed)) A_MST_DEL;

typedef struct{
	A_MST_CHANNEL_HEADER header;
	unsigned char mesg_type;
	unsigned short mesg_id; //It embedds the old tag
} __attribute__ ((packed)) A_MST_DEL_OLD_TAG;

typedef struct{
	A_MST_CHANNEL_HEADER header;
} __attribute__ ((packed)) A_MST_DUMMY;

typedef struct{
	A_MST_CHANNEL_HEADER header;
	unsigned char app_id;
	unsigned char nodeID;
	unsigned char ack; // 0/1
	unsigned short tx_id;
	unsigned short rx_id;
} __attribute__ ((packed)) A_PNP_IDS_NOTIFY;

#endif   /* ----- #ifndef FTT-GLOBAL2_FILE_HEADER_INC  ----- */

