/*****************************************************************************
 * M_Dispatcher.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#include "Ports.h"

#include "ftt-global.h"
//#include "SRDB.h"
#include "M_ECreg.h"
#include "M_Scheduler.h"
//#include "math_tools.h"
#include "ethernet.layer.h"
#include "M_SW_add_table.h"
//
#include "S_ftt-core.layer_embed.h"

//#define TRIGGER_DECODE
//#define BE_POLITE
////#define DISABLE_TX
 
static rtl_pthread_t ftt_dispatcher_th;
static FTT_SEMAPHORE start_dispatcher;

static unsigned char M_MAC[6];
static unsigned long M_ec_period=0; //us

void *ftt_dispatcher(void *t)
/****************************************************************************/
/*!
\brief This task is invoqued periodically (at the beginning of each EC)

	This task is invoqued periodically (at the beginning of each EC).\n

	- scans current EC (sched_ec),\n
	- builds respective master message and,\n
	- send it to the bus.

*****************************************************************************/
{
	unsigned short i;

	unsigned char *tmsg;

	FTT_PKT_TM_HEADERS *tm_pkt; /* Pointer to TM contents */
	TM_SYNC_MSG_INDEX *tm_sm_element;
	TM_ASYNC_MSG_INDEX *tm_am_element;
	TM_SIG_ACK_INDEX *tm_sig_ack_element;

	unsigned short tm_datalength; /* Data lenght of the TM frame */
	
	struct rtl_timespec next_activation; //used to time activations	
	unsigned char tm_seqno_temp;
	int ret;
	
	unsigned short ret_node_id;
	unsigned char ret_seq_no;
	unsigned short num_sig_acks;
	
	/* Wait for start */
	if ( FTT_SEMAPHORE_wait(&start_dispatcher) < 0 )
		return NULL;

//	PRINT_MSG("Dispatcher Started");

	/* get the current time and setup for the first tick */
	rtl_clock_gettime( RTL_CLOCK_REALTIME, &next_activation );
	while (1)
	{
		/* set the period */
		rtl_timespec_add_ns( &next_activation, 1000*M_ec_period );

		
		/* sleep */
		rtl_clock_nanosleep( RTL_CLOCK_REALTIME, RTL_TIMER_ABSTIME, &next_activation, NULL);
//		{
//			struct rtl_timespec timestamp;
//			rtl_clock_gettime( RTL_CLOCK_REALTIME, &timestamp );
//			PRINT_MSG("Dispatcher end: %lld", (timestamp.tv_sec*1000 + (timestamp.tv_nsec/1000000)));
//		}

		ECreg_SwapPlans();
		tm_seqno_temp = (unsigned char)EC_Get_and_Inc_seqno();
#ifdef BE_POLITE
		PRINT_MSG("ftt_dispatcher: Hello...!");
#endif

		/* Check  EC status */
		switch( disp_ec->status ){
			case OUT_OF_DATE:        /* Starting up :  do nothing */
			case UPDATED:            /* OK. Follow down to normal plan dispatching */
				//PRINT_MSG("ftt_dispatcher fine");
				break;
			case MISSED_DEADLINE:    /* Missed deadline detected by scheduler, terminate */
				PRINT_MSG("ftt_dispatcher Fatal error: missed deadline in synchronous message");
				return NULL;
				break;
			case BUSY_SCHED:         /* Scheduler did not build plan in time, terminate */
				PRINT_MSG("BUSY");
			case DISPATCHED:         /* Scheduler has not been started                  */
				PRINT_MSG("ftt_dispatcher Fatal error: Scheduler has not finished in time");
				return NULL;
				break;
			default:
				PRINT_MSG("ftt_dispatcher Fatal error: Invalid EC Status (Strange error!)");
				break;
		}

		if ( ETH_L_tx_reserve_buffer( &tmsg ) < 0 ){
			ERROR_MSG("Problems getting a new Tx buffer - Progamming error ");
		}

		tm_pkt = (FTT_PKT_TM_HEADERS *)tmsg;

		/* The source and destination address */
		ETH_L_Str2Addr((unsigned char *)"FF:FF:FF:FF:FF:FF",  tm_pkt->eth_header.eth_dest );

		ETH_L_CopyMAC_to_from(tm_pkt->eth_header.eth_src, M_MAC );
		
		/* The frame type */
		tm_pkt->eth_header.eth_type = htons ( ETH_FTT_TYPE );
		tm_pkt->pkt_header.type = htons ( FTT_MST_MSG );
		
	
		/* Given disp_ec, we can translate it into the TM */
		tm_pkt->tm_header.seq_no = tm_seqno_temp;
		tm_pkt->tm_header.ec_time_ms = M_ec_period/1000;
		tm_pkt->tm_header.nsm   = htons((unsigned short)disp_ec->sm.n_mesgs);
		tm_pkt->tm_header.nam   = htons((unsigned short)disp_ec->am.n_mesgs);

#ifdef TRIGGER_DECODE
		PRINT_MSG("TM: nsm=%2d nam=%2d",disp_ec->sm.n_mesgs,disp_ec->am.n_mesgs);
#endif
		/* Synchronous message data */
		tm_sm_element = (TM_SYNC_MSG_INDEX *)(tm_pkt->data);
		for( i=0; i < disp_ec->sm.n_mesgs; i++ ){
			tm_sm_element[i].SMesgId = htons( (unsigned short)(disp_ec->sm.msgs[i].id) );
			tm_sm_element[i].SMesg_frag_no = htons( disp_ec->sm.msgs[i].n_fragmentation );
			tm_sm_element[i].source_nodeID = disp_ec->sm.msgs[i].source_nodeID;
#ifdef TRIGGER_DECODE
			PRINT_MSG("      S_id:%04x   frag:%d  src:%d", disp_ec->sm.msgs[i].id, disp_ec->sm.msgs[i].n_fragmentation, disp_ec->sm.msgs[i].source_nodeID);			
#endif
		}
	
		/* Asynchronous message data */
		tm_am_element = (TM_ASYNC_MSG_INDEX *)( tm_pkt->data + ( disp_ec->sm.n_mesgs*sizeof(TM_SYNC_MSG_INDEX)) );
		for( i=0; i < disp_ec->am.n_mesgs; i++ ){
			tm_am_element[i].AMesgId = htons( (unsigned short)(disp_ec->am.msgs[i].id) );
			tm_am_element[i].AMesg_frag_no = htons(disp_ec->am.msgs[i].n_fragmentation);
			tm_am_element[i].source_nodeID = disp_ec->am.msgs[i].source_nodeID;
#ifdef TRIGGER_DECODE
			PRINT_MSG("      A_id:%04x   frag:%d  src:%d", disp_ec->am.msgs[i].id, disp_ec->am.msgs[i].n_fragmentation, disp_ec->am.msgs[i].source_nodeID);
#endif

		}
		

		/* Notify Signaling !ack */
		num_sig_acks = 0;
				
		tm_sig_ack_element = (TM_SIG_ACK_INDEX *)( tm_pkt->data + (disp_ec->sm.n_mesgs*sizeof(TM_SYNC_MSG_INDEX)) + (disp_ec->am.n_mesgs*sizeof(TM_ASYNC_MSG_INDEX)) );
		tm_pkt->tm_header.nsa = 0;
		
		while (EC_pull_Signaling_notification( &ret_node_id, &ret_seq_no ) == 0 ) {
			tm_sig_ack_element->nodeID = htons ( ret_node_id );
			tm_sig_ack_element->sig_seqno = ret_seq_no;
			tm_sig_ack_element++;
			num_sig_acks++;
		}
		tm_pkt->tm_header.nsa   = htons((unsigned short)num_sig_acks);
		
		/****************************/
		/* Prepare the transmission */
		/****************************/
		tm_datalength = FTT_PKT_TM_LOGICALSIZE_B_from_no(disp_ec->sm.n_mesgs, disp_ec->am.n_mesgs, num_sig_acks);
//		PRINT_MSG(" TM size:%d", tm_datalength);
//#define TRIGGER_PERIODIC
#ifdef TRIGGER_PERIODIC
#error hi
		/* set absolute delay to reduce the system jitter */
		{
		struct rtl_timespec temp_activation; //used to time activations
		temp_activation = next_activation;
		
		timespec_add_ns( &temp_activation, 1000*30 );
		
		/* sleep */
		rtl_clock_nanosleep( CLOCK_REALTIME, TIMER_ABSTIME, &temp_activation, NULL);
		}
#endif

#ifndef DISABLE_TX
		/* Send the message */
		if ( (ret = ETH_L_tx_send_messagebuffer(NULL, tm_datalength)) < 0 ){
			ERROR_MSG("Message not sent. ETH_L_tx_send_messagebuffer returned: %d", ret);
		}
#endif
		/* Declare the table as dispatched */
		disp_ec->status = DISPATCHED;

/* We will simulate that the TM was received, right now. So, it has to be decoded and the 
 * varibles that are produced here transmitted */
		{
			struct rtl_timespec TM_tx_time;

			rtl_clock_gettime( RTL_CLOCK_REALTIME, &TM_tx_time );
			S_eth_filter( tmsg, tm_datalength, TM_tx_time );
		}
		/* Activate scheduler() */
		WakeSheduler();

		/* Let's now downgrade the Node's connection aging time */
		M_SW_downall_lease_time();

	}//while 1
}
signed char M_DispatcherInit(unsigned char *mac_reg, unsigned long ec_period_reg )
{

	rtl_pthread_attr_t attr;
	struct rtl_sched_param sched_param;
	int ret;

	/* Store the MAC and EC_period in this module */
	ETH_L_CopyMAC_to_from( M_MAC, mac_reg );
	M_ec_period = ec_period_reg;

	/* Init semaphore to start periodic dispatch task */
	if ( (ret=FTT_SEMAPHORE_init(&start_dispatcher, 0)) < 0 ){	//Initialization locked
		ERROR_MSG("Couldn't initialize semaphore");
		return -1;
	}
	
	/******************************/
	/* Create ftt_dispatcher task */
	/******************************/
	rtl_pthread_attr_init( &attr );
	sched_param.sched_priority = rtl_sched_get_priority_max(RTL_SCHED_FIFO);
	rtl_pthread_attr_setschedparam( &attr, &sched_param );
#ifdef RTL_PRO	
	rtl_pthread_attr_setfp_np(&attr, 1);
#endif
	if ( (ret=rtl_pthread_create( &ftt_dispatcher_th, &attr, ftt_dispatcher, (void *)0 )) ){
		ERROR_MSG("Init Fatal Error: pthread_create returned %d", ret);
		return -2;
	}
#ifdef PRINT_THREAD_ID
#ifdef RTL_PRO
	ERROR_MSG("New thread id: %d", ftt_dispatcher_th);
#else
	ERROR_MSG("New thread id: %lu", ftt_dispatcher_th);
#endif
#endif

#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -M_DispatcherInit Ok");
#endif
	return 0;
}

void M_DispatcherClose(void)
{
	/* Clean up the semaphore */
	FTT_SEMAPHORE_destroy(&start_dispatcher);

	/* cancel the ftt_dispatcher thread */
	rtl_pthread_cancel( ftt_dispatcher_th );
	rtl_pthread_join( ftt_dispatcher_th, NULL );

#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -M_DispatcherClose Ok");
#endif
}

void M_DispatcherStartSystem(void)
{
	FTT_SEMAPHORE_post(&start_dispatcher); 
	return ;
}

