/*****************************************************************************
 * M_ECreg.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#include "Ports.h"

#include "M_ECreg.h"

static ECType ECs[2]; 

ECType *sched_ec;
ECType *disp_ec;

static unsigned long tm_seqn;


#define MAX_SIG_SLOTS (EC_MAX_SIG_MISSING_REQUESTS+1)


typedef struct{
	unsigned short nodeID;
	unsigned char  missing_sig_seq_no;
}SIG_ACK_IDX;

static SIG_ACK_IDX sig_ack_array[MAX_SIG_SLOTS];
static unsigned short sig_ack_rd;
static unsigned short sig_ack_wr;


void ECreg_Init(void)
{
	tm_seqn = 0; /* TM sequence number */

	/* Init plans and pointers to plans */
	ECs[0].status = OUT_OF_DATE;
	ECs[1].status = OUT_OF_DATE;
	sched_ec=&(ECs[0]);
	disp_ec=&(ECs[1]);
#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -ECreg_Init Ok");
#endif
	
	sig_ack_rd = sig_ack_wr = 0;
}

void ECreg_SwapPlans(void)
{
	ECType *temp_ec;

	/* Swap plans */
	temp_ec = sched_ec;
	sched_ec = disp_ec;
	disp_ec = temp_ec;
}

void ECreg_prepare2build(void)
{
	unsigned int j;
	/* Assuming the plans swap is performed by the Distpatcher,
	 * this function cleans and prepares the new scheduling plan */

	/* Start to build a new EC-schedule */
	sched_ec->status = BUSY_SCHED;		/* Signal that plan is being built */
	sched_ec->err_code = SCH_ERRCODE_OK;	/* No errors */

	/* SM's*/
#ifdef SW_UNICAST
	for (j=0; j<SW_MAX_PORTS; j++)
	{
		sched_ec->sm.ports_load[j].up_load = 0;
		sched_ec->sm.ports_load[j].down_load = 0;
		sched_ec->sm.ports_load[j].down_blocks_no = 1;
		sched_ec->sm.ports_load[j].down_blocks[0].begin = 0;
		sched_ec->sm.ports_load[j].down_blocks[0].end = 0;
		sched_ec->sm.ports_load[j].down_blocks[0].used = 0;
	}
#elif SW_BROADCAST
	sched_ec->sm.updown_load = 0;
#endif
	sched_ec->sm.n_mesgs = 0;
	
	/* AM's*/
	sched_ec->am.updown_load = 0;
	sched_ec->am.n_mesgs = 0;

	return;
}


void EC_printsched(void)
{
	int i; /* General counter */
	ECType *plan_p=sched_ec;

	PRINT_MSG("ECreg sched_ec:");
	PRINT_MSG("    Sync.  mesgs:");
	for(i=0;i < plan_p->sm.n_mesgs; i++)
	{
		PRINTF(" [%x]",plan_p->sm.msgs[i].id);
		//PRINTF(" [%x/%d]",plan_p->sm.messages[i].id, plan_p->sm.messages[i].sloted_size);
	}

	PRINT_MSG("    Async. mesgs:");
	for(i=0;i < plan_p->am.n_mesgs; i++)
	{
		PRINTF(" [%x]",plan_p->am.msgs[i].id);
	}

	PRINT_MSG("");

	return;
}

unsigned char EC_is_async_appID_sched(unsigned short id)
{
	int i; /* General counter */
	ECType *plan_p=sched_ec;

	for(i=0;i < plan_p->am.n_mesgs; i++)
		if (( (plan_p->am.msgs)[i].id & 0x0FFF) == id)
			return 1;
	return 0;
}

void EC_printdisp(void)
{
	int i; /* General counter */
	ECType *plan_p=disp_ec;

	PRINT_MSG("ECreg disp_ec:");
	PRINT_MSG("    Sync.  mesgs:");
	for(i=0;i < plan_p->sm.n_mesgs; i++)
	{
		PRINTF(" [%x]",plan_p->sm.msgs[i].id);
	}

	PRINT_MSG("    Async. mesgs:");
	for(i=0;i < plan_p->am.n_mesgs; i++)
	{
		printf(" [%x]",plan_p->am.msgs[i].id);
	}

	PRINT_MSG("");

	return;
}

unsigned long EC_Get_and_Inc_seqno(void)
{
	return (tm_seqn++);
}


signed char EC_push_Signaling_notification( unsigned short nodeID, unsigned char seq_no )
{
	/* Check if full */
	if ( ((sig_ack_wr+1)%MAX_SIG_SLOTS) == sig_ack_rd )
		return -1;

	sig_ack_array[sig_ack_wr].nodeID = nodeID;
	sig_ack_array[sig_ack_wr].missing_sig_seq_no = seq_no;
	
	sig_ack_wr = (sig_ack_wr+1)%MAX_SIG_SLOTS;

	return 0;
}

signed char EC_pull_Signaling_notification( unsigned short *nodeID, unsigned char *seq_no )
{
	/* Check if empty */
	if ( sig_ack_wr == sig_ack_rd )
		return -1;
	
	if (nodeID) *nodeID = sig_ack_array[sig_ack_rd].nodeID;
	if (seq_no) *seq_no = sig_ack_array[sig_ack_rd].missing_sig_seq_no;	
	
	sig_ack_rd = (sig_ack_rd+1)%MAX_SIG_SLOTS;
	return 0;
}
