/*****************************************************************************
 * M_Ready_queue.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#include "Ports.h"

#include "M_db.h"
#include "M_Ready_queue.h"

static unsigned char M_Ready_queue_init=0;

/**********************Scheduling queue*************************/

#define LIST_ARRAY_MAX_SIZE 2000
static READY_QUEUE_IDX ready_idx_heap[ LIST_ARRAY_MAX_SIZE ];

typedef struct{
	READY_QUEUE_IDX *list_base;
	SCHED_POLICIES Sorting_policy; //Scheduling policy
}READY_QUEUE;

static READY_QUEUE ready_queues[4];

//static READY_QUEUE_IDX *list_bases[4];

static void ready_queue_print( unsigned char queue_no )
{
	READY_QUEUE_IDX **idx_temp;

	idx_temp = &(ready_queues[queue_no].list_base);
	PRINTF("\n Start=");
	while (1){
		if (*idx_temp == NULL){
			PRINTF(" NULL");
			break;
		}
		PRINTF(" %p ->", *idx_temp);

		/* Increment */
		idx_temp = &((*idx_temp)->link);
	}
	return ;
}

signed char Ready_queues_Get_policy(unsigned char queue)
{
	if (queue >= 4){
		ERROR_MSG("Bad queue indentifier");
		return -1;
	}
	return ready_queues[queue].Sorting_policy;
}

signed char Ready_queues_Init(void)
{
	unsigned int i;
	for (i=0; i<LIST_ARRAY_MAX_SIZE; i++)
		ready_idx_heap[i].message_DB_pointer = NULL;
	ready_queues[SRT_QUEUE].list_base = NULL;
	ready_queues[SRT_QUEUE].Sorting_policy = SCHED_EDF;
	ready_queues[ART_H_QUEUE].list_base = NULL;
	ready_queues[ART_H_QUEUE].Sorting_policy = SCHEDD_FIFO;
	ready_queues[ART_S_QUEUE].list_base = NULL;
	ready_queues[ART_S_QUEUE].Sorting_policy = SCHEDD_FIFO;
	ready_queues[ART_U_QUEUE].list_base = NULL;
	ready_queues[ART_U_QUEUE].Sorting_policy = SCHEDD_FIFO;

	M_Ready_queue_init = 1;

#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -Ready_queues_Init Ok");
#endif
	return 0;
}
signed char Ready_queues_Init_check(void)
{
	if (M_Ready_queue_init == 0)
		return -1;
	return 0;
}

static signed int ready_idx_heap_get_free( READY_QUEUE_IDX **ret )
{
	/* One its free if the message_DB_pointer pointer is set to NULL */
	unsigned int i;
	for (i=0; i<LIST_ARRAY_MAX_SIZE; i++)
		if ( ready_idx_heap[i].message_DB_pointer == NULL )
			break;
	if (i == LIST_ARRAY_MAX_SIZE)
		return -1;
	*ret = &(ready_idx_heap[i]);
	return 0;
}

#define SCHED_RM_DECISION( idx_t ) ( MDB_Get_runtime_period((idx_t)->message_DB_pointer) )

#define SCHED_DM_DECISION( idx_t ) ( MDB_Get_runtime_deadline((idx_t)->message_DB_pointer) )

/* Once in the ready queue, the activation is negative */
#define SCHED_EDF_DECISION( idx_t ) ( (signed)( MDB_Get_runtime_deadline((idx_t)->message_DB_pointer) + MDB_Get_runtime_activation_counter((idx_t)->message_DB_pointer) ) )
//#define SCHED_EDF_DECISION( idx_t ) ( (signed)( MDB_Get_runtime_activation_counter((idx_t)->message_DB_pointer)-(MDB_Get_runtime_period((idx_t)->message_DB_pointer) - MDB_Get_runtime_deadline((idx_t)->message_DB_pointer)) ) )

inline static signed char Scheduling_fixed_parameter_calc( SCHED_POLICIES policy, int *ret_parameter, READY_QUEUE_IDX *item)
{
	switch(policy){
		case SCHED_EDF:	*ret_parameter = SCHED_EDF_DECISION( item );break; //How much for deadline
		case SCHED_RM:	*ret_parameter = SCHED_RM_DECISION( item );break; //The period
		case SCHED_DM:	*ret_parameter = SCHED_DM_DECISION( item );break; //The relative deadline
		case SCHEDD_FIFO:*ret_parameter = 0 ;break;
		default: ERROR_MSG("Invalid scheduling policy in queue: %d",policy ); return -1;
	}
	return 0;
}
inline static unsigned char Scheduling_decision( SCHED_POLICIES policy, int input_parameter, READY_QUEUE_IDX *list_item  )
{
//	PRINT_MSG("listed %p :%d",list_item, SCHED_EDF_DECISION( list_item ));
	switch(policy){
		case SCHED_EDF:	return ( input_parameter < SCHED_EDF_DECISION( list_item ) );
		case SCHED_RM:	return ( input_parameter < SCHED_RM_DECISION( list_item ) );
		case SCHED_DM:	return ( input_parameter < SCHED_DM_DECISION( list_item ) );
		case SCHEDD_FIFO:return ( 0 ); //It will go to the list tail
		default: ERROR_MSG("Invalid scheduling policy in queue: %d",policy ); return -1;
	}
	return 0;
}


static void ready_queue_include_by_sched_policy( unsigned char queue_no, READY_QUEUE_IDX *idx )
{
	READY_QUEUE_IDX **idx_temp;

	int decision_parameter_in;     //In this var is the scheduling dependent data
	                               // - It is static along all the survey while
	                               //   the other comparision is varies for each item in the list
	
	idx_temp = &(ready_queues[queue_no].list_base);

	if ( Scheduling_fixed_parameter_calc(ready_queues[queue_no].Sorting_policy, &decision_parameter_in, idx) < 0 )
		return;

//	PRINT_MSG("input %p :%d",idx,decision_parameter_in);
	while ( *idx_temp != NULL ){

		if ( Scheduling_decision( ready_queues[queue_no].Sorting_policy, decision_parameter_in, *idx_temp  ) )
			break;
		idx_temp = &((*idx_temp)->link);
	}/* Exit if end of list or insertion point */

	/* Insert it */
	idx->link = *idx_temp; 
	*idx_temp = idx;

//	PRINT_MSG("Queue %d added 0x%p", queue_no, idx );
//	ready_queue_print( queue_no );
	return ;

}

static signed char ready_queue_add_item( unsigned char queue_no, void *message, unsigned int packets_no, unsigned char source_nodeID) //extern
{
	READY_QUEUE_IDX *temp_p;

	if ( ready_idx_heap_get_free( &temp_p ) < 0 )
		return -1;
	temp_p->message_DB_pointer = message;
	temp_p->packets_no = packets_no;
	temp_p->source_nodeID = source_nodeID;
	temp_p->link = NULL;
	
//	MDB_Get_runtime_message_status(message) = SM_SCHEDULED;
	
	ready_queue_include_by_sched_policy( queue_no, temp_p );
	return 0;
}
static signed char ready_SM_list_add_item( READY_RETURN_ITEMS *one_item ){
	return ready_queue_add_item( SRT_QUEUE, one_item->message_p, one_item->pakets_no, one_item->source_nodeID);
}

static signed char ready_AM_list_add_item( READY_RETURN_ITEMS *one_item ){
	switch (one_item->am_type){
		case TMLN_HARD:
			return ready_queue_add_item( ART_H_QUEUE, one_item->message_p, one_item->pakets_no, one_item->source_nodeID);
			break;
		case TMLN_SOFT:
			return ready_queue_add_item( ART_S_QUEUE, one_item->message_p, one_item->pakets_no, one_item->source_nodeID);
			break;
		case TMLN_UNCONST:
			return ready_queue_add_item( ART_U_QUEUE, one_item->message_p, one_item->pakets_no, one_item->source_nodeID);
			break;
		default:
			ERROR_MSG("UNKNOWN TIMELINE CONSTRAIN");
			break;
	}
	return -1;
}

signed char ready_queue_watch_last_item(unsigned char queue_no, READY_QUEUE_IDX **item)
{
	if ((ready_queues[queue_no].list_base) == NULL){
		return -1;
	}

	/* One might think that I should masrk the var as SM_TRANSMITING
	 * I am not doing it here 'cause this message may still not be scheduled
	 * It will be set in ready_queue_decrease_last_item()
	 */
	*item=(ready_queues[queue_no].list_base);
	return 0;
}

static signed char ready_queue_erase_last_item(unsigned char queue_no)
{
	
	if ((ready_queues[queue_no].list_base) == NULL){
		return -1;
	}

	/* The message was fully dispatched - Refresh the activation counter */
	MDB_Update_activation( (ready_queues[queue_no].list_base)->message_DB_pointer );

	(ready_queues[queue_no].list_base)->message_DB_pointer=NULL;
	(ready_queues[queue_no].list_base) = (ready_queues[queue_no].list_base)->link;
	return 0;
}

signed char ready_queue_decrease_last_item( unsigned char queue_no )
{
	if ((ready_queues[queue_no].list_base) == NULL){
		return -1;
	}
	
	(ready_queues[queue_no].list_base)->packets_no--;
	if ((ready_queues[queue_no].list_base)->packets_no==0)
		ready_queue_erase_last_item(queue_no);
	return 0;
}

/* The following function are to replace the 
 * - ready_queue_watch_last_item
 * - ready_queue_decrease_last_item
 * approach.
 *
 * Although this new approach could be improved by avoiding the pointer indexing every time
 * accessing the queue.
 */

signed char ready_queue_last_item_pointer(unsigned char queue_no, READY_QUEUE_IDX **item)
{
	if ((ready_queues[queue_no].list_base) == NULL){
		*item = NULL;
		return -1;
	}

	/* One might think that I should masrk the var as SM_TRANSMITING
	 * I am not doing it here 'cause this message may still not be scheduled
	 * It will be set in ready_queue_decrease_last_item()
	 */
	*item=(ready_queues[queue_no].list_base);
	return 0;
}
signed char ready_queue_next_item_pointer(READY_QUEUE_IDX **item)
{
//	if ( (*item)->link == NULL ){
//		return -1;
//	}

	*item = (*item)->link;
	return 0;
}

static signed char ready_queue_erase_this_item( READY_QUEUE_IDX **item_p )
{
	
	if ((*item_p) == NULL){
		return -1;
	}

	/* The message was fully dispatched - Refresh the activation counter */
	MDB_Update_activation( (*item_p)->message_DB_pointer );

	(*item_p)->message_DB_pointer=NULL;
	(*item_p) = (*item_p)->link;
	return 0;
}

signed char ready_queue_decrease_item_pointer( unsigned char queue_no, READY_QUEUE_IDX **item )
{
	READY_QUEUE_IDX **temp;
	
	temp = &(ready_queues[queue_no].list_base);

	for ( temp = &(ready_queues[queue_no].list_base); (*temp)!=NULL; temp = &((*temp)->link) ){
		if ((*temp) == *item){
			/*This is the one */
			(*temp)->packets_no--;
			if ( (*temp)->packets_no == 0 )
				ready_queue_erase_this_item( temp );
			*item = (*temp);

			return 0;
		}
	}

	return -1;
}

#define ARRAY_MAX_NO	50
static READY_RETURN_ITEMS g_array_return[ARRAY_MAX_NO]; //This is made global to avoid thread stacking

signed char ready_queue_get_activations_from_db(void)
{
	unsigned short num_ret;
	unsigned short i;
	int ret;

	/* Get the activation list from DB and push it into the ready queue */

	num_ret=ARRAY_MAX_NO; /* this is the input parameter for the arrays' maximum size */
	if( (ret=MDB_SRT_get_runtime_activations( g_array_return, &num_ret )) < 0 ){
		ERROR_MSG("MDB_SRT_get_runtime_activations ret:%d", ret);
		return -1;
	}

	for (i=0; i<num_ret; i++){
		if ( (ret=ready_SM_list_add_item( &(g_array_return[i]))) < 0 ){
			ERROR_MSG("ready_SM_list_add_item ret:%d", ret);
			return -2;
		}
	}
	
	num_ret=ARRAY_MAX_NO; /* this is the input parameter for the arrays' maximum size */
	if( (ret=MDB_ART_get_runtime_activations( g_array_return, &num_ret )) < 0 ){
		ERROR_MSG("MDB_ART_get_runtime_activations ret:%d", ret);
		return -1;
	}

	for (i=0; i<num_ret; i++){
		if ( (ret=ready_AM_list_add_item( &(g_array_return[i]))) < 0 ){
			ERROR_MSG("ready_AM_list_add_item ret:%d", ret);
			return -3;
		}
	}

	return 0;
}

void ready_queue_all_print(void)
{
	ready_queue_print(SRT_QUEUE);
//	ready_queue_print(ART_H_QUEUE);
	ready_queue_print(ART_S_QUEUE);
//	ready_queue_print(ART_U_QUEUE);
}

#if 0
static signed char ready_queue_refresh_message_pointers( unsigned char queue_no )
/* This function goes for each item queued and updates the message pointer that
 * surely changed upon a SRDB_swap
 */
{
	READY_QUEUE_IDX **idx_temp;
	void * temp_p;
	unsigned int  m_id;

	idx_temp = &(ready_queues[queue_no].list_base);
	while (1){

		if (*idx_temp == NULL){
			ERROR_MSG("NULL");
			break;
		}
		
		/* Get the ID from the old plan */
		m_id = (*idx_temp)->message_DB_pointer->sm.m_id;

		/* Search that ID in the new one 
		 * it might even had disapeared! */
		temp_p = (void *) MDB_runtime_track_mesg_id_pointer( m_id );

		if (temp_p == NULL){
			/* Lets remove it */
			(*idx_temp)->message_DB_pointer=NULL;
			*idx_temp=(*idx_temp)->link;
		}

		(*idx_temp)->message_DB_pointer = temp_p;
		
		/* Increment */
		idx_temp = &((*idx_temp)->link);
	}
	return 0;
}

void Ready_queues_update_all_message_pointers( void )
{
	ready_queue_refresh_message_pointers( SRT_QUEUE );
	ready_queue_refresh_message_pointers( ART_H_QUEUE );
	ready_queue_refresh_message_pointers( ART_S_QUEUE );
	ready_queue_refresh_message_pointers( ART_U_QUEUE );
	return ;
}
#endif
