/*****************************************************************************
 * M_SRDB.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#define M_SRDB_C

#include "Ports.h"
#include "M_SRDB.h"

#include "ethernet.layer.h"

#include "ftt-global.h"
#include "M_db.h"
#include "M_master_ch.h"
#include "M_SW_add_table.h"
#include "igmp.h"

//#define FORCE_UNICAST_AS_MULTICAST

#define CHECK_LIST_ITEMS_NO	50
typedef enum
{
	CHECK_LIST_NOT_IN_USE=-1,  //used for internal control
	CHECK_LIST_NO_UPDATE=0,    //No updates are necessary in the SRDB or NRDB
	CHECK_LIST_UPDATE=1,       //Some updates are necesary in bothe SRDB and NRDB
	CHECK_LIST_FORCE_NEW=2,    //The message will be forced as new in the SRDB and NRDB
	CHECK_LIST_DELETE=3,       //The message will be deleted from SRDB and NRDB
	CHECK_LIST_CHECKED=4       //used for internal control
}CHECK_LIST_STATES;

inline static void CHECK_LIST_STATES_print(CHECK_LIST_STATES *state, unsigned char f)
{
	switch (*state){
		case CHECK_LIST_NOT_IN_USE: PRINTF("NOT_IN_USE"); break;
		case CHECK_LIST_NO_UPDATE: PRINTF("NO_UPDATE"); break;
		case CHECK_LIST_UPDATE: PRINTF("UPDATE"); break;
		case CHECK_LIST_DELETE: PRINTF("DELETE"); break;
		case CHECK_LIST_CHECKED: PRINTF("CHECKED"); break;
		default: PRINTF("WRONG!!!");
	}
	if (f) PRINTF("\n");
}

typedef struct {
	unsigned short type;
	unsigned short id;
}CHECK_LIST_ID;

#define CHECK_LIST_ID_COMPARE( _id1, _id2 ) ( *((unsigned int *)&_id1) ==  *((unsigned int *)&_id2) )


typedef struct{
	unsigned int no; //number of contents in the table
	struct item{
		CHECK_LIST_STATES state;
		CHECK_LIST_ID id;
		unsigned char id_tag;
	} items[CHECK_LIST_ITEMS_NO];
}CHECK_LIST_ITEM;

static CHECK_LIST_ITEM system_checklist;

/* Mutex to serialize access to the checklist 
 *  - Presently this would not be needed, but in case it becomes necessary in the future */
#define CHECK_LIST_LockerDeclaration   rtl_pthread_mutex_t system_checklist_access_mutex;
#define CHECK_LIST_Enter           rtl_pthread_mutex_lock(&system_checklist_access_mutex);
#define CHECK_LIST_Leave           rtl_pthread_mutex_unlock(&system_checklist_access_mutex);
#define CHECK_LIST_Init_locker     rtl_pthread_mutex_init(&system_checklist_access_mutex, 0);
#define CHECK_LIST_Destroy_locker   rtl_pthread_mutex_destroy(&system_checklist_access_mutex);
static CHECK_LIST_LockerDeclaration;

static void M_SRDB_system_checklist_cleanup(void)
{
	unsigned int i;

	system_checklist.no=0;
	for (i=0; i<CHECK_LIST_ITEMS_NO; i++)
		system_checklist.items[i].state=-1;
}

static void M_SRDB_system_checklist_init(void)
{
	M_SRDB_system_checklist_cleanup();
	CHECK_LIST_Init_locker;
}

static void M_SRDB_system_checklist_close(void)
{
	CHECK_LIST_Destroy_locker;
}

static void M_SRDB_system_checklist_setall_delete(void)
{
	unsigned int i,no=0;

	for (i=0; i<CHECK_LIST_ITEMS_NO; i++){
		/* If all elements have been considered */
		if( (no++)==system_checklist.no ) break;

		if ( !(system_checklist.items[i].state == CHECK_LIST_NOT_IN_USE) ){
			system_checklist.items[i].state = CHECK_LIST_DELETE;
		}
	}
}

static signed char M_SRDB_system_checklist_get_tag( CHECK_LIST_ID id )
{
	unsigned int i,no=0;

	for (i=0; i<CHECK_LIST_ITEMS_NO; i++){
		if ( !(system_checklist.items[i].state == CHECK_LIST_NOT_IN_USE) ){
			if ( CHECK_LIST_ID_COMPARE( system_checklist.items[i].id, id )){
				return system_checklist.items[i].id_tag;
			}
		}else
		/* If all elements+1 have been considered */
		if( (++no)>=system_checklist.no ) break;
	}
	return -1;
}

static void M_SRDB_system_checklist_set_status( CHECK_LIST_ID id, unsigned char status, signed char tag)
{
	unsigned int i,no=0;
	signed int fei=-1; //first empty item

	for (i=0; i<CHECK_LIST_ITEMS_NO; i++){
		if ( !(system_checklist.items[i].state == CHECK_LIST_NOT_IN_USE) ){
			if ( CHECK_LIST_ID_COMPARE(system_checklist.items[i].id, id ) ){
				system_checklist.items[i].state = status;
				if (tag>=0) system_checklist.items[i].id_tag = tag;
				return;
			}
		}else if (fei<0) fei=i;
		
		
		/* If all elements+1 have been considered */
		if( (++no)>=system_checklist.no ) break;
	}

	/* We have reached the end with no id found 
	 * Let's create one */
	if (i==CHECK_LIST_ITEMS_NO){
		if (fei<0){
			ERROR_MSG("Could not find space for new message - design problem: check CHECK_LIST_ITEMS_NO");
		}else{
			system_checklist.items[fei].id = id;
			system_checklist.items[fei].state = status;
			if (tag>=0) system_checklist.items[fei].id_tag = tag;
			system_checklist.no ++;
		}
	}else{
		if (fei<0){
			system_checklist.items[i+1].id = id;
			system_checklist.items[i+1].state = status;
			if (tag>=0) system_checklist.items[i+1].id_tag = tag;
			system_checklist.no ++;

		}else{
			system_checklist.items[fei].id = id;
			system_checklist.items[fei].state = status;
			if (tag>=0) system_checklist.items[fei].id_tag = tag;
			system_checklist.no ++;
		}
	}
}

//static signed int M_SRDB_system_checklist_search_deleted_and_do_it( CHECK_LIST_ID *id )
//{
//	unsigned int i,no=0;
//
//	for (i=0; i<CHECK_LIST_ITEMS_NO; i++){
//		if ( !(system_checklist.items[i].state == CHECK_LIST_NOT_IN_USE) )
//			if ( system_checklist.items[i].state == CHECK_LIST_DELETE){
//				system_checklist.items[i].state = CHECK_LIST_NOT_IN_USE;
//				system_checklist.no --;
//				*id = system_checklist.items[i].id;
//				return 0;
//			}
//		
//		/* If all elements have been considered */
//		if( (++no)>=system_checklist.no ) break;
//	}
//
//	/* No items set to delete */
//	return -1;
//}

//static signed int M_SRDB_system_checklist_search_state_and_set_as_checked(unsigned char state, CHECK_LIST_ID *id)
//{
//	unsigned int i,no=0;
//
//	for (i=0; i<CHECK_LIST_ITEMS_NO; i++){
//		if ( !(system_checklist.items[i].state == CHECK_LIST_NOT_IN_USE) )
//			if ( system_checklist.items[i].state == state){
//				system_checklist.items[i].state = CHECK_LIST_CHECKED;
//				*id = system_checklist.items[i].id;
//				return 0;
//			}
//		
//		/* If all elements have been considered */
//		if( (++no)>=system_checklist.no ) break;
//	}
//
//	/* No items set to delete */
//	return -1;
//}

static signed int M_SRDB_system_checklist_search_not_checked_and_check(CHECK_LIST_ID *id, unsigned char *tag)
{
	unsigned int i,no=0;
	CHECK_LIST_STATES old_state;

	for (i=0; i<CHECK_LIST_ITEMS_NO; i++){
		old_state = system_checklist.items[i].state;
		if ( !(old_state == CHECK_LIST_NOT_IN_USE) )
			if ( old_state != CHECK_LIST_CHECKED){				
				system_checklist.items[i].state = CHECK_LIST_CHECKED;
				*id = system_checklist.items[i].id;
				*tag = system_checklist.items[i].id_tag;
				return old_state;
			}
		
		/* If all elements have been considered */
		if( (++no)>=system_checklist.no ) break;
	}

	/* No items set to delete */
	return -1;
}

static void M_SRDB_system_checklist_remove_item(CHECK_LIST_ID *check_id)
{
	unsigned int i;
	for (i=0; i<CHECK_LIST_ITEMS_NO; i++)
		if ( !( system_checklist.items[i].state == CHECK_LIST_NOT_IN_USE ) )
			if ( CHECK_LIST_ID_COMPARE( system_checklist.items[i].id, *check_id) ){
				system_checklist.items[i].state = CHECK_LIST_NOT_IN_USE;
				break;
			}
	return ;
}

void M_SRDB_system_checklist_print(void)
{
	unsigned int i;

	PRINT_MSG("There are %d items in the checklist", system_checklist.no );
	for (i=0; i<CHECK_LIST_ITEMS_NO; i++)
		if ( !(system_checklist.items[i].state == CHECK_LIST_NOT_IN_USE) ){
			PRINT_MSG("item: %d   ID:0x%x Status: ", i, *((unsigned int *)&(system_checklist.items[i].id)) );
			CHECK_LIST_STATES_print(&(system_checklist.items[i].state),0);
		}
}

//////////////////////////Static table contents

#define STATIC_TABLE_ITEMS_NO	10

static GEN_MESSAGE_ITEM M_SRDB_static_table_contents[STATIC_TABLE_ITEMS_NO];
static MESSAGES_TABLE M_SRDB_static_table;

////////////////////The interface functions

signed char M_SRDB_Init(void)
{
	/* Check if M_db was initialized */
	if ( !MDB_Init_check()){
		ERROR_MSG("MDB_Init_check failed");
		return -1;
	}

	/* Initialize the internal Checklist table */
	M_SRDB_system_checklist_init();

	/* Initialize the static table */
	if ( MESSAGES_TABLE_Init( &M_SRDB_static_table, M_SRDB_static_table_contents, STATIC_TABLE_ITEMS_NO ) < 0){
		return -2;
	}
	
#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -M_SRDB_Init Ok");
#endif
	return 0;
}

signed char M_SRDB_Close(void)
{
	MESSAGES_TABLE_Close(&M_SRDB_static_table);
	M_SRDB_system_checklist_close();
#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -M_SRDB_Close Ok");
#endif
	return 0;
}

static inline signed char M_SRDB_determine_tx_mac(MESSAGE_REQUESITS *req_p, unsigned char *mac2produce )
{
	switch( req_p->sw_consumer_no ){
		case 0: ETH_L_Str2Addr((unsigned char *)"FF:FF:FF:FF:FF:FF", mac2produce);
			break;
#ifndef	FORCE_UNICAST_AS_MULTICAST
		case 1:
			if (!M_SW_get_mac( req_p->sw_consumer_idx[0], mac2produce )){
				ERROR_MSG("warning error: Could not resolve the MAC for this var 0x%x.\
						The node was still not registered.", req_p->m_id);
				return -1;
			}
			break;
#endif
		default:
			/* Multicast */
			/* This is the only situation in which the messages ID (class 3) mixes 
			 * with the MAC (class 2) */
			IGMP_setMulticastBase(mac2produce);

			/* the last 2 bytes - get the var id*/
			*((unsigned short *)&(mac2produce[4])) = htons((unsigned short)(req_p->m_id));
			break;
	}
	return 0;
}

signed char M_SRDB_Update(MESSAGES_TABLE *dyn_table)
{
	signed int i,ret;
	MESSAGE_REQUESITS *req_p;
	CHECK_LIST_ID check_id;
	unsigned char tag;
	unsigned char mac2produce[6];
	signed char next_tag_proposal;

	if (dyn_table==NULL) return -1;
//	PRINT_MSG("SRDB.c:>>>>>>>>>>> Runtime table BEFORE update");
//	MDB_runtime_print();

	MESSAGES_TABLE *stat_table = &M_SRDB_static_table;

	/* Lock the M_db for changes */
	MDB_Change_lock();

	/* Lock our static requirements table */
	MESSAGES_TABLE_EnterLock(stat_table);

	/* Lock the received dynamic requirements table */
	MESSAGES_TABLE_EnterLock(dyn_table);

	/* Lock the checklist */
	CHECK_LIST_Enter;

	/* With so many lockers this should be safer than Fort Knox */

//PRINT_MSG("->checklist before deleting all");
//M_SRDB_system_checklist_print();
	/* Prepare our internal checklist to track the variables that will be published to the slaves */
	M_SRDB_system_checklist_setall_delete();
//PRINT_MSG("->checklist after deleting all");
//M_SRDB_system_checklist_print();

	/* Update the static.... */
	for (i=0; i<(stat_table->items_no); i++ ){
		/* For each element in the static table */
		check_id.type = (stat_table->data)[i].message_type;
		check_id.id = (stat_table->data)[i].message_id;

		if ( (next_tag_proposal = M_SRDB_system_checklist_get_tag( check_id )) < 0)
			next_tag_proposal = -1;
		next_tag_proposal = ((unsigned char)(next_tag_proposal+1)) & FTT_VAR_ID_TAG_BASEMASK;
		
		if ( (ret = MDB_add_msg( (stat_table->data)[i].message_type, next_tag_proposal, &((stat_table->data)[i].req), (stat_table->data)[i].add_type/*overwrite_f*/ )) < 0 ){
			ERROR_MSG("Could not MDB_add_msg: probably design errror.");
			CHECK_LIST_Leave;
			MDB_Change_forget();
			MESSAGES_TABLE_LeaveLock(stat_table);
			MESSAGES_TABLE_LeaveLock(dyn_table);
			return -2;
		}

		switch ( ret ){
			case 0: /* No changes */
				M_SRDB_system_checklist_set_status( check_id, CHECK_LIST_NO_UPDATE, -1 ); break;
			case 1: /* Update changes */
				M_SRDB_system_checklist_set_status( check_id, CHECK_LIST_UPDATE, next_tag_proposal ); break;
			case 2: /* Force the changes */
				M_SRDB_system_checklist_set_status( check_id, CHECK_LIST_FORCE_NEW, next_tag_proposal ); break;
			default: ERROR_MSG("Wrong return - programming error");			
		}
	}

	/* Update the dynamic.... */
	for (i=0; i<(dyn_table->items_no); i++ ){
		/* For each element in the dynamic table */
		check_id.type = (dyn_table->data)[i].message_type;
		check_id.id = (dyn_table->data)[i].message_id;

		if ( (next_tag_proposal = M_SRDB_system_checklist_get_tag( check_id )) < 0)
			next_tag_proposal = -1;
		next_tag_proposal = ((unsigned char)(next_tag_proposal+1)) & FTT_VAR_ID_TAG_BASEMASK;

		if ( (ret = MDB_add_msg( (dyn_table->data)[i].message_type, next_tag_proposal, &((dyn_table->data)[i].req), (dyn_table->data)[i].add_type/*overwrite_f*/ )) < 0 ){
			ERROR_MSG("Could not MDB_add_msg: probably design errror.");
			CHECK_LIST_Leave;
			MDB_Change_forget();
			MESSAGES_TABLE_LeaveLock(stat_table);
			MESSAGES_TABLE_LeaveLock(dyn_table);
			return -2;	
		}

		switch ( ret ){
			case 0: /* No changes */
				M_SRDB_system_checklist_set_status( check_id, CHECK_LIST_NO_UPDATE, -1 ); break;
			case 1: /* Update changes */
				M_SRDB_system_checklist_set_status( check_id, CHECK_LIST_UPDATE, next_tag_proposal ); break;
			case 2: /* Force the changes */
				M_SRDB_system_checklist_set_status( check_id, CHECK_LIST_FORCE_NEW, next_tag_proposal ); break;
			default: ERROR_MSG("Wrong return - programming error");		
		}
	}

//PRINT_MSG("->checklist after update static&dynamic");
//M_SRDB_system_checklist_print();

	/* Purge this update in the M_db, then unlock */
	MDB_Change_submit(1 /*swapping*/ );

	/* Let's now propagate the M_db changesMDB_runtime_print(void) into the slaves */

	while ( (ret = M_SRDB_system_checklist_search_not_checked_and_check( &check_id, &tag )) != -1){
		switch(ret){
			case CHECK_LIST_DELETE: 
				M_master_ch_Publish_del( 1, check_id.type, check_id.id );

				/* Let's remove the message from the check list 
				 *  - This code was added very much late. It mght be doing something nasty
				 *  - Is it necessary to preserve the old deleted for any reason?
				 *     - assuming not.
				 */
				M_SRDB_system_checklist_remove_item( &check_id );
				break;
			
			case CHECK_LIST_FORCE_NEW:
			case CHECK_LIST_UPDATE: 
				//PRINT_MSG("Searching for: 0x%x", check_id.id );
				/* Search the var requirements in both tables */
		
				if ((req_p = MESSAGES_TABLE_Search_ID( dyn_table, check_id.type, check_id.id )) == NULL)
				if ((req_p = MESSAGES_TABLE_Search_ID( stat_table, check_id.type, check_id.id )) == NULL){
					ERROR_MSG("Desigh error. Could not find the message in the dynamic nor static tables");
					continue;
				}
				
				/* Determine the MAC address with which the producer transmits this var */
				if ( M_SRDB_determine_tx_mac( req_p, mac2produce ) < 0 ){
					ERROR_MSG("Couldn't determine the MAC - Abort this notification");
					continue;
				}
				
				/* Publish */
//				PRINT_MSG("    c: Publish order for Update/Forcenew");
				if ( M_master_ch_Publish_add( 
							1 /*hold*/, 
							check_id.type,
							check_id.id,
							tag,
							req_p, 
							mac2produce,
							((ret==CHECK_LIST_FORCE_NEW)?1:0) ) < 0 ){
					ERROR_MSG("Error publishing updates to slaves (M_master_ch_Publish_add)");
					continue;
				}

				break;
			case CHECK_LIST_NO_UPDATE: break;
			default: ERROR_MSG("Wrong return - programming error");
		}
	}

	M_master_ch_Publish_dummy(0/*!hold*/);
	
//M_SRDB_system_checklist_print();

	/* Unlock the checklist */
	CHECK_LIST_Leave;
	
	/* UnLock the tables */
	MESSAGES_TABLE_LeaveLock(stat_table);
	MESSAGES_TABLE_LeaveLock(dyn_table);
//PRINT_MSG(">>>>>>>>>>>M_SRDB_Update perfect exit ");

//	PRINT_MSG("SRDB.c:>>>>>>>>>>> Runtime table AFTER update");
//	MDB_runtime_print();
	return 0;
}

void M_SRDB_Hardcoded_Table_Print(void)
{
	PRINT_MSG("Hardcoded table");
	MESSAGES_TABLE_print( &M_SRDB_static_table, 0);
}

signed char M_SRDB_Hardcoded_Table_Append_var(MESSAGE_TYPE type, unsigned short id, MESSAGE_REQUESITS *req)
{
	int ret;
	unsigned char mac2produce[6];

	if (req==NULL) return -1;

	MESSAGES_TABLE_EnterLock( &M_SRDB_static_table );

	/* Lock the M_db for changes */
	MDB_Change_lock();

	/* Check if this var is already static */
	/*   - I'll ignore if it is dynamic because it shouldn't be */

	if (MESSAGES_TABLE_Search_ID( &M_SRDB_static_table, type, id)!=NULL ){
		ERROR_MSG("Static message could not be added because another one was found");
	}

	if ( MESSAGES_TABLE_Append_new( &M_SRDB_static_table, req, type, id, 0 /*not overwrite*/) < 0){
		ERROR_MSG("FATAL ERROR: Could not append to static table");
		MDB_Change_forget();
		MESSAGES_TABLE_LeaveLock( &M_SRDB_static_table );
		return -2;
	}

	ret = MDB_add_msg( type , 0/*tag*/,  req, 0 /*not overwrite*/ );
	if (ret<0){
		ERROR_MSG("Could not add message in M_db: design errror.");
		MDB_Change_forget();
		MESSAGES_TABLE_LeaveLock( &M_SRDB_static_table );
		return -3;
	}
	
	MDB_Change_submit(0 /*Merging*/ );


	/* Determine the MAC address with which the producer must produce this var */
	switch( req->sw_consumer_no ){
		case 0:
			ETH_L_Str2Addr((unsigned char *)"FF:FF:FF:FF:FF:FF", mac2produce);
		break;
	case 1:
		if (!M_SW_get_mac( req->sw_consumer_idx[0], mac2produce )){
			ERROR_MSG("Fatal error: could not resolve the MAC for production, BROADCAST set");
			ETH_L_Str2Addr((unsigned char *)"FF:FF:FF:FF:FF:FF", mac2produce);
		}
		break;
	default:
		/* Multicast it */
		/* This is the only situation in which the messages ID (class 3) mixes 
		 * with the MAC (class 2) */
		IGMP_setMulticastBase(mac2produce);

		/* the last 2 bytes */
		*((unsigned short *)&(mac2produce[4])) = htons((unsigned short)(req->m_id));
		break;
	}

	/* Publish */
	if (M_master_ch_Publish_add( 0 /*!hold*/, type, id, 0/*tag*/ , req, mac2produce, 1/*force add*/ )<0){
		ERROR_MSG("Fatal error: publishing updates to slaves.... let's keep going");
		/* If this error show up when registering M_master_ch_init that's normal... */
	}

	MESSAGES_TABLE_LeaveLock( &M_SRDB_static_table );

	return 0;
}

signed char M_SRDB_get_hardcoded_table( MESSAGES_TABLE **table )
{
	*table = &M_SRDB_static_table;
	return 0;
}


