/*****************************************************************************
 * M_SRDB.h:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

/*! \page SRDB SRDB - System Requirements DataBase
 This block is basically responsible for dealing with the system registration requests coming from the upper layer and applying those in the "db". At startup, it issues the creation of a broadcast channel that is used to synchronize the Master "db" with the other Slaves. The "db" compiles all the requirements of the admitted communication channels and also some dynamic properties that are useful to the scheduler. The db is implemented in a double buffering mechanism that enables a fast switch when doing some upgrade, i.e., registering/deregistering some new channel. Basically, a clone of the db is used to implement the new features. This process can run independently of the network operation. When the changes are finally submitted both dbs are compared updating the dynamic properties and swapping the db images. 
 
 Another mechanism implemented in the "db" is the update tagging. Whenever a message with a given ID is updated, i.e. same ID while some properties change, both versions of the variable are kept for a while. This is to maintain the normal flow of the system and the lifecycle of a message after entering the system under the old version tag. It basically enforces that after an update, only new-versioned transmissions will be allowed while the scheduler will be dequeueing on going transmissions with the old tag.
 
 */

#ifndef M_SRDB_H
#define M_SRDB_H

#include "Ports.h"
#include "ftt-core.layer_types.h"

/** Initializes the SRDB. \return 0 - OK */
extern signed char M_SRDB_Init(void);

/** Closes the SRDB. \return 0 - OK */
extern signed char M_SRDB_Close(void);

/** Updates the SRDB with a new table proposal. Using this API we are updating the current dynamic table.
 The dynamic table is the one not hardcoded in the FTT core layer. The dynamic table must be provided as a whole and not as increments to the currently running table.
 \return 0 - OK */
extern signed char M_SRDB_Update(MESSAGES_TABLE *dyn_table);

/** Adds a new variable to the hardcoded table. This should be handleded at the startup.
 \return 0 - OK */
extern signed char M_SRDB_Hardcoded_Table_Append_var(MESSAGE_TYPE type, unsigned short id, MESSAGE_REQUESITS *req);

/** Debug: Print the Hardcoded table */
extern void M_SRDB_Hardcoded_Table_Print(void);

/** Get properties stored in the hardcoded table. This is usefull for the admission control, to know how much load is hardcode in the FTT Core.
 \return 0 - OK */
extern signed char M_SRDB_get_hardcoded_table( MESSAGES_TABLE **table );

#endif
