/*****************************************************************************
 * M_Scheduler.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#include "Ports.h"

#include "M_Ready_queue.h"

#include "M_ECreg.h"
#include "M_SW_add_table.h"
#include "M_db.h"
#include "S_ftt-core.layer_embed.h"

//#define BE_POLITE
//#define DEBUG_SCHED_TIMING

static unsigned long M_Scheduduler_EC_period=0; //us

static unsigned char M_Scheduduler_init_flag=0;


#ifdef SLOTED_EC
#define GET_MESSAGE_SIZE_US(_message, _packet) ( (_packet == _message->packets_no)?\
		_message->real_size.Lastm_ssize*SLOTV_US :\
		_message->real_size.Onem_ssize*SLOTV_US)

#else

#define GET_MESSAGE_SIZE_US(_message, _packet) ( (_packet == _message->packets_no)?\
		_message->real_size.Lastm_bsize*BIT_TIME_US :\
		_message->real_size.Onem_bsize*BIT_TIME_US)
#endif


static unsigned char check_up_link(LINK_LOAD *l_load_p, unsigned long m_size_us, unsigned long load_limit)
{
	if (!( ( l_load_p->up_load + m_size_us ) < load_limit ))
		return 0;
	return 1;
}

static void update_up_link(LINK_LOAD *l_load_p, unsigned long m_size_us)
{
	l_load_p->up_load += m_size_us;

	return;
}

static unsigned char check_down_link(LINK_LOAD *l_load_p, LINK_LOAD *prod_link_p, unsigned long m_size_us, unsigned long load_limit)
{
	unsigned long load_start;
	unsigned short i;

	if (!( ( l_load_p->down_load + m_size_us ) < load_limit ))
		return 0;

	load_start = prod_link_p->up_load;

//	load_start += adicionar o header time or else : load_limit-x

	for (i=0 ; i<l_load_p->down_blocks_no; i++){
		
		/* if before this block */
		if (load_start < l_load_p->down_blocks[i].begin){
			if ( ( (load_limit - l_load_p->down_blocks[i-1].end) - (load_start - l_load_p->down_blocks[i-1].end) - l_load_p->down_blocks[i-1].used ) >= m_size_us )
				return 1;
			else
				return 0;
		
		}

		/* if inside this block */
		if (load_start <= l_load_p->down_blocks[i].end){
			if ( ( (load_limit - l_load_p->down_blocks[i].end) - l_load_p->down_blocks[i].used ) >= m_size_us )
				return 1;
			else
				return 0;
		
		}
	}
	/* all the blocks passed */
	if ( ( load_limit - load_start) >= m_size_us )
		return 1;
	else
		return 0;
}

static void update_down_link(LINK_LOAD *l_load_p, LINK_LOAD *prod_link_p, unsigned long m_size_us)
{
	unsigned long load_start;
	unsigned short i;
	signed short j;

	l_load_p->up_load += m_size_us;
#warning Zahid detected this!
//return;
	load_start = prod_link_p->up_load;

//	load_start += adicionar o header time or else : load_limit-x

	for (i=0 ; i<l_load_p->down_blocks_no; i++){
		
		/* if before this block */
		if (load_start < l_load_p->down_blocks[i].begin){
			if ( (load_start + m_size_us) < l_load_p->down_blocks[i].begin ){
				/* if isolated */

				/* shift all the bolcks */
				for ( j = l_load_p->down_blocks_no; j > i ; j--)
					l_load_p->down_blocks[j]=l_load_p->down_blocks[j-1];
				
				l_load_p->down_blocks_no ++;

				/* Now, I can change it */
				l_load_p->down_blocks[i].begin = load_start;
				l_load_p->down_blocks[i].end = load_start + m_size_us;
				l_load_p->down_blocks[i].used = l_load_p->down_blocks[i+1].used + (l_load_p->down_blocks[i+1].end-l_load_p->down_blocks[i+1].begin);
			}else{
				/* update this block */
				l_load_p->down_blocks[i].end = (l_load_p->down_blocks[i].end - l_load_p->down_blocks[i].begin) + m_size_us + load_start;
				l_load_p->down_blocks[i].begin = load_start;
			}
			
			for ( j = i-1; j >= 0 ; j--)
					l_load_p->down_blocks[j].used += m_size_us;
			
			return;
		}

		/* if inside this block */
		if (load_start <= l_load_p->down_blocks[i].end){
			l_load_p->down_blocks[i].end += m_size_us;

			if (i>0)
				for ( j = i-1; j >= 0 ; j--)
					l_load_p->down_blocks[j].used += m_size_us;

			return;
		
		}
	}

	/* all the blocks passed */
	l_load_p->down_blocks[i].begin = load_start;
	l_load_p->down_blocks[i].end = load_start + m_size_us;
	l_load_p->down_blocks[i].used = 0;
	l_load_p->down_blocks_no ++;

	return;
}

static unsigned char check_message_fit_multidimensinally(
		LINK_LOAD *current_ports_load,
		void *message,
		unsigned short packet,
		unsigned long load_limit
		)
{
	/* packet: 1,2,3,... */
	unsigned long m_size_us;	//message size in us
	unsigned char c;
	MESSAGE_REQUESITS *message_reqs = MDB_Get_runtime_info_reqs( message );
	MESSAGE_FRAGMENTS *message_frags = MDB_Get_runtime_info_frag( message );
	

	/* Get the message size */
	m_size_us = GET_MESSAGE_SIZE_US(message_frags, packet);

	/* Check if it is broadcast */
	if ( message_reqs->sw_consumer_no == 0){
		/* Then it's broadcast */

		/* Check the production link load */
		if (!check_up_link ( &(current_ports_load[ (message_reqs->sw_producer_idx)[0] ]), m_size_us, load_limit ))
			return 0;
		
		/* Search in all the remaining ports to see if it fits */
		for (c=0; c<SW_MAX_PORTS; c++)
		{
			/* skip myself and the sw_ports not in use */
			if ((c == (message_reqs->sw_producer_idx)[0] )||( !M_SW_is_idx_in_use(c) ))
				continue;
			
			if (!check_down_link( &(current_ports_load[c]), &(current_ports_load[ (message_reqs->sw_producer_idx)[0] ]),	m_size_us, load_limit))
				return 0;

		}

	}else
	{
		/* Check the production link load */
		if (!check_up_link ( &(current_ports_load[ (message_reqs->sw_producer_idx)[0] ]), m_size_us, load_limit ))
			return 0;

		/* Check the consumption links load */
		for (c=0; c<message_reqs->sw_consumer_no; c++){
			/* For each consume port */
			if (!check_down_link( &(current_ports_load[message_reqs->sw_consumer_idx[c]]), &(current_ports_load[ (message_reqs->sw_producer_idx)[0] ]),	m_size_us, load_limit))
				return 0;
		}
	}
	return 1;
}

static void update_message_load_multidimensinally(
		LINK_LOAD *current_ports_load,
		void *message,
		unsigned short packet
		)
{
	unsigned long m_size_us;	//message size in us
	unsigned char c;
	MESSAGE_REQUESITS *message_reqs = MDB_Get_runtime_info_reqs( message );
	MESSAGE_FRAGMENTS *message_frags = MDB_Get_runtime_info_frag( message );

	/* Get the message size */
	m_size_us = GET_MESSAGE_SIZE_US(message_frags, packet);

	/* Check if it is broadcast */
	if ( message_reqs->sw_consumer_no == 0){
		/* Then it's broadcast */

		/* Update all the remaining ports */
		for (c=0; c<SW_MAX_PORTS; c++)
		{
			/* skip myself and the sw_ports not in use */
			if ((c == (message_reqs->sw_producer_idx)[0] )||( !M_SW_is_idx_in_use(c) ))
				continue;

			update_down_link(
					&(current_ports_load[c]),
					&(current_ports_load[ (message_reqs->sw_producer_idx)[0] ]),
					m_size_us);
		}

		/* Update production link load */
		update_up_link(	&(current_ports_load[ (message_reqs->sw_producer_idx)[0] ]), m_size_us);

	}else
	{
		/* Check the consumption links load */
		for (c=0; c<message_reqs->sw_consumer_no; c++){
			/* For each consume port */
			update_down_link(
					&(current_ports_load[message_reqs->sw_consumer_idx[c]]),
					&(current_ports_load[ (message_reqs->sw_producer_idx)[0] ] ),
					m_size_us);
		}

		/* Update production link load */
		update_up_link(	&(current_ports_load[ (message_reqs->sw_producer_idx)[0] ]), m_size_us);

	}
	return;
}

static unsigned char check_message_fit_unidimensinally(unsigned long *current_load, void *message, unsigned short packet, unsigned long load_limit )
{
	unsigned long m_size_us;	//message size in us
	MESSAGE_FRAGMENTS *message_frags = MDB_Get_runtime_info_frag( message );

	/* Get the message size */
	m_size_us = GET_MESSAGE_SIZE_US(message_frags, packet);
	return ( (*current_load + m_size_us) < load_limit );

}
static void update_message_load_unidimensinally(
		unsigned long *current_load,
		void *message,
		unsigned short packet
		)
{
	unsigned long m_size_us;	//message size in us
	MESSAGE_FRAGMENTS *message_frags = MDB_Get_runtime_info_frag( message );

	/* Get the message size */
	m_size_us = GET_MESSAGE_SIZE_US(message_frags, packet);

	*current_load += m_size_us;
	return;
}

/************************************************************************/
/** @brief Process ID of scheduler task */
static rtl_pthread_t ftt_scheduler_th;

/** @brief Semaphore descriptor */
static FTT_SEMAPHORE sched_wake_semaph;

/* Prototypes */
static void *ftt_scheduler(void *t);



signed char SchedulerInit( unsigned long ec_period_reg )
{
	rtl_pthread_attr_t attr;
	struct rtl_sched_param sched_param;
	int ret; //temp

	/* Init semaphore to wake the scheduler */
	if ( FTT_SEMAPHORE_init(&sched_wake_semaph, 0) < 0){	//Initialization locked
		ERROR_MSG("Couldn't initialize semaphore");
		return -1;
	}
	
	/* Check some modules dependencies */
	if (Ready_queues_Init_check()){
		ERROR_MSG("Ready_queues_Init_check must be initialized before this one");
		return -2;
	}

	M_Scheduduler_EC_period = ec_period_reg;

	/******************************/
	/* Create ftt_scheduler  task *
	 * Hard realtime              *
	 * Waked by the Dispatcher    */
	/******************************/
	/* Important note: since the dispatcher may have some jitter when activating the schduler */
	/*  due to activation jitter and itself execution jitter, the mit should be reduced for */
	/*  an amount greatest than the max jitter, to avoid activation before mit */

	rtl_pthread_attr_init( &attr );
	sched_param.sched_priority = rtl_sched_get_priority_max(RTL_SCHED_FIFO) - 3;
	rtl_pthread_attr_setschedparam( &attr, &sched_param );
#ifdef RTL_PRO
	rtl_pthread_attr_setfp_np(&attr, 1);//////////////////////
#endif
	ret = rtl_pthread_create( &ftt_scheduler_th, &attr, ftt_scheduler, (void *)0 );
	if (ret){
		ERROR_MSG("Init Fatal Error: pthread_create returned %d", ret);
		return -3;
	}

#ifdef PRINT_THREAD_ID
#ifdef RTL_PRO
	ERROR_MSG("New thread id: %d", ftt_scheduler_th);
#else
	ERROR_MSG("New thread id: %lu", ftt_scheduler_th);
#endif
#endif

	M_Scheduduler_init_flag = 1;
#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -SchedulerInit Ok");
#endif

	return 0;
}

signed char SchedulerInit_check(void)
{
	if (M_Scheduduler_init_flag == 0)
		return -1;
	return 0;
}

void SchedulerClose(void)
{
	/* cancel the ftt_dispatcher thread */
	rtl_pthread_cancel( ftt_scheduler_th );
	rtl_pthread_join( ftt_scheduler_th, NULL );

	/* Clean up the semaphore */
	FTT_SEMAPHORE_destroy(&sched_wake_semaph);
#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -SchedulerClose Ok");
#endif
	return ;
}

void WakeSheduler(void)
{
#ifdef DEBUG_SCHED_TIMING
{
	struct rtl_timespec timestamp;
	rtl_clock_gettime( RTL_CLOCK_REALTIME, &timestamp );
	PRINT_MSG("->Waking: %lu",timestamp.tv_nsec);
}
#endif
	/* Activate ftt_scheduler() */
	FTT_SEMAPHORE_post(&sched_wake_semaph); 
	return ;
}

void *ftt_scheduler(void *t)
{
	READY_QUEUE_IDX *mesg;
	unsigned short fragmented_packet_no;
#ifdef NODE_ASYNC_REPLY
	struct rtl_timespec delaying_time;
#endif

	/* The two following are used to return the AM queue states frome the master built in Slave */
	NDB_AM_Get_sent_RETURN_TYPE ndb_return_AM_queues[STA_MAX_AM_VARS];
	unsigned short ndb_return_buf_size = STA_MAX_AM_VARS;


	while (1)
	{
		/* Wait for next invocation */
		if ( FTT_SEMAPHORE_wait(&sched_wake_semaph) < 0)
			return NULL;
#ifdef BE_POLITE
		PRINT_MSG("ftt_scheduler says:Hello...\n");
#endif
#ifdef DEBUG_SCHED_TIMING
{
	struct rtl_timespec timestamp;
	rtl_clock_gettime( RTL_CLOCK_REALTIME, &timestamp );
	PRINT_MSG("->Waked: %lu",timestamp.tv_nsec);
}
#endif

//		rtl_clock_gettime( CLOCK_REALTIME, &delaying_time );
//		PRINT_MSG ("Time1:%ld", delaying_time.tv_sec*1000000 + delaying_time.tv_nsec/1000 );
	
#ifdef NODE_ASYNC_REPLY
#ifdef RTL_PRO	
		delaying_time = rtl_timespec_from_ns(REAL_IDLE_TIME*1000); //wait 200us
#else
		delaying_time = rtl_timespec_from_ns(REAL_IDLE_TIME*1000); //wait 100ms
#endif
		rtl_clock_nanosleep( RTL_CLOCK_REALTIME, 0, &delaying_time, NULL);
#endif
//		rtl_clock_gettime( CLOCK_REALTIME, &delaying_time );
//		PRINT_MSG ("Time2:%d", delaying_time.tv_sec*1000000 + delaying_time.tv_nsec/1000 );

#ifdef DEBUG_SCHED_TIMING
{
	struct rtl_timespec timestamp;
	rtl_clock_gettime( RTL_CLOCK_REALTIME, &timestamp );
	PRINT_MSG("->After sched delay: %lu",timestamp.tv_nsec);
}
#endif	
		/* Lock DB */
		MDB_Access_Enter_and_Lock();
		/* Update the asynchronous held in the Master built-in Slave */
		NDB_AM_Get_sent( ndb_return_AM_queues, &ndb_return_buf_size);
		while ( ndb_return_buf_size-- ){
			MDB_ART_update_status (
					ndb_return_AM_queues[ndb_return_buf_size].tagged_id,
						0, /* Master source_nodeID */
				       	ndb_return_AM_queues[ndb_return_buf_size].num );
		}

		/* Get the new activations */
		if (ready_queue_get_activations_from_db() < 0)
			ERROR_MSG("Fatal ERROR.....");
	
		/*pre_dispatch the messages - Build the EC plan*/
		ECreg_prepare2build();
//EC_printsched();
		/* Lets handle the SM */
		while (1){
			if ( ready_queue_watch_last_item(SRT_QUEUE, &mesg)<0 ){
				/* There are no more messages to schedule */
				break;
			}

			//PRINT_MSG("scheduling : %d", MDB_Get_runtime_info_reqs(mesg->message_DB_pointer)->m_id );
			if (sched_ec->sm.n_mesgs == EC_MAX_SM_VARS){
				ERROR_MSG("Scheduler stopped due to constaints on EC_MAX_SM_VARS=%d",EC_MAX_SM_VARS);
				break;
			}

			fragmented_packet_no =  
				MDB_Get_runtime_packets_no(mesg->message_DB_pointer) //totalsize
				- mesg->packets_no // number of left packets
				+ 1; //plus one to result in a serialized seq_no 1,2,3,...

#warning "Check this multidimentional call in the ifdef"
#ifdef SW_UNICAST
			if (check_message_fit_multidimensinally(
					sched_ec->sm.ports_load,
					mesg->message_DB_pointer,
					fragmented_packet_no, 
					EC_MAX_us_SP( M_Scheduduler_EC_period )))
#else
			if (check_message_fit_unidimensinally(
					&(sched_ec->sm.updown_load),
					mesg->message_DB_pointer,
					fragmented_packet_no, 
					EC_MAX_us_SP( M_Scheduduler_EC_period )))
#endif
			{
				/* the message was accepted. Now lets include it... */
				sched_ec->sm.msgs[sched_ec->sm.n_mesgs].id = MDB_Get_runtime_id(mesg->message_DB_pointer);
				sched_ec->sm.msgs[sched_ec->sm.n_mesgs].source_nodeID = mesg->source_nodeID;

				
//				if (fragmented_packet_no == 1)
//					sched_ec->sm.msgs[sched_ec->sm.n_mesgs].id |= FRAG_TRIG_TYPE_MASK;
				sched_ec->sm.msgs[sched_ec->sm.n_mesgs].n_fragmentation = fragmented_packet_no;

#ifdef SLOTED_EC 
					sched_ec->sm.msgs[sched_ec->sm.n_mesgs].sloted_size = 
						GET_MESSAGE_SIZE_US( MDB_Get_runtime_info_frag( mesg->message_DB_pointer ), fragmented_packet_no)/SLOTV_US;
#endif
				/* Update number of synch. messages in EC */
				sched_ec->sm.n_mesgs ++;

				/* Update the included loads */
#ifdef SW_UNICAST			
				update_message_load_multidimensinally( sched_ec->sm.ports_load, mesg->message_DB_pointer, (unsigned char)fragmented_packet_no	);
#else
				update_message_load_unidimensinally( &(sched_ec->sm.updown_load), mesg->message_DB_pointer, (unsigned char)fragmented_packet_no );
#endif
				/* pop the message from the buffer */
				ready_queue_decrease_last_item(SRT_QUEUE);
			}else{
				/* The message doesn't fit */
				/* No priority inversion!!! */
				break;
			}
		}

		/* Lets handle the AM_H */
		while (1){
			if ( ready_queue_watch_last_item(ART_H_QUEUE, &mesg)<0 ){
				/* There are no more messages to schedule */
				break;
			}

			if (sched_ec->am.n_mesgs == EC_MAX_AM_VARS){
				ERROR_MSG("Scheduler stopped due to constaints on EC_MAX_AM_VARS=%d",EC_MAX_AM_VARS);
				break;
			}

			fragmented_packet_no =  
				MDB_Get_runtime_packets_no(mesg->message_DB_pointer) //totalsize
				- mesg->packets_no // number of left packets
				+ 1; //plus one to result in a serialized seq_no 1,2,3,...
	
			if (check_message_fit_unidimensinally(
					&(sched_ec->am.updown_load),
					mesg->message_DB_pointer,
					fragmented_packet_no, 
					EC_MAX_us_AP( M_Scheduduler_EC_period )))
			{
				/* the message was accepted. Now lets include it... */
				sched_ec->am.msgs[sched_ec->am.n_mesgs].id = MDB_Get_runtime_id(mesg->message_DB_pointer);
				sched_ec->am.msgs[sched_ec->am.n_mesgs].source_nodeID = mesg->source_nodeID;

				
//				if (fragmented_packet_no == 1)
//					sched_ec->am.msgs[sched_ec->am.n_mesgs].id |= FRAG_TRIG_TYPE_MASK;
				sched_ec->am.msgs[sched_ec->am.n_mesgs].n_fragmentation = fragmented_packet_no;
				
#ifdef SLOTED_EC
					sched_ec->am.msgs[sched_ec->am.n_mesgs].sloted_size = 
						GET_MESSAGE_SIZE_US(MDB_Get_runtime_info_frag( mesg->message_DB_pointer ), fragmented_packet_no)/SLOTV_US;
#endif
				/* Update number of asynch. messages in EC */
				sched_ec->am.n_mesgs ++;

				/* Update the included loads */
				update_message_load_unidimensinally( &(sched_ec->am.updown_load), mesg->message_DB_pointer, fragmented_packet_no );
				
				/* pop the message from the buffer */
				ready_queue_decrease_last_item(ART_H_QUEUE);

			}else{
				/* The message doesn't fit */
				/* No priority inversion!!! */
				break;
			}
		}
		/* Lets handle the AM_S */
		while (1){
			
			if ( ready_queue_watch_last_item(ART_S_QUEUE, &mesg)<0 ){
				/* There are no more messages to schedule */
				break;
			}
			
			if (sched_ec->am.n_mesgs == EC_MAX_AM_VARS){
				ERROR_MSG("Scheduler stopped due to constaints on EC_MAX_AM_VARS=%d",EC_MAX_AM_VARS);
				break;
			}
			
			fragmented_packet_no =  
			MDB_Get_runtime_packets_no(mesg->message_DB_pointer) //totalsize
			- mesg->packets_no // number of left packets
			+ 1; //plus one to result in a serialized seq_no 1,2,3,...
			
			if (check_message_fit_unidimensinally(
												  &(sched_ec->am.updown_load),
												  mesg->message_DB_pointer,
												  fragmented_packet_no, 
												  EC_MAX_us_AP( M_Scheduduler_EC_period )))
			{
				/* the message was accepted. Now lets include it... */
				sched_ec->am.msgs[sched_ec->am.n_mesgs].id = MDB_Get_runtime_id(mesg->message_DB_pointer);
				sched_ec->am.msgs[sched_ec->am.n_mesgs].source_nodeID = mesg->source_nodeID;

				
				//				if (fragmented_packet_no == 1)
				//					sched_ec->am.msgs[sched_ec->am.n_mesgs].id |= FRAG_TRIG_TYPE_MASK;
				sched_ec->am.msgs[sched_ec->am.n_mesgs].n_fragmentation = fragmented_packet_no;
				
#ifdef SLOTED_EC
				sched_ec->am.msgs[sched_ec->am.n_mesgs].sloted_size = 
				GET_MESSAGE_SIZE_US(MDB_Get_runtime_info_frag( mesg->message_DB_pointer ), fragmented_packet_no)/SLOTV_US;
#endif
				/* Update number of asynch. messages in EC */
				sched_ec->am.n_mesgs ++;
				
				/* Update the included loads */
				update_message_load_unidimensinally( &(sched_ec->am.updown_load), mesg->message_DB_pointer, fragmented_packet_no );
				
				/* pop the message from the buffer */
				ready_queue_decrease_last_item(ART_S_QUEUE);
				
			}else{
				/* The message doesn't fit */
				/* No priority inversion!!! */
				break;
			}
		}
		
		/* Lets handle the AM_U */
		while (1){
			if ( ready_queue_watch_last_item(ART_U_QUEUE, &mesg)<0 ){
				/* There are no more messages to schedule */
				break;
			}

			if (sched_ec->am.n_mesgs == EC_MAX_AM_VARS)
				break;

			fragmented_packet_no =  
				MDB_Get_runtime_packets_no(mesg->message_DB_pointer) //totalsize
				- mesg->packets_no // number of left packets
				+ 1; //plus one to result in a serialized seq_no 1,2,3,...
	
			if (check_message_fit_unidimensinally(
					&(sched_ec->am.updown_load),
					mesg->message_DB_pointer,
					fragmented_packet_no, 
					EC_MAX_us_AP( M_Scheduduler_EC_period )))
			{
				/* the message was accepted. Now lets include it... */
				sched_ec->am.msgs[sched_ec->am.n_mesgs].id = MDB_Get_runtime_id(mesg->message_DB_pointer);
				sched_ec->am.msgs[sched_ec->am.n_mesgs].source_nodeID = mesg->source_nodeID;

				
//				if (fragmented_packet_no == 1)
//					sched_ec->am.msgs[sched_ec->am.n_mesgs].id |= FRAG_TRIG_TYPE_MASK;
				sched_ec->am.msgs[sched_ec->am.n_mesgs].n_fragmentation = fragmented_packet_no;
				
#ifdef SLOTED_EC
					sched_ec->am.msgs[sched_ec->am.n_mesgs].sloted_size = 
						GET_MESSAGE_SIZE_US(MDB_Get_runtime_info_frag( mesg->message_DB_pointer ), fragmented_packet_no)/SLOTV_US;
#endif
				/* Update number of asynch. messages in EC */
				sched_ec->am.n_mesgs ++;

				/* Update the included loads */
				update_message_load_unidimensinally( &(sched_ec->am.updown_load), mesg->message_DB_pointer, fragmented_packet_no );
				
				/* pop the message from the buffer */
				ready_queue_decrease_last_item(ART_U_QUEUE);

			}else{
				/* The message doesn't fit */
				/* No priority inversion!!! */
				break;
			}
		}
		

		/* Downgrade the activations of all the SMessages */
		if (MDB_SRT_downgrade_runtime_activations()){
			/* There was a missed deadline */
			
			sched_ec->status=MISSED_DEADLINE;
			sched_ec->err_code=SCH_ERRCODE_SM_MISS_DEADLINE;
			//ERROR_MSG ("Scheduler fatal error: Missed deadline in Synchronous message");
		}
	
		/* Downgrade the activations of all the AMessages */
		if (MDB_ART_downgrade_runtime_activations()){
			/* There was a missed deadline */
			
	//		sched_ec->status=MISSED_DEADLINE;
	//		sched_ec->err_code=SCH_ERRCODE_AM_HARD_MISS_DEADLINE;
			//ERROR_MSG ("Scheduler fatal error: Missed deadline in Asynchronous message");
		}

	
		/* If plan sucessfully built, set plan status flag */
		if(sched_ec->status == BUSY_SCHED)
			sched_ec->status = UPDATED;

		/* Unlock */
		MDB_Access_UnLock();

//		EC_printsched();

#ifdef DEBUG_SCHED_TIMING
{
	struct rtl_timespec timestamp;
	rtl_clock_gettime( RTL_CLOCK_REALTIME, &timestamp );
	PRINT_MSG("->Ended: %lu",timestamp.tv_nsec);
}
#endif	
	}
	return NULL;
}
