/*****************************************************************************
 * M_Scheduler.h:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

/*! \page Scheduler Scheduler
 This block takes care of scheduling the messages(channels) listed in the "db". Schedule is taken on a per Elementary Cycle (EC) fashion, so that it is prepared for the next EC a schedule plan based on the current message status in the "db". The messages that get activated are put in a "Ready Queue" list that is kept sorted according to the current scheduling policy. The scheduler then performs for each of the ready messages (in order) a scan over the switch ports and sees if each packet fits within the EC length.
 
The product of this per-EC schedule is stored in the "ECReg", which is later compiled to form the Trigger Message (TM). The ECReg is a double buffering container that also detects scheduling overruns, i.e., detects if the scheduler couldn't finish in time for the next TM transmission.
 
 The Scheduler block works for the two kinds of traffic, Synchronous and Asynchronous. While for the Synchronous traffic activations are strictly periodic, for the asynchronous activations result of receiving the signaling message from the Slaves, stating the messages that have been queued and require a transmission schedule.
 
 */

#ifndef SCHEDULER_H
#define SCHEDULER_H

//#include "M_db.h"

/** @fn signed char SchedulerInit(void) 
 *  @brief This function initializes the ftt-master Scheduling module.
 *
 *  Initializes the Scheduler Task semaphore\n
 *  Creates the Scheduler Task\n
 *  The Scheduler Task will be called and locked
 *  @param - 
 *  @return (=0) - Everything OK.\n (<0) - Fatal error
 */
extern signed char SchedulerInit( unsigned long ec_period_reg );

/** @fn void SchedulerClose(void) 
 *  @brief This function closes the ftt-master Scheduling module.
 *
 *  Destroy the Scheduler Task \n
 *  Destroy the Scheduler Task semaphore
 *  @param - 
 *  @return -
 */
extern void SchedulerClose(void);

/** @fn void WakeSheduler(void) 
 *  @brief This function activates (triggers) the Scheduler Task
 *
 *  @param - 
 *  @return -
 */
extern void WakeSheduler(void);

extern void sched_all_list_update_message_pointers( void );

extern void sched_list_all_print(void);

#endif
