/*****************************************************************************
 * M_appPnP.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#include "Ports.h"

#include "M_master_ch.h"
/* This module has the function of signaling the App upon a AppPnP */
#define REQ_FIFO_SIZE 200



typedef struct
{
	signed short app_id; //if <0, means the node was removed
	unsigned char nodeID;
}REQ_ITEM_TYPE;
	
REQ_ITEM_TYPE req_array[REQ_FIFO_SIZE];
unsigned char req_array_wr, req_array_rd;

static FTT_SEMAPHORE req_array_semaph;



signed char M_appPnP_Init(void)
{

	/* when overlaped means empty */
	req_array_wr=0;
	req_array_rd=0;
	
	if ( FTT_SEMAPHORE_init(&req_array_semaph, 0) < 0){	//Initialization locked
		ERROR_MSG("Couldn't initialize semaphore");
		return -1;
	}
#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -M_appPnP_Init Ok");
#endif	
	return 0;
}


signed char M_appPnP_Close(void)
{
	FTT_SEMAPHORE_destroy(&req_array_semaph);
	
#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -M_appPnP_Close Ok");
#endif
	return 0;
}

signed char M_appPnP_Append_request(signed short app_id, unsigned char nodeID)
{
	/* Check if full */
	if ( ((req_array_wr+1)%REQ_FIFO_SIZE) ==(req_array_rd) ){
		return -1;
	}

	req_array[req_array_wr].app_id = app_id;
	req_array[req_array_wr].nodeID = nodeID;

	req_array_wr = (req_array_wr+1)%REQ_FIFO_SIZE;

	/* increment the semaphore */
	FTT_SEMAPHORE_post(&req_array_semaph);

	return 0;

}

signed char M_appPnP_Get_request_blocking(signed short *app_id, unsigned char *nodeID)
{
	/* hold on the semaphore */
	if ( FTT_SEMAPHORE_wait(&req_array_semaph) < 0 )
		return -1;

	/* Check if empty */
	if ( (req_array_wr) ==(req_array_rd) ){
		return -2;
	}

	*app_id = req_array[req_array_rd].app_id;
	*nodeID = req_array[req_array_rd].nodeID;

	req_array_rd = (req_array_rd+1)%REQ_FIFO_SIZE;

	return 0;
}

signed char M_appPnP_notify_Slave(unsigned char ok, unsigned char app_id, unsigned char nodeID, unsigned short id_tx, unsigned short id_rx)
{
	return M_master_ch_Publish_appPnPnotification( 
		0 /*hold*/,
		app_id,
	       	nodeID,
		ok,
	       	id_tx,
	       	id_rx );
}





