/*****************************************************************************
 * M_db.h:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#ifndef M_DB_H
#define M_DB_H

#include "ftt-global.h"
#include "ftt-core.layer_types.h"

/* For ready queue interface */
typedef struct{
	void *message_p; //This pointer directly identifies a variable inside M_db
	unsigned short pakets_no;
	TIMELINESS_CONSTRAINTS am_type;
	unsigned char source_nodeID;
}READY_RETURN_ITEMS;

extern signed char MDB_Init(unsigned char *mac_reg, unsigned long ec_period_reg );
extern signed char MDB_Close(void);
extern unsigned char MDB_Init_check(void);

extern signed char MDB_add_msg( MESSAGE_TYPE m_type, unsigned char tag, MESSAGE_REQUESITS *reqs, unsigned char overwrite_f);
extern unsigned char MDB_Change_locker_check(void);
extern signed char MDB_Change_lock(void);
extern signed char MDB_Change_forget(void);

extern signed char MDB_Change_submit(unsigned char swap_plans);

extern signed char MDB_SRT_get_runtime_activations( READY_RETURN_ITEMS *array_return_p, unsigned short *num_ret );
extern signed char MDB_ART_get_runtime_activations( READY_RETURN_ITEMS *array_return_p, unsigned short *num_ret );

extern unsigned char MDB_SRT_downgrade_runtime_activations( void );
extern unsigned char MDB_ART_downgrade_runtime_activations( void );

extern void MDB_runtime_print(void);
extern void MDB_backtime_print(void);

extern void MDB_Access_Enter_and_Lock(void);
extern void MDB_Access_UnLock(void);

extern signed short MDB_ART_update_status (unsigned short id, unsigned char source_nodeID, unsigned short update_size);


/* Message runtime state */
typedef enum
{
	SM_OPERATIONAL=0,   //SM_OPERATIONAL-message is waiting for activation
	SM_SCHEDULED,       //SM_SCHEDULED-message is ready for activation
	SM_TRANSMITING,     //SM_TRANSMITING-temporary state when a fragmented packet is in the middle of transmission.
	SM_ZOMBIE,          //SM_ZOMBIE-when the message is deleted but its load
	                    // must be considered for schedulability until its 
	                    // next activation. Only then is purged. When the
	                    // message is deleted in the SCHEDULED state it
	                    // can be imediatly removed if also removed from 
	                    // the sched_queue. When TRANSMITTING, if at least one
	                    // packet as already been sent then the message is
	                    // no longer SCHEDULED. Then when deleting it will 
	                    // became ZOMBIE but the variable will finish transmission.
	SM_WAITING          //SM_WAITING-When a message is virtualy added with 
	                    // the condition of being triggered after a certain
	                    // ZOMBIE message(s) have been purged.
}DYNAMIC_VAR_STATE;

typedef struct{
	unsigned short packets_no;      ///< n. of packets
	struct{
		unsigned long Onem_bsize;      ///< One message size in bits (including all overheads )
		unsigned char Onem_ssize;      ///< One Message size in SLOTS 
		unsigned long Lastm_bsize;     ///< Last message size in bits (including all overheads )
		unsigned char Lastm_ssize;     ///< Last Message size in SLOTS 
	}real_size;
}MESSAGE_FRAGMENTS;

inline static void MESSAGE_FRAGMENTS_print(MESSAGE_FRAGMENTS *frag, unsigned char f)
{
	PRINTF("%d packets (%ld and %ld)", frag->packets_no, frag->real_size.Onem_bsize, frag->real_size.Lastm_bsize);
	if (f) PRINTF("\n");
}

typedef struct{
	signed int activation_counter;
	signed int replenish_counter;
	DYNAMIC_VAR_STATE message_status;
#ifdef NODE_ASYNC_REPLY
	unsigned int queue_total_size;	
	unsigned short queue_size[256]; /* Used for the asynch traffic
									 * Each item refers to a different source ID. */
#endif
}MESSAGE_DYNAMIC_DATA;

inline static void MESSAGE_DYNAMIC_DATA_queue_size_clean( MESSAGE_DYNAMIC_DATA *dyn )
{
	int i;

	for (i=0; i<256; i++)
		(dyn->queue_size)[i]=0;

	dyn->queue_total_size = 0;

	return ;
}

inline static void MESSAGE_DYNAMIC_DATA_queue_size_increment( MESSAGE_DYNAMIC_DATA *dyn, unsigned char source_nodeID, unsigned short value )
{
	(dyn->queue_size)[source_nodeID] += value;
	dyn->queue_total_size += value;
	return ;
}

inline static unsigned short MESSAGE_DYNAMIC_DATA_queue_size_check_and_decrement( MESSAGE_DYNAMIC_DATA *dyn, unsigned char *ret_source_nodeID )
{
	int i;
	unsigned short ret;
	
	/* returns < 0 if */
	if ( !(dyn->queue_total_size) )
		return 0;
	
	for (i=0; i<256; i++){
		if ( (ret = (dyn->queue_size)[i]) ){
			/* This source_nodeID holds sth */
			(dyn->queue_size)[i] = 0;
			dyn->queue_total_size -= ret;
			*ret_source_nodeID = i;
			return ret;
		}
	}

	ERROR_MSG("Fatal-error: The queue_total_size and the queue_size array are inconsistent.");
	
	/* I'll zero the queue_total_size. Just in case. */
	dyn->queue_total_size = 0;
	
	return 0;
}

inline static void MESSAGE_DYNAMIC_DATA_print(MESSAGE_DYNAMIC_DATA *dyn, unsigned char f)
{
	PRINTF("(Acvtn:%d Status:%d Qsize[0]:%d)", dyn->activation_counter, dyn->message_status, (dyn->queue_size)[0]);
	if (f) PRINTF("\n");
}

extern MESSAGE_REQUESITS *MDB_Get_runtime_info_reqs( void *mesgtype_p );
extern MESSAGE_FRAGMENTS *MDB_Get_runtime_info_frag( void *mesgtype_p );
extern MESSAGE_DYNAMIC_DATA *MDB_Get_runtime_info_dynamic( void *mesgtype_p );
extern void MDB_Update_activation( void *mesgtype_p );

//Some MACROS to directly access DATA
#define MDB_Get_runtime_id(_p_)                  ( (MDB_Get_runtime_info_reqs(_p_))->m_id )
#define MDB_Get_runtime_period(_p_)              ( (MDB_Get_runtime_info_reqs(_p_))->m_period )
#define MDB_Get_runtime_deadline(__p)            ( (MDB_Get_runtime_info_reqs(__p))->m_deadline )
#define MDB_Get_runtime_activation_counter(__p)  ( (MDB_Get_runtime_info_dynamic(__p))->activation_counter )
#define MDB_Get_runtime_message_status(__p)      ( (MDB_Get_runtime_info_dynamic(__p))->message_status )
#define MDB_Get_runtime_packets_no(__p)          ( (MDB_Get_runtime_info_frag(__p))->packets_no )

typedef struct{
	struct {
		unsigned short type;
		unsigned short id;
	}id;
	unsigned char tag;
	MESSAGE_REQUESITS *reqs;
}MDB_DUMP_BACK_ITEM;

extern signed char MDB_Dump_back_runtime_reqs( signed short nodeID, MDB_DUMP_BACK_ITEM *array_p, unsigned short *no );


#endif

