/*****************************************************************************
 * M_eth_filter.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#include "Ports.h"

#include "ftt-global.h"

#include "ethernet.layer.h"
#include "M_db.h"

#include "M_SW_add_table.h"

#include "M_master_ch.h"
#include "M_appPnP.h"

#include "M_SRDB.h"
#include "S_ftt-core.layer_embed.h"

#include "M_ECreg.h"

//#define SLAVE_TIMEOUT 2 - Passed through command line


static void Master_ethernet_pkt_rx(unsigned char *rmsg, unsigned short size, struct rtl_timespec rx_timestamp)
{
	int ret;
	FTT_PKT_GENERIC_HEADERS *ftt_rmsg_pointer = (FTT_PKT_GENERIC_HEADERS *)rmsg;
//PRINTF("x");

	if ( ntohs(ftt_rmsg_pointer->eth_header.eth_type) != ETH_FTT_TYPE ){
		//ERROR_MSG("Warnning: Frame is not ETH_FTT_TYPE");
		return;
	}

//	{int i; PRINT_MSG ("DUUUUUUUMP:"); for (i=0; i<20; i++) PRINTF(" %x",rmsg[i+14]);}

	/* Get ftt frame type */
	switch( ntohs(ftt_rmsg_pointer->pkt_header.type) )
	{
		case FTT_MST_MSG: /* Trigger Message received!!! */
			PRINT_MSG("TM rx");
			break;
		case FTT_SDATA_MSG:/* Synchronous Data Message received */
		case FTT_ADATA_MSG: /* Asynchronous Data Message received */
//			PRINT_MSG("master: forwarding S|A DATA_MESG_ID to slave layer");
			S_eth_filter( rmsg, size, rx_timestamp);
			break;
#ifdef NODE_ASYNC_REPLY
		case FTT_PLUGnPLAY:
			//PRINT_MSG("PLUGnPLAY received");

#ifdef PLUG_OUT
			ret = M_SW_subscribe_node( ftt_rmsg_pointer->eth_header.eth_src , 1 );
#else
			ret = M_SW_subscribe_node( ftt_rmsg_pointer->eth_header.eth_src , 0 );
#endif

			if ( !(ret<0) ){
				PRINT_MSG("\n    c: NODE Plug&Play subscribed : %d", ret);
				if (M_master_ch_Send_nodeID( ftt_rmsg_pointer->eth_header.eth_src , ret) < 0){
					ERROR_MSG("The master could reply with the nodeID");
					M_SW_unsubscribe_node( ret );
				}
			}
			break;

		case FTT_ASTATUS_MSG:
			{
				FTT_PKT_ASM_HEADERS *ftt_asm_pointer = (FTT_PKT_ASM_HEADERS *)ftt_rmsg_pointer;
				ASM_MSG_QUEUE_INDEX *ftt_am_element;
				unsigned short idx;
				unsigned short nSm; /* Number off status reports */
				unsigned char last_seq_no;
				//PRINT_MSG("ASTATUS_MESG_ID Received");

				/* Let's now touch the node connection aging time */
				if ( M_SW_heartbeat( ftt_asm_pointer->asm_header.nodeID, ftt_asm_pointer->asm_header.seq_no, &last_seq_no ) == -2 ){
					PRINT_MSG("Slave %d missed sending some signaling messages.", ftt_asm_pointer->asm_header.nodeID);
					/* Ask for the respective retransmissions */
#ifdef SIG_RECOVER
					{
					unsigned char ch_idx;
					for (ch_idx=(last_seq_no+1)&0xFF; ch_idx!=ftt_asm_pointer->asm_header.seq_no ; ch_idx++ ) {
						EC_push_Signaling_notification( ftt_asm_pointer->asm_header.nodeID, ch_idx );
					}
					}
#endif
				}
				
				nSm = ntohs(ftt_asm_pointer->asm_header.nam );
				ftt_am_element = (ASM_MSG_QUEUE_INDEX *)(ftt_asm_pointer->data);

				for ( idx=0; idx<nSm; idx++){
					/* Check if it is an AppPnP request */
					if (ntohs(ftt_am_element[idx].AMesgId) == MASTER_BROADCAST_A_CH_ID){
						M_appPnP_Append_request( ntohs(ftt_am_element[idx].AMesgQueueLen), ftt_asm_pointer->asm_header.nodeID );
						continue;
					}

					MDB_Access_Enter_and_Lock();
					if ( ( ret=MDB_ART_update_status( ntohs(ftt_am_element[idx].AMesgId), ftt_asm_pointer->asm_header.nodeID , ntohs(ftt_am_element[idx].AMesgQueueLen) ) ) < 0 )
						/*One possible cause is upon a message removal... some signals are still floating in the system */
						ERROR_MSG("SRDB_ART_update_status error: %d with var:%x",ret,ntohs(ftt_am_element[idx].AMesgId) );
					MDB_Access_UnLock();

				}

			}
			break;
#endif
		case FTT_IDLE:
			{
				FTT_PKT_IDLE_HEADERS *ftt_idle_pointer = (FTT_PKT_IDLE_HEADERS *)ftt_rmsg_pointer;
				PRINT_MSG("IDDLE time AM message: %x", ntohs( ftt_idle_pointer->idle_header.id ) );
			}
			break;
			
		default:
			ERROR_MSG("Unknown pckt received");
			break;
	} /* End Switch(frametype) */
//PRINTF("X");
}

signed char M_eth_filter_Init(void)
{
	/* The ethernet socket must have already been created */
	ETH_L_rx_fun_register( Master_ethernet_pkt_rx );
	
#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -M_eth_filter_Init Ok");
#endif
	return 0;
}

signed char M_eth_filter_Close(void)
{
	ETH_L_rx_fun_unregister();
#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -M_eth_filter_Close Ok");
#endif
	return 0;
}
