/*****************************************************************************
 * M_master_ch.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/


#include "Ports.h"

#include "ftt-global.h"

#include "M_SRDB.h"
#include "ethernet.layer.h"
#include "S_ftt-core.layer_embed.h"

static unsigned char M_master_ch_sending_buff[MASTER_BROADCAST_A_CH_SIZE];

static unsigned char M_master_ch_channel_initialized;

signed char M_master_ch_Init(unsigned short master_nodeID)
{
	int ret;
	MESSAGE_REQUESITS req;
	
	M_master_ch_channel_initialized=0;
	
	
	if ( sizeof(A_MST_SET_NODEID)>MASTER_BROADCAST_A_CH_SIZE ) return -2;
	if ( sizeof(A_MST_ADD)>MASTER_BROADCAST_A_CH_SIZE ) return -2;
	if ( sizeof(A_MST_DEL)>MASTER_BROADCAST_A_CH_SIZE ) return -2;
	if ( sizeof(A_MST_DEL_OLD_TAG)>MASTER_BROADCAST_A_CH_SIZE ) return -2;
	if ( sizeof(A_MST_DUMMY)>MASTER_BROADCAST_A_CH_SIZE ) return -2;
	if ( sizeof(A_PNP_IDS_NOTIFY)>MASTER_BROADCAST_A_CH_SIZE ) return -2;	

	
	/* Initialize the Master broadcast asynchronous channel */

	/* Add the var in the SRDB, which will update the M_db and update the slaves by 
	 * calling M_master_ch_Publish_add in this module 
	 * That function will notify the producer and consumers of this var, but to do that
	 * it needs this comm channel. The cons. don't need notification since its
	 * broadcast (when so there is no notification). As for the producer since its inside 
	 * the master it also does not need the channel
	 */

	PRINT_MSG("Creating ftt-core broadcast channel ");
	req.m_id = MASTER_BROADCAST_A_CH_ID;
	req.m_size = MASTER_BROADCAST_A_CH_SIZE;
	req.m_max_size = MASTER_BROADCAST_A_CH_SIZE;
	req.m_period = 1;
	req.m_deadline = 1;
	req.m_init = 0;
	req.m_relative_id = -1;
	req.m_MTU = MASTER_BROADCAST_A_CH_MTU;
	req.m_timeliness = TMLN_HARD;
	req.sw_consumer_no = 0; //broadcast
	(req.sw_producer_idx)[0] = master_nodeID; //most probably it's 0
	req.sw_producer_no = 1;
	req.sw_producer_max_no = 1;
	
	if( (ret = M_SRDB_Hardcoded_Table_Append_var( ASYNC_MESSAGE, MASTER_BROADCAST_A_CH_ID, &req)) < 0 ){
		ERROR_MSG("M_SRDB_Append_on_static_vars ret:%d", ret);
		return -1;
	}
	
	/* By this point the var MASTER_BROADCAST_A_CH_ID should have 
	 * already been created in the Master slave's module */
	/* Let's bind the var and make it usable */

	if ( (ret = NDB_bind_mesg( ASYNC_MESSAGE, MASTER_BROADCAST_A_CH_ID, NULL, NULL,
							  0, /*mem_size - 0 'cause mem is controlled by the FTT */
							  20 /* FIFO size */
							  )) < 0){
		/* no signal, and no app memory supplied */
		ERROR_MSG("Couldn't bind the MASTER_BROADCAST_A_CH_ID. NDB_bind_mesg ret: %d", ret);
	}
	
	M_master_ch_channel_initialized=1; /* Means the channel has been created. */

#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -M_master_ch_Init Ok");
#endif
	
	return 0;
}

signed char M_master_ch_Close(void)
{
	//TODO M_SRDB_Remove_on_static_vars
#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -M_master_ch_Close Ok");
#endif
	return 0;
}

signed char M_master_ch_Send_nodeID(unsigned char *mac, unsigned short nodeID)
{
	A_MST_SET_NODEID *temp_p;
	signed char ret;

	/* Get a slot to send the data
	 * It is a non-blocking attempt 'cause this functions is called from the IRQ 
	 * context uppon a packet reception */

#ifdef PRINT_MASTER_NRDBUPDATE_COMMANDS_SENT
	PRINT_MSG("    c: Master Sending nodeID registration %d", nodeID);
#endif
	
	temp_p = (A_MST_SET_NODEID *)M_master_ch_sending_buff;

	temp_p->header.COMMAND_id = htons(FTT_CMD_SET_NODEID);
	temp_p->header.flags = htons(0x1234);

	temp_p->nodeID = htons(nodeID);
	ETH_L_CopyMAC_to_from( temp_p->mac, mac );

	/* I'm not sending to the Master's Slave.... useless*/
	
	ret = NDB_send(MASTER_BROADCAST_A_CH_ID, 0, temp_p, 1/* block on fifo*/);
	if (ret < 0){
		ERROR_MSG("Fatal errro NDB_send on channel MASTER_BROADCAST_A_CH_ID, failed ");
		return -1;
	}

	return 0;
}

signed char M_master_ch_Publish_add( unsigned char hold, unsigned short m_type, unsigned short m_id, unsigned char m_tag, MESSAGE_REQUESITS *req_p, unsigned char *m_prod_mac, unsigned char overwrite )
{
	A_MST_ADD *temp_p;
	unsigned char i;
	signed char ret;

	
	if ( (req_p->sw_consumer_no > 40) || (req_p->sw_producer_no > 40)){
		ERROR_MSG("The Master-to-slave command A_MST_ADD can't hold more than 40 Producers/Consumers.");
		return -1;
	}
	
#ifdef PRINT_MASTER_NRDBUPDATE_COMMANDS_SENT
	PRINT_MSG("    c: Master broadcasting add ftt-id: %s0x%x size:%d/%d T:%d Tx[0]:%d Rx[0]:%d", (m_type==SYNC_MESSAGE)?"S":"A", m_id, req_p->m_size, req_p->m_max_size, req_p->m_period, (req_p->sw_producer_idx)[0],(req_p->sw_consumer_idx)[0]  );
#endif
	
	temp_p = (A_MST_ADD *)M_master_ch_sending_buff;
	
	/* Fill the command */	
	temp_p->header.COMMAND_id = htons(FTT_CMD_ADD_MSG);
	temp_p->header.flags = htons( (hold!=0)<<8 );

	temp_p->adding_type = (overwrite != 0);
	temp_p->mesg_type = (unsigned char)m_type;
	temp_p->mesg_id = htons(m_id);
	temp_p->mesg_id_tag = m_tag;
	temp_p->mesg_size = htoni(req_p->m_size);
	temp_p->mesg_max_size = htoni(req_p->m_max_size);
	temp_p->mesg_MTU = htons(req_p->m_MTU);
	temp_p->mesg_period = htoni(req_p->m_period);
	ETH_L_CopyMAC_to_from( temp_p->mesg_production_mac, m_prod_mac );

	temp_p->mesg_producers_no = req_p->sw_producer_no;
	temp_p->mesg_producers_max_no = req_p->sw_producer_max_no;
	for (i=0; i<(req_p->sw_producer_no); i++)
		temp_p->mesg_producerID[i] = ((req_p->sw_producer_idx)[i]);
	
	temp_p->mesg_consumers_no = (req_p->sw_consumer_no);
	for (i=0; i<(req_p->sw_consumer_no); i++)
		temp_p->mesg_consumerID[i] = ((req_p->sw_consumer_idx)[i]);
	
	/* The loopback - Send the command to the Master's Slave module */
	S_master_decode( temp_p );
	
	if ( !M_master_ch_channel_initialized ) return 0; /* This prevents from sending on the hardcoded MASTER_BROADCAST_A_CH_ID */

	/* If the variable was correctly obtained send through the channel */
	ret = NDB_send(MASTER_BROADCAST_A_CH_ID, 0, temp_p, 1/* block on fifo*/);
	if (ret < 0){
		ERROR_MSG("Fatal errro NDB_send on channel MASTER_BROADCAST_A_CH_ID, failed ");
		return -1;
	}

	return 0;
}


signed char M_master_ch_Publish_del(
		unsigned char hold,
		unsigned short m_type,
		unsigned short m_id
		)
{
	A_MST_DEL *temp_p;
	signed char ret;
	
#ifdef PRINT_MASTER_NRDBUPDATE_COMMANDS_SENT
	PRINT_MSG("    c: Master broadcasting del ftt-id: %s0x%x", (m_type==SYNC_MESSAGE)?"S":"A", m_id );
#endif

	temp_p = (A_MST_DEL *)M_master_ch_sending_buff;

	temp_p->header.COMMAND_id = htons(FTT_CMD_DEL_MSG);
	temp_p->header.flags = htons( (hold!=0)<<8 );

	temp_p->mesg_type = (unsigned char)m_type;
	temp_p->mesg_id = htons(m_id);

	/* To the Master's Slave module */
	S_master_decode( temp_p ); 	

	ret = NDB_send(MASTER_BROADCAST_A_CH_ID, 0, temp_p, 1 /*block on fifo*/); /* It might be that when deleting 
																		 MASTER_BROADCAST_A_CH_ID this returns
																		 with error since the var is no longer there */
	if (ret < 0){
		ERROR_MSG("Fatal errro NDB_send on channel MASTER_BROADCAST_A_CH_ID, failed ");
		return -1;
	}

	return 0;

}

signed char M_master_ch_Publish_del_old_tag(
									unsigned char hold,
									unsigned short m_type,
									unsigned short m_id
									)
{
	A_MST_DEL_OLD_TAG *temp_p;
	signed char ret;
	
#ifdef PRINT_MASTER_NRDBUPDATE_COMMANDS_SENT
	PRINT_MSG("    c: Master broadcasting del old tag ftt-id: %s0x%04x", (m_type==SYNC_MESSAGE)?"S":"A", m_id );
#endif
	
	temp_p = (A_MST_DEL_OLD_TAG *)M_master_ch_sending_buff;

	temp_p->header.COMMAND_id = htons(FTT_CMD_DEL_OLD_TAG);
	temp_p->header.flags = htons( (hold!=0)<<8 );
	
	temp_p->mesg_type = (unsigned char)m_type;
	temp_p->mesg_id = htons(m_id);
	
	/* To the Master's Slave module */
	S_master_decode( temp_p ); 	
	
	ret = NDB_send(MASTER_BROADCAST_A_CH_ID, 0, temp_p, 1 /*block on fifo*/); /* It might be that when deleting 
																		 MASTER_BROADCAST_A_CH_ID this returns
																		 with error since the var is no longer there */
	if (ret < 0){
		ERROR_MSG("Fatal errro NDB_send on channel MASTER_BROADCAST_A_CH_ID, failed ");
		return -1;
	}
	
	return 0;
	
}


signed char M_master_ch_Publish_appPnPnotification(
		unsigned char hold,
		unsigned char app_id,
	    unsigned char nodeID,
		unsigned char ack,
	    unsigned short id_tx,
	    unsigned short id_rx )
{
	A_PNP_IDS_NOTIFY *temp_p;
	signed char ret;

#ifdef PRINT_MASTER_NRDBUPDATE_COMMANDS_SENT
	PRINT_MSG("    c: Master broadcasting i:appPnPnotification to slave %d", nodeID );
#endif
	
	temp_p = (A_PNP_IDS_NOTIFY *)M_master_ch_sending_buff;

	temp_p->header.COMMAND_id = htons(FTT_CMD_SET_APP_PNP_IDS);
	temp_p->header.flags = htons( (hold!=0)<<8 );

	temp_p->app_id = app_id;
	temp_p->nodeID = nodeID;
	temp_p->ack = (ack!=0);
	temp_p->tx_id = htons(id_tx);
	temp_p->rx_id = htons(id_rx);

//	/* To the Master's Slave module */
//	S_master_decode( temp_p );

	ret = NDB_send(MASTER_BROADCAST_A_CH_ID, 0, temp_p, 1/* block on fifo */);
	if (ret < 0){
		ERROR_MSG("Fatal errro NDB_send on channel MASTER_BROADCAST_A_CH_ID, failed ");
		return -1;
	}
	
	return 0;

}

signed char M_master_ch_Publish_dummy(unsigned char hold)
{
	A_MST_DUMMY *temp_p;
	signed char ret;

#ifdef PRINT_MASTER_NRDBUPDATE_COMMANDS_SENT
	PRINT_MSG("    c: Master broadcasting dummy");
#endif
	
	temp_p = (A_MST_DUMMY *)M_master_ch_sending_buff;

	temp_p->header.COMMAND_id = htons(FTT_CMD_DUMMY);
	temp_p->header.flags = htons( (hold!=0)<<8 );
	
	/* To the Master's Slave module */
	S_master_decode( temp_p );
	
	ret = NDB_send(MASTER_BROADCAST_A_CH_ID, 0, temp_p, 1/* block on fifo*/);
	if (ret < 0){
		ERROR_MSG("Fatal errro NDB_send on channel MASTER_BROADCAST_A_CH_ID, failed ");
		return -1;
	}

	return 0;
}

