/*****************************************************************************
 * M_master_ch.h:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/


/*! \page M_master_ch Master Channel
 This is responsible for establishing the connection with all Slaves in order to synchronize the distributed databases.

 Whenever a change is issued to the database through SRDB, the same change is notified to this entity, which then sends to the Slaves.
 
 This command is sent as an asynchronous Broadcast message that is hardcoded in the FTT Core. The transmission of the asynchronous message uses the replicated Slave code that is instantiated in the Master. However, this instantiated Slave is itself a Slave that holds its local db that must also be synchronized. But as it is now, this Slave will not receive a loopback of the broadcast channel. Hence, when preparing the command notification to be broadcast, this entity also shortcuts the message transmission /reception and issues the NRDB update directly in the Master.
 
 This  entity is also responsible for conveying through the Broadcast channel the plug 'n play notification upon adding a node.
 
 */

#ifndef M_MASTER_CH_H
#define M_MASTER_CH_H

/** Initializes the Master channel. \return 0 - OK */
extern signed char M_master_ch_Init(unsigned short master_nodeID);

/** Closes the Master channel. \return 0 - OK */
extern signed char M_master_ch_Close(void);

/** Delivers to the Slave Core a notification stating that the node referring as mac got the nodeID. This comes upon a new node comes in. \return 0 - OK */
extern signed char M_master_ch_Send_nodeID(unsigned char *mac, unsigned short nodeID);


/** Delivers to the Slave interface a notification stating that the node referring as nodeID has the following two channels created to use in the communication with the interface layer in the Master. \return 0 - OK */
extern signed char M_master_ch_Publish_appPnPnotification(
														  unsigned char hold,
														  unsigned char app_id,
														  unsigned char nodeID,
														  unsigned char ack,
														  unsigned short id_tx,
														  unsigned short id_rx );


#if defined(M_SRDB_C)
/** Orders an: Add the message with the following characteristics in the Slaves that are either producers or consumers of the variable in matter. \return 0 - OK */
extern signed char M_master_ch_Publish_add( 
		unsigned char hold, 
		unsigned short m_type, 
		unsigned short m_id, 
		unsigned char m_tag, 
		MESSAGE_REQUESITS *req_p, 
		unsigned char *m_prod_mac,
		unsigned char overwrite /* force adding as a new var */
		);
#endif


/** Orders a: Delete a message in the Slaves that have something with it. \return 0 - OK */
extern signed char M_master_ch_Publish_del(
		unsigned char hold,
		unsigned short m_type,
		unsigned short m_id
		);

/** Orders a: Delete the old tag in the Slaves. This comes after the Master switching to the new version of the given variable. 
 \return 0 - OK */
extern signed char M_master_ch_Publish_del_old_tag(
		unsigned char hold,
		unsigned short m_type,
		unsigned short m_id
		);

extern signed char M_master_ch_Publish_dummy(unsigned char hold);

#endif
