/*****************************************************************************
 * S_Dispatch.h:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

/*! \page S_Dispatcher Slave Dispatcher
  
 This is responsible for sending whatever is needed from this Slave, as a consequence of the reception of the Trigger Message. As defined by the protocol, upon the reception of the TM, the Slave decodes it, and figures out which messages it should transmit. This Dispatcher performs this decoding test and also sends those messages. After sending them all it sleeps for the time at which the Master should be starting the transmission of another TM and at this instant it sends to Master the signaling messages, holding the status of the asynchronous buffers in this Slave. 
 */

#ifndef S_DISPATCH_H
#define S_DISPATCH_H

#include "ftt-global.h"

/** Initializes the Slave Dispatcher. \return 0 - OK */
extern signed char S_Dispatch_Init(void);

/** Closes the Slave Dispatcher. */
extern void S_Dispatch_Close(void);

/** Wakes the Slave Dispatcher (after the reception of a new TM) */
extern void S_Dispatch_Wake(void);

extern void S_Dispatch_Set_ec_time( int ec_time_ms );
extern unsigned int S_Dispatch_Read_ec_time_us( void );
extern unsigned char S_Dispatcher_is_running(void);

#endif
