/*****************************************************************************
 * S_ECscheduled.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#include "Ports.h"

#include "ftt-global.h"
#include "S_ECscheduled.h"


/*! \struct STA_M2P_mesgtype 
\brief Produced message type */
typedef struct {
	unsigned short msg_id;		    ///< Id of the message + type
	unsigned short msg_frag_no;		///< Fragmentation number
	unsigned char  msg_source_nodeID;   ///< Source node ID
} STA_M2P_mesgtype;

/*! \struct STA_M2P_tabtype 
\brief Produced message table type */
typedef struct {
	unsigned short nsm;  ///< Number of Synch mesgs 
	unsigned short nam;  ///< Number of Asynch mesgs
	unsigned short nsa;  ///< Number of Missed_signaling messages
	STA_M2P_mesgtype Smesg2p[EC_MAX_SM_VARS];  ///< Message list
	STA_M2P_mesgtype Amesg2p[EC_MAX_AM_VARS];  ///< Message list
	STA_M2P_mesgtype sig2p[EC_MAX_SIG_MISSING_REQUESTS];  ///< Message list
	struct timespec tm_timestamp; /* TM timestamp */
				/* append seq_no */
} STA_M2P_tabtype;

/*!
\brief Table with the messages scheduled to be produced within the EC */
static STA_M2P_tabtype mesg2p_table;

signed char S_ECScheduled_Init(void)
{
//	tm_seqn = 0; /* TM sequence number */
	/* Reset the messages number */
	mesg2p_table.nsm=0;
	mesg2p_table.nam=0;
	mesg2p_table.nsa=0;

#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -S_ECScheduled_Init Ok");
#endif
	
	return 0;
}
signed char S_ECScheduled_Close(void)
{
#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -S_ECScheduled_Close Ok");
#endif
	
	return 0;
}

static unsigned char S_ECScheduled_GetSMessage( unsigned short *return_id, unsigned char *return_source_nodeID, unsigned short *return_frag_no )
/* The messages are given in reverse order!!!!! */
{
	if (mesg2p_table.nsm){
		mesg2p_table.nsm--;
		*return_id = mesg2p_table.Smesg2p[mesg2p_table.nsm].msg_id;
		*return_frag_no = mesg2p_table.Smesg2p[mesg2p_table.nsm].msg_frag_no;
		*return_source_nodeID = mesg2p_table.Smesg2p[mesg2p_table.nsm].msg_source_nodeID;
		return 1;
	}
	return 0;
}

static unsigned char S_ECScheduled_GetAMessage( unsigned short *return_id, unsigned char *return_source_nodeID, unsigned short *return_frag_no )
/* The messages are given in reverse order!!!!! */
{
	if (mesg2p_table.nam){
		mesg2p_table.nam--;
		*return_id = mesg2p_table.Amesg2p[mesg2p_table.nam].msg_id;
		*return_frag_no = mesg2p_table.Amesg2p[mesg2p_table.nam].msg_frag_no;
		*return_source_nodeID = mesg2p_table.Amesg2p[mesg2p_table.nam].msg_source_nodeID;
		return 1;
	}
	return 0;
}

unsigned char S_ECScheduled_GetMessage( unsigned short *return_id, unsigned char *return_source_nodeID, unsigned short *return_frag_no )
{
	if (S_ECScheduled_GetSMessage( return_id, return_source_nodeID, return_frag_no )){
		return 1;
	}else{
		return (S_ECScheduled_GetAMessage( return_id, return_source_nodeID, return_frag_no ));
	}
}

unsigned char S_ECScheduled_GetSigRequest( unsigned short *return_nodeid, unsigned char *return_seq_no )
{
	if (mesg2p_table.nsa){
		mesg2p_table.nsa--;
		if (return_nodeid) *return_nodeid = mesg2p_table.sig2p[mesg2p_table.nsa].msg_id;
		if (return_seq_no) *return_seq_no = (unsigned char) (mesg2p_table.sig2p[mesg2p_table.nsa].msg_frag_no);
		return 1;
	}
	return 0;
}

unsigned char S_ECScheduled_PutSMessage( unsigned short id, unsigned char source_nodeID, unsigned short frag_no )
/* The messages must be put in reverse order so that they are get in the proper order */
{
	if (mesg2p_table.nsm<EC_MAX_SM_VARS){
		mesg2p_table.Smesg2p[mesg2p_table.nsm].msg_id = id;
		mesg2p_table.Smesg2p[mesg2p_table.nsm].msg_source_nodeID = source_nodeID;
		mesg2p_table.Smesg2p[mesg2p_table.nsm].msg_frag_no = frag_no;
		mesg2p_table.nsm++;
		return 1;
	}
	return 0;
}

unsigned char S_ECScheduled_PutAMessage( unsigned short id, unsigned char source_nodeID, unsigned short frag_no )
/* The messages must be put in reverse order so that they are get in the proper order */
{
	if (mesg2p_table.nam<EC_MAX_AM_VARS){
		mesg2p_table.Amesg2p[mesg2p_table.nam].msg_id = id;
		mesg2p_table.Amesg2p[mesg2p_table.nam].msg_source_nodeID = source_nodeID;
		mesg2p_table.Amesg2p[mesg2p_table.nam].msg_frag_no = frag_no;
		mesg2p_table.nam++;
		return 1;
	}
	return 0;
}

unsigned char S_ECScheduled_PutSigRequest( unsigned short nodeID, unsigned char seq_no )
{
	if (mesg2p_table.nsa<EC_MAX_SIG_MISSING_REQUESTS ){
		mesg2p_table.sig2p[mesg2p_table.nsa].msg_id = nodeID;
		mesg2p_table.sig2p[mesg2p_table.nsa].msg_frag_no = seq_no;
		mesg2p_table.nsa++;
		return 1;
	}
	return 0;
}

void S_ECScheduled_CleanTables(void)
{
	mesg2p_table.nsm = 0;
	mesg2p_table.nam = 0;
	mesg2p_table.nsa = 0;

}

void S_ECScheduled_StoreTimestamp( struct timespec *timestamp ) {
	mesg2p_table.tm_timestamp = *timestamp;
}

struct timespec S_ECScheduled_ReadTimestamp( void ) {
	return mesg2p_table.tm_timestamp;
}
