/*****************************************************************************
 * S_NRDB.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#include "Ports.h"

#include "S_NRDB.h"
#include "S_db.h"

/* Mutex to serialize access to the DB */
#define NRDB_LockerDeclaration	rtl_pthread_mutex_t db_access_mutex;
#define NRDB_Enter              rtl_pthread_mutex_lock(&db_access_mutex);
#define NRDB_Leave              rtl_pthread_mutex_unlock(&db_access_mutex);
#define NRDB_Init_locker        rtl_pthread_mutex_init(&db_access_mutex, 0);
#define NRDB_Destroy_locker     rtl_pthread_mutex_destroy(&db_access_mutex);

#define NRDB_CondVarDeclaration rtl_pthread_cond_t db_change_cond;
#define NRDB_CondVarWait        rtl_pthread_cond_wait( &(db_change_cond), &(db_access_mutex) );
#define NRDB_CondVarBroadcast   rtl_pthread_cond_broadcast( &(db_change_cond) );
#define NRDB_CondVar_locker     rtl_pthread_cond_init( &(db_change_cond), NULL);
#define NRDB_CondVar_Destroy    rtl_pthread_cond_destroy( &(db_change_cond) );


static NRDB_LockerDeclaration; /* Mutex to serialize access to SRT between */
static NRDB_CondVarDeclaration;

signed char S_NRDB_Init(void)
{
	/* Check if S_db was boot */
	
	/* Create table access locker */
	NRDB_Init_locker;
	NRDB_CondVar_locker;
#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -S_NRDB_Init Ok");
#endif	
	return 0;

}
signed char S_NRDB_Close(void)
{
//	PRINT_MSG("Trying to destroy cond var");
	NRDB_CondVar_Destroy;
	NRDB_Destroy_locker;
#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -S_NRDB_Close Ok");
#endif	
	return 0;
}

void S_NRDB_Pre_Close(void)
{
	/* This function should be called prior to any 
	 * deassigning function of any module 
	 * This is because the CondVar when waiting for the sig
	 * should keep the mutex unlocked. And it does, but when 
	 * (in RTL) Ctrl-C is pressed it seems that the locker is
	 * locked which prevents any db access when deassigning 
	 * resources */
	/*So, let's unlock
	 * If the locker was not locked no problem anyway */
	NRDB_Leave;
}

signed char S_NRDB_add_msg( unsigned char m_type,
		unsigned short m_id,
		unsigned char m_tag,
		unsigned int m_size,
		unsigned int m_max_size,
		unsigned int m_period,
		unsigned short m_MTU,
		unsigned char *m_prod_mac,
		unsigned char m_prod_max_no,
		unsigned char overwrite //if 1 - force delete and add.
		)
{
	signed char ret;

	NRDB_Enter;
	if (overwrite)
		NDB_del_msg( m_type, m_id );

	ret = NDB_add_msg( m_type, m_id, m_tag, m_size, m_max_size, m_period, m_MTU, m_prod_mac, m_prod_max_no);
	NRDB_CondVarBroadcast;
	NRDB_Leave;

	return ret;
}

signed char S_NRDB_del_msg( unsigned char m_type, unsigned int m_id)
{
	signed char ret;

	NRDB_Enter;
	ret = NDB_del_msg( m_type, m_id);
	NRDB_CondVarBroadcast;
	NRDB_Leave;

	return ret;
}

signed char S_NRDB_clean_msg_tag( unsigned char m_type, unsigned int m_id, unsigned char m_tag)
{
	signed char ret;
	
	NRDB_Enter;
	ret = NDB_clean_msg_tag( m_type, m_id, m_tag);
	NRDB_Leave;
	
	return ret;
}

signed int S_NRDB_wait4_mesg(
		MESSAGE_TYPE m_type,
		unsigned int m_id)
{
	NRDB_Enter;

	PRINT_MSG("Waiting4_mesg instantiation...");
	while (NDB_get_current( m_type, m_id, NULL, NULL, NULL, NULL ) < 0){
		NRDB_CondVarWait;
	}
	
	NRDB_Leave;

	return 0;
}

signed int S_NRDB_wait4_not_mesg(
							 MESSAGE_TYPE m_type,
							 unsigned int m_id)
{
	NRDB_Enter;
	
	PRINT_MSG("Waiting4_mesg de-instantiation...");
	while (NDB_get_current( m_type, m_id, NULL, NULL, NULL, NULL ) >= 0){
		NRDB_CondVarWait;
	}
	
	NRDB_Leave;
	
	return 0;
}


signed char S_NRDB_bind_mesg(
		MESSAGE_TYPE m_type,
		unsigned int m_id,
		FTT_SIGNAL **ret_callback_signal,
		void *memory_point,
		unsigned int mem_size,
		unsigned char slots_no )
{
	signed char ret;
	
	NRDB_Enter;
	ret = NDB_bind_mesg( m_type, m_id, ret_callback_signal, memory_point, mem_size, slots_no);
	NRDB_Leave;
	
	return ret;
}

signed char S_NRDB_unbind_mesg( MESSAGE_TYPE m_type, unsigned short m_id)
{
	signed char ret;
	
	NRDB_Enter;
	ret = NDB_unbind_mesg( m_type, m_id );
	NRDB_Leave;
	
	return ret;
}

signed char S_NRDB_isbind_mesg( MESSAGE_TYPE m_type, unsigned short m_id)
{
	signed char ret;
	
	NRDB_Enter;
	ret = NDB_isbind_mesg( m_type, m_id );
	NRDB_Leave;
	
	return ret;
}

signed char S_NRDB_get_current(
		MESSAGE_TYPE m_type,
		unsigned int m_id,
		unsigned char *tag,
		unsigned int *cur_size,
		unsigned short *period )
{
	signed char ret;
	
	NRDB_Enter;
	ret = NDB_get_current( m_type, m_id, tag, cur_size, NULL, period);
	NRDB_Leave;
	
	return ret;
}

signed char S_NRDB_get_memory_pool_size(
		MESSAGE_TYPE m_type,
		unsigned int m_id,
		unsigned char slots_no,
		unsigned short *ret_mem_size )
{
	signed char ret;
	
	NRDB_Enter;
	ret = NDB_get_memory_pool_size( m_type, m_id, slots_no, ret_mem_size);
	NRDB_Leave;
	
	return ret;
}

signed char S_NRDB_printmesg( void )
{
	signed char ret;

	NRDB_Enter;
	ret = NDB_table_print();
	NRDB_Leave;

	return ret;
}
void S_NRDB_enter(void)
{
	NRDB_Enter;
	return;
}
void S_NRDB_leave(void)
{
	NRDB_Leave;
	return;
}
