/*****************************************************************************
 * S_appPnP.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/


#include "Ports.h"
#include "S_Dispatch.h"

/* This is a very naive approach to initialize the application layer
 * and make it synchronized with the one in the Master
 */


static unsigned char g_request_status = 0;
static unsigned char g_app_ID;

static unsigned short g_tx_channel_id;
static unsigned short g_rx_channel_id;
static unsigned char g_ack;

static FTT_SEMAPHORE g_app_semaph;


signed char S_appPnP_Init(void)
{
//	g_request_status = 0; //not asking
	if ( FTT_SEMAPHORE_init(&g_app_semaph, 0) < 0 ) {	//Initialization locked
		ERROR_MSG("Couldn't initialize semaphore");
		return -1;
	}

#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -S_appPnP_Init Ok");
#endif

	return 0;
}
void S_appPnP_Close(void)
{
	FTT_SEMAPHORE_destroy(&g_app_semaph);

#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -S_appPnP_Close Ok");
#endif
	return ;
}

signed char S_appPnP_Get_asynchronous_channel(unsigned char *app_ID, unsigned short *id_rx, unsigned short *id_tx, unsigned int *ec_len_us)
{
	g_app_ID = *app_ID;
	g_request_status += 1;

	/* now block waiting for the reception of the asynchronous RX and TX pair */
	if ( FTT_SEMAPHORE_wait(&g_app_semaph) < 0 )
		return -2;

	*id_rx = g_rx_channel_id;
	*id_tx = g_tx_channel_id;
	*app_ID = g_app_ID;
	*ec_len_us = S_Dispatch_Read_ec_time_us();

	if (g_ack) return 0;
	else return -1;
}

signed char S_appPnP_Master_Ack_OK( unsigned short rx_channel, unsigned short tx_channel, unsigned char app_id )
{
	g_tx_channel_id = tx_channel;
	g_rx_channel_id = rx_channel;
	g_app_ID = app_id;
	
	g_ack = 1;
	/**/
	FTT_SEMAPHORE_post(&g_app_semaph);
	return 0;
}

signed char S_appPnP_Master_Ack_KO(void)
{
	g_ack = 0;

	/**/
	FTT_SEMAPHORE_post(&g_app_semaph);
	return 0;
}


unsigned char S_appPnP_is_there_a_request( unsigned char *app_id )
{
	if (g_request_status){
		*app_id = g_app_ID;
		g_request_status --;
		return 1;
	}
	return 0;
}
