/*****************************************************************************
 * S_appPnP.h:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

/*! \page S_app_pnp Slave Application Plug 'n Play handler
 This is responsible for providing the interface layer the means to register itself in the Master. It uses the signaling mechanism to convey the announcement and the Broadcast channel to receive the confirmation, along with the two communication channels that are passed to the upper layer.
 */

#ifndef  S_APPPNP_H
#define  S_APPPNP_H

/** Initializes the Slave Application Plug 'n Play. \return 0 - OK */
extern signed char S_appPnP_Init(void);

/** Closes the Slave Application Plug 'n Play. */
extern void S_appPnP_Close(void);

/** Starts the registration request, asking the Master for two asynchronous channels. This is a blocking call performed in the scope of the application thread.
 \return 0 - OK 
 */
extern signed char S_appPnP_Get_asynchronous_channel(unsigned char *app_ID, unsigned short *id_rx, unsigned short *id_tx, unsigned int *ec_len_us);

/** Uppon a reception of the message that replies, this call updates the status and unblocks #S_appPnP_Get_asynchronous_channel . In this case it replies with a positive ACK. \return 0 - OK */
extern signed char S_appPnP_Master_Ack_OK( unsigned short rx_channel, unsigned short tx_channel, unsigned char app_id );

/** Upon a reception of the message that replies, this call updates the status and unblocks #S_appPnP_Get_asynchronous_channel . In this case it replies with a negative ACK. \return 0 - OK */
extern signed char S_appPnP_Master_Ack_KO(void);

/** This call is performed in the scope of the Dispatcher and asks this entity if there is a pending request that should be conveyed to the Master.
 \return 0 - no request is pending */
extern unsigned char S_appPnP_is_there_a_request( 
	unsigned char *app_id /**< [out] The application id of the pending request. */
	);


#endif   /* ----- #ifndef S_APPPNP_FILE_HEADER_INC  ----- */

