/*****************************************************************************
 * S_eth_filter.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#include "Ports.h"
#include "ftt-global.h"

#include "ethernet.layer.h"

#include "S_ECscheduled.h"
#include "S_Dispatch.h"
#include "S_nodeID.h"
#include "S_db.h"

//#define SLAVE_TM_DECODE /* cfg define */
//#define SLAVE_WATCH_DOG /* cfg define */

#ifndef FTT_SLAVE_COMPILE
#undef SLAVE_WATCH_DOG
#endif


#if defined(L26) & defined(SLAVE_WATCH_DOG)
#include <unistd.h>
#include <signal.h>
#endif

extern void S_NRDB_enter(void);
extern void S_NRDB_leave(void);

#ifdef SLAVE_WATCH_DOG
#define SLAVE_WATCH_DOG_START_VALUE 20 /*20=19ECs*/
static unsigned long watch_ec_len=50; //ms
static int watch_dog_counter=0;
static rtl_pthread_t watch_dog_th;

static void *watch_dog(void *t)
{
	struct rtl_timespec next_activation; //used to time activations	

	rtl_clock_gettime( RTL_CLOCK_REALTIME, &next_activation );
	while (1) {
		rtl_timespec_add_ns( &next_activation, watch_ec_len*1000000 );
		rtl_clock_nanosleep( RTL_CLOCK_REALTIME, RTL_TIMER_ABSTIME, &next_activation, NULL);
		if (watch_dog_counter)
			watch_dog_counter--;
		if (watch_dog_counter == 1){
#ifdef L26
			PRINT_MSG("Watch dog detected on my pid:%d", getpid() );
			kill( getpid(),SIGINT);
#else
			PRINT_MSG("Watch dog detected. Doing nothing for now." );
#endif
		}
		
		
	}
	return NULL;
}

#endif

void S_eth_filter(unsigned char *rmsg, unsigned short size, struct rtl_timespec rx_timestamp)
{

	FTT_PKT_GENERIC_HEADERS *ftt_rmsg_pointer= (FTT_PKT_GENERIC_HEADERS *)rmsg;
	int i; /* General counters */

	//PRINT_MSG("S_eth_filter in");
	//PRINT_MSG("Slave_ethernet_pkt_rx called at %ld", rx_timestamp.tv_sec*1000000 + rx_timestamp.tv_nsec/1000 );

	if ( ntohs( ftt_rmsg_pointer->eth_header.eth_type ) != ETH_FTT_TYPE ) {
		//ERROR_MSG("Rx_event Warnning: Frame is not ETH_FTT_TYPE ");
		return;
	}
		
	/* Get ftt frame type */
	switch( ntohs(ftt_rmsg_pointer->pkt_header.type) )
	{
		case FTT_MST_MSG: /* TM received */
			{
				FTT_PKT_TM_HEADERS *ftt_tm_pkt = (FTT_PKT_TM_HEADERS *)ftt_rmsg_pointer;
				TM_SYNC_MSG_INDEX *tm_sm_element;
				TM_ASYNC_MSG_INDEX *tm_am_element;
				TM_SIG_ACK_INDEX *tm_sig_ack_element;

				unsigned short nsm,nam,nsa;
				static unsigned char last_seq_no=255;
				static unsigned char first_run=1;
//PRINTF("+");
				if ( S_Dispatcher_is_running() ){
					ERROR_MSG("Analysing a new TM while S_Dispatcher still running ");
					/* This means that the dispatcher is taking too much time to encode and notify Tx */
                    while ( S_Dispatcher_is_running() ) PRINTF("=");

				}

				S_Set_Master_mac ( ftt_tm_pkt->eth_header.eth_src );

				/* Let us store the TM contents in the internal EC table */
				S_ECScheduled_CleanTables();

//TODO get the TM size and post the time of TM transmission to avoid jitter
				S_ECScheduled_StoreTimestamp( &rx_timestamp );
			
				/* Get number of sync. and asynch. messages in the EC */
				nsm = ntohs(ftt_tm_pkt->tm_header.nsm);
				nam = ntohs(ftt_tm_pkt->tm_header.nam);
				nsa = ntohs(ftt_tm_pkt->tm_header.nsa);
				
				S_Dispatch_Set_ec_time( ftt_tm_pkt->tm_header.ec_time_ms );
#ifdef SLAVE_WATCH_DOG
				watch_ec_len = ftt_tm_pkt->tm_header.ec_time_ms;
#endif

				if ( ++last_seq_no != ftt_tm_pkt->tm_header.seq_no ){
					if (!first_run)
						ERROR_MSG("TM: rx bad seq_no. Expected %d but received %d", last_seq_no, ftt_tm_pkt->tm_header.seq_no );
					last_seq_no = ftt_tm_pkt->tm_header.seq_no;
				}
				first_run = 0;

#ifdef SLAVE_TM_DECODE
				PRINT_MSG("TM:");
#endif

				/* Store messages to produce in ECSchedule table */
				/* 1: First synchronous messages (and asynchronous via aperiodic server) */

				tm_sm_element = (TM_SYNC_MSG_INDEX *)(ftt_tm_pkt->data);
				tm_am_element = (TM_ASYNC_MSG_INDEX *)( ftt_tm_pkt->data + nsm*sizeof(TM_SYNC_MSG_INDEX) );
				tm_sig_ack_element = (TM_SIG_ACK_INDEX *)( ftt_tm_pkt->data + nsm*sizeof(TM_SYNC_MSG_INDEX) + nam*sizeof(TM_ASYNC_MSG_INDEX) );
				
				for(i=(nsm-1); i>=0; i--){
					/* add to table in reverse sort */
					S_ECScheduled_PutSMessage( ntohs(tm_sm_element[i].SMesgId), tm_sm_element[i].source_nodeID, ntohs(tm_sm_element[i].SMesg_frag_no) );
#ifdef SLAVE_TM_DECODE
					PRINTF(" S%x-%d", ntohs(tm_sm_element[i].SMesgId),  ntohs(tm_sm_element[i].SMesg_frag_no) );
#endif
				}

				for(i=(nam-1); i>=0; i--){
					S_ECScheduled_PutAMessage( ntohs (tm_am_element[i].AMesgId), tm_am_element[i].source_nodeID, ntohs(tm_am_element[i].AMesg_frag_no) );
#ifdef SLAVE_TM_DECODE
					PRINTF(" A%x", ntohs(tm_am_element[i].AMesgId));
#endif
				}
				
				for(i=(nsa-1); i>=0; i--){
					if ( ntohs (tm_sig_ack_element[i].nodeID) == S_nodeID_read() ){
						PRINT_MSG ("Have to re-transmit seq_no: %d ", tm_sig_ack_element[i].sig_seqno );
						S_ECScheduled_PutSigRequest( ntohs (tm_sig_ack_element[i].nodeID), tm_sig_ack_element[i].sig_seqno );
					}
				}

				/* Activate the task that will send the messages (at appropriate time instants) to the ETh bus */
//				NDB_printmesg();
				S_Dispatch_Wake();
#ifdef SLAVE_WATCH_DOG
				watch_dog_counter = SLAVE_WATCH_DOG_START_VALUE;
#endif
			}
			break;
			
		case FTT_SDATA_MSG:	/* Synchronous Data Message received */
			{
				FTT_PKT_SDM_HEADERS *ftt_sdm_pointer = (FTT_PKT_SDM_HEADERS *)ftt_rmsg_pointer;

//				PRINT_MSG("Received SDATA_MESG %x  %d", ntohs( ftt_sdm_pointer->sdm_header.id ), ntohs( ftt_sdm_pointer->sdm_header.frag_seqno ) );
//PRINTF("s");
			
				if( NDB_test_consume_and_bind( ntohs( ftt_sdm_pointer->sdm_header.id ) ) < 0 ) {
					//PRINT_MSG("This SM is not a consumer or not bound of this...");
					break;
				}
				//PRINT_MSG("Is a consumer of this SM");
			
				S_NRDB_enter();
				NDB_set_data2rx(
						ntohs( ftt_sdm_pointer->sdm_header.id ),
						ftt_sdm_pointer->sdm_header.source_nodeID,
						ntohs( ftt_sdm_pointer->sdm_header.frag_seqno ),
						ftt_sdm_pointer->data,
						(ftt_sdm_pointer->sdm_header.flag_s & TXNEWDAT_MASK),
#ifdef QNX_TAGGING
						ntohi( ftt_sdm_pointer->sdm_header.tagging ),
#endif
						rx_timestamp);
				S_NRDB_leave();
			}
			break;

		case FTT_ADATA_MSG: /* Asynchronous Data Message received */
			{
				FTT_PKT_ADM_HEADERS *ftt_adm_pointer = (FTT_PKT_ADM_HEADERS *)ftt_rmsg_pointer;
			
//				PRINT_MSG("Received ADATA_MESG %x", ntohs( ftt_adm_pointer->adm_header.id ));

				if( NDB_test_consume_and_bind( ntohs( ftt_adm_pointer->adm_header.id ) ) < 0 ) {
				//	PRINTF(" will NOT consume or not ready to consume");
					break;
				}
				//PRINTF(" will consume");
			
				S_NRDB_enter();
				NDB_set_data2rx(
						ntohs( ftt_adm_pointer->adm_header.id ),
						ftt_adm_pointer->adm_header.source_nodeID,
						ntohs( ftt_adm_pointer->adm_header.frag_seqno),
						ftt_adm_pointer->data,
						(ftt_adm_pointer->adm_header.flag_s & TXNEWDAT_MASK),
#ifdef QNX_TAGGING
						ntohi( ftt_adm_pointer->adm_header.tagging ),
#endif
						rx_timestamp);
				S_NRDB_leave();
			}
			break;
		
		case FTT_PLUGnPLAY:
		case FTT_ASTATUS_MSG: /* Asynchronous Data Message received */
			//PRINT_MSG("Received ASTATUS_MESG_ID");
			break;

		case FTT_IDLE:
			{
				FTT_PKT_IDLE_HEADERS *ftt_idle_pointer = (FTT_PKT_IDLE_HEADERS *)ftt_rmsg_pointer;
				PRINT_MSG("IDDLE time AM message: %x", ntohs( ftt_idle_pointer->idle_header.id ) );
			}
			break;

		default:
			ERROR_MSG(" Received an ilegal message id");
			break;

	} /* Switch(frametype) */
	return ;
}

signed char S_eth_filter_Init(void)
{
#ifdef FTT_SLAVE_COMPILE
	PRINT_MSG("Initializing the S_eth_filter as the main rceiver");
	ETH_L_rx_fun_register( S_eth_filter );
	
#ifdef SLAVE_WATCH_DOG
	if ( rtl_pthread_create( &watch_dog_th, NULL, watch_dog, NULL )<0 ){
		ERROR_MSG("Init Fatal Error: pthread_create");
		return -1;
	}
#endif
	
#else //FTT_MASTER_COMPILE

#endif

#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -S_eth_filter_Init Ok");
#endif
	
	return 0;

}

void S_eth_filter_Close(void)
{
#ifdef FTT_SLAVE_COMPILE
#ifdef SLAVE_WATCH_DOG
	rtl_pthread_cancel( watch_dog_th );
	rtl_pthread_join( watch_dog_th, NULL );
#endif
	ETH_L_rx_fun_unregister();
#endif

#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -S_eth_filter_Close Ok");
#endif
}

