/*****************************************************************************
 * S_eth_filter.h:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

/*! \page S_eth_filter Slave Ehernet Filter 
 This is responsible for receiving all packets from the network card, decoding them individually and establishing the interactions with the different entities in the Slave. For example, the Slave may receive a Plug 'n Play reply or a TM or a regular Synchronous/Asynchronous message. */

#ifndef S_ETH_FILER_H
#define S_ETH_FILER_H

/** Initializes the Slave Ethernet Filter. \return 0 - OK */
extern signed char S_eth_filter_Init(void);

/** Closes the Slave Ethernet Filter. */
extern void S_eth_filter_Close(void);

/** Shortcut to call the reception of a message. 
 * On a normal Slave implementation this isn't required, each Slave registers in the Ethernet layer asking for the reception of all the packets in this function. However, when instantiating this code in the Master, another entity will be responsible for receiving those packets. It is that entity that detects if the message is directed to the Slave and directly calls the Slave Ethernet Filter using this function. */
extern void S_eth_filter(unsigned char *rmsg, unsigned short size, struct rtl_timespec rx_timestamp);


#endif
