/*****************************************************************************
 * S_ftt-core.layer.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#define S_FTT_CORE_LAYER_C

#include "Ports.h"

#include "ethernet.layer.h"

#include "ftt-core.layer_types.h"

#include "S_nodeID.h"
#include "S_allocator.h"
#include "S_db.h"
#include "S_ECscheduled.h"
#include "S_igmp_queue.h"
#include "S_Dispatch.h"
#include "S_eth_filter.h"
#include "S_NRDB.h"
#include "S_master_ch.h"

#ifdef FTT_SLAVE_COMPILE
#include "S_appPnP.h"
#include "S_sig_backup.h"
#endif

signed char S_FTT_CORE_L_init(unsigned char *mac)
{
	int ret;
	
#ifdef SLAVE_POLITE
	PRINT_MSG("Initializing the Core layer - Slave");
#endif
	/* Check if Ethernet was initialized */
	/* Net initialization */
	if ( ETH_L_Init_check() < 0 ){
		ERROR_MSG("Init Fatal Error: ETH_L_Init_check failed");
		return -3;
	}


	/* Initialize memory blocks */
	/*  - allocator */
	if ( (ret=Allocator_init()) < 0){
		ERROR_MSG("Allocator_init returned: %d",ret);
		return -1;
	}
	
	/*  - igmp_queue */
	if ( (ret=S_igmp_queue_init()) < 0){
		ERROR_MSG("S_igmp_queue_init returned: %d",ret);
		return -5;
	}

	/*  - db */
	if ( (ret=NDB_Init()) < 0){
		ERROR_MSG("NDB_Init returned: %d",ret);
		return -2;
	}

	S_NRDB_Init();

	/*  - node_id */
	S_nodeID_Init(mac);

#ifdef FTT_SLAVE_COMPILE
	S_appPnP_Init();
	S_sig_backup_Init();
#endif
	
	/*  - ECscheduled */
	S_ECScheduled_Init();
	
	/* Initialize functional blocks */
	if ( (ret=S_Dispatch_Init()) < 0)
	{
		ERROR_MSG("S_Dispatch_Init returned: %d",ret);
		return -4;
	}

	/*  - S_eth_filter */
	if ( (ret=S_eth_filter_Init()) < 0){
		ERROR_MSG("S_eth_filter_Init returned: %d",ret);
		return -4;
	}

	/*  - S_master_channel */
	S_master_ch_Init();

	return 0;
}

signed char S_FTT_CORE_L_close(void) {
#ifdef SLAVE_POLITE
	PRINT_MSG("Closing the Core layer - Slave");
#endif
	S_NRDB_Pre_Close();

//	PRINT_MSG("S_master_ch_Close");
	S_master_ch_Close();
	
//	PRINT_MSG("S_eth_filter_Close");
	S_eth_filter_Close();
	
//	PRINT_MSG("S_Dispatch_Close");
	S_Dispatch_Close();

//	PRINT_MSG("S_ECScheduled_Close");
	S_ECScheduled_Close();

#ifdef FTT_SLAVE_COMPILE
//	PRINT_MSG("S_appPnP_Close");
	S_sig_backup_Close();
	S_appPnP_Close();
#endif	

//	PRINT_MSG("S_nodeID_Close");
	S_nodeID_Close();
	
//	PRINT_MSG("S_NRDB_Close");
	S_NRDB_Close();

//	PRINT_MSG("S_db_Close");
	NDB_Close();
	
//	PRINT_MSG("S_igmp_queue_close");
	S_igmp_queue_close();
	
//	PRINT_MSG("Allocator_close");
	Allocator_close();

//	PRINT_MSG("S_FTT_CORE_L_close OK");
	return 0;
}

#ifdef FTT_SLAVE_COMPILE
signed char S_FTT_CORE_L_Get_asynchronous_buffers( unsigned char *app_ID, unsigned short *id_rx, unsigned short *id_tx, unsigned int *ec_len_us)
{

	return S_appPnP_Get_asynchronous_channel(app_ID, id_rx, id_tx, ec_len_us);
}
#endif


signed int S_FTT_CORE_L_Wait4var(
		MESSAGE_TYPE m_type,
		unsigned int m_id)
{
	//It will return the message mem size
	return S_NRDB_wait4_mesg( m_type, m_id );
}

signed int S_FTT_CORE_L_Wait4notvar(
		MESSAGE_TYPE m_type,
		unsigned int m_id)
{
	//It will return the message mem size
	return S_NRDB_wait4_not_mesg( m_type, m_id );
}

signed char S_FTT_CORE_L_bind_mesg(
		MESSAGE_TYPE m_type,
		unsigned int m_id,
		FTT_SIGNAL **ret_callback_signal,
		void *memory_point,
		unsigned int mem_size,
		unsigned char slots_no )
{
	return S_NRDB_bind_mesg( m_type, m_id, ret_callback_signal, memory_point, mem_size, slots_no);
}

signed char S_FTT_CORE_L_unbind_mesg(
		MESSAGE_TYPE m_type,
		unsigned short m_id)
{
	return S_NRDB_unbind_mesg( m_type, m_id );
}

signed char S_FTT_CORE_L_isbind_mesg(
		MESSAGE_TYPE m_type,
		unsigned short m_id)
{
	return S_NRDB_isbind_mesg( m_type, m_id );
}

signed char S_FTT_CORE_L_get_memory_pool_size(
		MESSAGE_TYPE m_type,
		unsigned int m_id,
		unsigned char slots_no,
		unsigned short *ret_mem_size )
{
	return S_NRDB_get_memory_pool_size( m_type, m_id, slots_no, ret_mem_size);
}

signed char S_FTT_CORE_L_get_current(
									 MESSAGE_TYPE m_type,
									 unsigned int m_id,
									 unsigned char *tag,
									 unsigned int *cur_size,
									 unsigned short *period )
{
	return S_NRDB_get_current( m_type, m_id, tag, cur_size, period);
	
}

signed char S_FTT_CORE_pre_produce(unsigned short MesgID, unsigned char **return_p)
{
	return NDB_pre_produce(MesgID, return_p);
}

signed char S_FTT_CORE_pre_produce_abort(unsigned short MesgID)
{
	return NDB_pre_produce_abort(MesgID);
}

signed char S_FTT_CORE_produce( unsigned short m_id, unsigned char tag, unsigned char *buf )
{
	return NDB_produce( m_id, tag, buf);
}

signed char S_FTT_CORE_consume_status( unsigned short MesgID )
{
	return NDB_consume_status( MesgID );
}

unsigned char *S_FTT_CORE_pre_consume(unsigned short MesgID, unsigned char *ret_tag, unsigned char *ret_status )
{
	return NDB_pre_consume(MesgID, ret_tag, ret_status);
}

signed char S_FTT_CORE_consume(unsigned short MesgID, unsigned char *ret_tag, unsigned char *mesg_value, unsigned char *ret_status )
{
	return NDB_consume(MesgID, ret_tag, mesg_value, ret_status);
}

signed char S_FTT_CORE_pre_send(unsigned short MesgID, unsigned char alloc_block, unsigned char **return_p)
{
	return NDB_pre_send(MesgID, alloc_block, return_p);
}

signed char S_FTT_CORE_pre_send_abort(unsigned short MesgID)
{
	return NDB_pre_send_abort(MesgID);
}

signed char S_FTT_CORE_send( unsigned short m_id, unsigned char tag, unsigned char *buf, unsigned char alloc_block )
{
	return NDB_send( m_id, tag, buf, alloc_block);
}

signed char S_FTT_CORE_receive_status( unsigned short MesgID )
{
	return NDB_receive_status( MesgID );
}

unsigned char *S_FTT_CORE_pre_receive(unsigned short MesgID, unsigned char *ret_tag, unsigned char *ret_status )
{
	return NDB_pre_receive(MesgID, ret_tag, ret_status);
}

signed char S_FTT_CORE_receive(unsigned short MesgID, unsigned char *ret_tag, unsigned char *mesg_value, unsigned char *ret_status )
{
	return NDB_receive(MesgID, ret_tag, mesg_value, ret_status);
}

#ifdef FTT_MASTER_COMPILE
/* This is only for the master since for the slaves it's PnP network driven */
signed char S_FTT_CORE_L_Set_NodeID(unsigned char id)
{
	S_nodeID_write(id);
	return 0;
}
#endif

void S_FTT_CORE_L_print_db(void)
{
	S_NRDB_printmesg();
}

