/*****************************************************************************
 * S_master_ch.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/


#include "Ports.h"
#include "ethernet.layer.h"
#include "ftt-global.h"
#include "S_nodeID.h"
#include "S_db.h"
#include "S_NRDB.h"
#include "S_igmp_queue.h"
#include "S_appPnP.h"


//#define DECODE

void S_master_decode(void *temp_p);

#ifdef FTT_SLAVE_COMPILE
static rtl_pthread_t master_asynch_handler_th;
static FTT_SIGNAL *master_asynch_sig;

static void *master_asynch_handler(void *t)
{
	void *temp_p;
	//unsigned short commandID;

	//pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS,NULL);
	while (1){
		//PRINT_MSG("S_master_ch.c: waitting for master_asynch_sig");
		FTT_wait4signal( master_asynch_sig );
		//PRINT_MSG("S_master_ch.c: Got a message from the Master ftt-core");	
		if ( (temp_p = (void *)NDB_pre_receive(MASTER_BROADCAST_A_CH_ID, NULL, NULL/*status*/)) == NULL){
			ERROR_MSG("NDB_pre_receive on var:MASTER_BROADCAST_A_CH_ID returned NULL");
			continue;
		}
		//commandID = ntohs( ((A_MST_CHANNEL_HEADER *)temp_p)->COMMAND_id);

		S_master_decode( temp_p );
		NDB_receive(MASTER_BROADCAST_A_CH_ID, NULL, NULL, NULL/*status*/);
	}
	return NULL;
}
#endif

signed char S_master_ch_Init(void)
{
#ifdef FTT_SLAVE_COMPILE
	rtl_pthread_attr_t attr;
	struct rtl_sched_param sched_param;

	int ret;
	unsigned char broadcast[6];

	/* Check if S_NRDB initialized */

	/* Create Master assynchronous channel (Conmsume) */
	ETH_L_Str2Addr((unsigned char *)"FF:FF:FF:FF:FF:FF", broadcast);
	if ( (ret=S_NRDB_add_msg( 1 /*Async consumer*/,
			MASTER_BROADCAST_A_CH_ID,
			0 /*tag*/,
			MASTER_BROADCAST_A_CH_SIZE,
			MASTER_BROADCAST_A_CH_SIZE,
			0,/*m_period*/
			MASTER_BROADCAST_A_CH_MTU,
			broadcast,
			1 /*m_prod_max_no*/,
			0 ))<0 ){
		ERROR_MSG("COULD NOT ADD MASTER ASYNC CHANNEL");
	}

	if ( (ret=S_NRDB_bind_mesg( 1 /*Async*/,
			MASTER_BROADCAST_A_CH_ID,
			&master_asynch_sig,
			NULL, /* means the FTT gives the mem */
			0, /*mem_size - 0 'cause mem is controlled by the FTT */
			20 /* FIFO size */ ))<0 ){
		ERROR_MSG("S_NRDB_bind_mesg returned: %d", ret);
	}

	rtl_pthread_attr_init( &attr );
//	sched_param.sched_priority = rtl_sched_get_priority_min(RTL_SCHED_FIFO);
	sched_param.sched_priority = rtl_sched_get_priority_max(RTL_SCHED_FIFO)-2;
	rtl_pthread_attr_setschedparam( &attr, &sched_param );
#ifdef RTL_PRO
	rtl_pthread_attr_setfp_np(&attr, 1);
#endif
	rtl_pthread_create( &master_asynch_handler_th, &attr, master_asynch_handler, (void*)0);

#ifdef PRINT_THREAD_ID
#ifdef RTL_PRO
	ERROR_MSG("New thread id: %d", master_asynch_handler_th);
#else
	ERROR_MSG("New thread id: %lu", master_asynch_handler_th);
#endif
#endif

#endif

#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -S_master_ch_Init Ok");
#endif
	return 0;
}

signed char S_master_ch_Close(void)
{
#ifdef FTT_SLAVE_COMPILE
	rtl_pthread_cancel( master_asynch_handler_th );
	rtl_pthread_join( master_asynch_handler_th, NULL );

//	PRINT_MSG("S_master_ch.c: Going to del channel");
	S_NRDB_del_msg( 1 /*Async consumer*/, MASTER_BROADCAST_A_CH_ID);
#endif
	
#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -S_master_ch_Close Ok");
#endif
	
	return 0;
}

void S_master_decode(void *temp_p)
{
	unsigned short commandID;
	int ret;
	unsigned char i;

	commandID = ntohs( ((A_MST_CHANNEL_HEADER *)temp_p)->COMMAND_id);

	switch(commandID)
	{
		case FTT_CMD_SET_NODEID:
			/* Check if this is my MAC */
			if (ETH_L_CompareMAC(((A_MST_SET_NODEID *)temp_p)->mac, S_Get_My_mac())){
				S_nodeID_write((unsigned char) ntohs( ((A_MST_SET_NODEID *)temp_p)->nodeID ));
#ifdef PRINT_SLAVE_NRDBUPDATE_COMMANDS
				PRINT_MSG("    c: slave registered as %d ",(unsigned char) ntohs( ((A_MST_SET_NODEID *)temp_p)->nodeID ));
#endif
			}
			//PRINT_MSG("\t S_nodeID_write(%d)",(unsigned char) ntohs( ((A_MST_SET_NODEID *)temp_p)->nodeID ));
			break;

		case FTT_CMD_SET_APP_PNP_IDS:
			/* Check if already got nodeID */
			if (S_nodeID_read()<0)
				break;

			/* Check if the message is for this node (remember that the channel is broadcast) */
			if ((((A_PNP_IDS_NOTIFY *)temp_p)->nodeID)!=(unsigned char)S_nodeID_read())
				break;

#ifdef PRINT_SLAVE_NRDBUPDATE_COMMANDS
			PRINT_MSG("    c: slave %d notifying interface", (unsigned char)S_nodeID_read());
#endif

			if (((A_PNP_IDS_NOTIFY *)temp_p)->ack)
				S_appPnP_Master_Ack_OK(
						ntohs(((A_PNP_IDS_NOTIFY *)temp_p)->rx_id), 
						ntohs(((A_PNP_IDS_NOTIFY *)temp_p)->tx_id),
						((A_PNP_IDS_NOTIFY *)temp_p)->app_id );
			else
				S_appPnP_Master_Ack_KO();

			break;

		case FTT_CMD_ADD_MSG:
				
			if (S_nodeID_read()<0){
				PRINT_MSG("Received Add in S_master_ch but nodeID not registered");
				break;
			}
				
			/* Check if my nodeID is producer of this */
			
			
			for (i=0; i<((A_MST_ADD *)temp_p)->mesg_producers_no; i++ ){
				if ( ((A_MST_ADD *)temp_p)->mesg_producerID[i] == (unsigned char)S_nodeID_read() ){
#ifdef PRINT_SLAVE_NRDBUPDATE_COMMANDS
					PRINT_MSG("    c: slave %d is adding ftt-id: %s0x%x (tag:%d) as producer with size:%d/%d T:%d ",
							  S_nodeID_read(),
							  (((((A_MST_ADD *)temp_p)->mesg_type) & 0x0F )==SYNC_MESSAGE)?"S":"A",
							  ntohs( ((A_MST_ADD *)temp_p)->mesg_id),
							  ((A_MST_ADD *)temp_p)->mesg_id_tag,
							  ntohi( ((A_MST_ADD *)temp_p)->mesg_size),
							  ntohi( ((A_MST_ADD *)temp_p)->mesg_max_size),
							  ntohi( ((A_MST_ADD *)temp_p)->mesg_period) );
#endif
					
					ret=S_NRDB_add_msg( 
									   ((((A_MST_ADD *)temp_p)->mesg_type) & 0x0F) | 0x10 /*m_type or'ed with the direction */,
									   ntohs( ((A_MST_ADD *)temp_p)->mesg_id)/*m_id*/,
									   ((A_MST_ADD *)temp_p)->mesg_id_tag,
									   ntohi( ((A_MST_ADD *)temp_p)->mesg_size)/*m_size*/,
									   ntohi( ((A_MST_ADD *)temp_p)->mesg_max_size)/*m_max_size*/,
									   ntohi( ((A_MST_ADD *)temp_p)->mesg_period)/*m_period*/,
									   ntohs( ((A_MST_ADD *)temp_p)->mesg_MTU)/*m_MTU*/,
									   ((A_MST_ADD *)temp_p)->mesg_production_mac /* *m_prod_mac*/,
									   ((A_MST_ADD *)temp_p)->mesg_producers_max_no,
									   ((A_MST_ADD *)temp_p)->adding_type );
					
					if ( ret<0 )
						ERROR_MSG( "S_NRDB_add_msg for producer returned: %d", ret);
					
					break;
				}
			}
				

			/* Check if my nodeID is consumer of this */
			for (i=0; i<((A_MST_ADD *)temp_p)->mesg_consumers_no; i++ ){
				if ( ((A_MST_ADD *)temp_p)->mesg_consumerID[i] == (unsigned char)S_nodeID_read() ){
#ifdef PRINT_SLAVE_NRDBUPDATE_COMMANDS
				PRINT_MSG("    c: slave %d is adding ftt-id: %s0x%x (tag:%d) as consumer with size:%d/%d T:%d",
						  S_nodeID_read(),
						  (((((A_MST_ADD *)temp_p)->mesg_type) & 0x0F)==SYNC_MESSAGE)?"S":"A",
						  ntohs( ((A_MST_ADD *)temp_p)->mesg_id),
						  ((A_MST_ADD *)temp_p)->mesg_id_tag,
						  ntohi( ((A_MST_ADD *)temp_p)->mesg_size),
						  ntohi( ((A_MST_ADD *)temp_p)->mesg_max_size),
						  ntohi( ((A_MST_ADD *)temp_p)->mesg_period) );
#endif
					ret=S_NRDB_add_msg(
						((((A_MST_ADD *)temp_p)->mesg_type) & 0x0F) | 0x00 /*m_type or'ed with the direction */,
						ntohs( ((A_MST_ADD *)temp_p)->mesg_id)/*m_id*/,
						((A_MST_ADD *)temp_p)->mesg_id_tag,
						ntohi( ((A_MST_ADD *)temp_p)->mesg_size)/*m_size*/,
						ntohi( ((A_MST_ADD *)temp_p)->mesg_max_size)/*m_max_size*/,
						ntohi( ((A_MST_ADD *)temp_p)->mesg_period)/*m_period*/,
						ntohs( ((A_MST_ADD *)temp_p)->mesg_MTU)/*m_MTU*/,
						((A_MST_ADD *)temp_p)->mesg_production_mac,
						((A_MST_ADD *)temp_p)->mesg_producers_max_no,
						((A_MST_ADD *)temp_p)->adding_type );
					
					if ( ret<0 )
						ERROR_MSG( "S_NRDB_add_msg for consumer returned: %d", ret);

					break;
				}
			}
			
#if 0   
			//I've now decided not to delete the var in the consumers. This will be done explecitely, later.
			/////////////////
			if ( i == ((A_MST_ADD *)temp_p)->mesg_consumers_no ) {
				/* Previous for cycle ended without a break */
				if ( S_NRDB_soft_del_msg( ((((A_MST_ADD *)temp_p)->mesg_type) & 0x0F) /*m_type*/, ntohs( ((A_MST_ADD *)temp_p)->mesg_id)/*m_id*/ ) >= 0 ){
#ifdef PRINT_SLAVE_NRDBUPDATE_COMMANDS
					PRINT_MSG("    c: slave %d soft-deleted ftt-id: %s0x%x",
							  S_nodeID_read(),
							  (((((A_MST_ADD *)temp_p)->mesg_type) & 0x0F)==SYNC_MESSAGE)?"S":"A",
							  ntohs( ((A_MST_ADD *)temp_p)->mesg_id) );
#endif
					;
				}
			}
#endif
		
			break;

		case FTT_CMD_DEL_MSG:
				
			if (S_nodeID_read()<0){
				PRINT_MSG("Received Del in S_master_ch but nodeID not registered");
				break;
			}
			
			/* try delete - if this nodeID is not Producer/Consumer no crise */
			if ( S_NRDB_del_msg( ((((A_MST_DEL *)temp_p)->mesg_type) & 0x0F) /*m_type*/, ntohs( ((A_MST_DEL *)temp_p)->mesg_id)/*m_id*/ ) < 0 )
				break;
#ifdef PRINT_SLAVE_NRDBUPDATE_COMMANDS
			PRINT_MSG("    c: slave %d deleting ftt-id: %s0x%x",
					  S_nodeID_read(),
					  ( (((((A_MST_DEL *)temp_p)->mesg_type) & 0x0F)) == SYNC_MESSAGE )?"S":"A",
					  ntohs( ((A_MST_DEL *)temp_p)->mesg_id) );
#endif
			break;

		case FTT_CMD_DEL_OLD_TAG:
			
			if (S_nodeID_read()<0){
				PRINT_MSG("Received Del in S_master_ch but nodeID not registered");
				break;
			}
			
			/* try delete - if this nodeID is not Producer/Consumer no crise */
			if ( S_NRDB_clean_msg_tag( 
									  ((((A_MST_DEL_OLD_TAG *)temp_p)->mesg_type) & 0x0F) /*m_type*/, 
									  ntohs( ((A_MST_DEL_OLD_TAG *)temp_p)->mesg_id)/*m_id*/, 
									  FTT_VAR_ID_GET_TAG (ntohs( ((A_MST_DEL_OLD_TAG *)temp_p)->mesg_id) ) /*tag*/
									  ) < 0 )
				break;
#ifdef PRINT_SLAVE_NRDBUPDATE_COMMANDS
			PRINT_MSG("    c: slave %d cleaning old tag: %s0x%x",
					  S_nodeID_read(),
					  ( (((((A_MST_DEL_OLD_TAG *)temp_p)->mesg_type) & 0x0F)) == SYNC_MESSAGE )?"S":"A",
					  ntohs( ((A_MST_DEL_OLD_TAG *)temp_p)->mesg_id) );
#endif
			break;
			
		case FTT_CMD_DUMMY:
#ifdef DECODE
			PRINT_MSG("Got a master control message: FTT_CMD_DUMMY");
#endif
			break;
		default:
			PRINT_MSG (" S_master_ch.c: commandID unhandled: %d ", commandID);
			break;		
	}
}

