/*****************************************************************************
 * S_sig_backup.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#include "Ports.h"

#define MAX_SIG_STORAGE_SIZE 1400

typedef struct{
	unsigned char  data[MAX_SIG_STORAGE_SIZE];
	unsigned short size;
}SINGLE_SEQ_NO_BLOCK;

static SINGLE_SEQ_NO_BLOCK seq_no_array[256];


signed char S_sig_backup_Init(void)
{
	short i;
	
	for (i=0; i<256; i++)
		seq_no_array[i].size = 0;
	
	return 0;
}

signed char S_sig_backup_Close(void)
{
	
	return 0;
}

signed char S_sig_backup_push( unsigned char seq_no, void *mem_p, unsigned short size )
{
	if (!mem_p)	return -1;
	
	if (size>MAX_SIG_STORAGE_SIZE) {
		ERROR_MSG("You're trying to store a message greater than MAX_SIG_STORAGE_SIZE (%d). Backup has been denyed", MAX_SIG_STORAGE_SIZE);
		seq_no_array[seq_no].size = 0;
		return -2;
	}
	
	if (size) memcpy( seq_no_array[seq_no].data, mem_p, size );
	seq_no_array[seq_no].size = size;
	return 0;
}

signed char S_sig_backup_pull( unsigned char seq_no, void **mem_p, unsigned short *ret_size )
{
	*mem_p = NULL;
	
	if ( (*ret_size = seq_no_array[seq_no].size) )
		*mem_p = seq_no_array[seq_no].data;
	
	return 0;
}


