/*****************************************************************************
 * request_cmds.h:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/


#ifndef  REMOTE_REPORT_H
#define  REMOTE_REPORT_H

#include "Ports.h"
#include "ftt-interface.layer_types.h"

#define BROADCAST_REP_MAX_SIZE 80

#define BROADCAST_REP_ID (M_INT_NODE_ASYNC_ID_BASE)


typedef enum
{
	REP_ADD_NODE=0,
	REP_DEL_NODE
} REP_COMMANDS;


typedef struct{
	unsigned short app_node_id;
} __attribute__ ((packed)) REP_ADD_NODE_DATA;

inline static void REP_ADD_NODE_DATA_hton(REP_ADD_NODE_DATA *p)
{
	p->app_node_id = htons(p->app_node_id);
	return;
}

typedef struct{
	unsigned short app_node_id;
} __attribute__ ((packed)) REP_DEL_NODE_DATA;

inline static void REP_DEL_NODE_DATA_hton(REP_DEL_NODE_DATA *p)
{
	p->app_node_id = htons(p->app_node_id);
	return;
}

typedef struct{
	REP_COMMANDS cmd_id;
	union {
		REP_ADD_NODE_DATA add;
		REP_DEL_NODE_DATA del;
	} cmd;
} __attribute__ ((packed)) REP_COMMAND;

inline static void REP_COMMAND_hton(REP_COMMAND *p)
{
	
	switch (p->cmd_id) {
		case REP_ADD_NODE:
			REP_ADD_NODE_DATA_hton( &(p->cmd.add) );
			break;
		case REP_DEL_NODE:
			REP_DEL_NODE_DATA_hton( &(p->cmd.del) );
			break;
		default:
			PRINT_MSG("Bad command here!");
			break;
	}
	
	return;
}

inline static void REP_COMMAND_ntoh(REP_COMMAND *p)
{
	switch (p->cmd_id) {
		case REP_ADD_NODE:
			REP_ADD_NODE_DATA_hton( &(p->cmd.add) );
			break;
		case REP_DEL_NODE:
			REP_DEL_NODE_DATA_hton( &(p->cmd.del) );
			break;
		default:
			PRINT_MSG("Bad command here!");
			break;
	}

	return;
}


#endif   /* ----- #ifndef REQUEST_CMDS_H  ----- */

