/*****************************************************************************
 * request_cmds.h:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/


#ifndef  REQUEST_CMDS_H
#define  REQUEST_CMDS_H

#include "Ports.h"
#include "ftt-interface.layer_types.h"

#define REQ_CMDS_MAX_SIZE 1400 // -This is the maximum load available
                               //  to transport several commands within
			       //  an AM channel
#define REQ_REPLY_MAX_SIZE 70
#define REQ_CMDS_MAX_NODES (M_INT_MAX_APP_NODES )

typedef enum
{
	REQ_ADD_FULL_MSG=0,
	REQ_DEL_FULL_MSG,
	REQ_ADDMODIFY_LOAD_REQS,
	REQ_ADDMODIFY_QOS_REQS,
	REQ_REMOVE_LOAD_REQS,
	REQ_ADD_QOS_PAIR_REQS,
	REQ_ADD_QOS_PRIO_REQS,
	REQ_ADD_ONE_END_CONECTION,
	REQ_DEL_ONE_END_CONECTION,
	REQ_GET_FTT_ID_DETAILS,
	REQ_CHG_PERIOD,
	REQ_REMOTE_REPORT,
	REQ_SUBMIT
} REQ_COMMANDS;

inline static void REQ_COMMANDS_print(REQ_COMMANDS *req_cmd, unsigned char f)
{
	switch (*req_cmd){
		case REQ_ADD_FULL_MSG: PRINTF("REQ_ADD_FULL_MSG"); break;
		case REQ_DEL_FULL_MSG: PRINTF("REQ_DEL_FULL_MSG"); break;
		case REQ_ADDMODIFY_LOAD_REQS: PRINTF("REQ_ADDMODIFY_LOAD_REQS"); break;
		case REQ_ADDMODIFY_QOS_REQS: PRINTF("REQ_ADDMODIFY_QOS_REQS"); break;
		case REQ_REMOVE_LOAD_REQS: PRINTF("REQ_REMOVE_LOAD_REQS"); break;
		case REQ_ADD_QOS_PAIR_REQS: PRINTF("REQ_ADD_QOS_PAIR_REQS"); break;
		case REQ_ADD_QOS_PRIO_REQS: PRINTF("REQ_ADD_QOS_PRIO_REQS"); break;
		case REQ_ADD_ONE_END_CONECTION: PRINTF("REQ_ADD_ONE_END_CONECTION"); break;
		case REQ_DEL_ONE_END_CONECTION: PRINTF("REQ_DEL_ONE_END_CONECTION"); break;
		case REQ_GET_FTT_ID_DETAILS: PRINTF("REQ_GET_FTT_ID_DETAILS"); break;
		case REQ_REMOTE_REPORT: PRINTF("REQ_REMOTE_REPORT"); break;
		case REQ_SUBMIT: PRINTF("REQ_SUBMIT"); break;
		default: PRINTF("WRONG!!!");
	}
	if (f) PRINTF("\n");
}


typedef struct{
	M_INT_APP_MSG_ID app_msg_id;
	unsigned int mesg_size;
	unsigned int mesg_max_size;
	unsigned char mesg_max_prod_no;
	unsigned short mesg_period;
	unsigned short mesg_MTU;
	unsigned char mesg_producers_no;
	unsigned char mesg_producerID[REQ_CMDS_MAX_NODES];
	unsigned char mesg_consumers_no;	//if 0 broadcast
	unsigned char mesg_consumerID[REQ_CMDS_MAX_NODES];
} __attribute__ ((packed)) REQ_ADD_FULL_MSG_DATA;

inline static void REQ_ADD_FULL_MSG_DATA_hton(REQ_ADD_FULL_MSG_DATA *p)
{
	p->app_msg_id.type = htons(p->app_msg_id.type);
	p->app_msg_id.id = htons(p->app_msg_id.id);
	p->mesg_size = htoni(p->mesg_size);
	p->mesg_max_size = htoni(p->mesg_max_size);
	p->mesg_max_prod_no = p->mesg_max_prod_no;
	p->mesg_period = htons(p->mesg_period);
	p->mesg_MTU = htons(p->mesg_MTU);

	return;
}

typedef struct{
	M_INT_APP_MSG_ID app_msg_id;
} __attribute__ ((packed)) REQ_DEL_FULL_MSG_DATA;

inline static void REQ_DEL_FULL_MSG_DATA_hton(REQ_DEL_FULL_MSG_DATA *p)
{
	p->app_msg_id.type = htons(p->app_msg_id.type);
	p->app_msg_id.id = htons(p->app_msg_id.id);
	return;
}

typedef struct{
	M_INT_APP_MSG_ID app_msg_id;
	unsigned int mesg_size;
	unsigned int mesg_max_size;
	unsigned char mesg_max_prod_no;
	unsigned short mesg_period;
	unsigned short mesg_MTU;
} __attribute__ ((packed)) REQ_ADDMODIFY_LOAD_REQS_DATA;

inline static void REQ_ADDMODIFY_LOAD_REQS_DATA_hton(REQ_ADDMODIFY_LOAD_REQS_DATA *p)
{
	p->app_msg_id.type = htons(p->app_msg_id.type);
	p->app_msg_id.id = htons(p->app_msg_id.id);
	p->mesg_size = htoni(p->mesg_size);
	p->mesg_max_size = htoni(p->mesg_max_size);
	p->mesg_max_prod_no = p->mesg_max_prod_no;
	p->mesg_period = htons(p->mesg_period);
	p->mesg_MTU = htons(p->mesg_MTU);
	return;
}

typedef struct{
	M_INT_APP_MSG_ID app_msg_id;
	signed int mesg_size;
	signed int mesg_period;
	signed short priority;
} __attribute__ ((packed)) REQ_ADDMODIFY_QOS_REQS_DATA; //Initialize w one pair

inline static void REQ_ADDMODIFY_QOS_REQS_DATA_hton(REQ_ADDMODIFY_QOS_REQS_DATA *p)
{
	p->app_msg_id.type = htons(p->app_msg_id.type);
	p->app_msg_id.id = htons(p->app_msg_id.id);
	p->mesg_size = htoni(p->mesg_size);
	p->mesg_period = htoni(p->mesg_period);
	p->priority = htons(p->priority);
	return;
}

typedef struct{
	M_INT_APP_MSG_ID app_msg_id;
} __attribute__ ((packed)) REQ_REMOVE_LOAD_REQS_DATA; //Initialize w one pair

inline static void REQ_REMOVE_LOAD_REQS_DATA_hton(REQ_REMOVE_LOAD_REQS_DATA *p)
{
	p->app_msg_id.type = htons(p->app_msg_id.type);
	p->app_msg_id.id = htons(p->app_msg_id.id);
	return;
}

typedef struct{
	M_INT_APP_MSG_ID app_msg_id;
	signed int mesg_size;
	signed int mesg_period;
} __attribute__ ((packed)) REQ_ADD_QOS_PAIR_REQS_DATA;

inline static void REQ_ADD_QOS_PAIR_REQS_DATA_hton(REQ_ADD_QOS_PAIR_REQS_DATA *p)
{
	p->app_msg_id.type = htons(p->app_msg_id.type);
	p->app_msg_id.id = htons(p->app_msg_id.id);
	p->mesg_size = htoni(p->mesg_size);
	p->mesg_period = htoni(p->mesg_period);
	return;
}

typedef struct{
	M_INT_APP_MSG_ID app_msg_id;
	signed short priority;
} __attribute__ ((packed)) REQ_ADD_QOS_PRIO_REQS_DATA;

inline static void REQ_ADD_QOS_PRIO_REQS_DATA_hton(REQ_ADD_QOS_PRIO_REQS_DATA *p)
{
	p->app_msg_id.type = htons(p->app_msg_id.type);
	p->app_msg_id.id = htons(p->app_msg_id.id);
	p->priority = htons(p->priority);
	return;
}

typedef struct{
	M_INT_APP_MSG_ID app_msg_id;
	unsigned char mesg_prodconsID;
	unsigned char direction; //0-1
} __attribute__ ((packed)) REQ_ADD_ONE_END_CONECTION_DATA;

inline static void REQ_ADD_ONE_END_CONECTION_DATA_hton(REQ_ADD_ONE_END_CONECTION_DATA *p)
{
	p->app_msg_id.type = htons(p->app_msg_id.type);
	p->app_msg_id.id = htons(p->app_msg_id.id);
	return;
}

typedef struct{
	M_INT_APP_MSG_ID app_msg_id;
	unsigned char mesg_prodconsID;
} __attribute__ ((packed)) REQ_DEL_ONE_END_CONECTION_DATA;

inline static void REQ_DEL_ONE_END_CONECTION_DATA_hton(REQ_DEL_ONE_END_CONECTION_DATA *p)
{
	p->app_msg_id.type = htons(p->app_msg_id.type);
	p->app_msg_id.id = htons(p->app_msg_id.id);
	return;
}

typedef struct{
	M_INT_APP_MSG_ID app_msg_id;
} __attribute__ ((packed)) REQ_GET_FTT_ID_DETAILS_DATA;

inline static void REQ_GET_FTT_ID_DETAILS_DATA_hton(REQ_GET_FTT_ID_DETAILS_DATA *p)
{
	p->app_msg_id.type = htons(p->app_msg_id.type);
	p->app_msg_id.id = htons(p->app_msg_id.id);
	return;
}

typedef struct{
	M_INT_APP_MSG_ID app_msg_id;
	unsigned short mesg_period;
} __attribute__ ((packed)) REQ_CHG_PERIOD_DATA;

inline static void REQ_CHG_PERIOD_DATA_hton(REQ_CHG_PERIOD_DATA *p)
{
	p->app_msg_id.type = htons(p->app_msg_id.type);
	p->app_msg_id.id = htons(p->app_msg_id.id);
	p->mesg_period = htoni(p->mesg_period);
	return;
}

typedef struct{
	unsigned char seq_no;
} __attribute__ ((packed)) REQ_REMOTE_REPORT_DATA;

inline static void REQ_REMOTE_REPORT_DATA_hton(REQ_REMOTE_REPORT_DATA *p)
{
	return;
}

typedef struct{
	unsigned char seq_no;
} __attribute__ ((packed)) REQ_SUBMIT_DATA;

inline static void REQ_SUBMIT_DATA_hton(REQ_SUBMIT_DATA *p)
{
	return;
}

inline static unsigned short GET_CMD_DATA_SIZE(REQ_COMMANDS cmd){
	switch(cmd){
		case REQ_ADD_FULL_MSG:
			return (sizeof(REQ_ADD_FULL_MSG_DATA));
			break;
		case REQ_DEL_FULL_MSG:
			return (sizeof(REQ_DEL_FULL_MSG_DATA));
			break;
		case REQ_ADDMODIFY_LOAD_REQS:
			return (sizeof(REQ_ADDMODIFY_LOAD_REQS_DATA));
			break;
		case REQ_ADDMODIFY_QOS_REQS:
			return (sizeof(REQ_ADDMODIFY_QOS_REQS_DATA));
			break;
		case REQ_REMOVE_LOAD_REQS:
			return (sizeof(REQ_REMOVE_LOAD_REQS_DATA));
			break;
		case REQ_ADD_QOS_PAIR_REQS:
			return (sizeof(REQ_ADD_QOS_PAIR_REQS_DATA));
			break;
		case REQ_ADD_QOS_PRIO_REQS:
			return (sizeof(REQ_ADD_QOS_PRIO_REQS_DATA));
			break;
		case REQ_ADD_ONE_END_CONECTION:
			return (sizeof(REQ_ADD_ONE_END_CONECTION_DATA));
			break;
		case REQ_DEL_ONE_END_CONECTION:
			return (sizeof(REQ_DEL_ONE_END_CONECTION_DATA));
			break;
		case REQ_GET_FTT_ID_DETAILS:
			return (sizeof(REQ_GET_FTT_ID_DETAILS_DATA));
			break;
		case REQ_CHG_PERIOD:
			return (sizeof(REQ_CHG_PERIOD_DATA));
			break;
		case REQ_REMOTE_REPORT:
			return (sizeof(REQ_REMOTE_REPORT_DATA));
			break;
		case REQ_SUBMIT:
			return (sizeof(REQ_SUBMIT_DATA));
			break;
		default:
			return 0;
			break;
	}
}


typedef struct{
	REQ_COMMANDS cmd_type;
	unsigned short cmd_size; //command size excluding this header
} __attribute__ ((packed)) CMD_TRANSPORT_CONTAINER_H; //header

typedef struct{
	CMD_TRANSPORT_CONTAINER_H header; //number of commands whithin this packet
	char data[];
} __attribute__ ((packed)) CMD_TRANSPORT_CONTAINER;

inline static unsigned short GET_CMD_TRANSPORT_CONTAINER_FULL_SIZE(REQ_COMMANDS cmd){
	return GET_CMD_DATA_SIZE(cmd) + sizeof(CMD_TRANSPORT_CONTAINER_H);
}
inline static unsigned short GET_CMD_TRANSPORT_CONTAINER_FULL_SIZE_WITH_POINTER(CMD_TRANSPORT_CONTAINER *pointer){
	return (pointer->header.cmd_size + sizeof(CMD_TRANSPORT_CONTAINER_H));
}

inline static void CMD_TRANSPORT_CONTAINER_hton(CMD_TRANSPORT_CONTAINER *p)
{
	switch(p->header.cmd_type){
		case REQ_ADD_FULL_MSG:
			REQ_ADD_FULL_MSG_DATA_hton((REQ_ADD_FULL_MSG_DATA *)(p->data));
			break;
		case REQ_DEL_FULL_MSG:
			REQ_DEL_FULL_MSG_DATA_hton((REQ_DEL_FULL_MSG_DATA *)(p->data));
			break;
		case REQ_ADDMODIFY_LOAD_REQS:
			REQ_ADDMODIFY_LOAD_REQS_DATA_hton((REQ_ADDMODIFY_LOAD_REQS_DATA *)(p->data));
			break;
		case REQ_ADDMODIFY_QOS_REQS:
			REQ_ADDMODIFY_QOS_REQS_DATA_hton((REQ_ADDMODIFY_QOS_REQS_DATA *)(p->data));
			break;
		case REQ_REMOVE_LOAD_REQS:
			REQ_REMOVE_LOAD_REQS_DATA_hton((REQ_REMOVE_LOAD_REQS_DATA *)(p->data));
			break;
		case REQ_ADD_QOS_PAIR_REQS:
			REQ_ADD_QOS_PAIR_REQS_DATA_hton((REQ_ADD_QOS_PAIR_REQS_DATA *)(p->data));
			break;
		case REQ_ADD_QOS_PRIO_REQS:
			REQ_ADD_QOS_PRIO_REQS_DATA_hton((REQ_ADD_QOS_PRIO_REQS_DATA *)(p->data));
			break;
		case REQ_ADD_ONE_END_CONECTION:
			REQ_ADD_ONE_END_CONECTION_DATA_hton((REQ_ADD_ONE_END_CONECTION_DATA *)(p->data));
			break;
		case REQ_DEL_ONE_END_CONECTION:
			REQ_DEL_ONE_END_CONECTION_DATA_hton((REQ_DEL_ONE_END_CONECTION_DATA *)(p->data));
			break;
		case REQ_GET_FTT_ID_DETAILS:
			REQ_GET_FTT_ID_DETAILS_DATA_hton((REQ_GET_FTT_ID_DETAILS_DATA *)(p->data));
			break;
		case REQ_CHG_PERIOD:
			REQ_CHG_PERIOD_DATA_hton((REQ_CHG_PERIOD_DATA *)(p->data));
			break;
		case REQ_REMOTE_REPORT:
			REQ_REMOTE_REPORT_DATA_hton((REQ_REMOTE_REPORT_DATA *)(p->data));
			break;
		case REQ_SUBMIT:
			REQ_SUBMIT_DATA_hton((REQ_SUBMIT_DATA *)(p->data));
			break;
		default:
			break;
	}
	//htons cmd_type char
	p->header.cmd_size = htons(p->header.cmd_size);
	return;
}

inline static void CMD_TRANSPORT_CONTAINER_ntoh(CMD_TRANSPORT_CONTAINER *p)
{
	/* I'm using the same 'cause p->header.cmd_type is char 
	 * Otherwise it would be necessary */
	CMD_TRANSPORT_CONTAINER_hton(p);
	return;
}


typedef struct{
	unsigned char cmds_no; //number of commands whithin this packet
	CMD_TRANSPORT_CONTAINER cmds_array[];
} __attribute__ ((packed)) CMDS_TRANSPORT;


inline static signed char CMDS_TRANSPORT_init(CMDS_TRANSPORT *command_data)
{
	command_data->cmds_no = 0;
	return 0;
}

inline static void *CMDS_TRANSPORT_append_cmd( CMDS_TRANSPORT *command_data, REQ_COMMANDS cmd_type, void *cmd)
{
	unsigned short i;
	unsigned short current_total_cmd_size;
	CMD_TRANSPORT_CONTAINER *container_p;
	
	current_total_cmd_size = sizeof(CMDS_TRANSPORT);
	if (current_total_cmd_size != 1) ERROR_MSG("Check these structures again");
	
	//Point into the last position
	for (i=0; i < command_data->cmds_no; i++){
		container_p = (CMD_TRANSPORT_CONTAINER *)((char *)command_data + current_total_cmd_size);
		current_total_cmd_size += GET_CMD_TRANSPORT_CONTAINER_FULL_SIZE_WITH_POINTER(container_p);
	}
	//current_total_cmd_size is the size already booked
	if ( ( current_total_cmd_size + GET_CMD_TRANSPORT_CONTAINER_FULL_SIZE(cmd_type) ) > REQ_CMDS_MAX_SIZE ){
		//ERROR can't append more this command, wait for the next train!
		return NULL;
	}
	
	//Come aboard
	container_p = (CMD_TRANSPORT_CONTAINER *)((char *)command_data + current_total_cmd_size);
	container_p->header.cmd_type = cmd_type;
	container_p->header.cmd_size = GET_CMD_DATA_SIZE(cmd_type);

	if (cmd != NULL)
		memcpy(&(container_p->data), cmd, container_p->header.cmd_size);

	//Increment the "Train"
	((CMDS_TRANSPORT *)command_data)->cmds_no ++;

//	PRINT_MSG("S_int_req_transport_append_command Returning pointer: %p",&(container_p->data) );
	return ((void *)&(container_p->data));
}

inline static void CMDS_TRANSPORT_hton( CMDS_TRANSPORT *command_data )
{
	CMD_TRANSPORT_CONTAINER *temp;
	unsigned short i;
	unsigned short cmd_size;

	
	temp = (CMD_TRANSPORT_CONTAINER *)&( command_data->cmds_array );
	for (i=0; i < command_data->cmds_no; i++){
		cmd_size = GET_CMD_TRANSPORT_CONTAINER_FULL_SIZE_WITH_POINTER(temp);
		CMD_TRANSPORT_CONTAINER_hton(temp);
		temp = (CMD_TRANSPORT_CONTAINER *)( (char *)temp + cmd_size);
	}
	
	//htons cmds_no -char
	return;
}

inline static void CMDS_TRANSPORT_ntoh( CMDS_TRANSPORT *command_data )
{
	CMD_TRANSPORT_CONTAINER *temp;
	unsigned short i;

	//ntohs cmds_no -char
	
	temp = (CMD_TRANSPORT_CONTAINER *)&( command_data->cmds_array );
	for (i=0; i < command_data->cmds_no; i++){
		CMD_TRANSPORT_CONTAINER_ntoh(temp);
		temp = (CMD_TRANSPORT_CONTAINER *)( (char *)temp + GET_CMD_TRANSPORT_CONTAINER_FULL_SIZE_WITH_POINTER(temp));
	}
	return;
}

inline static void CMDS_TRANSPORT_print(CMDS_TRANSPORT *command_data)
{
	CMD_TRANSPORT_CONTAINER *temp;
	unsigned short i;
	
	PRINT_MSG("Received a set with %d commands",((CMDS_TRANSPORT *)command_data)->cmds_no);

	temp = (CMD_TRANSPORT_CONTAINER *)&( ((CMDS_TRANSPORT *)command_data)->cmds_array );
	for (i=0; i<((CMDS_TRANSPORT *)command_data)->cmds_no; i++){
		PRINT_MSG("Command %d: type %d   Size %d", 
				i,
				temp->header.cmd_type,
				temp->header.cmd_size
				);
		temp = (CMD_TRANSPORT_CONTAINER *)((char *)temp + GET_CMD_TRANSPORT_CONTAINER_FULL_SIZE_WITH_POINTER(temp));
	}
}

/* The reply goes direct, no command type, no commands concatenation */

typedef struct{
	unsigned char seq_no;
	signed char submission_result;
	signed int last_command_return;
} __attribute__ ((packed)) REQ_REPLY;

inline static void REQ_REPLY_hton(REQ_REPLY *p)
{
	p->last_command_return = htoni(p->last_command_return);
	return;
}
inline static void REQ_REPLY_ntoh(REQ_REPLY *p)
{
	p->last_command_return = ntohi(p->last_command_return);
	return;
}


inline static void REQ_REPLY_print(REQ_REPLY *rep_cmd, unsigned char f)
{
	PRINT_MSG("Reply command: Submission result: %d   last entered command return: %d",
			rep_cmd->submission_result,
			rep_cmd->last_command_return
			);
	if (f) PRINTF("\n");
}

#endif   /* ----- #ifndef REQUEST_CMDS_H  ----- */

