/*****************************************************************************
 * M_int_app_report.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/


#include "Ports.h"



typedef struct{
	int node_ID;
	unsigned char event_state; // 1=in
} APP_REPORT_TYPE;

#define MAX_APP_REPORT_EVENTS 500

static APP_REPORT_TYPE app_report_array[MAX_APP_REPORT_EVENTS];
static int app_report_rd, app_report_wr;
static FTT_SEMAPHORE app_report_sem;

signed char M_int_app_report_init(void)
{
	app_report_rd=app_report_wr=0;
	FTT_SEMAPHORE_init( &app_report_sem , 0);
	return 0;
}

signed char M_int_app_report_close(void)
{
	app_report_rd=app_report_wr=0;
	FTT_SEMAPHORE_destroy( &app_report_sem );
	return 0;
}

signed char M_int_app_report_put( unsigned char event, unsigned int node_app_id )
{
	if ( (app_report_wr+1)%MAX_APP_REPORT_EVENTS == app_report_rd){
		ERROR_MSG("Cannot place more events... Full!");
		return -1;
	}
	app_report_array[app_report_wr].node_ID = node_app_id;
	app_report_array[app_report_wr].event_state = event;
	app_report_wr = (app_report_wr+1)%MAX_APP_REPORT_EVENTS;
	
	FTT_SEMAPHORE_post( &app_report_sem );

	return 0;
}

signed char M_int_app_report_get( unsigned char *event, unsigned int *node_app_id )
{

	if ( FTT_SEMAPHORE_wait( &app_report_sem ) < 0)
		return -1;
	
	if (app_report_rd == app_report_wr){
		ERROR_MSG("Event queues is inconsistent! Empty after a sem_wait");
		return -2;
	}
	
	if (node_app_id!= NULL) *node_app_id = app_report_array[app_report_rd].node_ID;
	if (event!= NULL) *event = app_report_array[app_report_rd].event_state;
	app_report_rd = (app_report_rd+1)%MAX_APP_REPORT_EVENTS;

	
	return 0;
}



