/*****************************************************************************
 * M_int_async_ch.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/


#include "Ports.h"
#include "M_ftt-core.layer.h"
#include "M_ftt-management.layer.h"
#include "M_int_async_ch_threads.h"
#include "M_int_nodes_db.h"

#include "M_ftt-management.layer.h"
#include "request_cmds.h"

#ifdef PLUGNPLAY_REPORT
#include "M_int_app_report.h"
#include "M_int_app_remote_report.h"
#endif

static rtl_pthread_t int_async_th;

static void *int_async(void *t)
{
	int ftt_id_2rx_in_master, ftt_id_2tx_from_master;
	unsigned short id_2tx_from_master, id_2rx_in_master;
	signed short app_id;
	unsigned char nodeID;
	int ret;

	M_FTT_MANAGEMENT_L_APP_ID var_id;
	M_FTT_MANAGEMENT_L_LOAD_REQUIREMENTS load_req;
	M_FTT_MANAGEMENT_L_CONNECTION_SET con_req;

	while (1){
		
		/* Blocks waiting for another Slave Interface layer connection */
		if ( M_FTT_CORE_L_get_appPnP( &app_id, &nodeID) < 0){
			ERROR_MSG("Possibly the signal has been destroyed.");
			return NULL;
		}
		
		/* if asking for app_id == 0, ask for an available one */
		if (app_id == M_INT_APP_MASTER_ID) {
			ret = M_int_nodes_db_Get_a_free_appID( );
			if (ret >= 0) app_id = ret;
			PRINT_MSG("Master assigning an app_id %d", app_id );

		}

		if (app_id>=0){
			PRINT_MSG("\ni: app_id %d from node %d  bootstrapping...", app_id, nodeID );

			/*Make the app_id <-> nodeID association */
			ret = M_int_nodes_db_Add( nodeID, app_id );
			if ( ret == -2 ){
				/* This nodeID was already assigned */
				/* Let's destroy the remote commands handler */

				M_int_async_ch_thread_destroy( nodeID );

				/* Now, deallocate all the contracts that this node takes part */
				M_FTT_MANAGEMENT_L_changes_lock();

				M_FTT_MANAGEMENT_L_del_node( nodeID );
				M_FTT_MANAGEMENT_L_changes_unlock_submiting();

				/*Finally remove it from the DB */
				M_int_nodes_db_remove(nodeID);

				/* Now... we can add a new one! Even with new app_id */
				ret = M_int_nodes_db_Add( nodeID, app_id );
			}

			if ( ret < 0 ){
				ERROR_MSG("This app node ID was probably already assigned");
				M_FTT_CORE_L_notify_appPnP_KO(app_id, nodeID);
				continue;
			}

			/* Let's now assign two async_ids for these new async channels 
			 * of this app_layer 
			 * As a side note: since the master doesn't need a pair of ID's M_INT_NODE_ASYNC_ID_BASE*/
			id_2tx_from_master = M_INT_NODE_ASYNC_ID_BASE + 1/*The Bradcast ch*/ + (app_id-1/*0idx*/)*2;
			id_2rx_in_master = M_INT_NODE_ASYNC_ID_BASE + 1/*The Bradcast ch*/ + (app_id-1/*0idx*/)*2 +1;
			//PRINT_MSG("The Base: %x   IDs: %x  and  %x",M_INT_NODE_ASYNC_ID_BASE, id_2tx_from_master, id_2rx_in_master);


			load_req.m_size = REQ_CMDS_MAX_SIZE;
			load_req.m_max_size = REQ_CMDS_MAX_SIZE;
			load_req.m_max_prod_no = 1;
			load_req.m_period = load_req.m_deadline = 3;
			load_req.m_init=0;
			load_req.m_relative_id = -1;
			load_req.m_MTU=1400;
			load_req.m_timeliness = M_FTT_MANAGEMENT_HARD_RT;

			/* Set those asynchronous IDs */
			M_FTT_MANAGEMENT_L_changes_lock();

			con_req.Producers_no = 1;
			(con_req.Producer_nodeID)[0] = nodeID;
			con_req.Consumers_no = 1;
			(con_req.Consumer_nodeID)[0] = M_FTT_CORE_L_get_master_nodeID();
			con_req.Production_type = M_FTT_MANAGEMENT_P_UNICAST;

			var_id.type = M_FTT_MANAGEMENT_AM;
			var_id.id = id_2rx_in_master;
	
			ftt_id_2rx_in_master = M_FTT_MANAGEMENT_L_add_msg_complete(
					var_id,
					&load_req,
					NULL, //no qos
					&con_req);
		
			con_req.Producers_no = 1;
			(con_req.Producer_nodeID)[0] = M_FTT_CORE_L_get_master_nodeID();
			con_req.Consumers_no = 1;
			(con_req.Consumer_nodeID)[0] = nodeID;
			con_req.Production_type = M_FTT_MANAGEMENT_P_UNICAST;

			var_id.type = M_FTT_MANAGEMENT_AM;
			var_id.id = id_2tx_from_master;
			load_req.m_size = REQ_REPLY_MAX_SIZE;
			load_req.m_max_size = REQ_REPLY_MAX_SIZE;
			load_req.m_max_prod_no = 1;


			ftt_id_2tx_from_master  = M_FTT_MANAGEMENT_L_add_msg_complete(
					var_id,
					&load_req,
					NULL, //no qos
					&con_req);
//PRINT_MSG("ftt_id_2tx_from_master: %x", ftt_id_2tx_from_master);

			ret = M_FTT_MANAGEMENT_L_changes_unlock_submiting();

//PRINT_MSG("M_int_async_ch.c printing after adding the node async messages");		
//M_FTT_MANAGEMENT_L_print();
//M_FTT_CORE_L_print_db();


			/* Notify the Slaves layer */
			if ((ftt_id_2rx_in_master<0)||(ftt_id_2tx_from_master<0)||(ret < 0)){
				/* There isn't much to do here except notify the error */
				ERROR_MSG(" The interface layer asynchronous channels were not created - The node was not registered");

				/* Un register this nodeID from the reference table */
				M_int_nodes_db_remove( nodeID );
				M_FTT_CORE_L_notify_appPnP_KO((unsigned char)app_id, nodeID);
			}else{
		
//				PRINT_MSG("Sending the the ftt_ids(0x%x,0x%x) to the erspective slave: node ID %d   app_id %d", ftt_id_2tx_from_master, ftt_id_2rx_in_master, nodeID, app_id);
//				PRINT_MSG("Sending the the app_ids(0x%x,0x%x) to the erspective slave: node ID %d   app_node_id %d", id_2tx_from_master, id_2rx_in_master, nodeID, app_id);
				M_FTT_CORE_L_notify_appPnP_OK( (unsigned char) app_id, nodeID, ftt_id_2rx_in_master/*id_tx*/, ftt_id_2tx_from_master/*id_rx*/);
				/* Create the async communication endpoints for this new node in the master */
				M_int_async_ch_thread_create( nodeID, (unsigned short)ftt_id_2tx_from_master, (unsigned short)ftt_id_2rx_in_master );
			}

#ifdef PLUGNPLAY_REPORT
			M_int_app_report_put( 1 /*event_plug-in*/, app_id );
			M_int_app_remote_report_put( 1 /*event_plug-in*/, app_id );
#endif
			PRINT_MSG("");

		}else{ //(app_id_id<0) - remove nodeID

			/* De-assign the threads created for the interface communication with this node */
			M_int_async_ch_thread_destroy( nodeID );
		
			app_id = -app_id; /* This code was added to solve the signedness issue */

			if ( M_int_nodes_db_Get_appID( nodeID, (unsigned short *)&app_id ) >= 0 ){
				/* Get the two management layer IDs */
				id_2tx_from_master = M_INT_NODE_ASYNC_ID_BASE + 1/*The Bradcast ch*/ + (app_id-1/*0idx*/)*2;
				id_2rx_in_master = M_INT_NODE_ASYNC_ID_BASE + 1/*The Bradcast ch*/ + (app_id-1/*0idx*/)*2 +1;

				/* Remove all connections related to this node */
				M_FTT_MANAGEMENT_L_changes_lock();
				M_FTT_MANAGEMENT_L_del_node( nodeID );
				M_FTT_MANAGEMENT_L_changes_unlock_submiting();
				
				/* Remove the comm channels */
				M_FTT_MANAGEMENT_L_changes_lock();

				var_id.type = M_FTT_MANAGEMENT_AM; var_id.id = id_2rx_in_master;
				M_FTT_MANAGEMENT_L_del_msg( var_id );
	
				var_id.type = M_FTT_MANAGEMENT_AM; var_id.id = id_2tx_from_master;
				M_FTT_MANAGEMENT_L_del_msg( var_id );

				M_FTT_MANAGEMENT_L_changes_unlock_submiting();

			}
			M_int_nodes_db_remove( nodeID );
#ifdef PLUGNPLAY_REPORT
			M_int_app_remote_report_put( 0 /*event_unplug*/, app_id );
#endif
		}
	}

	return NULL;
}

signed char M_int_async_ch_Init(unsigned char app_id)
{
	rtl_pthread_attr_t attr;
	struct rtl_sched_param sched_param;
	int ret;

	if ( !M_int_nodes_db_Init_check() ){
		ERROR_MSG("Initialize first the M_int_nodes_db module");
		return -1;
	}
	
	M_int_nodes_db_Add( M_FTT_CORE_L_get_master_nodeID(), app_id );

	/* Initialize the thread that will wait for the asynch channels */
	rtl_pthread_attr_init( &attr );
	sched_param.sched_priority = rtl_sched_get_priority_min(RTL_SCHED_FIFO);
	rtl_pthread_attr_setschedparam( &attr, &sched_param );
#ifdef RTL_PRO
	rtl_pthread_attr_setfp_np(&attr, 1);
#endif
	if ( (ret=rtl_pthread_create( &int_async_th, &attr, int_async, (void *)0 )) ){
		ERROR_MSG("Init Fatal Error: pthread_create returned %d", ret);
		return -2;
	}

#ifdef PRINT_THREAD_ID
#ifdef RTL_PRO
	ERROR_MSG("New thread id: %d", int_async_th);
#else
	ERROR_MSG("New thread id: %lu", int_async_th);
#endif
#endif

#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -M_int_async_ch_Init Ok");
#endif
	
	return 0;

}

signed char M_int_async_ch_Close(void)
{
	/* cancel the int_async thread */
	rtl_pthread_cancel( int_async_th );
	rtl_pthread_join( int_async_th, NULL );
	
#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -M_int_async_ch_Close Ok");
#endif
		
	return 0;
}


