/*****************************************************************************
 * M_int_async_ch_threads.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#include "Ports.h"
#include "M_ftt-core.layer.h"
#include "S_ftt-core.layer.h"

#include "request_cmds.h"
#include "M_int_req_queuing.h"

#include "S_ftt-interface.layer.h"

#define M_INT_ASYNC_CH_THREADS_MAX_SIZE 100
#define M_INT_ASYNC_CH_THREADS_STACK_SIZE 30000

typedef struct{
	unsigned char in_use;
	rtl_pthread_t interface_async_rx_th;
	unsigned short amsgID_rx;
	unsigned short amsgID_tx;
	unsigned short nodeID;
	unsigned char thread_stack[M_INT_ASYNC_CH_THREADS_STACK_SIZE];
}M_INT_ASYNC_CH_THREADS_ITEM;

static M_INT_ASYNC_CH_THREADS_ITEM M_INT_ASYNC_CH_THREADS_array[M_INT_ASYNC_CH_THREADS_MAX_SIZE];

static void M_INT_ASYNC_CH_THREADS_array_init(void)
{
	unsigned short i;
	for (i=0; i<M_INT_ASYNC_CH_THREADS_MAX_SIZE; i++){
		M_INT_ASYNC_CH_THREADS_array[i].in_use = 0;
	}
}


static M_INT_ASYNC_CH_THREADS_ITEM *M_INT_ASYNC_CH_THREADS_array_get_item(void)
{
	unsigned short i;
	for (i=0; i<M_INT_ASYNC_CH_THREADS_MAX_SIZE; i++){
		if ( !(M_INT_ASYNC_CH_THREADS_array[i].in_use) ){
			M_INT_ASYNC_CH_THREADS_array[i].in_use = 1;
			return &(M_INT_ASYNC_CH_THREADS_array[i]);
		}
	}
	return NULL;
}

static M_INT_ASYNC_CH_THREADS_ITEM *M_INT_ASYNC_CH_THREADS_array_find_item( unsigned short nodeID )
{
	unsigned short i;
	for (i=0; i<M_INT_ASYNC_CH_THREADS_MAX_SIZE; i++)
		if ( M_INT_ASYNC_CH_THREADS_array[i].in_use )
			if ( M_INT_ASYNC_CH_THREADS_array[i].nodeID == nodeID )
				return &(M_INT_ASYNC_CH_THREADS_array[i]);
	return NULL;
}
static signed char M_INT_ASYNC_CH_THREADS_array_free_item (M_INT_ASYNC_CH_THREADS_ITEM * p)
{
	p->in_use = 0;
	return 0;
}

static void *interface_async_rx(void *t)
{
	int ret;
	unsigned short me_amsgID_rx = ((M_INT_ASYNC_CH_THREADS_ITEM *)t)->amsgID_rx;
	unsigned short me_amsgID_tx = ((M_INT_ASYNC_CH_THREADS_ITEM *)t)->amsgID_tx;
	unsigned char me_amsgID_rx_tag, me_amsgID_tx_tag;
	unsigned short me_nodeID = ((M_INT_ASYNC_CH_THREADS_ITEM *)t)->nodeID;
	FTT_SIGNAL *rx_sig;
	signed int return_code;

	unsigned char *rx_msg_pointer;
	unsigned char *tx_msg_pointer;
	
	CMD_TRANSPORT_CONTAINER *temp;
	unsigned short i;


//PRINT_MSG(">>>>>>>>>>>>>>>>>>creating %x %x", me_amsgID_rx, me_amsgID_tx);

	if ( (ret=S_FTT_CORE_L_bind_mesg( 1 /*AM*/, me_amsgID_rx, &rx_sig, NULL,
									 0, /*mem_size - 0 'cause mem is controlled by the FTT */
									 10 /* FIFO size */
									 ))<0){
		ERROR_MSG("S_FTT_CORE_L_bind_mesg rx returned error: %d", ret);
		ERROR_MSG("The ID gave for RX could not be registered");
		return NULL;
	}

	if ( S_FTT_CORE_L_get_current( 1 /*AM*/, me_amsgID_rx, &me_amsgID_rx_tag, NULL, NULL ) < 0){
		ERROR_MSG("Error getting var tag");
		return NULL;
	}

	if ( (ret=S_FTT_CORE_L_bind_mesg( 1 /*AM*/, me_amsgID_tx, NULL, NULL,
									 0, /*mem_size - 0 'cause mem is controlled by the FTT */
									 10 /* FIFO size */
									 ))<0){
		ERROR_MSG("S_FTT_CORE_L_bind_mesg rx returned error: %d", ret);
		ERROR_MSG("The ID gave for TX could not be registered");
		return NULL;
	}

	if ( S_FTT_CORE_L_get_current( 1 /*AM*/, me_amsgID_tx, &me_amsgID_tx_tag, NULL, NULL ) < 0){
		ERROR_MSG("Error getting var tag");
		return NULL;
	}

	while (1){
//		PRINT_MSG("Waiting for the next rx_sig in the AM:0x%x",me_amsgID_rx);
//M_FTT_CORE_L_print_db();
		FTT_wait4signal( rx_sig );

//		PRINT_MSG("Received one \"request packet\" from slave nodeID %d  in AM 0x%x", me_nodeID, me_amsgID_rx );

		rx_msg_pointer = S_FTT_CORE_pre_receive(me_amsgID_rx, NULL, NULL/*status*/);

		/* Decode from network data */
		CMDS_TRANSPORT_ntoh( (CMDS_TRANSPORT *)rx_msg_pointer );

//		CMDS_TRANSPORT_print( (CMDS_TRANSPORT *)rx_msg_pointer);

		/* Let's then analyse and compute the commands */
		/* For each command container - submit */
		temp = (CMD_TRANSPORT_CONTAINER *)&( ((CMDS_TRANSPORT *)rx_msg_pointer)->cmds_array );
		for (i=0; i<((CMDS_TRANSPORT *)rx_msg_pointer)->cmds_no; i++){
			//PRINT_MSG("Going to submit no. %d th:", i );
			ret = M_int_req_queuing_Submit_req(
					me_nodeID,
					me_amsgID_rx,
					temp->header.cmd_type,
					(unsigned char *)(&(temp->data)),
					&return_code );

			//PRINT_MSG("M_int_req_queuing_Submit_req return %d", ret );

			/* If the command was to submit, let's notify the sender */
			switch( ret ){
				case 0:
					//PRINT_MSG("The command was queued");
					break;
				case 1:
				case 2:
				case 3:
				case 4:	/* Send it */
					//PRINT_MSG("The command(s) were purged return: %d", ret );
					//M_FTT_CORE_L_print_db();
					//NDB_printmesg( );
					/* Get memory buffer */
					if ( S_FTT_CORE_pre_send(me_amsgID_tx, 1, (unsigned char **)(&tx_msg_pointer) ) < 0 ){
						ERROR_MSG("Command not replied - could not get AM buffer");
						break;
					}

					/* Encode the command */
					//((REQ_REPLY *)tx_msg_pointer)->seq_no = ;
					((REQ_REPLY *)tx_msg_pointer)->submission_result = ret;
					((REQ_REPLY *)tx_msg_pointer)->last_command_return = return_code;
					//PRINT_MSG("Sending reply"); REQ_REPLY_print( (REQ_REPLY *)tx_msg_pointer, 0);

					/* Encode to network data and send */
					REQ_REPLY_hton( (REQ_REPLY *)tx_msg_pointer );
					S_FTT_CORE_send( me_amsgID_tx, me_amsgID_tx_tag, NULL, 0);
					break;
				
				default:
					ERROR_MSG(" This last command had design problems ");
					break;
			}

			/* increment to next command container */
			temp = (CMD_TRANSPORT_CONTAINER *)((unsigned char *)temp + GET_CMD_TRANSPORT_CONTAINER_FULL_SIZE_WITH_POINTER(temp));
		}
		
		ret = S_FTT_CORE_receive( me_amsgID_rx, NULL, NULL, NULL/*status*/);
		//PRINT_MSG("S_FTT_CORE_receive returned %d", ret );
	}

	return NULL;
}
signed char M_int_async_ch_thread_init( void )
{
	M_INT_ASYNC_CH_THREADS_array_init();

#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -M_int_async_ch_thread_init Ok");
#endif
	return 0;
}

signed char M_int_async_ch_thread_close( void )
{
	unsigned short i;
	for (i=0; i<M_INT_ASYNC_CH_THREADS_MAX_SIZE; i++){
		if (M_INT_ASYNC_CH_THREADS_array[i].in_use){
			;
			/* destroy thread */
			rtl_pthread_cancel( M_INT_ASYNC_CH_THREADS_array[i].interface_async_rx_th );
			rtl_pthread_join( M_INT_ASYNC_CH_THREADS_array[i].interface_async_rx_th, NULL );
			M_INT_ASYNC_CH_THREADS_array[i].in_use = 0;
		}
	}
	
#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -M_int_async_ch_thread_close Ok");
#endif
	return 0;
}

signed char M_int_async_ch_thread_create( unsigned char nodeID, unsigned short id_tx, unsigned short id_rx )
{
	rtl_pthread_attr_t attr;
	struct rtl_sched_param sched_param;
	int ret;
	M_INT_ASYNC_CH_THREADS_ITEM *temp;

	/* id_tx follows the direction master to slave
	 * id_rx follows the direction slave to master
	 */

#ifdef MASTER_PRINT_INTERFACE_REQUESTS
//	PRINT_MSG("i: Creating interface endpoints. For nodeID %d    (0x%x,0x%x)", nodeID, id_tx, id_rx);
#endif

	if ( (temp = M_INT_ASYNC_CH_THREADS_array_get_item()) == NULL ){
		ERROR_MSG("M_INT_ASYNC_CH_THREADS_array_get_item failed");
		return -1;
	}

	temp->amsgID_rx = id_rx;
	temp->amsgID_tx = id_tx;
	temp->nodeID = nodeID;

	/* Initialize the thread that will wait for the asynch channels */
	rtl_pthread_attr_init( &attr );
	sched_param.sched_priority = rtl_sched_get_priority_min(RTL_SCHED_FIFO);
	rtl_pthread_attr_setschedparam( &attr, &sched_param );

#ifdef RTL_PRO
	/* In RTLinux to online instantiate threads we have to give the stack */
	rtl_pthread_attr_setstacksize(&attr, M_INT_ASYNC_CH_THREADS_STACK_SIZE);
	rtl_pthread_attr_setstackaddr(&attr, temp->thread_stack);
	rtl_pthread_attr_setfp_np(&attr, 1);
#else
	pthread_attr_setstack( &attr, temp->thread_stack, M_INT_ASYNC_CH_THREADS_STACK_SIZE);
#endif
	if ( (ret=rtl_pthread_create( &(temp->interface_async_rx_th), &attr, interface_async_rx, (void *)temp )) != 0 ){
		ERROR_MSG("pthread_create returned %d", ret);
		return -1;
	}
#ifdef PRINT_THREAD_ID
#ifdef RTL_PRO
	ERROR_MSG("New thread id: %d", (temp->interface_async_rx_th));
#else
	ERROR_MSG("New thread id: %lu", (temp->interface_async_rx_th));
#endif
#endif
	return 0;
}

signed char M_int_async_ch_thread_destroy( unsigned char nodeID )
{
	M_INT_ASYNC_CH_THREADS_ITEM *temp;

	PRINT_MSG("i: Destroying interface endpoints for nodeID %d", nodeID );

	/* Locate the thread data */
	if ( (temp = M_INT_ASYNC_CH_THREADS_array_find_item( nodeID )) == NULL ){
		return -1;
	}

	/* cancel the int_async thread */
	rtl_pthread_cancel( (temp->interface_async_rx_th) );
	rtl_pthread_join( (temp->interface_async_rx_th), NULL );

	M_INT_ASYNC_CH_THREADS_array_free_item ( temp );

	return 0;
}

