/*****************************************************************************
 * M_int_nodes_db.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#include "Ports.h"
#include "ftt-interface.layer_types.h"

typedef struct{
	unsigned char nodeID;
	unsigned char app_id;
}NODES_TYPE;

static NODES_TYPE nodes_array[M_INT_MAX_APP_NODES];	//254 number of admissible nodes
static unsigned short nodes_array_no=0;

static unsigned char M_int_nodes_db_Init_flag=0;

signed char M_int_nodes_db_Init( void )
{
	nodes_array_no = 0;
	M_int_nodes_db_Init_flag=1;

	return 0;
}

signed char M_int_nodes_db_Close( void )
{
	M_int_nodes_db_Init_flag=0;
	return 0;
}

signed char M_int_nodes_db_Init_check( void )
{
	return M_int_nodes_db_Init_flag;
}

signed char M_int_nodes_db_Get_nodeID( unsigned short app_id, unsigned short *nodeID ) /* nodeID may be NULL */
{
	unsigned short i;
	for (i=0; i<nodes_array_no; i++ ){
		if ( nodes_array[i].app_id == app_id ){
			if (nodeID!=NULL)
				*nodeID = nodes_array[i].nodeID;
			return i;
		}
	}

	/* it found no match */
	return -1;
}

signed char M_int_nodes_db_Get_appID( unsigned short nodeID, unsigned short *app_id ) /* app_id may be NULL */
{
	unsigned short i;
	for (i=0; i<nodes_array_no; i++ ){
		if ( nodes_array[i].nodeID == nodeID ){
			if (app_id!=NULL)
				*app_id = nodes_array[i].app_id;
			return i;
		}
	}

	/* it found no match */
	return -1;
}

signed short M_int_nodes_db_Get_a_free_appID( void )
{
	unsigned short i;

	/* TRy different app_ids - could start from 1 but should make no difference */
	for (i=0; i<M_INT_MAX_APP_NODES; i++) {
		if ( M_int_nodes_db_Get_nodeID(i, NULL) < 0 )	return i; //the app_id that is free
	}
	return -1;
}

/* we are assuming that that the writting accesses comes from a single point */
signed char M_int_nodes_db_Add( unsigned short nodeID, unsigned short app_id )
{
	if ( (nodes_array_no+1) == M_INT_MAX_APP_NODES){
		ERROR_MSG("No room for more assigning nodes");
		return -1;
	}

	/* Check if this nodeID alrady exists */
	if ( !( M_int_nodes_db_Get_appID( nodeID, NULL ) < 0 ) ){
//		PRINT_MSG("This nodeID is already registered. I'll abort. For now.... ");
		return -2; //Don't change the error
	}

	/* Check if this appID alrady exists */
	if ( !( M_int_nodes_db_Get_nodeID( app_id, NULL ) < 0 ) ){
//		PRINT_MSG("This nodeID is already registered. I'll abort. For now.... ");
		return -3;
	}

	nodes_array[nodes_array_no].nodeID = nodeID;
	nodes_array[nodes_array_no].app_id = app_id;
	nodes_array_no ++;
	return 0;
}

signed char M_int_nodes_db_remove( unsigned short node_id )
{
	int ret;
	int i;
	/* Check if this nodeID alrady exists */
	if ( (ret = M_int_nodes_db_Get_appID( node_id, NULL )) < 0 ){
		PRINT_MSG("M_int_nodes_db_remove: This nodeID was not registered.");
		return 0;
	}

	/* Remove the item ret */
	for (i=ret+1; i<nodes_array_no; i++ )
		nodes_array[i-1] = nodes_array[i];

	nodes_array_no --;
	
	
	return 0;
}

void M_int_nodes_db_PrintTable( void )
{
	int i;
	PRINT_MSG("Printing the AppNodeID <-> nodeID association: (%d items)",nodes_array_no );
	for ( i=0; i< nodes_array_no; i++){
		PRINT_MSG("(%d,%d)", nodes_array[i].app_id,  nodes_array[i].nodeID );
	}
	return;
}


