/*****************************************************************************
 * M_int_req_queuing.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/


#include "Ports.h"
#include "M_ftt-management.layer.h"

#include "request_cmds.h"
#include "remote_report_cmds.h"

#include "M_int_req_queue.h"
#include "M_int_nodes_db.h"


/* DB changes locker */
#define RQ_QUEUE_LockerDeclaration  rtl_pthread_mutex_t rq_queue_access_mutex;
#define RQ_QUEUE_Enter              rtl_pthread_mutex_lock(&rq_queue_access_mutex);
#define RQ_QUEUE_Leave              rtl_pthread_mutex_unlock(&rq_queue_access_mutex);
#define RQ_QUEUE_Init_locker        rtl_pthread_mutex_init(&rq_queue_access_mutex, 0);
#define RQ_QUEUE_Destroy_Locker     rtl_pthread_mutex_destroy(&rq_queue_access_mutex);
static RQ_QUEUE_LockerDeclaration;	//This var is to the application access into DB changes

signed char M_int_req_queuing_Init(void)
{

	RQ_QUEUE_Init_locker;

#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -M_int_req_queuing_Init Ok");
#endif
	
	return 0;
}

signed char M_int_req_queuing_Close(void)
{
	RQ_QUEUE_Destroy_Locker;

#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -M_int_req_queuing_Close Ok");
#endif

	return 0;
}

static signed char M_int_req_queuing_Submit_cmd( unsigned char nodeID_source, REQ_COMMANDS command_type, unsigned char *command_data, int *return_code)
{
	unsigned char i;

	switch(command_type){
 		case REQ_ADD_FULL_MSG:
 			{
				M_FTT_MANAGEMENT_L_LOAD_REQUIREMENTS gen_req;
				M_FTT_MANAGEMENT_L_CONNECTION_SET connect_req;
				M_FTT_MANAGEMENT_L_APP_ID message_app_id;
	
				message_app_id.id = ((REQ_ADD_FULL_MSG_DATA *)command_data)->app_msg_id.id;
				switch(((REQ_ADD_FULL_MSG_DATA *)command_data)->app_msg_id.type){
					case M_INT_MSG_SYNCH:
						message_app_id.type = M_FTT_MANAGEMENT_SM;
						break;
					case M_INT_MSG_ASYNCH_H:
						message_app_id.type = M_FTT_MANAGEMENT_AM;
						gen_req.m_timeliness = M_FTT_MANAGEMENT_HARD_RT;
						break;
					case M_INT_MSG_ASYNCH_S:
						message_app_id.type = M_FTT_MANAGEMENT_AM;
						gen_req.m_timeliness = M_FTT_MANAGEMENT_SOFT_RT;
						break;
					case M_INT_MSG_ASYNCH_U:
					default:
						message_app_id.type = M_FTT_MANAGEMENT_AM;
						gen_req.m_timeliness = M_FTT_MANAGEMENT_UNCONSTRAINED;
						break;
				}

#ifdef MASTER_PRINT_INTERFACE_REQUESTS
				PRINT_MSG("i: REQ_ADD_FULL_MSG %s  id:%x  [C=%d, T=%d] ....Prod_no:%d [0:%d], Cons_no:%d [0:%d]",
						  (message_app_id.type == M_FTT_MANAGEMENT_SM)?" SYNC":" ASYNC",
						  message_app_id.id,
						  ((REQ_ADD_FULL_MSG_DATA *)command_data)->mesg_size,
						  ((REQ_ADD_FULL_MSG_DATA *)command_data)->mesg_period,
						  ((REQ_ADD_FULL_MSG_DATA *)command_data)->mesg_producers_no,
						  (((REQ_ADD_FULL_MSG_DATA *)command_data)->mesg_producerID)[0],
						  ((REQ_ADD_FULL_MSG_DATA *)command_data)->mesg_consumers_no,
						  (((REQ_ADD_FULL_MSG_DATA *)command_data)->mesg_consumerID)[0]
						  );
#endif
				
				gen_req.m_size = ((REQ_ADD_FULL_MSG_DATA *)command_data)->mesg_size;
				gen_req.m_max_size = ((REQ_ADD_FULL_MSG_DATA *)command_data)->mesg_max_size;
				gen_req.m_max_prod_no = ((REQ_ADD_FULL_MSG_DATA *)command_data)->mesg_max_prod_no;
				gen_req.m_period = ((REQ_ADD_FULL_MSG_DATA *)command_data)->mesg_period;
				gen_req.m_deadline = ((REQ_ADD_FULL_MSG_DATA *)command_data)->mesg_period;
				gen_req.m_init = -1;
				gen_req.m_MTU = ((REQ_ADD_FULL_MSG_DATA *)command_data)->mesg_MTU;

				/* Let's now translate the connection requirements with our addressing table */
				connect_req.Producers_no = ((REQ_ADD_FULL_MSG_DATA *)command_data)->mesg_producers_no;
				for (i=0; i<connect_req.Producers_no; i++ )
					if ( M_int_nodes_db_Get_nodeID( (((REQ_ADD_FULL_MSG_DATA *)command_data)->mesg_producerID)[i], &(connect_req.Producer_nodeID[i]) ) < 0 ){
						ERROR_MSG("Couldn't find a nodeID for this");
						*return_code = -10;
						goto end_of_while2789563478563485;
					}	//connect_req.Producer_nodeID = ((REQ_ADD_FULL_MSG_DATA *)command_data)->mesg_producerID;

				connect_req.Consumers_no = ((REQ_ADD_FULL_MSG_DATA *)command_data)->mesg_consumers_no;
				for (i=0; i<connect_req.Consumers_no; i++ )
					if ( M_int_nodes_db_Get_nodeID( (((REQ_ADD_FULL_MSG_DATA *)command_data)->mesg_consumerID)[i], &(connect_req.Consumer_nodeID[i]) ) < 0 ){
						ERROR_MSG("Couldn't find a nodeID for this");
						*return_code = -11;
						goto end_of_while2789563478563485;
					}//connect_req.Consumer_nodeID[i] = (((REQ_ADD_FULL_MSG_DATA *)command_data)->mesg_consumerID)[i]; //[mesg_consumers_no]

				connect_req.Production_type = M_FTT_MANAGEMENT_P_BROADCAST;

				//PRINT_MSG("Preparing to call M_FTT_MANAGEMENT_L_add_msg_complete");
				*return_code = M_FTT_MANAGEMENT_L_add_msg_complete(
						message_app_id,
						&gen_req,
						NULL,
						&connect_req );
 			}
 			break;
		case REQ_DEL_FULL_MSG:
			{
				M_FTT_MANAGEMENT_L_APP_ID message_app_id;
				message_app_id.id = ((REQ_DEL_FULL_MSG_DATA *)command_data)->app_msg_id.id;
				message_app_id.type = (((REQ_DEL_FULL_MSG_DATA *)command_data)->app_msg_id.type == M_INT_MSG_SYNCH) ? M_FTT_MANAGEMENT_SM: M_FTT_MANAGEMENT_AM;
#ifdef MASTER_PRINT_INTERFACE_REQUESTS
				PRINT_MSG("i: REQ_DEL_FULL_MSG %s  id:%x", (message_app_id.type == M_FTT_MANAGEMENT_SM)?" SYNC":" ASYNC",message_app_id.id);
#endif
				*return_code = M_FTT_MANAGEMENT_L_del_msg( message_app_id );


			}
			break;
		case REQ_ADDMODIFY_LOAD_REQS:
			{
				M_FTT_MANAGEMENT_L_APP_ID message_app_id;
				M_FTT_MANAGEMENT_L_LOAD_REQUIREMENTS gen_req;
	
				message_app_id.id = ((REQ_ADDMODIFY_LOAD_REQS_DATA *)command_data)->app_msg_id.id;
				switch(((REQ_ADD_FULL_MSG_DATA *)command_data)->app_msg_id.type){
					case M_INT_MSG_SYNCH:
						message_app_id.type = M_FTT_MANAGEMENT_SM;
						break;
					case M_INT_MSG_ASYNCH_H:
						message_app_id.type = M_FTT_MANAGEMENT_AM;
						gen_req.m_timeliness = M_FTT_MANAGEMENT_HARD_RT;
						break;
					case M_INT_MSG_ASYNCH_S:
						message_app_id.type = M_FTT_MANAGEMENT_AM;
						gen_req.m_timeliness = M_FTT_MANAGEMENT_SOFT_RT;
						break;
					case M_INT_MSG_ASYNCH_U:
					default:
						message_app_id.type = M_FTT_MANAGEMENT_AM;
						gen_req.m_timeliness = M_FTT_MANAGEMENT_UNCONSTRAINED;
						break;
				}
#ifdef MASTER_PRINT_INTERFACE_REQUESTS
				PRINT_MSG("i: REQ_ADDMODIFY_LOAD_REQS %s  id:%x  [C=%d, T=%d]",
						  (message_app_id.type == M_FTT_MANAGEMENT_SM)?" SYNC":" ASYNC",
						  message_app_id.id,
						  ((REQ_ADDMODIFY_LOAD_REQS_DATA *)command_data)->mesg_size,
						  ((REQ_ADDMODIFY_LOAD_REQS_DATA *)command_data)->mesg_period );
#endif

				gen_req.m_size = ((REQ_ADDMODIFY_LOAD_REQS_DATA *)command_data)->mesg_size;
				gen_req.m_max_size = ((REQ_ADDMODIFY_LOAD_REQS_DATA *)command_data)->mesg_max_size;
				gen_req.m_max_prod_no = ((REQ_ADDMODIFY_LOAD_REQS_DATA *)command_data)->mesg_max_prod_no;
				gen_req.m_period = ((REQ_ADDMODIFY_LOAD_REQS_DATA *)command_data)->mesg_period;
				gen_req.m_deadline = ((REQ_ADDMODIFY_LOAD_REQS_DATA *)command_data)->mesg_period;
				gen_req.m_init = -1;
				gen_req.m_MTU = ((REQ_ADDMODIFY_LOAD_REQS_DATA *)command_data)->mesg_MTU;

				*return_code = M_FTT_MANAGEMENT_L_add_msg_load(
						message_app_id,
						&gen_req );
 			}
 			break;
 		case REQ_ADDMODIFY_QOS_REQS:
 			{
				M_FTT_MANAGEMENT_L_APP_ID message_app_id;
				M_FTT_MANAGEMENT_L_QOS_REQUIREMENTS qos_req;
				message_app_id.id = ((REQ_ADDMODIFY_QOS_REQS_DATA *)command_data)->app_msg_id.id;
				message_app_id.type = (((REQ_ADDMODIFY_QOS_REQS_DATA *)command_data)->app_msg_id.type == M_INT_MSG_SYNCH) ? M_FTT_MANAGEMENT_SM: M_FTT_MANAGEMENT_AM;
#ifdef MASTER_PRINT_INTERFACE_REQUESTS
				PRINT_MSG("i: REQ_ADDMODIFY_QOS_REQS %s  id:%x", (message_app_id.type == M_FTT_MANAGEMENT_SM)?" SYNC":" ASYNC",message_app_id.id);

#endif
				//PRINT_MSG(" size: %d", ((REQ_ADDMODIFY_QOS_REQS_DATA *)command_data)->mesg_size);

				qos_req.qos_pairs_no = 1;
				qos_req.qos_pairs[0].size = ((REQ_ADDMODIFY_QOS_REQS_DATA *)command_data)->mesg_size;
				qos_req.qos_pairs[0].period = ((REQ_ADDMODIFY_QOS_REQS_DATA *)command_data)->mesg_period;
				qos_req.qos_share = ((REQ_ADDMODIFY_QOS_REQS_DATA *)command_data)->priority;
				*return_code = M_FTT_MANAGEMENT_L_add_msg_qos (
						message_app_id,
						&qos_req );
 			}
			break;

 		case REQ_REMOVE_LOAD_REQS:
			{
				M_FTT_MANAGEMENT_L_APP_ID message_app_id;
				
				message_app_id.id = ((REQ_REMOVE_LOAD_REQS_DATA *)command_data)->app_msg_id.id;
				switch(((REQ_ADD_FULL_MSG_DATA *)command_data)->app_msg_id.type){
					case M_INT_MSG_SYNCH: message_app_id.type = M_FTT_MANAGEMENT_SM; break;
					case M_INT_MSG_ASYNCH_H:
					case M_INT_MSG_ASYNCH_S:
					case M_INT_MSG_ASYNCH_U:
					default: message_app_id.type = M_FTT_MANAGEMENT_AM; break;
				}
#ifdef MASTER_PRINT_INTERFACE_REQUESTS
				PRINT_MSG("i: REQ_REMOVE_LOAD_REQS %s  id:%x", (message_app_id.type == M_FTT_MANAGEMENT_SM)?" SYNC":" ASYNC",message_app_id.id);
#endif
				
				*return_code = M_FTT_MANAGEMENT_L_del_msg( message_app_id );
				//I should be deleting just the load parameter. But since the connecting slaves have to re-attach again... 
				//they will have to send the attach request, which they are already sending!
			}
 			break;

		case REQ_ADD_QOS_PAIR_REQS:
 			{
				M_FTT_MANAGEMENT_L_APP_ID message_app_id;
				M_FTT_MANAGEMENT_L_QOS_REQUIREMENTS qos_req;
				message_app_id.id = ((REQ_ADD_QOS_PAIR_REQS_DATA *)command_data)->app_msg_id.id;
				message_app_id.type = (((REQ_ADD_QOS_PAIR_REQS_DATA *)command_data)->app_msg_id.type == M_INT_MSG_SYNCH) ? M_FTT_MANAGEMENT_SM: M_FTT_MANAGEMENT_AM;
#ifdef MASTER_PRINT_INTERFACE_REQUESTS
				PRINT_MSG("i: REQ_ADD_QOS_PAIR_REQS %s  id:%x", (message_app_id.type == M_FTT_MANAGEMENT_SM)?" SYNC":" ASYNC",message_app_id.id);
#endif


				qos_req.qos_pairs_no = 0; //ADD on pair... it fails if qos is not already created
				qos_req.qos_pairs[0].size = ((REQ_ADD_QOS_PAIR_REQS_DATA *)command_data)->mesg_size;
				qos_req.qos_pairs[0].period = ((REQ_ADD_QOS_PAIR_REQS_DATA *)command_data)->mesg_period;
				*return_code = M_FTT_MANAGEMENT_L_add_msg_qos (
						message_app_id,
						&qos_req );
 			}
			break;

		case REQ_ADD_QOS_PRIO_REQS:
 			{
				M_FTT_MANAGEMENT_L_APP_ID message_app_id;
				message_app_id.id = ((REQ_ADD_QOS_PRIO_REQS_DATA *)command_data)->app_msg_id.id;
				message_app_id.type = (((REQ_ADD_QOS_PRIO_REQS_DATA *)command_data)->app_msg_id.type == M_INT_MSG_SYNCH) ? M_FTT_MANAGEMENT_SM: M_FTT_MANAGEMENT_AM;
#ifdef MASTER_PRINT_INTERFACE_REQUESTS
				PRINT_MSG("i: REQ_ADD_QOS_PRIO_REQS %s  id:%x  prio:%d", (message_app_id.type == M_FTT_MANAGEMENT_SM)?" SYNC":" ASYNC",message_app_id.id, ((REQ_ADD_QOS_PRIO_REQS_DATA *)command_data)->priority);
#endif

				*return_code = M_FTT_MANAGEMENT_L_chg_qos_prio(
						message_app_id,
						((REQ_ADD_QOS_PRIO_REQS_DATA *)command_data)->priority );
 			}
			break;

 		case REQ_ADD_ONE_END_CONECTION:
			{
				M_FTT_MANAGEMENT_L_APP_ID message_app_id;
				unsigned short temp_id;
				unsigned char direction;

				message_app_id.id = ((REQ_ADD_ONE_END_CONECTION_DATA *)command_data)->app_msg_id.id;
				message_app_id.type = (((REQ_ADD_ONE_END_CONECTION_DATA *)command_data)->app_msg_id.type == M_INT_MSG_SYNCH) ? M_FTT_MANAGEMENT_SM: M_FTT_MANAGEMENT_AM;
#ifdef MASTER_PRINT_INTERFACE_REQUESTS
				PRINT_MSG("i: REQ_ADD_ONE_END_CONECTION %s  id:%x", (message_app_id.type == M_FTT_MANAGEMENT_SM)?" SYNC":" ASYNC",message_app_id.id);
#endif
				if ( M_int_nodes_db_Get_nodeID( ((REQ_ADD_ONE_END_CONECTION_DATA *)command_data)->mesg_prodconsID, &(temp_id) ) < 0 ){
					ERROR_MSG("Couldn't find a nodeID for this");
					*return_code = -10;
					goto end_of_while2789563478563485;
				}

				direction = ((REQ_ADD_ONE_END_CONECTION_DATA *)command_data)->direction;
				*return_code = M_FTT_MANAGEMENT_L_add_msg_connection( message_app_id, temp_id, direction );
			}
			break;

 		case REQ_DEL_ONE_END_CONECTION:
			{
				M_FTT_MANAGEMENT_L_APP_ID message_app_id;
				unsigned short temp_id;

				message_app_id.id = ((REQ_DEL_ONE_END_CONECTION_DATA *)command_data)->app_msg_id.id;
				message_app_id.type = (((REQ_DEL_ONE_END_CONECTION_DATA *)command_data)->app_msg_id.type == M_INT_MSG_SYNCH) ? M_FTT_MANAGEMENT_SM: M_FTT_MANAGEMENT_AM;
#ifdef MASTER_PRINT_INTERFACE_REQUESTS
				PRINT_MSG("i: REQ_DEL_ONE_END_CONECTION %s  id:%x", (message_app_id.type == M_FTT_MANAGEMENT_SM)?" SYNC":" ASYNC",message_app_id.id);
#endif
				if ( M_int_nodes_db_Get_nodeID( ((REQ_DEL_ONE_END_CONECTION_DATA *)command_data)->mesg_prodconsID, &(temp_id) ) < 0 ){
					ERROR_MSG("Couldn't find a nodeID for this");
					*return_code = -10;
					goto end_of_while2789563478563485;
				}

				*return_code = M_FTT_MANAGEMENT_L_del_msg_connection( message_app_id, temp_id );
			}
			break;
			
		case REQ_GET_FTT_ID_DETAILS:
		{
			M_FTT_MANAGEMENT_L_APP_ID message_app_id;
			
			message_app_id.id = ((REQ_GET_FTT_ID_DETAILS_DATA *)command_data)->app_msg_id.id;
			message_app_id.type = (((REQ_GET_FTT_ID_DETAILS_DATA *)command_data)->app_msg_id.type == M_INT_MSG_SYNCH) ? M_FTT_MANAGEMENT_SM: M_FTT_MANAGEMENT_AM;
#ifdef MASTER_PRINT_INTERFACE_REQUESTS
			PRINT_MSG("i: REQ_GET_FTT_ID_DETAILS %s  id:%x", (message_app_id.type == M_FTT_MANAGEMENT_SM)?" SYNC":" ASYNC",message_app_id.id);
#endif
			
			*return_code = M_FTT_MANAGEMENT_L_get_msg_ftt_id( message_app_id );
		}
			break;
			
			
 		case REQ_CHG_PERIOD:
			{
				M_FTT_MANAGEMENT_L_APP_ID message_app_id;

				message_app_id.id = ((REQ_CHG_PERIOD_DATA *)command_data)->app_msg_id.id;
				message_app_id.type = (((REQ_CHG_PERIOD_DATA *)command_data)->app_msg_id.type == M_INT_MSG_SYNCH) ? M_FTT_MANAGEMENT_SM: M_FTT_MANAGEMENT_AM;
#ifdef MASTER_PRINT_INTERFACE_REQUESTS
				PRINT_MSG("i: REQ_CHG_PERIOD %s  id:%x", (message_app_id.type == M_FTT_MANAGEMENT_SM)?" SYNC":" ASYNC",message_app_id.id);
#endif

				*return_code = M_FTT_MANAGEMENT_L_chg_period( message_app_id, ((REQ_CHG_PERIOD_DATA *)command_data)->mesg_period );
			}
			break;
			
		case REQ_REMOTE_REPORT:
		{
			unsigned short temp_id;
			M_FTT_MANAGEMENT_L_APP_ID message_app_id;

			
#ifdef MASTER_PRINT_INTERFACE_REQUESTS
			PRINT_MSG("i: REQ_REMOTE_REPORT node:%d", nodeID_source);
#endif
			temp_id = nodeID_source;
		
			message_app_id.type = M_FTT_MANAGEMENT_AM;
			message_app_id.id = BROADCAST_REP_ID;

			*return_code = M_FTT_MANAGEMENT_L_add_msg_connection( message_app_id, temp_id,  1 /*consumer*/ );

		}
			break;			

		default:
 			ERROR_MSG("Command issued not implemented (at least for now). Check in REQ_COMMANDS for the code %d", command_type );
			return -1;
 	}
end_of_while2789563478563485:
	/* return_code holds the command return from the issued command
	 * Notice that only when this function returns 0, return_code will be touched */
	return 0;
}

/* This function prepares to enqueue the command
 * -It determines the UniversalID
 * -Enqueue the command whith the U_ID
 * -Determines if it is a End-of-group command to dequeue all the
 *  commands and send them to the lower layer
 *
 *  It returns:
 *   <0 on errors
 *   0  on command queued
 *   1  on REQ_SUBMIT success: return_code has the last command result
 *   4  on REQ_SUBMIT some command failure
 *   2  on REQ_SUBMIT final submission failure
 *   3  on REQ_SUBMIT admission failure
 */
signed char M_int_req_queuing_Submit_req(
		unsigned char nodeID_source,
		unsigned short app_asynch_id,
		REQ_COMMANDS command_type,
		unsigned char *command_data,
		signed int *return_code )
{
	int ret;
	signed char fun_ret;
	unsigned int universal_source_id;
	unsigned short command_size;

	M_INT_MSG_TYPE g_command_type;
	unsigned char g_command_data[300];


	/* Determine the universal tag for this source */
	/* -the universal tag is to group the commands according to their source when 
	 * making group commands */
	universal_source_id = (((unsigned int)nodeID_source)<<16) + app_asynch_id;

	command_size = GET_CMD_DATA_SIZE(command_type);
	
	if (command_size>300){
	       ERROR_MSG("Design error - command_size allocated too short ");
      	       return -1;
	}

	/* Enter mutex */
	RQ_QUEUE_Enter;


	if (command_type != REQ_SUBMIT){
		/* enqueue it */
		//PRINT_MSG("Appending to req queue with UI: %d command type: %d",universal_source_id, command_type);
		ret = M_int_req_queue_Append_cmd( universal_source_id, command_type, command_data, command_size);
		fun_ret = 0;
	}else{
		/* We have received the REQ_SUBMIT. Obviously we will not queue it to be delivered in the FTT_MANAGENT.
		 * Instead, let's submit the command(s) already queued
		*/

#ifdef MASTER_PRINT_INTERFACE_REQUESTS
//		PRINT_MSG("i: M_int_req_queuing_Submit_req");
#endif
		/* Lock the Management layer */
		M_FTT_MANAGEMENT_L_changes_lock();
		//PRINT_MSG("Getting commands from req queue with UI:%d",universal_source_id);
		while(  M_int_req_queue_Get_cmd( universal_source_id, &g_command_type, g_command_data) >= 0  ){
			/* Send it to the Management layer */
			if ( M_int_req_queuing_Submit_cmd( nodeID_source, g_command_type, g_command_data, &ret ) < 0 ){
				/* Oops bad command, or something else: ret isn't touched */
			}else{
			//PRINT_MSG("M_int_req_queuing_Submit_cmd went ok ret: %d", ret);
				if (ret < 0){
					/* Oops, there is a command in trouble. I can't do much to help */
					ERROR_MSG(" Command %d submission into ManagementLayer ret: %d \nAborting all others", g_command_type, ret);
					M_int_req_queue_Abort_cmds( universal_source_id );
					M_FTT_MANAGEMENT_L_changes_unlock_aborting();
					RQ_QUEUE_Leave;
					return 4;
				}
			}
		}

	//	PRINT_MSG("Stopped getting commands returned with %d",ret);
	
		/* Let's pass the admission control before submitting */
		if ( M_FTT_MANAGEMENT_L_changes_Admission_Control() < 0 ){
			RQ_QUEUE_Leave;
			return 3;
		}
	
		if ( M_FTT_MANAGEMENT_L_changes_unlock_submiting() < 0 ){
			fun_ret = 2; /* error */
		}else{
			*return_code = ret;
			fun_ret = 1;
		}
		
//		{
//		static int qq=0;
//
//		PRINT_MSG("Successfully changed the management layer:%d", qq);
//		M_FTT_MANAGEMENT_L_print();
//		qq++;
//		}

	}

	RQ_QUEUE_Leave;

	return fun_ret;
}
