/*****************************************************************************
 * S_ftt-interface.layer.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/


#include "Ports.h"
#include "S_int_async_ch.h"
#include "S_int_req_dispatch.h"
#include "S_int_bind_table.h"



#ifdef PLUGNPLAY_REPORT
#ifdef FTT_SLAVE_COMPILE
#include "S_int_app_remote_report.h"
#else
#include "M_int_app_report.h"
#endif
#endif

#include "ftt-interface.layer_types.h"
#include "S_ftt-interface.layer.h"

static unsigned char S_FTT_INTERFACE_L_my_app_node_id;

static unsigned char S_FTT_INTERFACE_L_var_in_atomic_change;


signed char S_FTT_INTERFACE_L_init(unsigned char node_app_id)
{

#ifdef SLAVE_POLITE
	PRINT_MSG("Initializing the Interface layer - Slave");
#endif

	S_FTT_INTERFACE_L_my_app_node_id = node_app_id;
	S_FTT_INTERFACE_L_var_in_atomic_change = 0;
#ifdef FTT_SLAVE_COMPILE
	/* If the app_id is 0, means to do not use the app_id */
	S_int_async_ch_Init(node_app_id);
	/* when S_int_async_ch_Init succeds doesn't mean that thee buffers are
	 * created - they will be eventually */
#endif
	S_int_req_dispatch_Init();
	S_int_bind_table_Init();
	
#ifdef PLUGNPLAY_REPORT
#ifdef FTT_SLAVE_COMPILE
	S_int_app_remote_report_init();
#endif
#endif

	return 0;
}

signed char S_FTT_INTERFACE_L_close(void)
{
#ifdef SLAVE_POLITE
	PRINT_MSG("Closing the Interface layer - Slave");
#endif
	
#ifdef PLUGNPLAY_REPORT
#ifdef FTT_SLAVE_COMPILE
	S_int_app_remote_report_close();
#endif
#endif

	S_int_bind_table_Close();

	S_int_req_dispatch_Close();
#ifdef FTT_SLAVE_COMPILE
	S_int_async_ch_Close();
#endif

	return 0;
}


signed int S_FTT_INTERFACE_L_init_block(unsigned int *ret_ec_len_us)
{
#ifdef FTT_SLAVE_COMPILE

	/* Block until the interface layer is fully initialized
	 *  - notice that after S_FTT_INTERFACE_L_init the system might still not be fully initialized */
	S_FTT_INTERFACE_L_my_app_node_id = 	S_int_Block_on_initialization(ret_ec_len_us);
	/*I'm also updating the app_node_id in case the initialization was carried out with 0 */
	return S_FTT_INTERFACE_L_my_app_node_id;
#else
	if (ret_ec_len_us) *ret_ec_len_us = 0;
	/* For now, I don't care about the Master*/
	return 0;
#endif

}


static signed char S_FTT_INTERFACE_L_attach( unsigned char tx_rx, M_INT_APP_MSG_ID app_id, unsigned char wait4var_f )
{
	int ret;
	unsigned short ftt_id;
	unsigned char app_node_id;

	app_node_id = S_FTT_INTERFACE_L_my_app_node_id;

	/* Check if this var is already bound */
	if ( S_int_bind_table_Isthere( app_id, NULL, NULL ) ){
		ERROR_MSG (" This app_id is already listed in the interface ftt_id/signal table");
		return -1;
	}

	/* Let's then create the message. But first:
	 *   - Send a request to add my (nodeID)
	 *   - This request should return, if OK, the ftt_id as well as the type */
	ret = S_int_req_dispatch_CmdAddconnection( &app_id, &app_node_id, (tx_rx)?1 /*cons*/: 0/*prod*/ );
	if ( ret < 0 ){
		/* Problems to get the correspondent ftt_id */
		ERROR_MSG("Couldn't attach this connection in the master. S_int_req_Send_cmd_add_connection ret: %d", ret);
		return -2;
	}
	ftt_id = ret;
	PRINT_MSG("S_int_req_dispatch_CmdAddconnection returned from master: app_id %x->ftt_id %x", *((unsigned int *)&app_id), ftt_id);

	if (wait4var_f)
		if ( (ret = S_FTT_CORE_L_Wait4var( (app_id.type == M_INT_MSG_SYNCH) ? SYNC_MESSAGE : ASYNC_MESSAGE, ftt_id)) < 0 ){
			ERROR_MSG("Problems with S_FTT_CORE_L_Wait4var ret: %d", ret);
			return -3;
		}
	
	//PRINT_MSG("i: Mapping app_id: %x to ftt_id %x", app_id.id, ftt_id );
	
	/* write the ftt_id to table */
	if ( S_int_bind_table_Add_ftt_id( app_id, ftt_id ) < 0 )
		return -4;

	return 0;
}


signed char S_FTT_INTERFACE_L_attach_rx( M_INT_APP_MSG_ID app_id  )
{
	return S_FTT_INTERFACE_L_attach( 1/*rx*/, app_id, 1 /*wait4var_f*/);
}
signed char S_FTT_INTERFACE_L_attach_tx( M_INT_APP_MSG_ID app_id )
{
	return S_FTT_INTERFACE_L_attach( 0/*tx*/, app_id, 1 /*wait4var_f*/);
}

signed char S_FTT_INTERFACE_L_attach_details( M_INT_APP_MSG_ID app_id )
{
	int ret;
	unsigned short ftt_id;
//	unsigned char app_node_id;
	
//	app_node_id = S_FTT_INTERFACE_L_my_app_node_id;
	
	/* Check if this var is already bound */
	if ( S_int_bind_table_Isthere( app_id, NULL, NULL ) ){
		ERROR_MSG (" This app_id is already listed in the interface ftt_id/signal table");
		return -1;
	}
	
	/* Let's then create the message. But first:
	 *   - Send a request to add my (nodeID)
	 *   - This request should return, if OK, the ftt_id as well as the type */
	ret = S_int_req_dispatch_CmdDetailConnection( &app_id );
	if ( ret < 0 ){
		/* Problems to get the correspondent ftt_id */
		/* Probably this Var is not present in the Master */
		//ERROR_MSG("Couldn't attach_detail this connection in the master. S_int_req_dispatch_CmdUpdateConnection ret: %d", ret);
		return -2;
	}
	ftt_id = ret;
	//PRINT_MSG("S_int_req_dispatch_CmdUpdateConnection returned from master: app_id %x->ftt_id %x", *((unsigned int *)&app_id), ftt_id);
	
	//PRINT_MSG("i: Mapping app_id: %x to ftt_id %x", app_id.id, ftt_id );
	
	/* write the ftt_id to table */
	if ( S_int_bind_table_Add_ftt_id( app_id, ftt_id ) < 0 )
		return -4;
	
	return 0;
}


signed char S_FTT_INTERFACE_L_dettach( M_INT_APP_MSG_ID app_id )
{
	int ret;
	unsigned char app_node_id;
	unsigned short ftt_id;

	
	app_node_id = S_FTT_INTERFACE_L_my_app_node_id;
	
	if ( !S_int_bind_table_Isthere( app_id, &ftt_id, NULL ) ){ /* Message is not here */
		return -1;
	}
	
	/* First, unbind, just in case! */
	S_FTT_INTERFACE_L_unbind( app_id); /* It already unbinds from the table */


	/* Let's then dettach in the Master:
	 *   - Send a request to add my (nodeID)
	 *   - This request should return, if OK */
	ret = S_int_req_dispatch_CmdDelconnection( &app_id, &app_node_id );
	if ( ret < 0 ){
		/* Problems to get the correspondent ftt_id */
		ERROR_MSG("Couldn't dettach this var in the master. S_int_req_dispatch_CmdDelconnection ret: %d", ret);
		return -2;
	}
	
	if ( (ret = S_FTT_CORE_L_Wait4notvar( (app_id.type == M_INT_MSG_SYNCH) ? SYNC_MESSAGE : ASYNC_MESSAGE, ftt_id)) < 0 ){
		ERROR_MSG("Problems with S_FTT_CORE_L_Wait4notvar ret: %d", ret);
		return -3;
	}
	
	/* Delete from the table */
	S_int_bind_table_Del( app_id );
	
	return 0;
}

signed char S_FTT_INTERFACE_L_bind( M_INT_APP_MSG_ID app_id, void *mem, unsigned int mem_size, unsigned char slots_no )
{
	int ret;
	unsigned short ftt_id;
	FTT_SIGNAL *sig;


	/* Check if this var is in the message table and is bound */
	ret = S_int_bind_table_Isthere( app_id, &ftt_id, NULL );
	if (ret == 2) {
		ERROR_MSG (" Already bound ");
		return -1;	
	}
	if (ret == 0) {
		/* Not in the message table */
		return -2;
	}
	
	/* Let's then bind the message */
	ret = S_FTT_CORE_L_bind_mesg( 
			(app_id.type == M_INT_MSG_SYNCH) ? SYNC_MESSAGE : ASYNC_MESSAGE,
		       	ftt_id,
		       	&sig,
		       	mem, mem_size, slots_no);
	if ( ret < 0 ){
		ERROR_MSG("S_FTT_CORE_L_bind_mesg ret: %d", ret);
		return -3;
	}

	/* Finally we can update our bind table */
	if ( S_int_bind_table_Add_signal( app_id, sig ) < 0 )
		return -4;

	return 0;

}

signed char S_FTT_INTERFACE_L_unbind( M_INT_APP_MSG_ID app_id)
{
	int ret;
	unsigned short ftt_id;

	
	if ( !S_int_bind_table_Isthere( app_id, &ftt_id, NULL ) ){
		ERROR_MSG (" This Message is not attached here ");
		return -1;
	}
	
	/* I could have checked if the message is bound, but (just in case) I'll
	 * remove it from the table and unbind in FTT-core */
	
	/* Finally we can update our bind table */
	S_int_bind_table_Del_signal( app_id );
	
	if ( (ret = S_FTT_CORE_L_unbind_mesg( (app_id.type == M_INT_MSG_SYNCH) ? SYNC_MESSAGE : ASYNC_MESSAGE, ftt_id )) < 0 ){
		ERROR_MSG("S_FTT_CORE_L_unbind_mesg ret: %d", ret);
//		return -2;
	}
	
	return 0;	
}

signed char S_FTT_INTERFACE_L_isbind( M_INT_APP_MSG_ID app_id )
{
	unsigned short ftt_id;
	
	if ( !S_int_bind_table_Isthere( app_id, &ftt_id, NULL ) ){
		//ERROR_MSG (" This Message is not attached here ");
		return 0;
	}
	
	return S_FTT_CORE_L_isbind_mesg( (app_id.type == M_INT_MSG_SYNCH) ? SYNC_MESSAGE : ASYNC_MESSAGE, ftt_id);
}

signed short S_FTT_INTERFACE_L_var_details( M_INT_APP_MSG_ID app_id, unsigned int *current_var_size, unsigned short *current_var_period )
{
	int ret;
	unsigned short ftt_id;
	unsigned char tag;

	/* Check if this var is already attached  */
	if ( !S_int_bind_table_Isthere( app_id, &ftt_id, NULL ) ){
		ERROR_MSG (" Not in the table");
		return -1;
	}

	if ( (ret = S_FTT_CORE_L_get_current(
			(app_id.type == M_INT_MSG_SYNCH) ? SYNC_MESSAGE : ASYNC_MESSAGE,
			ftt_id,
			&tag,
			current_var_size,
			current_var_period ) ) < 0 ){
		return -2;
	}

	return tag;
}

signed char S_FTT_INTERFACE_L_pre_send( M_INT_APP_MSG_ID app_id, unsigned char block_on_reservation, void **mem)
{
	unsigned short ftt_id;
	signed char ret;

	/* Check if this var is already bound */
	if ( S_int_bind_table_Isthere( app_id, &ftt_id, NULL ) != 2 ){
		ERROR_MSG (" Var not bound at the interface bind table.");
		return -1;
	}

	if ( mem == NULL )
		return -2;

	if (app_id.type == M_INT_MSG_SYNCH) ret = S_FTT_CORE_pre_produce( ftt_id, (unsigned char **)mem );
	else ret = S_FTT_CORE_pre_send( ftt_id, block_on_reservation, (unsigned char **)mem );
	
	if (ret == -7) return -3; // FIFO/CAB full
	if (ret == -10) return -4; // Semaphore was destroyed while waiting.
	if (ret == -3) return -1;

	return 0;
}

signed char S_FTT_INTERFACE_L_pre_send_abort( M_INT_APP_MSG_ID app_id )
{
	unsigned short ftt_id;

	/* Check if this var is already bound */
	if ( S_int_bind_table_Isthere( app_id, &ftt_id, NULL /*signal*/ ) != 2 ){
		ERROR_MSG (" Var not bound at the interface bind table.");
		return -1;
	}

	if (app_id.type == M_INT_MSG_SYNCH){
		if ( S_FTT_CORE_pre_produce_abort( ftt_id ) < 0 )
			return -1;
	}else{
		if ( S_FTT_CORE_pre_send_abort( ftt_id ) < 0 )
			return -1;
	}

	return 0;
}

signed char S_FTT_INTERFACE_L_send ( M_INT_APP_MSG_ID app_id, unsigned char tag, void *mem, unsigned char block_on_reservation, unsigned char block_on_tx)
{
	unsigned short ftt_id;
	FTT_SIGNAL *sig;
	int ret;


	/* Check if this var is already bound */
	if ( S_int_bind_table_Isthere( app_id, &ftt_id, &sig ) != 2 ){
		ERROR_MSG (" Var not bound at the interface bind table.");
		return -1;
	}
	if (app_id.type == M_INT_MSG_SYNCH)
		ret = S_FTT_CORE_produce( ftt_id, tag, mem );
	else
		ret = S_FTT_CORE_send( ftt_id, tag, mem, block_on_reservation );

	if (ret < 0) {
		if ( ret == -5 ) return -2; // deprecated tag
		if ( ret == -7 ) return -3; // FIFO/CAB full
		if ( ret == -10) return -4; // Semaphore was destroyed while waiting.
		if ( ret == -3 ) return -1;
		return -5;
	}

	/* If block */
	if (block_on_tx) FTT_wait4signal( sig );

	return 0;
}

signed char S_FTT_INTERFACE_L_pre_receive ( M_INT_APP_MSG_ID app_id, unsigned char *ret_tag, void **mem, RX_FLAGS flags )
{
	unsigned short ftt_id;
	signed char status;
	FTT_SIGNAL *sig;

	/* Check if this var is already bound */
	if ( S_int_bind_table_Isthere( app_id, &ftt_id, &sig ) != 2 ){
		ERROR_MSG (" Var not bound at the interface bind table.");
		return -1;
	}

	if ( mem == NULL ) return -2;

	if (app_id.type == M_INT_MSG_SYNCH){
		while(1){
			if ( (status = S_FTT_CORE_consume_status (ftt_id)) < 0 )
				return -3;

			if (flags == RX_NO_BLOCK) break;
			if ( (flags == RX_BLOCK_NEW_RX) && (status != 0) ) break;
			if ( (flags == RX_BLOCK_NEW_TX) && (status == 2) ) break;

			/* Let's block waiting for the next rx */
			//PRINT_MSG("Blocking for the next Receiced message of SM %x",ftt_id );
			if ( FTT_wait4signal( sig ) < 0 ){
				ERROR_MSG("The signal was destroyed while waitting for it");
				S_int_bind_table_Del_signal( app_id );
				return -10;
			}
		}

		if ( (*mem = S_FTT_CORE_pre_consume( ftt_id, ret_tag, NULL )) == NULL )
			return -4;
	}else{
		while(1){
			if ( (status = S_FTT_CORE_receive_status (ftt_id)) < 0 )
				return -3;

			if (flags == RX_NO_BLOCK) break;
			if ( (flags == RX_BLOCK_NEW_RX) && (status != 0) ) break;
			if ( (flags == RX_BLOCK_NEW_TX) && (status != 0) ) break;

			/* Let's block waiting for the nex rx */
			//PRINT_MSG("Blocking for the next Receiced message of AM %x",ftt_id );
			if ( FTT_wait4signal( sig ) < 0 ){
				ERROR_MSG("The signal was destroyed while waitting for it");
				S_int_bind_table_Del_signal( app_id );
				return -10;
			}

			//PRINT_MSG("Passed");
		}

		if ( (*mem = S_FTT_CORE_pre_receive( ftt_id, ret_tag, NULL )) == NULL )
			return -5;
	}
	return 0;


}

signed char S_FTT_INTERFACE_L_receive( M_INT_APP_MSG_ID app_id, unsigned char *ret_tag, void *mem, RX_FLAGS flags )
{
	unsigned short ftt_id;
	unsigned char status;
	FTT_SIGNAL *sig;

	/* Check if this var is already bound */
	if ( S_int_bind_table_Isthere( app_id, &ftt_id, &sig ) != 2 ){
		ERROR_MSG (" Var not bound at the interface bind table.");
		return -1;
	}


	if (app_id.type == M_INT_MSG_SYNCH){
		while(1){

			if ( S_FTT_CORE_consume( ftt_id, ret_tag, mem, &status ) < 0 )
				return -3;

			if (flags == RX_NO_BLOCK) break;
			if ( (flags == RX_BLOCK_NEW_RX) && (status != 0) ) break;
			if ( (flags == RX_BLOCK_NEW_TX) && (status == 2) ) break;

			/* Let's block waiting for the next rx */
			PRINT_MSG("Blocking for the next Receiced message of SM %x",ftt_id );
			if ( FTT_wait4signal( sig ) < 0 ){
				PRINT_MSG("The signal was destroyed while waitting for it");
				S_int_bind_table_Del_signal( app_id );
				return -10;
			}

		}
	}else{
		while(1){

			if ( S_FTT_CORE_receive( ftt_id, ret_tag, mem, &status ) < 0 )
				return -3;

			if (flags == RX_NO_BLOCK) break;
			if ( (flags == RX_BLOCK_NEW_RX) && (status != 0) ) break;
			if ( (flags == RX_BLOCK_NEW_TX) && (status == 0) ) break;

			/* Let's block waiting for the next rx */
//			PRINT_MSG("Blocking for the next Receiced message of AM %x",ftt_id );
			if ( FTT_wait4signal( sig ) < 0 ){
				PRINT_MSG("The signal was destroyed while waitting for it");
				S_int_bind_table_Del_signal( app_id );
				return -10;
			}

		}
	}
	return 0;
}

signed char S_FTT_INTERFACE_L_block ( M_INT_APP_MSG_ID app_id )
{
	unsigned short ftt_id;
	FTT_SIGNAL *sig;


	/* Check if this var is already bound */
	if ( S_int_bind_table_Isthere( app_id, &ftt_id, &sig ) != 2 ){
		ERROR_MSG (" Var not bound at the interface bind table.");
		return -1;
	}

	/* If block */
	FTT_wait4signal( sig );

	return 0;
}


signed char S_FTT_INTERFACE_L_var_Atomic_change_Start( void )
{
	int ret;
	
	if (S_FTT_INTERFACE_L_var_in_atomic_change){
		ERROR_MSG("Already in the middle of an atomic change request");
		return -1;
	}

	S_FTT_INTERFACE_L_var_in_atomic_change = 1;
	if ( (ret = S_int_req_dispatch_GroupPrepare()) < 0 ){
		ERROR_MSG("S_int_req_dispatch_GroupPrepare ret: %d", ret);
		S_FTT_INTERFACE_L_var_in_atomic_change = 0;
		return -2;
	}
	
	return 0;
}

signed char S_FTT_INTERFACE_L_var_Atomic_change_Stop( void )
{
	int ret;
	
	if (!S_FTT_INTERFACE_L_var_in_atomic_change){
		ERROR_MSG("Not in an atomic change request");
		return -1;
	}
	
	if ( (ret = S_int_req_dispatch_GroupSubmit()) < 0 ){
		ERROR_MSG("S_int_req_dispatch_GroupSubmit ret: %d", ret);
		return -2;
	}	
	
	S_FTT_INTERFACE_L_var_in_atomic_change = 0;
	return 0;
}

signed char S_FTT_INTERFACE_L_var_add_load ( 
		M_INT_APP_MSG_ID id,
		unsigned int size,
		unsigned int max_size,
		unsigned char max_prod_no,
		unsigned short MTU,
		unsigned short period,
		unsigned short relative_offset /* Not used - Here for API backward compatibility */
		)
{
	int ret;

	if ( (ret = S_int_req_dispatch_CmdAddLoad( 
					&id,
					&size,
					&max_size,
					&max_prod_no,
					&period,
					&MTU )) < 0){
	       ERROR_MSG("S_int_req_Send_cmd_addLoad returned: %d ", ret);
	       return -2;
	}

	return 0;
}

signed char S_FTT_INTERFACE_L_var_rm_load ( M_INT_APP_MSG_ID id	)
{
	int ret;
	
	//	PRINT_MSG("Going for S_int_req_dispatch_CmdAddLoad");
	if ( (ret = S_int_req_dispatch_CmdRemoveLoad(  &id )) < 0){
		ERROR_MSG("S_int_req_dispatch_CmdRemoveLoad returned: %d ", ret);
		return -2;
	}
	
	return 0;
}

signed char S_FTT_INTERFACE_L_var_add_full ( 
		M_INT_APP_MSG_ID id,
		unsigned int size,
		unsigned int max_size,
		unsigned char max_prod_no,
		unsigned short MTU,
		unsigned short period,
		unsigned char sender_ftt_app_id,
		unsigned char receiver_ftt_app_id
		)
{
	int ret;
	
	if ( (ret = S_int_req_dispatch_CmdAdd( 
										  &id,
										  &size,
										  &max_size,
										  &max_prod_no,
										  &period,
										  &MTU,
										  1, /*1 - producer*/
										  &sender_ftt_app_id,
										  1, /*1 - consumer*/
										  &receiver_ftt_app_id
										  )) < 0){
		ERROR_MSG("S_int_req_Send_cmd_addLoad returned: %d ", ret);
		return -2;
	}
	
	return 0;
}

signed char S_FTT_INTERFACE_L_var_rm_full ( M_INT_APP_MSG_ID id	)
{
	int ret;
	
	//	PRINT_MSG("Going for S_int_req_dispatch_CmdDel");
	if ( (ret = S_int_req_dispatch_CmdDel(  &id )) < 0){
		ERROR_MSG("S_int_req_dispatch_CmdDel returned: %d ", ret);
		return -2;
	}
	
	return 0;
}

signed char S_FTT_INTERFACE_L_var_req_change_period ( M_INT_APP_MSG_ID id, unsigned short period )
{
	int ret;

	if ( (ret = S_int_req_dispatch_CmdChangePeriod( 
					&id,
					&period )	) < 0){
	       ERROR_MSG("S_FTT_INTERFACE_L_var_req_change_period returned: %d ", ret);
	       return -2;
	}

	return 0;
}

signed char S_FTT_INTERFACE_L_var_req_change_qos ( M_INT_APP_MSG_ID id, signed short prio, unsigned char pairs_no, S_FTT_INTERFACE_L_QOS_PAIR *pairs )
/* if prio<0 don't touch the prio */
{
	int ret;
	int i;

	if (!S_FTT_INTERFACE_L_var_in_atomic_change)
		if ( (ret = S_int_req_dispatch_GroupPrepare()) < 0 ){
			ERROR_MSG("S_int_req_dispatch_GroupPrepare ret: %d", ret);
			return -1;
		}

	if (pairs_no == 0){
	       ERROR_MSG("pairs_no error");
	       return -2;	
	}

	PRINT_MSG("Going for S_int_req_dispatch_CmdAddQoS");
	if ( (ret = S_int_req_dispatch_CmdAddQoS(
					&id,
					&(pairs[0].size),
					&(pairs[0].period),
					&prio )	) < 0){
	       ERROR_MSG("S_int_req_Send_cmd_addQoS returned: %d ", ret);
	       return -3;
	}

	for (i=1; i<pairs_no; i++){
		if ( (ret = S_int_req_dispatch_CmdAddQoS_pair(
						&id,
						&(pairs[i].size),
						&(pairs[i].period) ) ) < 0){
	 		ERROR_MSG("S_int_req_dispatch_CmdAddQoS_pair %d returned: %d ", i, ret);
		  	return -3;
		}
	}

	if (!S_FTT_INTERFACE_L_var_in_atomic_change)
		if ( (ret = S_int_req_dispatch_GroupSubmit()) < 0 ){
			ERROR_MSG("S_int_req_dispatch_GroupSubmit ret: %d", ret);
			return -4;
		}	
	
	return 0;
}

signed char S_FTT_INTERFACE_L_var_req_change_qos_prio ( M_INT_APP_MSG_ID id, unsigned short prio )
{
	int ret;

	if ( (ret = S_int_req_dispatch_CmdChangeQoS_prio( 
					&id,
					prio )	) < 0){
	       ERROR_MSG("S_int_req_dispatch_CmdChangePeriod returned: %d ", ret);
	       return -2;
	}

	return 0;
}


signed char S_FTT_INTERFACE_L_nodes_status( unsigned char *event, unsigned int *node_app_id )
{
	
#ifdef PLUGNPLAY_REPORT
#ifdef FTT_SLAVE_COMPILE
	S_int_app_remote_report_get( event, node_app_id );
#else
	M_int_app_report_get( event, node_app_id );
#endif
#else
	return -1;
#endif
	return 0;
}


