/*****************************************************************************
 * M_int_app_report.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/


#include "Ports.h"

#include "S_int_app_remote_report.h"
#include "remote_report_cmds.h"

#include "S_ftt-core.layer.h"


typedef struct{
	int node_ID;
	unsigned char event_state; // 1=in
} APP_REPORT_TYPE;

#define MAX_APP_REPORT_EVENTS 500

static APP_REPORT_TYPE app_report_array[MAX_APP_REPORT_EVENTS];
static int app_report_rd, app_report_wr;
static FTT_SEMAPHORE app_report_sem;


static FTT_SEMAPHORE app_report_init_sem;
static unsigned char app_report_init_sem_flag;


static signed char S_int_app_remote_report_put( unsigned char event, unsigned int node_app_id )
{
	if ( (app_report_wr+1)%MAX_APP_REPORT_EVENTS == app_report_rd){
		ERROR_MSG("Cannot place more events... Full!");
		return -1;
	}
	app_report_array[app_report_wr].node_ID = node_app_id;
	app_report_array[app_report_wr].event_state = event;
	app_report_wr = (app_report_wr+1)%MAX_APP_REPORT_EVENTS;
	
	FTT_SEMAPHORE_post( &app_report_sem );

	return 0;
}

signed char S_int_app_remote_report_get( unsigned char *event, unsigned int *node_app_id )
{
	
	if (!app_report_init_sem_flag){
		app_report_init_sem_flag=1;
		FTT_SEMAPHORE_post( &app_report_init_sem );
	}
	

	if ( FTT_SEMAPHORE_wait( &app_report_sem ) < 0)
		return -1;
	
	if (app_report_rd == app_report_wr){
		ERROR_MSG("Event queues is inconsistent! Empty after a sem_wait");
		return -2;
	}
	
	if (node_app_id!= NULL) *node_app_id = app_report_array[app_report_rd].node_ID;
	if (event!= NULL) *event = app_report_array[app_report_rd].event_state;
	app_report_rd = (app_report_rd+1)%MAX_APP_REPORT_EVENTS;

	
	return 0;
}


///////////////////////////

static unsigned char remote_report_quit_f;
static rtl_pthread_t remote_report_th;

unsigned char test[100000];

static void *remote_report_thread(void *t)
{
	int ftt_var_id;
	FTT_SIGNAL *signal;
	unsigned char buff[BROADCAST_REP_MAX_SIZE];
	int ret;

	/* Wait before establishing the connectivity to the master */
	if ( FTT_SEMAPHORE_wait( &app_report_init_sem ) < 0)
		return NULL;
	
	/* Connect as Rx */	
//re_attach:
	if (remote_report_quit_f)
		return NULL;
	
	ret = S_int_req_dispatch_CmdREQ_REMOTE_REPORT( );

	ftt_var_id = ret;
re_bind:
	S_FTT_CORE_L_Wait4var( ASYNC_MESSAGE, ftt_var_id );
	
	ret = S_FTT_CORE_L_bind_mesg( ASYNC_MESSAGE, ftt_var_id, &signal, NULL, 0, 10);

	while (!remote_report_quit_f) {
		//PRINT_MSG("Receiving on ftt_var_id:%x", ftt_var_id);
		if ( FTT_wait4signal( signal ) < 0 ){
			PRINT_MSG("BROADCAST_REP_ID: The signal was destroyed while waitting for it");
			goto re_bind;
		}
		
		if ( S_FTT_CORE_receive( ftt_var_id, NULL, buff, NULL /*status*/ ) < 0 ){
			PRINT_MSG ("BROADCAST_REP_ID consume error");
			return NULL;
		}
		
		REP_COMMAND_ntoh( (REP_COMMAND *) buff );
		
		switch ( ((REP_COMMAND *) buff)->cmd_id ) {
			case REP_ADD_NODE:
				//PRINT_MSG("Received a remote report: ADD! - node %d", ((REP_COMMAND *) buff)->cmd.add.app_node_id );
				S_int_app_remote_report_put( 1 /*in*/, ((REP_COMMAND *) buff)->cmd.add.app_node_id );
				break;
			case REP_DEL_NODE:
				//PRINT_MSG("Received a remote report: DEL! - node %d", ((REP_COMMAND *) buff)->cmd.del.app_node_id );
				S_int_app_remote_report_put( 0 /*out*/, ((REP_COMMAND *) buff)->cmd.add.app_node_id );
				break;
			default:
				PRINT_MSG("Received a bad remote report!");
				break;
		}
		
	}
	
	return NULL;
}



signed char S_int_app_remote_report_init(void)
{
	rtl_pthread_attr_t attr;
	struct rtl_sched_param sched_param;
	int ret;
	
	app_report_rd=app_report_wr=0;
	FTT_SEMAPHORE_init( &app_report_sem , 0);
	
	
	/*Init sem that waits for a request to start receiving reports */
	FTT_SEMAPHORE_init( &app_report_init_sem , 0);
	app_report_init_sem_flag = 0;

	
	remote_report_quit_f = 0;
	
	/* Initialize the thread */
	rtl_pthread_attr_init( &attr );
	sched_param.sched_priority = rtl_sched_get_priority_min(RTL_SCHED_FIFO);
	rtl_pthread_attr_setschedparam( &attr, &sched_param );
#ifdef RTL_PRO
	rtl_pthread_attr_setfp_np(&attr, 1);
#endif
	if ( (ret=rtl_pthread_create( &remote_report_th, &attr, remote_report_thread, (void *)0 )) ){
		ERROR_MSG("Init Fatal Error: pthread_create returned %d", ret);
		return -1;
	}
	
	
	return 0;
}

signed char S_int_app_remote_report_close(void)
{
//	M_INT_APP_MSG_ID app_id;

	remote_report_quit_f = 1;
	
	rtl_pthread_cancel( remote_report_th );
	rtl_pthread_join( remote_report_th, NULL );
	
//	if ( app_report_init_sem_flag ){
//		app_id.type = M_INT_MSG_ASYNCH_H;
//		app_id.id = BROADCAST_REP_ID;

		/* Close the channel */
//		S_FTT_INTERFACE_L_unbind( app_id );
//		S_FTT_INTERFACE_L_dettach( app_id );
//	}

	app_report_init_sem_flag = 0;
	FTT_SEMAPHORE_destroy( &app_report_init_sem );
	
	
	app_report_rd=app_report_wr=0;
	FTT_SEMAPHORE_destroy( &app_report_sem );
	return 0;
}

