/*****************************************************************************
 * S_int_bind_table.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#include "Ports.h"

#include "S_ftt-core.layer.h"
#include "ftt-interface.layer_types.h"

/* This table stores and references the ftt_ids once
 * the app_ids are binded
 */

#define S_INT_BIND_TABLE_SIZE	50 //This should represent the maximum numbers of vars per slave (minus the two asyncs)

typedef struct{
	unsigned char in_use;
	M_INT_APP_MSG_ID app_id;
	unsigned short ftt_id;
	FTT_SIGNAL *sig;
}S_INT_BIND_TABLE_ITEM;


static S_INT_BIND_TABLE_ITEM S_int_bind_table[S_INT_BIND_TABLE_SIZE];
static rtl_pthread_mutex_t S_int_bind_table_locker;  //locker

#define BIND_TABLE_LOCK_IN  rtl_pthread_mutex_lock(&S_int_bind_table_locker)
#define BIND_TABLE_LOCK_OUT rtl_pthread_mutex_unlock(&S_int_bind_table_locker);

signed char S_int_bind_table_Init( void )
{
	unsigned short i;
	for (i=0; i<S_INT_BIND_TABLE_SIZE; i++){
		S_int_bind_table[i].in_use = 0; //in use
	}
	rtl_pthread_mutex_init(&S_int_bind_table_locker, 0);
	return 0;
}
signed char S_int_bind_table_Close( void )
{
	rtl_pthread_mutex_destroy(&S_int_bind_table_locker);
	return 0;
}

//signed char S_int_bind_table_Add( M_INT_APP_MSG_ID app_id, unsigned short ftt_id, FTT_SIGNAL *sig )
//{
//	/* Check if this app_id was already registered
//	 *  - if it was, I can not overwrite it */
//
//	signed short empty_slot = -1;
//	unsigned short i;
//
//	for (i=0; i<S_INT_BIND_TABLE_SIZE; i++){
//		if (S_int_bind_table[i].in_use){
//			if ( M_INT_APP_MSG_ID_COMPARE(S_int_bind_table[i].app_id, app_id) )
//				return 0;
//		}else{
//			if ( empty_slot < 0 )
//				empty_slot = i;
//		}
//	}
//	
//	if ( empty_slot < 0 ){
//		ERROR_MSG("The bind_table is full");
//		return -1;
//	}
//	S_int_bind_table[empty_slot].in_use = 1;
//
//	/* Let's then store the id association */
//	S_int_bind_table[empty_slot].app_id = app_id;
//	S_int_bind_table[empty_slot].ftt_id = ftt_id;
//	S_int_bind_table[empty_slot].sig = sig;
//
//	return 1;
//}

signed char S_int_bind_table_Add_ftt_id( M_INT_APP_MSG_ID app_id, unsigned short ftt_id )
{
	/* Check if this app_id was already registered
	 *  - if it is and already binded, don't touch 
	 *  - if it isn't add a new slot for this var
	 */

	signed short empty_slot = -1;
	unsigned short i;

	BIND_TABLE_LOCK_IN;
	
	for (i=0; i<S_INT_BIND_TABLE_SIZE; i++){
		if (S_int_bind_table[i].in_use){
			if ( M_INT_APP_MSG_ID_COMPARE( S_int_bind_table[i].app_id, app_id) ){
				if ( S_int_bind_table[i].sig != NULL ){
					BIND_TABLE_LOCK_OUT;
					return -1;
				}
				S_int_bind_table[i].ftt_id = ftt_id;
				BIND_TABLE_LOCK_OUT;
				return 0;
			}
		}else{
			if ( empty_slot < 0 )
				empty_slot = i;
		}
	}
	
	if ( empty_slot < 0 ){
		ERROR_MSG("The bind_table is full");
		BIND_TABLE_LOCK_OUT;
		return -1;
	}
	S_int_bind_table[empty_slot].in_use = 1;

	/* Let's then store the id association */
	S_int_bind_table[empty_slot].app_id = app_id;
	S_int_bind_table[empty_slot].ftt_id = ftt_id;
	S_int_bind_table[empty_slot].sig = NULL;
	
	BIND_TABLE_LOCK_OUT;

	return 0;
}

signed char S_int_bind_table_Add_signal( M_INT_APP_MSG_ID app_id, FTT_SIGNAL *sig )
{
	/* Check if this app_id was already registered
	 *  - if it was, I can not overwrite it */

	unsigned short i;

	BIND_TABLE_LOCK_IN;
	for (i=0; i<S_INT_BIND_TABLE_SIZE; i++){
		if (S_int_bind_table[i].in_use){
			if ( M_INT_APP_MSG_ID_COMPARE(S_int_bind_table[i].app_id, app_id) ){
				if ( S_int_bind_table[i].sig != NULL ){
					BIND_TABLE_LOCK_OUT;
					return -1;
				}
				S_int_bind_table[i].sig = sig;
				BIND_TABLE_LOCK_OUT;
				return 0;
			}
		}
	}
	
	BIND_TABLE_LOCK_OUT;

	return -2;
}

signed char S_int_bind_table_Del( M_INT_APP_MSG_ID app_id )
{
	unsigned short i;

	BIND_TABLE_LOCK_IN;
	/* Locate it */
	for (i=0; i<S_INT_BIND_TABLE_SIZE; i++){
		if (S_int_bind_table[i].in_use)
			if ( M_INT_APP_MSG_ID_COMPARE(S_int_bind_table[i].app_id, app_id) ){
				S_int_bind_table[i].in_use = 0;
				BIND_TABLE_LOCK_OUT;
				return 0;
			}
	}
	BIND_TABLE_LOCK_OUT;
	return -1;
}

signed char S_int_bind_table_Del_signal( M_INT_APP_MSG_ID app_id )
{
	unsigned short i;
	
	BIND_TABLE_LOCK_IN;
	/* Locate it */
	for (i=0; i<S_INT_BIND_TABLE_SIZE; i++){
		if (S_int_bind_table[i].in_use)
			if ( M_INT_APP_MSG_ID_COMPARE(S_int_bind_table[i].app_id, app_id) ){
				if (S_int_bind_table[i].sig == NULL){
					/* Not bound */
					BIND_TABLE_LOCK_OUT;
					return 1;
				}
				S_int_bind_table[i].sig = NULL;
				BIND_TABLE_LOCK_OUT;
				return 0;
			}
	}
	BIND_TABLE_LOCK_OUT;
	return -1;
}


unsigned char S_int_bind_table_Isthere( M_INT_APP_MSG_ID app_id, unsigned short *ftt_id, FTT_SIGNAL **ret_sig )
{
	unsigned short i;

	BIND_TABLE_LOCK_IN;
	/* Locate it */
	for (i=0; i<S_INT_BIND_TABLE_SIZE; i++){
		if (S_int_bind_table[i].in_use)
			if ( M_INT_APP_MSG_ID_COMPARE(S_int_bind_table[i].app_id, app_id) ){
				if (ftt_id != NULL) *ftt_id = S_int_bind_table[i].ftt_id;
				if (ret_sig != NULL) *ret_sig = S_int_bind_table[i].sig;
				if (S_int_bind_table[i].sig == NULL){
					/* if not bound */
					BIND_TABLE_LOCK_OUT;
					return 1;
				}else{
					/* then it's already bound */
					BIND_TABLE_LOCK_OUT;
					return 2;
				}
			}
	}
	BIND_TABLE_LOCK_OUT;
	return 0;
}

