/*****************************************************************************
 * S_int_req_dispatch.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/


#include "S_int_async_ch.h"
#include "request_cmds.h"
#ifdef FTT_MASTER_COMPILE
#include "M_int_req_queuing.h"
#endif

/*This locker is used to protect the communication channel with the MAster (two asyncs)
 * so that while we are waiting for the master reply another request does not come in the middle*/
#define MASTER_CMDS_LockerDeclaration  rtl_pthread_mutex_t master_cmds_access_mutex;
#define MASTER_CMDS_Enter              rtl_pthread_mutex_lock(&master_cmds_access_mutex);
#define MASTER_CMDS_Leave              rtl_pthread_mutex_unlock(&master_cmds_access_mutex);
#define MASTER_CMDS_Init_locker        rtl_pthread_mutex_init(&master_cmds_access_mutex, 0);
#define MASTER_CMDS_Destroy_Locker     rtl_pthread_mutex_destroy(&master_cmds_access_mutex);
static MASTER_CMDS_LockerDeclaration;	//This var is to the application access into DB changes

static unsigned char S_int_req_GlobalContainer[REQ_CMDS_MAX_SIZE];
static unsigned char S_int_req_GlobalContainer_in_use = 0;


signed char S_int_req_dispatch_Init(void)
{
	S_int_req_GlobalContainer_in_use = 0;
	MASTER_CMDS_Init_locker;
	return 0;
}

signed char S_int_req_dispatch_Close(void)
{
	MASTER_CMDS_Destroy_Locker;
	return 0;
}

static signed char S_int_req_Encode_cmd_add( 
		void *command_data,
		M_INT_APP_MSG_ID *app_msg_id,
		unsigned int *mesg_size,
		unsigned int *mesg_max_size,
		unsigned char *mesg_max_prod_no,
		unsigned short *mesg_period,
		unsigned short *mesg_MTU,
		unsigned char *mesg_producers_no,
		unsigned char *mesg_producerID,
		unsigned char *mesg_consumers_no,
		unsigned char *mesg_consumerID
		)
{
	unsigned short i;
	REQ_ADD_FULL_MSG_DATA *add_cmd_data = command_data;


	add_cmd_data->app_msg_id.id = app_msg_id->id;
	add_cmd_data->app_msg_id.type = app_msg_id->type;
	add_cmd_data->mesg_size = *mesg_size;
	add_cmd_data->mesg_max_size = *mesg_max_size;
	add_cmd_data->mesg_max_prod_no = *mesg_max_prod_no;
	add_cmd_data->mesg_period = *mesg_period;
	add_cmd_data->mesg_MTU = *mesg_MTU;
	add_cmd_data->mesg_producers_no = *mesg_producers_no;
	add_cmd_data->mesg_consumers_no = *mesg_consumers_no;

	for (i=0; i<*mesg_producers_no; i++){
		(add_cmd_data->mesg_producerID)[i] = mesg_producerID[i];
	}
	
	for (i=0; i<*mesg_consumers_no; i++){
		(add_cmd_data->mesg_consumerID)[i] = mesg_consumerID[i];
	}

	return 0;
}
static signed char S_int_req_Encode_cmd_del(
		void *command_data,
		M_INT_APP_MSG_ID *app_msg_id
		)
{
	REQ_DEL_FULL_MSG_DATA *del_cmd_data = command_data;

	del_cmd_data->app_msg_id.id = app_msg_id->id;
	del_cmd_data->app_msg_id.type = app_msg_id->type;
	
	return 0;
}

static signed char S_int_req_Encode_cmd_add_load( 
		void *command_data,
		M_INT_APP_MSG_ID *app_msg_id,
		unsigned int *mesg_size,
		unsigned int *mesg_max_size,
		unsigned char *mesg_max_prod_no,
		unsigned short *mesg_period,
		unsigned short *mesg_MTU
		)
{
	REQ_ADDMODIFY_LOAD_REQS_DATA *add_cmd_data = command_data;

	add_cmd_data->app_msg_id.id = app_msg_id->id;
	add_cmd_data->app_msg_id.type = app_msg_id->type;
	add_cmd_data->mesg_size = *mesg_size;
	add_cmd_data->mesg_max_size = *mesg_max_size;
	add_cmd_data->mesg_max_prod_no = *mesg_max_prod_no;
	add_cmd_data->mesg_period = *mesg_period;
	add_cmd_data->mesg_MTU = *mesg_MTU;
	return 0;
}

static signed char S_int_req_Encode_cmd_add_qos( 
		void *command_data,
		M_INT_APP_MSG_ID *app_msg_id,
		signed int *mesg_size,
		signed int *mesg_period,
		signed short *mesg_priority
		)
{
	REQ_ADDMODIFY_QOS_REQS_DATA *add_cmd_data = command_data;

	add_cmd_data->app_msg_id.id = app_msg_id->id;
	add_cmd_data->app_msg_id.type = app_msg_id->type;
	add_cmd_data->mesg_size = *mesg_size;
	add_cmd_data->mesg_period = *mesg_period;
	add_cmd_data->priority = *mesg_priority;
	return 0;
}

static signed char S_int_req_Encode_cmd_add_qos_pair( 
		void *command_data,
		M_INT_APP_MSG_ID *app_msg_id,
		signed int *mesg_size,
		signed int *mesg_period
		)
{
	REQ_ADD_QOS_PAIR_REQS_DATA *add_cmd_data = command_data;

	add_cmd_data->app_msg_id.id = app_msg_id->id;
	add_cmd_data->app_msg_id.type = app_msg_id->type;
	add_cmd_data->mesg_size = *mesg_size;
	add_cmd_data->mesg_period = *mesg_period;
	return 0;
}

static signed char S_int_req_Encode_cmd_remove_load( 
		void *command_data,
		M_INT_APP_MSG_ID *app_msg_id
		)
{
	REQ_REMOVE_LOAD_REQS_DATA *add_cmd_data = command_data;

	add_cmd_data->app_msg_id.id = app_msg_id->id;
	add_cmd_data->app_msg_id.type = app_msg_id->type;
	return 0;
}

static signed char S_int_req_Encode_cmd_chg_qos_prio( 
		void *command_data,
		M_INT_APP_MSG_ID *app_msg_id,
		signed short *mesg_priority
		)
{
	REQ_ADD_QOS_PRIO_REQS_DATA *add_cmd_data = command_data;

	add_cmd_data->app_msg_id.id = app_msg_id->id;
	add_cmd_data->app_msg_id.type = app_msg_id->type;
	add_cmd_data->priority = *mesg_priority;
	return 0;
}

static signed char S_int_req_Encode_cmd_add_con( 
		void *command_data,
		M_INT_APP_MSG_ID *app_msg_id,
		unsigned char *mesg_prodconsID,
		unsigned char direction	)
{
	REQ_ADD_ONE_END_CONECTION_DATA *cmd_data = command_data;

	cmd_data->app_msg_id.id = app_msg_id->id;
	cmd_data->app_msg_id.type = app_msg_id->type;
	cmd_data->mesg_prodconsID = *mesg_prodconsID;
	cmd_data->direction = (direction == 0)? 0 : 1;

	return 0;
}

static signed char S_int_req_Encode_cmd_del_con(
		void *command_data,
		M_INT_APP_MSG_ID *app_msg_id,
		unsigned char *mesg_prodconsID )
{
	REQ_DEL_ONE_END_CONECTION_DATA *cmd_data = command_data;

	cmd_data->app_msg_id.id = app_msg_id->id;
	cmd_data->app_msg_id.type = app_msg_id->type;
	cmd_data->mesg_prodconsID = *mesg_prodconsID;

	return 0;
}

static signed char S_int_req_Encode_cmd_detail_connection(
		void *command_data,
		M_INT_APP_MSG_ID *app_msg_id
		)
{
	REQ_GET_FTT_ID_DETAILS_DATA *cmd_data = command_data;

	cmd_data->app_msg_id.id = app_msg_id->id;
	cmd_data->app_msg_id.type = app_msg_id->type;
	return 0;
}

static signed char S_int_req_Encode_cmd_chg_period(
		void *command_data,
		M_INT_APP_MSG_ID *app_msg_id,
		unsigned short *mesg_period )
{
	REQ_CHG_PERIOD_DATA *cmd_data = command_data;
	cmd_data->app_msg_id.id = app_msg_id->id;
	cmd_data->app_msg_id.type = app_msg_id->type;
	cmd_data->mesg_period = *mesg_period;

	return 0;
}

static signed char S_int_req_Encode_cmd_REQ_REMOTE_REPORT( void *command_data )
{
	//REQ_REMOTE_REPORT *cmd_data = command_data;

	return 0;
}


static signed char S_int_req_Encode_cmd_submit( 
		void *command_data
		)
{
	return 0;
}



signed char S_int_req_dispatch_GroupPrepare(void)
{
	if (S_int_req_GlobalContainer_in_use)
		return -1;

	MASTER_CMDS_Enter;
	S_int_req_GlobalContainer_in_use = 1;
	if ( CMDS_TRANSPORT_init((CMDS_TRANSPORT *)S_int_req_GlobalContainer) < 0 ){
		MASTER_CMDS_Leave;
		return -2;
	}
	return 0;

}

signed char S_int_req_dispatch_GroupSubmit(void)
{
#ifdef FTT_SLAVE_COMPILE
	void *cmd_container_data_p;
#endif
	int ret;
	int return_code;

	if (!S_int_req_GlobalContainer_in_use)
		return -1;
	S_int_req_GlobalContainer_in_use = 0;

#ifdef FTT_SLAVE_COMPILE
	while ( (cmd_container_data_p = (void *)CMDS_TRANSPORT_append_cmd( (CMDS_TRANSPORT *)S_int_req_GlobalContainer, REQ_SUBMIT, NULL)) == NULL){
		/* Command doen't fit 
		 *  - Let's flush the accumulated ones and re-send */

		/* Encode to network data */
		CMDS_TRANSPORT_hton( (CMDS_TRANSPORT *) S_int_req_GlobalContainer);
		
		/* Sending order */
		if ( (ret = S_int_async_ch_SendCMD( S_int_req_GlobalContainer, 0 /*let's not care with it*/ )) < 0 ){
			ERROR_MSG("S_int_async_ch_SendCMD returned: %d", ret);
			MASTER_CMDS_Leave;
			return -4;
		}
		CMDS_TRANSPORT_init((CMDS_TRANSPORT *)S_int_req_GlobalContainer);
	}
	
	S_int_req_Encode_cmd_submit (cmd_container_data_p);
	
	/* Encode to network data */
	CMDS_TRANSPORT_hton( (CMDS_TRANSPORT *) S_int_req_GlobalContainer);

	/* Sending order */
	if ( (ret = S_int_async_ch_SendCMD( S_int_req_GlobalContainer, 0 /*let's not care with it*/ )) < 0 ){
		ERROR_MSG("S_int_async_ch_SendCMD returned: %d", ret);
		MASTER_CMDS_Leave;
		return -4;
	}

	/* Block here, waiting for the reply - The reply will come in the S_int_req_GlobalContainer */
	if ( (ret = S_int_async_ch_GetCMD_blocking ( S_int_req_GlobalContainer, REQ_CMDS_MAX_SIZE )) < 0 ){
		ERROR_MSG("S_int_async_ch_GetCMD_blocking returned: %d", ret);
		MASTER_CMDS_Leave;
		return -5;
	}

	/* Decode from network data */
	REQ_REPLY_ntoh((REQ_REPLY *)S_int_req_GlobalContainer);
	
//	PRINT_MSG("After blocking the last_command_return: %d", ((REQ_REPLY *)S_int_req_GlobalContainer)->last_command_return);


	if ( ((REQ_REPLY *)S_int_req_GlobalContainer)->submission_result != 1 ){
//		PRINT_MSG("i: Request submission failure in master. Printing error:");
//		REQ_REPLY_print( (REQ_REPLY *)S_int_req_GlobalContainer, 0);
		MASTER_CMDS_Leave;
		return -6;
	}
	return_code = ((REQ_REPLY *)S_int_req_GlobalContainer)->last_command_return;

#endif

#ifdef FTT_MASTER_COMPILE
	S_int_req_Encode_cmd_submit (S_int_req_GlobalContainer);

	if ( (ret = M_int_req_queuing_Submit_req(
			0,
			0,
			REQ_SUBMIT,
			S_int_req_GlobalContainer,
			&return_code )) < 0 ){
		ERROR_MSG("M_int_req_queuing_Submit_req returned: %d", ret);
	}
	if (ret != 1){
		PRINT_MSG("REquest submission failure in master");
		MASTER_CMDS_Leave;
		return -6;
	}
#endif
	MASTER_CMDS_Leave;
	return return_code;
}


signed char S_int_req_dispatch_CmdAdd(
		M_INT_APP_MSG_ID *app_msg_id,
		unsigned int *mesg_size,
		unsigned int *mesg_max_size,
		unsigned char *mesg_max_prod_no,
		unsigned short *mesg_period,
		unsigned short *mesg_MTU,
		unsigned char mesg_producers_no,
		unsigned char *mesg_producerID,
		unsigned char mesg_consumers_no,
		unsigned char *mesg_consumerID
		)
{
	unsigned char in_a_group_req = 0;
#ifdef FTT_SLAVE_COMPILE
	void *cmd_container_data_p;
#endif
	int return_code = 0;
	int ret;

	if ( S_int_req_dispatch_GroupPrepare() < 0 ){
		/* Already in a group - means that the cmd will 
		 * only be appended, not submitted */
		in_a_group_req = 1;
	}

#ifdef FTT_SLAVE_COMPILE
	while ( (cmd_container_data_p = (void *)CMDS_TRANSPORT_append_cmd( (CMDS_TRANSPORT *)S_int_req_GlobalContainer, REQ_ADD_FULL_MSG, NULL)) == NULL){
		/* Command doen't fit 
		 *  - Let's flush the accumulated ones and re-send */

		/* Encode to network data */
		CMDS_TRANSPORT_hton( (CMDS_TRANSPORT *) S_int_req_GlobalContainer);
		
		/* Sending order */
		if ( (ret = S_int_async_ch_SendCMD( S_int_req_GlobalContainer, 0 /*let's not care with it*/ )) < 0 ){
			ERROR_MSG("S_int_async_ch_SendCMD returned: %d", ret);
			return -4;
		}
		CMDS_TRANSPORT_init((CMDS_TRANSPORT *)S_int_req_GlobalContainer);
	}

	if (S_int_req_Encode_cmd_add(
		cmd_container_data_p,
		app_msg_id,
		mesg_size,
		mesg_max_size,
		mesg_max_prod_no,
		mesg_period,
		mesg_MTU,
		&mesg_producers_no,
		mesg_producerID,
		&mesg_consumers_no,
		mesg_consumerID
		)<0)
		return -3;
#endif

#ifdef FTT_MASTER_COMPILE
	if (S_int_req_Encode_cmd_add(
		S_int_req_GlobalContainer,
		app_msg_id,
		mesg_size,
		mesg_max_size,
		mesg_max_prod_no,
		mesg_period,
		mesg_MTU,
		&mesg_producers_no,
		mesg_producerID,
		&mesg_consumers_no,
		mesg_consumerID
		)<0)
		return -3;

	if ( (ret = M_int_req_queuing_Submit_req(
			0,
			0,
			REQ_ADD_FULL_MSG,
			S_int_req_GlobalContainer,
			&return_code )) < 0 ){
		ERROR_MSG("M_int_req_queuing_Submit_req returned: %d", ret);
	}
#endif

	if ( !in_a_group_req ){
		return_code = S_int_req_dispatch_GroupSubmit();
	}

	return return_code;
}


signed char S_int_req_dispatch_CmdDel(
		M_INT_APP_MSG_ID *app_msg_id
		)
{
	unsigned char in_a_group_req = 0;
#ifdef FTT_SLAVE_COMPILE
	void *cmd_container_data_p;
#endif
	int return_code = 0;
	int ret;

	if ( S_int_req_dispatch_GroupPrepare() < 0 ){
		/* Already in a group - means that the cmd will 
		 * only be appended, not submitted */
		in_a_group_req = 1;
	}

#ifdef FTT_SLAVE_COMPILE
	while ( (cmd_container_data_p = (void *)CMDS_TRANSPORT_append_cmd( (CMDS_TRANSPORT *)S_int_req_GlobalContainer, REQ_DEL_FULL_MSG, NULL)) == NULL){
		/* Command doen't fit 
		 *  - Let's flush the accumulated ones and re-send */
	
		/* Encode to network data */
		CMDS_TRANSPORT_hton( (CMDS_TRANSPORT *) S_int_req_GlobalContainer);
		
		/* Sending order */
		if ( (ret = S_int_async_ch_SendCMD( S_int_req_GlobalContainer, 0 /*let's not care with it*/ )) < 0 ){
			ERROR_MSG("S_int_async_ch_SendCMD returned: %d", ret);
			return -4;
		}
		CMDS_TRANSPORT_init((CMDS_TRANSPORT *)S_int_req_GlobalContainer);
	}

	if ( S_int_req_Encode_cmd_del(
		cmd_container_data_p,
		app_msg_id ) < 0 )
		return -3;
#endif

#ifdef FTT_MASTER_COMPILE
	if ( S_int_req_Encode_cmd_del(
		S_int_req_GlobalContainer,
		app_msg_id ) < 0 )
		return -3;

	if ( (ret = M_int_req_queuing_Submit_req(
			0,
			0,
			REQ_DEL_FULL_MSG,
			S_int_req_GlobalContainer,
			&return_code )) < 0 ){
		ERROR_MSG("M_int_req_queuing_Submit_req returned: %d", ret);
	}
#endif

	if ( !in_a_group_req ){
		return_code = S_int_req_dispatch_GroupSubmit();
	}

	return return_code;

}


signed int S_int_req_dispatch_CmdAddLoad(
		M_INT_APP_MSG_ID *app_msg_id,
		unsigned int *mesg_size,
		unsigned int *mesg_max_size,
		unsigned char *mesg_max_prod_no,
		unsigned short *mesg_period,
		unsigned short *mesg_MTU
		)
{
	unsigned char in_a_group_req = 0;
#ifdef FTT_SLAVE_COMPILE
	void *cmd_container_data_p;
#endif
	int return_code = 0;
	int ret;

	if ( S_int_req_dispatch_GroupPrepare() < 0 ){
		/* Already in a group - means that the cmd will 
		 * only be appended, not submitted */
		in_a_group_req = 1;
	}

#ifdef FTT_SLAVE_COMPILE
	while ( (cmd_container_data_p = (void *)CMDS_TRANSPORT_append_cmd( (CMDS_TRANSPORT *)S_int_req_GlobalContainer, REQ_ADDMODIFY_LOAD_REQS, NULL)) == NULL){
		/* Command doen't fit 
		 *  - Let's flush the accumulated ones and re-send */

		/* Encode to network data */
		CMDS_TRANSPORT_hton( (CMDS_TRANSPORT *) S_int_req_GlobalContainer);
		
		/* Sending order */
		if ( (ret = S_int_async_ch_SendCMD( S_int_req_GlobalContainer, 0 /*let's not care with it*/ )) < 0 ){
			ERROR_MSG("S_int_async_ch_SendCMD returned: %d", ret);
			return -4;
		}
		CMDS_TRANSPORT_init((CMDS_TRANSPORT *)S_int_req_GlobalContainer);
	}

	if (S_int_req_Encode_cmd_add_load(
		cmd_container_data_p,
		app_msg_id,
		mesg_size,
		mesg_max_size,
		mesg_max_prod_no,
		mesg_period,
		mesg_MTU
		)<0)
		return -3;
#endif

#ifdef FTT_MASTER_COMPILE
	if (S_int_req_Encode_cmd_add_load(
		S_int_req_GlobalContainer,
		app_msg_id,
		mesg_size,
		mesg_max_size,
		mesg_max_prod_no,
		mesg_period,
		mesg_MTU
		)<0)
		return -3;

	if ( (ret = M_int_req_queuing_Submit_req(
			0,
			0,
			REQ_ADDMODIFY_LOAD_REQS,
			S_int_req_GlobalContainer,
			&return_code )) < 0 ){
		ERROR_MSG("M_int_req_queuing_Submit_req returned: %d", ret);
	}
#endif

	if ( !in_a_group_req ){
		return_code = S_int_req_dispatch_GroupSubmit();
	}

	return return_code;
}


signed int S_int_req_dispatch_CmdAddQoS(
		M_INT_APP_MSG_ID *app_msg_id,
		signed int *mesg_size,
		signed int *mesg_period,
		signed short *mesg_priority
		)
{
	unsigned char in_a_group_req = 0;
#ifdef FTT_SLAVE_COMPILE
	void *cmd_container_data_p;
#endif
	int return_code = 0;
	int ret;

	if ( S_int_req_dispatch_GroupPrepare() < 0 ){
		/* Already in a group - means that the cmd will 
		 * only be appended, not submitted */
		in_a_group_req = 1;
	}

#ifdef FTT_SLAVE_COMPILE
	while ( (cmd_container_data_p = (void *)CMDS_TRANSPORT_append_cmd( (CMDS_TRANSPORT *)S_int_req_GlobalContainer, REQ_ADDMODIFY_QOS_REQS, NULL)) == NULL){
		/* Command doen't fit 
		 *  - Let's flush the accumulated ones and re-send */

		/* Encode to network data */
		CMDS_TRANSPORT_hton( (CMDS_TRANSPORT *) S_int_req_GlobalContainer);
		
		/* Sending order */
		if ( (ret = S_int_async_ch_SendCMD( S_int_req_GlobalContainer, 0 /*let's not care with it*/ )) < 0 ){
			ERROR_MSG("S_int_async_ch_SendCMD returned: %d", ret);
			return -4;
		}
		CMDS_TRANSPORT_init((CMDS_TRANSPORT *)S_int_req_GlobalContainer);
	}

	if (S_int_req_Encode_cmd_add_qos(
		cmd_container_data_p,
		app_msg_id,
		mesg_size,
		mesg_period,
		mesg_priority
		)<0)
		return -3;
#endif

#ifdef FTT_MASTER_COMPILE
	if (S_int_req_Encode_cmd_add_qos(
		S_int_req_GlobalContainer,
		app_msg_id,
		mesg_size,
		mesg_period,
		mesg_priority
		)<0)
		return -3;

	if ( (ret = M_int_req_queuing_Submit_req(
			0,
			0,
			REQ_ADDMODIFY_QOS_REQS,
			S_int_req_GlobalContainer,
			&return_code )) < 0 ){
		ERROR_MSG("M_int_req_queuing_Submit_req returned: %d", ret);
	}
#endif

	if ( !in_a_group_req ){
		return_code = S_int_req_dispatch_GroupSubmit();
	}

	return return_code;
}

signed int S_int_req_dispatch_CmdAddQoS_pair(
		M_INT_APP_MSG_ID *app_msg_id,
		signed int *mesg_size,
		signed int *mesg_period
		)
{
	unsigned char in_a_group_req = 0;
#ifdef FTT_SLAVE_COMPILE
	void *cmd_container_data_p;
#endif
	int return_code = 0;
	int ret;

	if ( S_int_req_dispatch_GroupPrepare() < 0 ){
		/* Already in a group - means that the cmd will 
		 * only be appended, not submitted */
		in_a_group_req = 1;
	}

#ifdef FTT_SLAVE_COMPILE
	while ( (cmd_container_data_p = (void *)CMDS_TRANSPORT_append_cmd( (CMDS_TRANSPORT *)S_int_req_GlobalContainer, REQ_ADD_QOS_PAIR_REQS, NULL)) == NULL){
		/* Command doen't fit 
		 *  - Let's flush the accumulated ones and re-send */
	
		/* Encode to network data */
		CMDS_TRANSPORT_hton( (CMDS_TRANSPORT *) S_int_req_GlobalContainer);
		
		/* Sending order */
		if ( (ret = S_int_async_ch_SendCMD( S_int_req_GlobalContainer, 0 /*let's not care with it*/ )) < 0 ){
			ERROR_MSG("S_int_async_ch_SendCMD returned: %d", ret);
			return -4;
		}
		CMDS_TRANSPORT_init((CMDS_TRANSPORT *)S_int_req_GlobalContainer);
	}

	if (S_int_req_Encode_cmd_add_qos_pair(
		cmd_container_data_p,
		app_msg_id,
		mesg_size,
		mesg_period
		)<0)
		return -3;
#endif

#ifdef FTT_MASTER_COMPILE
	if (S_int_req_Encode_cmd_add_qos_pair(
		S_int_req_GlobalContainer,
		app_msg_id,
		mesg_size,
		mesg_period
		)<0)
		return -3;

	if ( (ret = M_int_req_queuing_Submit_req(
			0,
			0,
			REQ_ADD_QOS_PAIR_REQS,
			S_int_req_GlobalContainer,
			&return_code )) < 0 ){
		ERROR_MSG("M_int_req_queuing_Submit_req returned: %d", ret);
	}
#endif

	if ( !in_a_group_req ){
		return_code = S_int_req_dispatch_GroupSubmit();
	}

	return return_code;
}

signed int S_int_req_dispatch_CmdRemoveLoad(M_INT_APP_MSG_ID *app_msg_id)
{
	unsigned char in_a_group_req = 0;
#ifdef FTT_SLAVE_COMPILE
	void *cmd_container_data_p;
#endif
	int return_code = 0;
	int ret;
	
	if ( S_int_req_dispatch_GroupPrepare() < 0 ){
		/* Already in a group - means that the cmd will 
		 * only be appended, not submitted */
		in_a_group_req = 1;
	}
	
#ifdef FTT_SLAVE_COMPILE
	while ( (cmd_container_data_p = (void *)CMDS_TRANSPORT_append_cmd( (CMDS_TRANSPORT *)S_int_req_GlobalContainer, REQ_REMOVE_LOAD_REQS, NULL)) == NULL){
		/* Command doen't fit 
		 *  - Let's flush the accumulated ones and re-send */
		
		/* Encode to network data */
		CMDS_TRANSPORT_hton( (CMDS_TRANSPORT *) S_int_req_GlobalContainer);
		
		/* Sending order */
		if ( (ret = S_int_async_ch_SendCMD( S_int_req_GlobalContainer, 0 /*let's not care with it*/ )) < 0 ){
			ERROR_MSG("S_int_async_ch_SendCMD returned: %d", ret);
			return -4;
		}
		CMDS_TRANSPORT_init((CMDS_TRANSPORT *)S_int_req_GlobalContainer);
	}
	
	if (S_int_req_Encode_cmd_remove_load( cmd_container_data_p, app_msg_id ) < 0)
		return -3;
#endif
	
#ifdef FTT_MASTER_COMPILE
	if (S_int_req_Encode_cmd_remove_load( S_int_req_GlobalContainer, app_msg_id) < 0)
		return -3;
	
	if ( (ret = M_int_req_queuing_Submit_req(
											 0,
											 0,
											 REQ_REMOVE_LOAD_REQS,
											 S_int_req_GlobalContainer,
											 &return_code )) < 0 ){
		ERROR_MSG("M_int_req_queuing_Submit_req returned: %d", ret);
	}
#endif
	
	if ( !in_a_group_req ){
		return_code = S_int_req_dispatch_GroupSubmit();
	}
	
	return return_code;
}



signed int S_int_req_dispatch_CmdAddconnection(
		M_INT_APP_MSG_ID *app_msg_id,
		unsigned char *mesg_prodconsID,
		unsigned char direction
		)
{
	unsigned char in_a_group_req = 0;
#ifdef FTT_SLAVE_COMPILE
	void *cmd_container_data_p;
#endif
	int return_code = 0;
	int ret;

	if ( S_int_req_dispatch_GroupPrepare() < 0 ){
		/* Already in a group - means that the cmd will 
		 * only be appended, not submitted */
		in_a_group_req = 1;
	}

#ifdef FTT_SLAVE_COMPILE
	while ( (cmd_container_data_p = (void *)CMDS_TRANSPORT_append_cmd( (CMDS_TRANSPORT *)S_int_req_GlobalContainer, REQ_ADD_ONE_END_CONECTION, NULL)) == NULL){
		/* Command doen't fit 
		 *  - Let's flush the accumulated ones and re-send */
	
		/* Encode to network data */
		CMDS_TRANSPORT_hton( (CMDS_TRANSPORT *) S_int_req_GlobalContainer);
		
		/* Sending order */
		if ( (ret = S_int_async_ch_SendCMD( S_int_req_GlobalContainer, 0 /*let's not care with it*/ )) < 0 ){
			ERROR_MSG("S_int_async_ch_SendCMD returned: %d", ret);
			return -4;
		}
		CMDS_TRANSPORT_init((CMDS_TRANSPORT *)S_int_req_GlobalContainer);
	}

	if ( S_int_req_Encode_cmd_add_con( cmd_container_data_p, app_msg_id, mesg_prodconsID, direction ) < 0 )
		return -3;
#endif

#ifdef FTT_MASTER_COMPILE
	if ( S_int_req_Encode_cmd_add_con( S_int_req_GlobalContainer, app_msg_id, mesg_prodconsID, direction ) < 0 )
		return -3;

	if ( (ret = M_int_req_queuing_Submit_req(
			0,
			0,
			REQ_ADD_ONE_END_CONECTION,
			S_int_req_GlobalContainer,
			&return_code )) < 0 ){
		ERROR_MSG("M_int_req_queuing_Submit_req returned: %d", ret);
	}
#endif

	if ( !in_a_group_req ){
		return_code = S_int_req_dispatch_GroupSubmit();
	}

	return return_code;
}


signed int S_int_req_dispatch_CmdDelconnection(
		M_INT_APP_MSG_ID *app_msg_id,
		unsigned char *mesg_prodconsID
		)
{
	unsigned char in_a_group_req = 0;
#ifdef FTT_SLAVE_COMPILE
	void *cmd_container_data_p;
#endif
	int return_code = 0;
	int ret;

	if ( S_int_req_dispatch_GroupPrepare() < 0 ){
		/* Already in a group - means that the cmd will 
		 * only be appended, not submitted */
		in_a_group_req = 1;
	}

#ifdef FTT_SLAVE_COMPILE
	while ( (cmd_container_data_p = (void *)CMDS_TRANSPORT_append_cmd( (CMDS_TRANSPORT *)S_int_req_GlobalContainer, REQ_DEL_ONE_END_CONECTION, NULL)) == NULL){
		/* Command doen't fit 
		 *  - Let's flush the accumulated ones and re-send */
	
		/* Encode to network data */
		CMDS_TRANSPORT_hton( (CMDS_TRANSPORT *) S_int_req_GlobalContainer);
		
		/* Sending order */
		if ( (ret = S_int_async_ch_SendCMD( S_int_req_GlobalContainer, 0 /*let's not care with it*/ )) < 0 ){
			ERROR_MSG("S_int_async_ch_SendCMD returned: %d", ret);
			return -4;
		}
		CMDS_TRANSPORT_init((CMDS_TRANSPORT *)S_int_req_GlobalContainer);
	}

	if ( S_int_req_Encode_cmd_del_con( cmd_container_data_p, app_msg_id, mesg_prodconsID ) < 0 )
		return -3;
#endif

#ifdef FTT_MASTER_COMPILE
	if ( S_int_req_Encode_cmd_del_con( S_int_req_GlobalContainer, app_msg_id, mesg_prodconsID ) < 0 )
		return -3;

	if ( (ret = M_int_req_queuing_Submit_req(
			0,
			0,
			REQ_DEL_ONE_END_CONECTION,
			S_int_req_GlobalContainer,
			&return_code )) < 0 ){
		ERROR_MSG("M_int_req_queuing_Submit_req returned: %d", ret);
	}
#endif

	if ( !in_a_group_req ){
		return_code = S_int_req_dispatch_GroupSubmit();
	}

	return return_code;
}

signed int S_int_req_dispatch_CmdDetailConnection( M_INT_APP_MSG_ID *app_msg_id )
{
	unsigned char in_a_group_req = 0;
#ifdef FTT_SLAVE_COMPILE
	void *cmd_container_data_p;
#endif
	int return_code = 0;
	int ret;
	
	if ( S_int_req_dispatch_GroupPrepare() < 0 ){
		/* Already in a group - means that the cmd will 
		 * only be appended, not submitted */
		in_a_group_req = 1;
	}
	
#ifdef FTT_SLAVE_COMPILE
	while ( (cmd_container_data_p = (void *)CMDS_TRANSPORT_append_cmd( (CMDS_TRANSPORT *)S_int_req_GlobalContainer, REQ_GET_FTT_ID_DETAILS, NULL)) == NULL){
		/* Command doen't fit 
		 *  - Let's flush the accumulated ones and re-send */
		
		/* Encode to network data */
		CMDS_TRANSPORT_hton( (CMDS_TRANSPORT *) S_int_req_GlobalContainer);
		
		/* Sending order */
		if ( (ret = S_int_async_ch_SendCMD( S_int_req_GlobalContainer, 0 /*let's not care with it*/ )) < 0 ){
			ERROR_MSG("S_int_async_ch_SendCMD returned: %d", ret);
			return -4;
		}
		CMDS_TRANSPORT_init((CMDS_TRANSPORT *)S_int_req_GlobalContainer);
	}
	
	if ( S_int_req_Encode_cmd_detail_connection( cmd_container_data_p, app_msg_id ) < 0 )
		return -3;
#endif
	
#ifdef FTT_MASTER_COMPILE
	if ( S_int_req_Encode_cmd_detail_connection( S_int_req_GlobalContainer, app_msg_id ) < 0 )
		return -3;
	
	if ( (ret = M_int_req_queuing_Submit_req(
											 0,
											 0,
											 REQ_GET_FTT_ID_DETAILS,
											 S_int_req_GlobalContainer,
											 &return_code )) < 0 ){
		ERROR_MSG("M_int_req_queuing_Submit_req returned: %d", ret);
	}
#endif
	
	if ( !in_a_group_req ){
		return_code = S_int_req_dispatch_GroupSubmit();
	}
	
	return return_code;
}

signed char S_int_req_dispatch_CmdChangePeriod(
		M_INT_APP_MSG_ID *app_msg_id,
		unsigned short *mesg_period)
{
	unsigned char in_a_group_req = 0;
#ifdef FTT_SLAVE_COMPILE
	void *cmd_container_data_p;
#endif
	int return_code = 0;
	int ret;

	if ( S_int_req_dispatch_GroupPrepare() < 0 ){
		/* Already in a group - means that the cmd will 
		 * only be appended, not submitted */
		in_a_group_req = 1;
	}

#ifdef FTT_SLAVE_COMPILE
	while ( (cmd_container_data_p = (void *)CMDS_TRANSPORT_append_cmd( (CMDS_TRANSPORT *)S_int_req_GlobalContainer, REQ_CHG_PERIOD, NULL)) == NULL){
		/* Command doen't fit 
		 *  - Let's flush the accumulated ones and re-send */
	
		/* Encode to network data */
		CMDS_TRANSPORT_hton( (CMDS_TRANSPORT *) S_int_req_GlobalContainer);
		
		/* Sending order */
		if ( (ret = S_int_async_ch_SendCMD( S_int_req_GlobalContainer, 0 /*let's not care with it*/ )) < 0 ){
			ERROR_MSG("S_int_async_ch_SendCMD returned: %d", ret);
			return -4;
		}
		CMDS_TRANSPORT_init((CMDS_TRANSPORT *)S_int_req_GlobalContainer);
	}

	if ( S_int_req_Encode_cmd_chg_period(
		cmd_container_data_p,
		app_msg_id, mesg_period ) < 0 )
		return -3;
#endif

#ifdef FTT_MASTER_COMPILE
	if ( S_int_req_Encode_cmd_chg_period(
		S_int_req_GlobalContainer,
		app_msg_id, mesg_period ) < 0 )
		return -3;

	if ( (ret = M_int_req_queuing_Submit_req(
			0,
			0,
			REQ_CHG_PERIOD,
			S_int_req_GlobalContainer,
			&return_code )) < 0 ){
		ERROR_MSG("M_int_req_queuing_Submit_req returned: %d", ret);
	}
#endif

	if ( !in_a_group_req ){
		return_code = S_int_req_dispatch_GroupSubmit();
	}

	return return_code;

}


signed char S_int_req_dispatch_CmdChangeQoS_prio(
		M_INT_APP_MSG_ID *app_msg_id,
		unsigned short prio)
{
	unsigned char in_a_group_req = 0;
#ifdef FTT_SLAVE_COMPILE
	void *cmd_container_data_p;
#endif
	int return_code = 0;
	int ret;

	if ( S_int_req_dispatch_GroupPrepare() < 0 ){
		/* Already in a group - means that the cmd will 
		 * only be appended, not submitted */
		in_a_group_req = 1;
	}

#ifdef FTT_SLAVE_COMPILE
	while ( (cmd_container_data_p = (void *)CMDS_TRANSPORT_append_cmd( (CMDS_TRANSPORT *)S_int_req_GlobalContainer, REQ_ADD_QOS_PRIO_REQS, NULL)) == NULL){
		/* Command doen't fit 
		 *  - Let's flush the accumulated ones and re-send */
	
		/* Encode to network data */
		CMDS_TRANSPORT_hton( (CMDS_TRANSPORT *) S_int_req_GlobalContainer);
		
		/* Sending order */
		if ( (ret = S_int_async_ch_SendCMD( S_int_req_GlobalContainer, 0 /*let's not care with it*/ )) < 0 ){
			ERROR_MSG("S_int_async_ch_SendCMD returned: %d", ret);
			return -4;
		}
		CMDS_TRANSPORT_init((CMDS_TRANSPORT *)S_int_req_GlobalContainer);
	}

	if ( S_int_req_Encode_cmd_chg_qos_prio(
		cmd_container_data_p,
		app_msg_id, (signed short *)&prio ) < 0 )
		return -3;
#endif

#ifdef FTT_MASTER_COMPILE
	if ( S_int_req_Encode_cmd_chg_qos_prio(
		S_int_req_GlobalContainer,
		app_msg_id, (signed short *)&prio ) < 0 )
		return -3;

	if ( (ret = M_int_req_queuing_Submit_req(
			0,
			0,
			REQ_ADD_QOS_PRIO_REQS,
			S_int_req_GlobalContainer,
			&return_code )) < 0 ){
		ERROR_MSG("M_int_req_queuing_Submit_req returned: %d", ret);
	}
#endif

	if ( !in_a_group_req ){
		return_code = S_int_req_dispatch_GroupSubmit();
	}

	return return_code;

}

signed char S_int_req_dispatch_CmdREQ_REMOTE_REPORT(void)
{
	unsigned char in_a_group_req = 0;
#ifdef FTT_SLAVE_COMPILE
	void *cmd_container_data_p;
#endif
	int return_code = 0;
	int ret;
	
	if ( S_int_req_dispatch_GroupPrepare() < 0 ){
		/* Already in a group - means that the cmd will 
		 * only be appended, not submitted */
		in_a_group_req = 1;
	}
	
#ifdef FTT_SLAVE_COMPILE
	while ( (cmd_container_data_p = (void *)CMDS_TRANSPORT_append_cmd( (CMDS_TRANSPORT *)S_int_req_GlobalContainer, REQ_REMOTE_REPORT, NULL)) == NULL){
		/* Command doen't fit 
		 *  - Let's flush the accumulated ones and re-send */
		
		/* Encode to network data */
		CMDS_TRANSPORT_hton( (CMDS_TRANSPORT *) S_int_req_GlobalContainer);
		
		/* Sending order */
		if ( (ret = S_int_async_ch_SendCMD( S_int_req_GlobalContainer, 0 /*let's not care with it*/ )) < 0 ){
			ERROR_MSG("S_int_async_ch_SendCMD returned: %d", ret);
			return -4;
		}
		CMDS_TRANSPORT_init((CMDS_TRANSPORT *)S_int_req_GlobalContainer);
	}
	
	if ( S_int_req_Encode_cmd_REQ_REMOTE_REPORT( S_int_req_GlobalContainer ) < 0 )
		return -3;
#endif
	
#ifdef FTT_MASTER_COMPILE
	if ( S_int_req_Encode_cmd_REQ_REMOTE_REPORT( S_int_req_GlobalContainer ) < 0 )
		return -3;
	
	if ( (ret = M_int_req_queuing_Submit_req(
											 0,
											 0,
											 REQ_REMOTE_REPORT,
											 S_int_req_GlobalContainer,
											 &return_code )) < 0 ){
		ERROR_MSG("M_int_req_queuing_Submit_req returned: %d", ret);
	}
#endif
	
	if ( !in_a_group_req ){
		return_code = S_int_req_dispatch_GroupSubmit();
	}
	
	return return_code;
	
}

