/*****************************************************************************
 * M_ftt-management.layer.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#include "Ports.h"
#include "ftt-management.layer_types.h"
#include "M_ftt-core.layer.h"

#include "M_manag_db.h"
#include "M_manag_update.h"
#include "M_manag_qos.h"

static GEN_MESSAGE_ITEM  M_FTT_MANAGEMENT_core_table_contents[MESSAGE_IDS_MAX];
static MESSAGES_TABLE M_FTT_MANAGEMENT_core_table;


static unsigned char M_FTT_MANAGEMENT_L_Admission_passed;

signed char M_FTT_MANAGEMENT_L_init(void)
{
	int ret;
	MESSAGES_TABLE *ftt_core_static_table;

#ifdef MASTER_POLITE
	PRINT_MSG("Initializing the Management layer - Master");
#endif
	M_MANAG_DB_Init();

	if ( (ret=M_FTT_CORE_L_get_hardcoded_table( &ftt_core_static_table )) < 0 ){
		ERROR_MSG("M_FTT_CORE_L_get_hardcoded_table ret: %d",ret);
		return -1;
	}

	if ( (ret=M_manag_Import_Core_Table( ftt_core_static_table )) < 0 ){
		ERROR_MSG("M_FTT_CORE_L_get_hardcoded_table ret: %d",ret);
		return -2;
	}

	/* Initialize the table used to export the system requirements into the Core layer */
	MESSAGES_TABLE_Init( &M_FTT_MANAGEMENT_core_table, M_FTT_MANAGEMENT_core_table_contents, MESSAGE_IDS_MAX);

	return 0;
}

signed char M_FTT_MANAGEMENT_L_close(void)
{
#ifdef MASTER_POLITE
	PRINT_MSG("Closing the Management layer - Master");
#endif

	MESSAGES_TABLE_Close(&M_FTT_MANAGEMENT_core_table);
	M_MANAG_DB_Close();

	return 0;
}



signed char M_FTT_MANAGEMENT_L_changes_lock(void)
{
	M_MANAG_DB_Changes_lock();
	M_FTT_MANAGEMENT_L_Admission_passed=0;
	return 0;
}

signed char M_FTT_MANAGEMENT_L_changes_Admission_Control(void)
{
	if (M_manag_Admission_test() == 0){
		M_FTT_MANAGEMENT_L_Admission_passed = 1;
#ifdef PRINT_MANAGEMENT_REQUESTS
		PRINT_MSG("  m: Passed the Admission Control");
#endif
		return 0;
	}else{
		M_FTT_MANAGEMENT_L_Admission_passed = 0;
#ifdef PRINT_MANAGEMENT_REQUESTS
		PRINT_MSG("  m: NOT Passed the Admission Control");
#else
		PRINT_MSG ("Admission control fail");
#endif
		return -1;
	}
	return 0;
}

signed char M_FTT_MANAGEMENT_L_changes_unlock_aborting(void)
{
	M_MANAG_DB_Changes_abort_unlock();
	return 0;
}


signed char M_FTT_MANAGEMENT_L_changes_unlock_submiting(void)
{
	int ret;

	if ( !M_FTT_MANAGEMENT_L_Admission_passed )
		if (M_FTT_MANAGEMENT_L_changes_Admission_Control() < 0)
			return -1;

	if ( (ret = M_manag_QoS_distribution( )) < 0){
		ERROR_MSG("M_manag_QoS_distribution returned: %d", ret);
		return -2;
	}

//PRINT_MSG("manag runtime table before submition");
//M_MANAG_DB_Print_runtime();
	M_MANAG_DB_Changes_submit_unlock(); //transforms the backtime  into runtime

//PRINT_MSG("manag runtime table before exporting");
//M_MANAG_DB_Print_runtime();

	/* Now let's Make the QoS mesurements and publish to the Core layer */
	if ( (ret=M_manag_Export_Core_Table(  &M_FTT_MANAGEMENT_core_table )) < 0 ){
		ERROR_MSG("M_manag_Export_Core_Table ret: %d",ret);
		return -2;
	}

//PRINT_MSG("Successfully exported: Let's print it before sending to the underlayer");
//MESSAGES_TABLE_print(&M_FTT_MANAGEMENT_core_table, 1);

	/* Let's finally send the f*ing table downwards */
	if ( (ret = M_FTT_CORE_L_update_requirements(&M_FTT_MANAGEMENT_core_table)) < 0 ){
		ERROR_MSG("Error updating the table in the ftt-core.layer M_FTT_CORE_L_update_requirements ret: %d",ret);
		return -3;	
	}

//	PRINT_MSG("----------------Successfully sent");

	return 0;
}

void M_FTT_MANAGEMENT_L_print( unsigned char detailed )
{
	M_MANAG_DB_Print_runtime(detailed);
}

signed int M_FTT_MANAGEMENT_L_get_msg_ftt_id( M_FTT_MANAGEMENT_L_APP_ID app_id )
{
	return M_MANAG_DB_Get_ftt_id( app_id );
}

signed int M_FTT_MANAGEMENT_L_add_msg_complete( 
		M_FTT_MANAGEMENT_L_APP_ID app_id,
		M_FTT_MANAGEMENT_L_LOAD_REQUIREMENTS *load_req,
		M_FTT_MANAGEMENT_L_QOS_REQUIREMENTS *qos_req,
		M_FTT_MANAGEMENT_L_CONNECTION_SET *con_req)
{
	signed int ret = M_MANAG_DB_Add_requirements( app_id, load_req, qos_req, con_req, 0 /* no force new */ );
	M_FTT_MANAGEMENT_L_Admission_passed = 0;
	return ret;
}

signed char M_FTT_MANAGEMENT_L_del_msg(
		M_FTT_MANAGEMENT_L_APP_ID app_id)
{
	return M_MANAG_DB_Del(app_id);
}

signed int M_FTT_MANAGEMENT_L_add_msg_load( 
		M_FTT_MANAGEMENT_L_APP_ID app_id,
		M_FTT_MANAGEMENT_L_LOAD_REQUIREMENTS *load_req)
{
	signed int ret= M_MANAG_DB_Add_requirements( app_id, load_req, NULL, NULL, 0 );
	M_FTT_MANAGEMENT_L_Admission_passed = 0;
	return ret;
}

signed int M_FTT_MANAGEMENT_L_add_msg_qos(
		M_FTT_MANAGEMENT_L_APP_ID app_id,
		M_FTT_MANAGEMENT_L_QOS_REQUIREMENTS *qos_req)
{
	signed int ret= M_MANAG_DB_Add_requirements( app_id, NULL, qos_req, NULL, 0 );
	M_FTT_MANAGEMENT_L_Admission_passed = 0;
	return ret;
}

signed int M_FTT_MANAGEMENT_L_add_msg_connection(
		M_FTT_MANAGEMENT_L_APP_ID app_id,
		M_FTT_MANAGEMENT_L_CONNECTION nodeID,
		unsigned char direction )
/* direction: 0-producer    1-consumer */
{
	signed int ret= M_MANAG_DB_Add_connection( app_id, nodeID, direction );
	M_FTT_MANAGEMENT_L_Admission_passed = 0;
	return ret;
}

signed int M_FTT_MANAGEMENT_L_del_msg_connection(
		M_FTT_MANAGEMENT_L_APP_ID app_id,
		M_FTT_MANAGEMENT_L_CONNECTION nodeID )
{
	return M_MANAG_DB_Del_connection( app_id, nodeID );
}

signed int M_FTT_MANAGEMENT_L_del_node(
		M_FTT_MANAGEMENT_L_CONNECTION nodeID )
{
	return M_MANAG_DB_Del_node( nodeID );
}

signed int M_FTT_MANAGEMENT_L_chg_period(
		M_FTT_MANAGEMENT_L_APP_ID app_id,
		unsigned short period )
{
	signed int ret= M_MANAG_DB_Chg_period( app_id, period );
	M_FTT_MANAGEMENT_L_Admission_passed = 0;
	return ret;
}

signed int M_FTT_MANAGEMENT_L_chg_qos_prio(
		M_FTT_MANAGEMENT_L_APP_ID app_id,
		unsigned short prio )
{
	signed int ret= M_MANAG_DB_Chg_qos_prio( app_id, prio );
	M_FTT_MANAGEMENT_L_Admission_passed = 0; //not necessary, but stil...
	return ret;
}


