/*****************************************************************************
 * M_manag_db.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#include "Ports.h"
#include "M_ftt-management.layer.h"
#include "M_manag_db.h"

static M_MANAG_DB_TABTYPE m_manag_db_table[2];

static M_MANAG_DB_TABTYPE *m_manag_db_table_runtime;
static M_MANAG_DB_TABTYPE *m_manag_db_table_backtime;


signed char M_MANAG_DB_Init(void)
{
	m_manag_db_table_runtime = &(m_manag_db_table[0]);
	m_manag_db_table_backtime = &(m_manag_db_table[1]);

	M_MANAG_DB_TABTYPE_Init(m_manag_db_table_runtime);
	M_MANAG_DB_TABTYPE_Init(m_manag_db_table_backtime);

//PRINT_MSG("m_manag_db_table_runtime pointer in in: %p", &m_manag_db_table_runtime);
//PRINT_MSG("m_manag_db_table_backtime pointer in in: %p", &m_manag_db_table_backtime);
#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -M_MANAG_DB_Init Ok");
#endif
	return 0;
}

signed char M_MANAG_DB_Close(void)
{
	M_MANAG_DB_TABTYPE_Close(m_manag_db_table_runtime);
	
#ifdef PRINT_MODULES_INIT
	PRINT_MSG(" -M_MANAG_DB_Close Ok");
#endif
	return 0;
}

signed char M_MANAG_DB_Changes_lock(void)
{
	int i;

	/* locks */
	M_MANAG_DB_TABTYPE_EnterLock(m_manag_db_table_backtime);

	/* Prepare backtime table for changes */
	M_MANAG_DB_TABTYPE_EnterLock(m_manag_db_table_runtime);
//	*m_manag_db_table_backtime = *m_manag_db_table_runtime;
	m_manag_db_table_backtime->m_nvars = m_manag_db_table_runtime->m_nvars;
	for (i=0; i<MESSAGE_IDS_MAX*2; i++)
		(m_manag_db_table_backtime->m_message)[i] = (m_manag_db_table_runtime->m_message)[i];
	for (i=0; i<MESSAGE_IDS_MAX; i++){
		(m_manag_db_table_backtime->am_ids)[i] = (m_manag_db_table_runtime->am_ids)[i];
		(m_manag_db_table_backtime->sm_ids)[i] = (m_manag_db_table_runtime->sm_ids)[i];
	}

	M_MANAG_DB_TABTYPE_LeaveLock(m_manag_db_table_runtime);

	return 0;

}

signed char M_MANAG_DB_Changes_abort_unlock(void)
{
	M_MANAG_DB_TABTYPE_LeaveLock(m_manag_db_table_backtime);
	return 0;
}

signed char M_MANAG_DB_Changes_submit_unlock(void)
{
	M_MANAG_DB_TABTYPE *temp;

	/* Check lock flag */
	if ( !M_MANAG_DB_TABTYPE_IsLocked( m_manag_db_table_backtime ) ){
		ERROR_MSG("Backtime not locked");
		return -1;
	}

	M_MANAG_DB_TABTYPE_EnterLock(m_manag_db_table_runtime);
	temp=m_manag_db_table_backtime;
	m_manag_db_table_backtime = m_manag_db_table_runtime;
	m_manag_db_table_runtime=temp;
	M_MANAG_DB_TABTYPE_LeaveLock(m_manag_db_table_runtime);

	/* Leave the back locker */
	M_MANAG_DB_TABTYPE_LeaveLock(m_manag_db_table_backtime);

	return 0;
}

static unsigned int M_MANAG_DB_Access_counter; //used when exporting the table, 
                                               //for retrieving one by one
signed char M_MANAG_DB_Access_lock(void)
{
	/* locks */
	M_MANAG_DB_TABTYPE_EnterLock(m_manag_db_table_runtime);

	/* Reset the access counrer */
	M_MANAG_DB_Access_counter=0;
	return 0;
}

signed char M_MANAG_DB_Access_unlock(void)
{
	/* Leave the run locker */
	M_MANAG_DB_TABTYPE_LeaveLock(m_manag_db_table_runtime);
	return 0;
}


signed int M_MANAG_DB_Add_blacklisted_ftt_id(
		unsigned short ftt_id,
		unsigned char m_type,
		M_FTT_MANAGEMENT_L_LOAD_REQUIREMENTS *load_p,
		M_FTT_MANAGEMENT_L_CONNECTION_SET *con_p
		)
{
	signed int ret;

	/* This function is to be called right after this module initialization */

	/* Check lock flag */
	if ( !M_MANAG_DB_TABTYPE_IsLocked( m_manag_db_table_backtime ) ){
		ERROR_MSG("Backtime not locked");
		return -1;
	}

	/* Parameters validation */	
	if (ftt_id >= MESSAGE_IDS_MAX )
		return -2;

	if ( !M_FTT_MANAGEMENT_L_LOAD_REQUIREMENTS_validate( load_p ) )
		return -3;

	if ( !M_FTT_MANAGEMENT_L_CONNECTION_SET_validate( con_p ) )
		return -4;

	if ( ( ret = M_MANAG_DB_TABTYPE_Get_item_not_inuse(m_manag_db_table_backtime) ) < 0 )
		return -5;

	m_manag_db_table_backtime->m_message[ret].ftt_id_idx = ftt_id;
	m_manag_db_table_backtime->m_message[ret].new_flag = 0;
	m_manag_db_table_backtime->m_message[ret].con.req = *con_p;
	m_manag_db_table_backtime->m_message[ret].con.status = 1;
	m_manag_db_table_backtime->m_message[ret].load.req = *load_p;
	m_manag_db_table_backtime->m_message[ret].load.status = 1;

	switch (m_type){
		case M_FTT_MANAGEMENT_AM:
			if ((m_manag_db_table_backtime->am_ids)[ftt_id])
				return -6;
			m_manag_db_table_backtime->m_message[ret].type_and_id.type = M_FTT_MANAGEMENT_AM_DUMMY;
			(m_manag_db_table_backtime->am_ids)[ftt_id]=1;
			break;
		case M_FTT_MANAGEMENT_SM:
			if ((m_manag_db_table_backtime->sm_ids)[ftt_id])
				return -6;
			m_manag_db_table_backtime->m_message[ret].type_and_id.type = M_FTT_MANAGEMENT_SM_DUMMY;
			(m_manag_db_table_backtime->sm_ids)[ftt_id]=1;
			break;
		default:
			ERROR_MSG("Illegal message type");
			return -5;
	}
	
	m_manag_db_table_backtime->m_nvars ++;
	
	return 0;
}

static signed int M_MANAG_DB_get_ftt_id_entry(unsigned short m_type)
{
	/* Searches for an available FTT id */
//	int ret;
	unsigned short i;
	unsigned char *temp_p;

	switch(m_type){
		case M_FTT_MANAGEMENT_AM: temp_p = m_manag_db_table_backtime->am_ids; break;
		case M_FTT_MANAGEMENT_SM: temp_p = m_manag_db_table_backtime->sm_ids; break;
		default: return -1;	
	}

	for (i=0; i<MESSAGE_IDS_MAX; i++){
		if (temp_p[i] == 0){
			/* Found available ID */
			temp_p[i] = 1;
			break;
		}
	}
	if (i==MESSAGE_IDS_MAX)
		return -2;

	return i;
}

static signed char M_MANAG_DB_del_ftt_id_entry(unsigned short m_type, unsigned int ftt_id)
{
	unsigned char *temp_p;

	switch(m_type){
		case M_FTT_MANAGEMENT_AM: temp_p = m_manag_db_table_backtime->am_ids; break;
		case M_FTT_MANAGEMENT_SM: temp_p = m_manag_db_table_backtime->sm_ids; break;
		default: return -1;	
	}

	if ( !(ftt_id < MESSAGE_IDS_MAX) )
		return -2;

	temp_p[ftt_id] = 0;

	return 0;
}

static signed char M_MANAG_DB_locate_id( M_FTT_MANAGEMENT_L_APP_ID type_and_id, unsigned short *ret_ftt_id )
{
	unsigned short i, counter=0;

	for (i=0; i<(MESSAGE_IDS_MAX*2); i++){
		if ( M_FTT_MANAGEMENT_L_APP_ID_COMPARE( (m_manag_db_table_backtime->m_message)[i].type_and_id, type_and_id) ){
			/* Found ID */
			if (ret_ftt_id) *ret_ftt_id = (m_manag_db_table_backtime->m_message)[i].ftt_id_idx;
			return i;
		}
		if ( (++counter) == m_manag_db_table_backtime->m_nvars )
			break;
	}
	return -1;
}

static signed char M_MANAG_DB_allocate_id( M_FTT_MANAGEMENT_L_APP_ID type_and_id, unsigned short *ret_ftt_id )
{
	signed int free_slot;
	unsigned short i, counter;
	int ret;
	/* Search duplication while saving the first empty slot */
	free_slot = -1;

	counter = 0;
	for (i=0; i<(MESSAGE_IDS_MAX*2); i++){
		if ( M_FTT_MANAGEMENT_L_APP_ID_COMPARE( (m_manag_db_table_backtime->m_message)[i].type_and_id, type_and_id) ){
			/* Found ID */
			//PRINT_MSG("Found ID: id was already here");
			if (ret_ftt_id) *ret_ftt_id = (m_manag_db_table_backtime->m_message)[i].ftt_id_idx;
			return i;
		}
		if (free_slot == -1){
			if ( (m_manag_db_table_backtime->m_message)[i].type_and_id.type == M_FTT_MANAGEMENT_NOT_INUSE )
				free_slot = i;
		} else{
			if ( (++counter) == m_manag_db_table_backtime->m_nvars )
				break;
		}
	}

	/* At this point:
	 *   -no duplication
	 *   -if free_slot==-1 then full */
	if (free_slot == -1){
		ERROR_MSG("No available slot");
		return -1;
	}

	if ( (ret = M_MANAG_DB_get_ftt_id_entry( type_and_id.type )) < 0  )
		return -3;


	(m_manag_db_table_backtime->m_message)[free_slot].type_and_id = type_and_id;
	(m_manag_db_table_backtime->m_message)[free_slot].ftt_id_idx = ret;
	(m_manag_db_table_backtime->m_message)[free_slot].new_flag = 1;

	/* This is a new slot, let's reset the requirements */
	(m_manag_db_table_backtime->m_message)[free_slot].con.status = 0;
	(m_manag_db_table_backtime->m_message)[free_slot].load.status = 0;
	(m_manag_db_table_backtime->m_message)[free_slot].qos.status = 0;
	m_manag_db_table_backtime->m_nvars ++;

	if (ret_ftt_id) *ret_ftt_id = (unsigned short) ret;

	return free_slot;
}

static signed char M_MANAG_DB_deallocate_id( M_FTT_MANAGEMENT_L_APP_ID type_and_id )
{
	unsigned short i;

	for (i=0; i<(MESSAGE_IDS_MAX*2); i++){
		if ( M_FTT_MANAGEMENT_L_APP_ID_COMPARE ((m_manag_db_table_backtime->m_message)[i].type_and_id, type_and_id) ){
			/* Found ID */

			/* Remove this ftt_id from the list */
			M_MANAG_DB_del_ftt_id_entry(
					(m_manag_db_table_backtime->m_message)[i].type_and_id.type,
				       	(m_manag_db_table_backtime->m_message)[i].ftt_id_idx);


			(m_manag_db_table_backtime->m_message)[i].type_and_id.type = M_FTT_MANAGEMENT_NOT_INUSE;

			return 0;
		}
	}

	/* Could not find this ID */
	return -1;
}


signed int M_MANAG_DB_Get_ftt_id( M_FTT_MANAGEMENT_L_APP_ID app_id)
{
	int ret;
	unsigned short ftt_id;
	
	/* Check lock flag */
	if ( !M_MANAG_DB_TABTYPE_IsLocked( m_manag_db_table_backtime ) ){
		ERROR_MSG("Backtime not locked");
		return -1;
	}
	
	/* Get a slot (if it doen't already has)
	 *  -It will associate the slot to an ftt_id
	 *  -Remember that this slot ID might have already been there. With data! */
	if ( (ret = M_MANAG_DB_locate_id( app_id, &ftt_id )) < 0 ){
		ERROR_MSG("M_MANAG_DB_locate_id ret: %d - This ID is not yet registererd", ret);
		return -2;
	}
	
	return ftt_id;
}


signed int M_MANAG_DB_Add_requirements( 
		M_FTT_MANAGEMENT_L_APP_ID app_id,
		M_FTT_MANAGEMENT_L_LOAD_REQUIREMENTS *load_req,
		M_FTT_MANAGEMENT_L_QOS_REQUIREMENTS *qos_req,
		M_FTT_MANAGEMENT_L_CONNECTION_SET *connect_req,
		unsigned char force_new
		)
{
	int ret;
	//unsigned short i;
	M_MANAG_DB_ITEM_TYPE *item_slot;
	unsigned short ftt_id;
	signed short old_prio;

	/* Check lock flag */
	if ( !M_MANAG_DB_TABTYPE_IsLocked( m_manag_db_table_backtime ) ){
		ERROR_MSG("Backtime not locked");
		return -1;
	}

	#ifdef MASTER_PRINT_MANAGEMENT_UPDATES
	PRINT_MSG("mmmm - M_MANAG_DB_Add_requirements");
	#endif

	/* Get a slot (if it doen't already has)
	 *  -It will associate the slot to an ftt_id
	 *  -Remember that this slot ID might have already been there. With data! 
	 *  -The new_flag struct goes 1 if first allocation */
	if ( (ret = M_MANAG_DB_allocate_id( app_id, &ftt_id )) < 0 ){
		ERROR_MSG("M_MANAG_DB_allocate_id ret: %d", ret);
		return -2;
	}
	#ifdef MASTER_PRINT_MANAGEMENT_UPDATES
	PRINT_MSG("mmm app_id:"); M_FTT_MANAGEMENT_L_APP_ID_print(&app_id,0);
	#endif

	item_slot = &( (m_manag_db_table_backtime->m_message)[ret] );

	if (force_new) item_slot->new_flag = 1;

	/* Lets check the Connections - overwriting */
	if (connect_req!=NULL){
		/* Hey dude! we have something to connect */
		/* The connections that could might be registered will disappear */
		item_slot->con.req = *connect_req;

		if ( M_FTT_MANAGEMENT_L_CONNECTION_SET_validate( &(item_slot->con.req) ) )
			item_slot->con.status = 1; /* Possibly complete */
		else
			item_slot->con.status = 2; /* Incomplete */
		#ifdef MASTER_PRINT_MANAGEMENT_UPDATES
		PRINT_MSG("mmm connection reqs...");
		#endif
	}
	
	if (load_req!=NULL){
		/* Let's overwrite the existing requirements */
		item_slot->load.status = 0;
		if ( M_FTT_MANAGEMENT_L_LOAD_REQUIREMENTS_validate( load_req ) ){
			item_slot->load.status = 1;
			item_slot->load.req = *load_req;
			/* Re-check with the qos parameters */
			if (item_slot->qos.status == 1){
				if ( M_FTT_MANAGEMENT_L_QOS_REQUIREMENTS_validate( &(item_slot->qos.req), load_req ) ){
					item_slot->qos.status = 1;
				}
				else{
					item_slot->qos.status = 0;
					return -4;
				}
			}
		}
		#ifdef MASTER_PRINT_MANAGEMENT_UPDATES
		PRINT_MSG("mmm load reqs...");
		#endif
	}
	
	if (qos_req!=NULL){
		/* Let's overwite the existing requirements */
		if (item_slot->load.status == 1){
			switch( M_FTT_MANAGEMENT_L_QOS_REQUIREMENTS_validate( qos_req, &(item_slot->load.req) ) ){
				case 0:
					item_slot->qos.status = 0;
					return -3;
					break;
				case 1: /* qos_pairs_no > 0 */
					/* Cpy the all structure */
					if (item_slot->load.status == 1) old_prio = item_slot->qos.req.qos_share;
					else old_prio = 0;

					/* Cpy the all structure */
					item_slot->qos.req = *qos_req;
					item_slot->qos.status = 1;

					/* Check if the prio was to keep as it was */
					if (item_slot->qos.req.qos_share < 0) item_slot->qos.req.qos_share = old_prio;
					break;
				case 2: /* qos_pairs_no == 0 */
				default:
					/* Append only one pair */

					/* It's mandatory that other DATA has already been added */
					if ( item_slot->qos.status == 0 ) return -3;

					/* Special validation */
					(item_slot->qos.req.qos_pairs)[item_slot->qos.req.qos_pairs_no] = 
						qos_req->qos_pairs[0];
					(item_slot->qos.req.qos_pairs_no)++;
					if ( ! M_FTT_MANAGEMENT_L_QOS_REQUIREMENTS_validate( &(item_slot->qos.req), &(item_slot->load.req) ) ){
						/*Failed - let's remove */
						ERROR_MSG("Failed on add_qos_pair validation ");
						(item_slot->qos.req.qos_pairs_no)--;
						return -3;
					}
					
			}
		}else{
			item_slot->qos.status = 0;
			ERROR_MSG("You are trying to set qos data with no Load requirements");
		}
		#ifdef MASTER_PRINT_MANAGEMENT_UPDATES
		PRINT_MSG("mmm load reqs...");
		#endif
	}
	
	#ifdef MASTER_PRINT_MANAGEMENT_UPDATES
	PRINT_MSG("mmm load.status%d qos.status%d con.status%d",item_slot->load.status, item_slot->qos.status, item_slot->con.status );
	#endif
	return ftt_id;
}

signed char M_MANAG_DB_Del( 
		M_FTT_MANAGEMENT_L_APP_ID app_id )
{
	/* Check lock flag */
	if ( !M_MANAG_DB_TABTYPE_IsLocked( m_manag_db_table_backtime ) ){
		ERROR_MSG("Backtime not locked");
		return -1;
	}

	if (M_MANAG_DB_deallocate_id( app_id ) < 0)
		return -2;
	return 0;
}

signed int M_MANAG_DB_Add_connection( M_FTT_MANAGEMENT_L_APP_ID app_id, M_FTT_MANAGEMENT_L_CONNECTION nodeID, unsigned char direction )
/* direction: 0-producer    1-consumer */
{
	int ret;
	//unsigned short i;
	M_MANAG_DB_ITEM_TYPE *item_slot;
	unsigned short ftt_id;
	
	/* Check lock flag */
	if ( !M_MANAG_DB_TABTYPE_IsLocked( m_manag_db_table_backtime ) ){
		ERROR_MSG("Backtime not locked");
		return -1;
	}
	#ifdef MASTER_PRINT_MANAGEMENT_UPDATES
	PRINT_MSG("mmmmmmmmmmmmmmmmmmmmm - M_MANAG_DB_Add_connection");
	#endif

	/* Get a slot (if it doen't already has)
	 *  -It will associate the slot to an ftt_id
	 *  -Remember that this slot ID might have already been there. With data! */
	if ( (ret = M_MANAG_DB_allocate_id( app_id, &ftt_id )) < 0 ){
		ERROR_MSG("M_MANAG_DB_allocate_id ret: %d", ret);
		return -2;
	}

	#ifdef MASTER_PRINT_MANAGEMENT_UPDATES
	PRINT_MSG("mmm app_id:"); M_FTT_MANAGEMENT_L_APP_ID_print(&app_id,0);
	#endif

	item_slot = &( (m_manag_db_table_backtime->m_message)[ret] );


	/* Merge the connection */
	if ( item_slot->con.status == 0 ){
		/* Initialize the connection structure */
		M_FTT_MANAGEMENT_L_CONNECTION_SET_clear( &(item_slot->con.req) );
	}

	/* Now we can simply merge the connection in the structure */
	switch (direction){
		case 0: /* Producer */
			ret = M_FTT_MANAGEMENT_L_CONNECTION_SET_merge_producer( &(item_slot->con.req), nodeID);
			break;
		default:
			ret = M_FTT_MANAGEMENT_L_CONNECTION_SET_merge_consumer( &(item_slot->con.req), nodeID);
			/* Consumer */
		break;	
	}

	if (ret < 0){
		ERROR_MSG("Most probabbly you are merging a producer already occupied ");
		return -3;
	}

	if ( M_FTT_MANAGEMENT_L_CONNECTION_SET_validate( &(item_slot->con.req) ) )
		item_slot->con.status = 1; /* Possibly complete */
	else
		item_slot->con.status = 2; /* Incomplete */

	#ifdef MASTER_PRINT_MANAGEMENT_UPDATES
	PRINT_MSG("mmm load.status%d qos.status%d con.status%d",item_slot->load.status, item_slot->qos.status, item_slot->con.status );
	#endif
	
	return ftt_id;
}

static signed char M_MANAG_DB_remove_con (M_MANAG_DB_ITEM_TYPE *item_slot, M_FTT_MANAGEMENT_L_CONNECTION nodeID)
{
	unsigned short i;
	if (item_slot->con.status != 0){
		//if ( item_slot->con.req.Producers_no )
		//	if (item_slot->con.req.Producer_nodeID == nodeID){
		//		item_slot->con.req.Producers_no = 0;
		//		//return 0;
		//	}
		
		for (i=0; i<item_slot->con.req.Producers_no; i++){
			if ( (item_slot->con.req.Producer_nodeID)[i] == nodeID ){
				for (; i<(item_slot->con.req.Producers_no-1); i++)
					(item_slot->con.req.Producer_nodeID)[i] = (item_slot->con.req.Producer_nodeID)[i+1];
				item_slot->con.req.Producers_no --;
				break;
			}
		}
	
		for (i=0; i<item_slot->con.req.Consumers_no; i++){
			if ( (item_slot->con.req.Consumer_nodeID)[i] == nodeID ){
				for (; i<(item_slot->con.req.Consumers_no-1); i++)
					(item_slot->con.req.Consumer_nodeID)[i] = (item_slot->con.req.Consumer_nodeID)[i+1];
				item_slot->con.req.Consumers_no --;
				break;
			}
		}
	}
	
	if ( M_FTT_MANAGEMENT_L_CONNECTION_SET_validate( &(item_slot->con.req) ) )
		item_slot->con.status = 1; /* Possibly complete */
	else
		item_slot->con.status = 2; /* Incomplete */

	/* Check if the registry got empty with no data associated */
	if ( ( item_slot->con.req.Consumers_no == 0 ) && ( item_slot->con.req.Producers_no == 0 ) ){
		if ( (item_slot->load.status == 0) && (item_slot->qos.status == 0) )
			M_MANAG_DB_Del( item_slot->type_and_id );
	}
	return 0;
}

signed int M_MANAG_DB_Del_connection( M_FTT_MANAGEMENT_L_APP_ID app_id, M_FTT_MANAGEMENT_L_CONNECTION nodeID )
{
	/* seek the app_id */
	unsigned short i;

	/* Check lock flag */
	if ( !M_MANAG_DB_TABTYPE_IsLocked( m_manag_db_table_backtime ) ){
		ERROR_MSG("Backtime not locked");
		return -1;
	}

	for (i=0; i<(MESSAGE_IDS_MAX*2); i++){
		if ( M_FTT_MANAGEMENT_L_APP_ID_COMPARE( (m_manag_db_table_backtime->m_message)[i].type_and_id, app_id) ){
			/* Found ID */
			break;
		}
	}

	if ( i == (MESSAGE_IDS_MAX*2) )	/* Could not find this ID */
		return -2;

	M_MANAG_DB_remove_con( &((m_manag_db_table_backtime->m_message)[i]), nodeID);

	return 0;
}

signed int M_MANAG_DB_Del_node( M_FTT_MANAGEMENT_L_CONNECTION nodeID )
{

	unsigned short i;

	/* Check lock flag */
	if ( !M_MANAG_DB_TABTYPE_IsLocked( m_manag_db_table_backtime ) ){
		ERROR_MSG("Backtime not locked");
		return -1;
	}

	for (i=0; i<(MESSAGE_IDS_MAX*2); i++){
		if ( (m_manag_db_table_backtime->m_message)[i].type_and_id.type != M_FTT_MANAGEMENT_NOT_INUSE )
			M_MANAG_DB_remove_con( &((m_manag_db_table_backtime->m_message)[i]), nodeID);
	}

	return 0;
}

signed int M_MANAG_DB_Chg_period( 
		M_FTT_MANAGEMENT_L_APP_ID app_id,
		unsigned short period
		)
{
	int ret;
	M_MANAG_DB_ITEM_TYPE *item_slot;

	/* Check lock flag */
	if ( !M_MANAG_DB_TABTYPE_IsLocked( m_manag_db_table_backtime ) ){
		ERROR_MSG("Backtime not locked");
		return -1;
	}

	#ifdef MASTER_PRINT_MANAGEMENT_UPDATES
	PRINT_MSG("mmmm - M_MANAG_DB_Chg_period");
	#endif

	/* Get a slot (if it doen't already has)
	 *  -It will associate the slot to an ftt_id
	 *  -Remember that this slot ID might have already been there. With data! 
	 *  -The new_flag struct goes 1 if first allocation */
	if ( (ret = M_MANAG_DB_locate_id( app_id, NULL )) < 0 ){
		ERROR_MSG("M_MANAG_DB_locate_id ret: %d", ret);
		return -2;
	}
	#ifdef MASTER_PRINT_MANAGEMENT_UPDATES
	PRINT_MSG("mmm app_id:"); M_FTT_MANAGEMENT_L_APP_ID_print(&app_id,0);
	#endif

	item_slot = &( (m_manag_db_table_backtime->m_message)[ret] );

	if ( item_slot->load.status == 0 ){
		ERROR_MSG("Changing period with no load requirements");
		return -1;
	}
	
	item_slot->load.req.m_period = period;
	item_slot->load.req.m_deadline = period;

	item_slot->load.status = ( M_FTT_MANAGEMENT_L_LOAD_REQUIREMENTS_validate( &(item_slot->load.req) ) != 0 );
	item_slot->qos.status = ( M_FTT_MANAGEMENT_L_QOS_REQUIREMENTS_validate( &(item_slot->qos.req), &(item_slot->load.req) ) != 0 );
	
	#ifdef MASTER_PRINT_MANAGEMENT_UPDATES
	PRINT_MSG("mmm load.status%d qos.status%d con.status%d",item_slot->load.status, item_slot->qos.status, item_slot->con.status );
	#endif

	return 0;
}
signed int M_MANAG_DB_Chg_qos_prio( 
		M_FTT_MANAGEMENT_L_APP_ID app_id,
		unsigned short prio
		)
{
	int ret;
	M_MANAG_DB_ITEM_TYPE *item_slot;

	/* Check lock flag */
	if ( !M_MANAG_DB_TABTYPE_IsLocked( m_manag_db_table_backtime ) ){
		ERROR_MSG("Backtime not locked");
		return -1;
	}

	#ifdef MASTER_PRINT_MANAGEMENT_UPDATES
	PRINT_MSG("mmmm - M_MANAG_DB_Chg_qos_prio");
	#endif

	/* Get a slot (if it doen't already has)
	 *  -It will associate the slot to an ftt_id
	 *  -Remember that this slot ID might have already been there. With data! 
	 *  -The new_flag struct goes 1 if first allocation */
	if ( (ret = M_MANAG_DB_locate_id( app_id, NULL )) < 0 ){
		ERROR_MSG("M_MANAG_DB_locate_id ret: %d", ret);
		return -2;
	}
	#ifdef MASTER_PRINT_MANAGEMENT_UPDATES
	PRINT_MSG("mmm app_id:"); M_FTT_MANAGEMENT_L_APP_ID_print(&app_id,0);
	#endif

	item_slot = &( (m_manag_db_table_backtime->m_message)[ret] );

	if ( item_slot->qos.status == 0 ){
		ERROR_MSG("Changing qos priority with no qos requirements");
		return -1;
	}
	
	item_slot->qos.req.qos_share = prio;

	item_slot->qos.status = ( M_FTT_MANAGEMENT_L_QOS_REQUIREMENTS_validate( &(item_slot->qos.req), &(item_slot->load.req) ) != 0 );
	
	#ifdef MASTER_PRINT_MANAGEMENT_UPDATES
	PRINT_MSG("mmm load.status%d qos.status%d con.status%d",item_slot->load.status, item_slot->qos.status, item_slot->con.status );
	#endif

	return 0;
}

signed char M_MANAG_DB_Get_runtime_table( M_MANAG_DB_TABTYPE ***table )
{
	*table = &(m_manag_db_table_runtime);
	return 0;
}

signed char M_MANAG_DB_Get_backtime_table( M_MANAG_DB_TABTYPE ***table )
{
	if ( !M_MANAG_DB_TABTYPE_IsLocked( m_manag_db_table_backtime ) ){
		ERROR_MSG("Backtime not locked");
		return -1;
	}

	*table = &(m_manag_db_table_backtime);
	return 0;
}

signed char M_MANAG_DB_Get_next_runtime_item(
		unsigned short *ftt_id,
		unsigned char *m_type,
		M_FTT_MANAGEMENT_L_LOAD_REQUIREMENTS **load_p,
		M_FTT_MANAGEMENT_L_QOS_REQUIREMENTS **qos_p,
		M_FTT_MANAGEMENT_L_CONNECTION_SET **con_p
		)
{
	M_MANAG_DB_ITEM_TYPE *temp_p;

	/* Check lock flag */
	if ( !M_MANAG_DB_TABTYPE_IsLocked( m_manag_db_table_runtime ) ){
		ERROR_MSG("Runtime not locked");
		return -1;
	}

	/* Check if there are items to retrieve */
	if ( M_MANAG_DB_Access_counter == m_manag_db_table_runtime->m_nvars )
		return -2;

	temp_p = &(m_manag_db_table_runtime->m_message[M_MANAG_DB_Access_counter]);

	*ftt_id = temp_p->ftt_id_idx;
	*m_type = temp_p->type_and_id.type;

	*load_p = NULL;
	if (temp_p->load.status)
		*load_p = &(temp_p->load.req);

	*qos_p = NULL;
	if (temp_p->qos.status)
		*qos_p = &(temp_p->qos.req);

	*con_p = NULL;
	if (temp_p->con.status)
		*con_p = &(temp_p->con.req);

	/* Increment the access counter */
	M_MANAG_DB_Access_counter ++;

	return 0;
}

void M_MANAG_DB_Print_runtime( unsigned char detailed )
{
	M_MANAG_DB_TABTYPE_EnterLock(m_manag_db_table_runtime);

	M_MANAG_DB_TABTYPE_Print(m_manag_db_table_runtime, detailed);

	M_MANAG_DB_TABTYPE_LeaveLock(m_manag_db_table_runtime);
}

