/*****************************************************************************
 * M_manag_db.h:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#ifndef  M_MANAG_DB_H
#define  M_MANAG_DB_H

#include "ftt-management.layer_types.h"

#define MESSAGE_IDS_MAX 50

typedef struct{
	unsigned char status; //0-not set    1- set complete   2- set incomplete
	M_FTT_MANAGEMENT_L_CONNECTION_SET req;
}M_FTT_MANAG_DB_CON_TYPE;

typedef struct{
	unsigned char status; //0-1
	M_FTT_MANAGEMENT_L_LOAD_REQUIREMENTS req;
}M_FTT_MANAG_DB_LOAD_TYPE;

typedef struct{
	unsigned char status; //0-1
	M_FTT_MANAGEMENT_L_QOS_REQUIREMENTS req;
}M_FTT_MANAG_DB_QOS_TYPE;


typedef struct{
	M_FTT_MANAGEMENT_L_APP_ID type_and_id;
	unsigned char new_flag; //goes 1 when instantiated, goes 0 when sent down
	signed short ftt_id_idx; //ID with no type
	M_FTT_MANAG_DB_CON_TYPE con;
	M_FTT_MANAG_DB_LOAD_TYPE load;
	M_FTT_MANAG_DB_QOS_TYPE qos;
}M_MANAG_DB_ITEM_TYPE;


typedef struct{
	unsigned char m_nvars;
	M_MANAG_DB_ITEM_TYPE m_message[MESSAGE_IDS_MAX*2];

	unsigned char am_ids[MESSAGE_IDS_MAX]; //ftt_ids 0/1
	unsigned char sm_ids[MESSAGE_IDS_MAX];

	rtl_pthread_mutex_t mutex_locker;  //locker
	unsigned char locker_flag;	// 0/1
}M_MANAG_DB_TABTYPE;

inline static signed char M_MANAG_DB_TABTYPE_Init(M_MANAG_DB_TABTYPE *p)
{
	unsigned int i;
	
	p->m_nvars = 0;
	for (i=0; i<(MESSAGE_IDS_MAX*2); i++){
		p->m_message[i].type_and_id.type = M_FTT_MANAGEMENT_NOT_INUSE;
	}

	for (i=0; i<MESSAGE_IDS_MAX; i++){
		p->am_ids[i] = 0;
		p->sm_ids[i] = 0;
	}

	rtl_pthread_mutex_init(&(p->mutex_locker), 0);
	p->locker_flag = 0;

	return 0;
}
inline static signed char M_MANAG_DB_TABTYPE_Close(M_MANAG_DB_TABTYPE *p)
{
	rtl_pthread_mutex_destroy(&(p->mutex_locker));
	p->locker_flag = 0;
	return 0;
}
inline static signed char M_MANAG_DB_TABTYPE_EnterLock(M_MANAG_DB_TABTYPE *p)
{
	rtl_pthread_mutex_lock(&(p->mutex_locker));
	p->locker_flag = 1;
	return 0;
}
inline static signed char M_MANAG_DB_TABTYPE_LeaveLock(M_MANAG_DB_TABTYPE *p)
{
	p->locker_flag = 0;
	rtl_pthread_mutex_unlock(&(p->mutex_locker));
	return 0;
}
inline static signed char M_MANAG_DB_TABTYPE_IsLocked(M_MANAG_DB_TABTYPE *p)
{
	return p->locker_flag;
}

inline static signed int M_MANAG_DB_TABTYPE_Get_item_not_inuse(M_MANAG_DB_TABTYPE *p)
{
	unsigned int i;
	for (i=0; i<(MESSAGE_IDS_MAX*2); i++ ){
		if (p->m_message[i].type_and_id.type == M_FTT_MANAGEMENT_NOT_INUSE)
			return i;
	}
	
	return -1;
}

extern void M_MANAG_DB_TABTYPE_Print(M_MANAG_DB_TABTYPE *p, unsigned char detailed_flag);

extern signed char M_MANAG_DB_Init(void);
extern signed char M_MANAG_DB_Close(void);
extern signed char M_MANAG_DB_Changes_lock(void);
extern signed char M_MANAG_DB_Changes_abort_unlock(void);
extern signed char M_MANAG_DB_Changes_submit_unlock(void);

extern signed char M_MANAG_DB_Access_lock(void);
extern signed char M_MANAG_DB_Access_unlock(void);

extern signed int M_MANAG_DB_Get_ftt_id( M_FTT_MANAGEMENT_L_APP_ID app_id);

extern signed int M_MANAG_DB_Add_requirements( 
		M_FTT_MANAGEMENT_L_APP_ID app_id,
		M_FTT_MANAGEMENT_L_LOAD_REQUIREMENTS *load_req,
		M_FTT_MANAGEMENT_L_QOS_REQUIREMENTS *qos_req,
		M_FTT_MANAGEMENT_L_CONNECTION_SET *connect_req,
		unsigned char force_new
		);

extern signed char M_MANAG_DB_Del( M_FTT_MANAGEMENT_L_APP_ID app_id );

extern signed int M_MANAG_DB_Add_connection( M_FTT_MANAGEMENT_L_APP_ID app_id, M_FTT_MANAGEMENT_L_CONNECTION nodeID, unsigned char direction );

extern signed int M_MANAG_DB_Del_connection( M_FTT_MANAGEMENT_L_APP_ID app_id, M_FTT_MANAGEMENT_L_CONNECTION nodeID );

extern signed int M_MANAG_DB_Add_blacklisted_ftt_id(
		unsigned short ftt_id,
		unsigned char m_type,
		M_FTT_MANAGEMENT_L_LOAD_REQUIREMENTS *load_p,
		M_FTT_MANAGEMENT_L_CONNECTION_SET *con_p
		);

extern signed int M_MANAG_DB_Del_node( M_FTT_MANAGEMENT_L_CONNECTION nodeID );

extern signed int M_MANAG_DB_Chg_period( 
		M_FTT_MANAGEMENT_L_APP_ID app_id,
		unsigned short period
		);

extern signed int M_MANAG_DB_Chg_qos_prio( 
		M_FTT_MANAGEMENT_L_APP_ID app_id,
		unsigned short prio
		);

extern signed char M_MANAG_DB_Get_next_runtime_item(
		unsigned short *ftt_id,
		unsigned char *m_type,
		M_FTT_MANAGEMENT_L_LOAD_REQUIREMENTS **load_p,
		M_FTT_MANAGEMENT_L_QOS_REQUIREMENTS **qos_p,
		M_FTT_MANAGEMENT_L_CONNECTION_SET **con_p
		);
extern signed char M_MANAG_DB_Get_runtime_table(M_MANAG_DB_TABTYPE ***table);

extern signed char M_MANAG_DB_Get_backtime_table( M_MANAG_DB_TABTYPE ***table );

extern void M_MANAG_DB_Print_runtime( unsigned char detailed );

#endif   /* ----- #ifndef M_MANAG_DB_FILE_HEADER_INC  ----- */

