/*****************************************************************************
 * M_manag_qos.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#include "Ports.h"

#include "M_manag_qos.h"
#include "M_manag_db.h"

#include "M_ftt-core.layer.h"

#define MAX_NODES_NO 50 //Each node conects in a switch port
#define MAX_NO_VARS_PER_LINK 10000

#define MAX_NO_OF_VARS_SAME_PRIO 30

//#define POLITEE

typedef struct{
	double up_link_load;
	double down_link_load;
	unsigned short down_link_vars_no;
	M_MANAG_DB_ITEM_TYPE *down_link_vars_direct_id[MAX_NO_VARS_PER_LINK];

	double down_link_indirect_load;

	int aux_up_no;
	int aux_down_no;

	unsigned char in_use;
}PER_NODE_LOAD;

static PER_NODE_LOAD nodes_load[MAX_NODES_NO]; //the nodes will be indexed by the NodeID they got




//static void nodes_load_print(void)
//{
//	int i;
//	double val1, val2;
//	PRINT_MSG("nodes_load structure print:");
//	for ( i=0; i<MAX_NODES_NO; i++ ){
//		val1 = nodes_load[i].up_link_load/1000000;
//		val2 = (nodes_load[i].down_link_load + nodes_load[i].down_link_indirect_load)*1e-6;
//		if (nodes_load[i].in_use){
//			PRINT_MSG("NodeID%d  load_up: %d.%dMbps load_down:%d.%dMbps",
//				i,
//				(int)(val1), (int)(10*(val1-(int)val1)),
//				(int)(val2), (int)(10*(val2-(int)val2)) );
//		}
//	}
//}

static inline unsigned char message_filter_out( M_MANAG_DB_ITEM_TYPE *temp_p, M_FTT_MANAGEMENT_L_MSG_TYPE msg_type_filter )
{
	switch (temp_p->type_and_id.type){
		case M_FTT_MANAGEMENT_AM:
		case M_FTT_MANAGEMENT_AM_DUMMY:
//				PRINTF("A");
			if ( (msg_type_filter != M_FTT_MANAGEMENT_AM) && (msg_type_filter != M_FTT_MANAGEMENT_AM_DUMMY) )
				return 1;
			break;
		case M_FTT_MANAGEMENT_SM:
		case M_FTT_MANAGEMENT_SM_DUMMY:
//				PRINTF("S");
			if ( (msg_type_filter != M_FTT_MANAGEMENT_SM) && (msg_type_filter != M_FTT_MANAGEMENT_SM_DUMMY) )
				return 1;
			break;
		default:
			return 1;
	}
		
	/* Let's check if this is a valid message */
	if ( (temp_p->load.status != 1) || (temp_p->con.status != 1) ){
		return 1;
	}

	return 0;
}

static inline unsigned char is_var_in_uplink( M_MANAG_DB_ITEM_TYPE *temp_p, unsigned short uplink )
{
	unsigned short i;
	for (i=0; i<temp_p->con.req.Producers_no; i++)
		if ( (temp_p->con.req.Producer_nodeID)[i] == uplink )
			return 1;
	return 0;
//	return ( temp_p->con.req.Producer_nodeID == uplink );
}

static inline unsigned char is_var_in_downlink( M_MANAG_DB_ITEM_TYPE *temp_p, unsigned short downlink )
{
	unsigned short i;
	for (i=0; i<temp_p->con.req.Consumers_no; i++)
		if ( (temp_p->con.req.Consumer_nodeID)[i] == downlink )
			return 1;
	return 0;
}

static inline unsigned short get_max_interf_var_period( M_MANAG_DB_ITEM_TYPE *temp_p )
{
	/* Since I don't know which will be the assigned period I have to use the minimum */
	int i;
	int min_T;
	int temp;

	/* get the smaller T */
	min_T = temp_p->load.req.m_period; /* Start */
	if (temp_p->qos.status == 1) /* is qos defined */
		for (i=0; i < (temp_p->qos.req.qos_pairs_no); i++){
			temp = (temp_p->qos.req.qos_pairs)[i].period;
			if (temp < 0) temp=-temp;
			if ((temp != 0) && (temp < min_T))
				min_T = temp;
		}

	return min_T;
}

static inline unsigned short get_max_interf_var_size( M_MANAG_DB_ITEM_TYPE *temp_p )
{
	/* Since I don't know which will be the assigned period I have to use the minimum */
	int i;
	int max_C;
	int temp;

	/* get the smaller T */
	max_C = temp_p->load.req.m_size; /* Start */
	if (temp_p->qos.status == 1) /* is qos defined */
		for (i=0; i < (temp_p->qos.req.qos_pairs_no); i++){
			temp = (temp_p->qos.req.qos_pairs)[i].size;
			if (temp < 0) temp=-temp;
			if ((temp != 0) && (temp < max_C))
				max_C = temp;
		}

	return max_C;
}

/* returns the number of vars */
static signed int M_manag_populate_links_min_load( M_FTT_MANAGEMENT_L_MSG_TYPE msg_type_filter, double *max_pkt_load )
/*fills the auxiliar structure nodes_load with the minimum requirements.*/
{
	unsigned short i,j;
	double load_i;
	unsigned int load_i_byte_size;
	unsigned long int load_i_us;
	
	double load_i_pkt_load;


	M_MANAG_DB_TABTYPE **back_table;
	M_MANAG_DB_ITEM_TYPE *temp_p;
	unsigned short counter=0;
	unsigned long int ec_time = M_FTT_CORE_L_get_EC_time();

	unsigned short temp_min_T;
	unsigned int temp_max_J_interference;
	unsigned int temp_J_interference;
	unsigned short temp_max_T; //for the minimum prio
	M_MANAG_DB_ITEM_TYPE *lowest_prio_p;
	unsigned char uplink;

	RET_SCHED_POLICIES policy = M_FTT_CORE_L_ret_sm_sched_policy();
#ifdef POLITEE
	PRINT_MSG(" populating the load var:");
#endif

	/*Clear the nodes load*/
	for ( i=0; i<MAX_NODES_NO; i++ ){
		nodes_load[i].up_link_load = 0;
		nodes_load[i].down_link_load = 0;
		nodes_load[i].down_link_vars_no = 0;
		nodes_load[i].down_link_indirect_load = 0;
		nodes_load[i].in_use = 0;
	}

	/* Get the backtime table */
	if ( M_MANAG_DB_Get_backtime_table( &back_table ) < 0 )
		return -1;

	*max_pkt_load=0;

	/* For each var in the backtime table */
	for (i=0; i< (*back_table)->m_nvars; i++ ){
//		PRINT_MSG("%d ", i);

		/* Filter out */
		if ( message_filter_out( temp_p = &(((*back_table)->m_message)[i]), msg_type_filter ) )
			continue;

//		PRINTF("pass");

		counter ++;

		load_i_byte_size = M_FTT_CORE_L_get_message_bytesize(
				(msg_type_filter == M_FTT_MANAGEMENT_SM)?SYNC_MESSAGE:ASYNC_MESSAGE,
				(temp_p->load.req.m_size),
				(temp_p->load.req.m_MTU ) );
		load_i_us = (temp_p->load.req.m_period) * ec_time/*us*/;

		load_i = (double)(load_i_byte_size * 8) /( (double)(load_i_us) * 1e-6);

		load_i_pkt_load = (double)(M_FTT_CORE_L_get_max_per_pkt_bytesize(
				(msg_type_filter == M_FTT_MANAGEMENT_SM)?SYNC_MESSAGE:ASYNC_MESSAGE,
				(temp_p->load.req.m_size),
				(temp_p->load.req.m_MTU ) ) * 8) /( (double)(1) * ec_time/*us*/ * 1e-6);
		
		//TODO make this size the maximum qos size
		*max_pkt_load = (load_i_pkt_load > *max_pkt_load) ? load_i_pkt_load : *max_pkt_load;

		/* Check who produces */

		/****************
		 * I MIGHT HAVE TO RECONSIDER THE ADMISSION CONTROL WITH MANYCAST!
		 * For now i'm only checking producer[0]
		 ****************/
		if ( (temp_p->con.req.Producer_nodeID)[0] >= MAX_NODES_NO){
			ERROR_MSG(" Registered nodeID too big or internal struc too short");
			return -1;
		}
		nodes_load[(temp_p->con.req.Producer_nodeID)[0]].in_use = 1;
		nodes_load[(temp_p->con.req.Producer_nodeID)[0]].up_link_load += load_i;

		/* Check who consumes */
		for (j=0; j<temp_p->con.req.Consumers_no; j++){
			if ( (temp_p->con.req.Consumer_nodeID)[j] >= MAX_NODES_NO){
				ERROR_MSG(" Registered nodeID too big or internal struc too short");
				return -2;
			}
			nodes_load[(temp_p->con.req.Consumer_nodeID)[j]].in_use = 1;
			nodes_load[(temp_p->con.req.Consumer_nodeID)[j]].down_link_load += load_i;
			(nodes_load[(temp_p->con.req.Consumer_nodeID)[j]].down_link_vars_direct_id)[nodes_load[(temp_p->con.req.Consumer_nodeID)[j]].down_link_vars_no++] =  temp_p;
		}
	}

	/* let's now get the indirect load assigned to each downlink */
	for ( i=0; i<MAX_NODES_NO; i++ ){
		if ( !(nodes_load[i].in_use) ) continue;
		if ( !(nodes_load[i].down_link_vars_no) ) continue; //if no directs, then no indirects...
	
		/* Get the minimum T from the directs to this downlink */
		temp_min_T = get_max_interf_var_period((nodes_load[i].down_link_vars_direct_id)[0]);
		for (j=1; j<nodes_load[i].down_link_vars_no; j++ )
			if ( get_max_interf_var_period((nodes_load[i].down_link_vars_direct_id)[j]) < temp_min_T)
				temp_min_T = get_max_interf_var_period((nodes_load[i].down_link_vars_direct_id)[j]);

		/* Get the uplinks maximum interference for this downlink */
		temp_max_J_interference = 0;
		for ( uplink=0; uplink<MAX_NODES_NO; uplink++){
			if ( !(nodes_load[uplink].in_use) ) continue;

			/* Search the lowest priority direct coming from uplink, if any */
			lowest_prio_p = NULL;
			temp_max_T = 0;
			for (j=0; j<nodes_load[i].down_link_vars_no; j++ ){
				temp_p = (nodes_load[i].down_link_vars_direct_id)[j];
				if ( (temp_p->con.req.Producer_nodeID)[0] == uplink )
					if ( get_max_interf_var_period(temp_p) > temp_max_T ){
						temp_max_T = get_max_interf_var_period(temp_p)>temp_max_T;
						lowest_prio_p = temp_p;
					}
			}
			
			if (lowest_prio_p == NULL) continue;

			/* Now, with this lowest_prio_p, let's get the sum(C_interf) from the uplink */
			temp_J_interference=0;

			/* for each var in the uplink */
				/* if the message is not directed to this downlink */
					/* if priority higher */
						/* sum the var */
			for (j=0; j<(*back_table)->m_nvars; j++ ){
				if ( message_filter_out( temp_p = &(((*back_table)->m_message)[j]), msg_type_filter ) )	continue; /* Filter out */
				if ( is_var_in_uplink( temp_p, uplink ) )
					if ( !is_var_in_downlink( temp_p, i ) )
						switch (policy){
							case RET_SCHEDD_EDF:
								temp_J_interference += M_FTT_CORE_L_get_message_bytesize(
										(msg_type_filter == M_FTT_MANAGEMENT_SM)?SYNC_MESSAGE:ASYNC_MESSAGE,
										get_max_interf_var_size(temp_p),
										(temp_p->load.req.m_MTU ) );
								break;
							case RET_SCHEDD_RM:
							default: 
								/*if temp_p is higher or equal prio < T */
								if ( get_max_interf_var_period(temp_p) > get_max_interf_var_period(lowest_prio_p) )
									break;
								temp_J_interference += M_FTT_CORE_L_get_message_bytesize(
										(msg_type_filter == M_FTT_MANAGEMENT_SM)?SYNC_MESSAGE:ASYNC_MESSAGE,
										get_max_interf_var_size(temp_p),
										(temp_p->load.req.m_MTU ) );
								break;
						}

			}
			
			if (temp_J_interference>temp_max_J_interference)
				temp_max_J_interference=temp_J_interference;

		}

		/* compute the indirect load for this downlink */
		nodes_load[i].down_link_indirect_load = (double)(temp_max_J_interference*8)/( (double)( temp_min_T * ec_time) * 1e-6);
	}


	return counter;
}

static signed int M_manag_links_available_load( M_FTT_MANAGEMENT_L_MSG_TYPE msg_type_filter )
{
	int ret;
	double traffic_slot_load;
	double available_load;
//	unsigned short vars_no;
	int i;
	RET_SCHED_POLICIES policy;
	double max_pkt_load;

	/* Get the actual load */
	if ( (ret = M_manag_populate_links_min_load( msg_type_filter, &max_pkt_load ) ) < 0 ){
		ERROR_MSG("M_manag_populate_links_min_load(M_FTT_MANAGEMENT_SM) ret: %d", ret);
		return -1;
	}

	/* change the minimum load for the available one */
	traffic_slot_load = M_FTT_CORE_L_get_SM_slot_load( );
	policy = M_FTT_CORE_L_ret_sm_sched_policy();

	switch(policy){
		case RET_SCHEDD_EDF: available_load = traffic_slot_load - max_pkt_load; break;
		case RET_SCHEDD_RM: available_load = (traffic_slot_load - max_pkt_load)*(0.69); break;
//#warning actualizar com o limite rm correcto
		default: ERROR_MSG("Unespected scheduling policy"); return -2;
	}
//PRINT_MSG("The available load will be: %g   max_pkt_load: %g     traffic_slot_load:%g ", available_load, max_pkt_load, traffic_slot_load );

	for ( i=0; i<MAX_NODES_NO; i++ )
		if (nodes_load[i].in_use){
			nodes_load[i].up_link_load = available_load - nodes_load[i].up_link_load;
			nodes_load[i].down_link_load = available_load - nodes_load[i].down_link_load - nodes_load[i].down_link_indirect_load;
		}

	/* Notice that now nodes_load represents the spare BW to be distributed 
	 * It already accounts with the indirect load that might appear. This is not optimized, though!!!*/

	return 0;
}

static signed int M_manag_prepare_for_distribution( M_FTT_MANAGEMENT_L_MSG_TYPE msg_type_filter )
{
	unsigned short i;
	M_MANAG_DB_TABTYPE **back_table;
	M_MANAG_DB_ITEM_TYPE *temp_p;

	/* Get the backtime table */
	if ( M_MANAG_DB_Get_backtime_table( &back_table ) < 0 )
		return -1;

	for (i=0; i< (*back_table)->m_nvars; i++ ){
		temp_p = &(((*back_table)->m_message)[i]);

		/* Filter */
		switch (temp_p->type_and_id.type){
			case M_FTT_MANAGEMENT_AM:
			case M_FTT_MANAGEMENT_AM_DUMMY:
				if ( (msg_type_filter != M_FTT_MANAGEMENT_AM) && (msg_type_filter != M_FTT_MANAGEMENT_AM_DUMMY) )
					continue;
				break;
			case M_FTT_MANAGEMENT_SM:
			case M_FTT_MANAGEMENT_SM_DUMMY:
				if ( (msg_type_filter != M_FTT_MANAGEMENT_SM) && (msg_type_filter != M_FTT_MANAGEMENT_SM_DUMMY) )
					continue;
				break;
			default:
				continue;
		}

		if ( (temp_p->qos.status != 1) ) continue;

		temp_p->qos.req.qos_given_load = -1.0;
//PRINT_MSG("---> preparing dist: %d", (int)temp_p->qos.req.qos_given_load );		
	}
	
	return 0;
}

static signed short M_manag_get_highest_prio( M_FTT_MANAGEMENT_L_MSG_TYPE msg_type_filter, unsigned short *ret_idx, unsigned short *ret_size )
{
	unsigned short i;
	M_MANAG_DB_TABTYPE **back_table;
	M_MANAG_DB_ITEM_TYPE *temp_p;
	signed short max_prio = -1; //0-minimum prio
	unsigned short idx_max = MAX_NO_OF_VARS_SAME_PRIO;

	*ret_size = 0;

	/* Get the backtime table */
	if ( M_MANAG_DB_Get_backtime_table( &back_table ) < 0 )
		return -1;

	/* For each var in the backtime table */
	for (i=0; i< (*back_table)->m_nvars; i++ ){
		temp_p = &(((*back_table)->m_message)[i]);

		/* Filter */
		switch (temp_p->type_and_id.type){
			case M_FTT_MANAGEMENT_AM:
			case M_FTT_MANAGEMENT_AM_DUMMY:
				if ( (msg_type_filter != M_FTT_MANAGEMENT_AM) && (msg_type_filter != M_FTT_MANAGEMENT_AM_DUMMY) )
					continue;
				break;
			case M_FTT_MANAGEMENT_SM:
			case M_FTT_MANAGEMENT_SM_DUMMY:
				if ( (msg_type_filter != M_FTT_MANAGEMENT_SM) && (msg_type_filter != M_FTT_MANAGEMENT_SM_DUMMY) )
					continue;
				break;
			default:
				continue;
		}

		/* Let's check if this is a valid message to be added into the QoS stuff */
		if (temp_p->qos.status != 1) continue;

		/* Check if this var was already computed */
		if ( temp_p->qos.req.qos_given_load > 0 ) continue;

		/* extend the max_prio's array if the same prio */
		if ( temp_p->qos.req.qos_share == max_prio ){
			if (*ret_size == idx_max ) return -2;
			ret_idx[*ret_size] = i;
			(*ret_size)++;
		}
		
		/* check if new max found - reset the out array*/
		if ( temp_p->qos.req.qos_share > max_prio ){
			max_prio = temp_p->qos.req.qos_share;
			ret_idx[0] = i;
			*ret_size = 1;
		}
	}
//PRINT_MSG("returning:%d", max_prio);
	return max_prio;
}

static signed short M_manag_clear_nodeID_vars_no( void )
{
	unsigned short i;

	for ( i=0; i<MAX_NODES_NO; i++ )
		if (nodes_load[i].in_use){
			nodes_load[i].aux_up_no = 0;
			nodes_load[i].aux_down_no = 0;
		}
	return 0;
}


static double Compute_load_for_var( M_FTT_MANAGEMENT_L_MSG_TYPE type, unsigned int size, unsigned int period, unsigned short MTU )
{

	switch (type){
		case M_FTT_MANAGEMENT_AM:
		case M_FTT_MANAGEMENT_AM_DUMMY:
			return M_FTT_CORE_L_get_message_load( ASYNC_MESSAGE, size, period, MTU );
				break;
		case M_FTT_MANAGEMENT_SM:
		case M_FTT_MANAGEMENT_SM_DUMMY:
			return M_FTT_CORE_L_get_message_load( SYNC_MESSAGE, size, period, MTU );
				break;
		default:
				return -1;
	}
	return -1;
}

static unsigned int Compute_period_for_var( M_FTT_MANAGEMENT_L_MSG_TYPE type, unsigned int size, unsigned short MTU, double load )
{

	switch (type){
		case M_FTT_MANAGEMENT_AM:
		case M_FTT_MANAGEMENT_AM_DUMMY:
			return M_FTT_CORE_L_get_corresponding_period_ticks( ASYNC_MESSAGE, size, MTU, load );
				break;
		case M_FTT_MANAGEMENT_SM:
		case M_FTT_MANAGEMENT_SM_DUMMY:
			return M_FTT_CORE_L_get_corresponding_period_ticks( SYNC_MESSAGE, size, MTU, load );
				break;
		default:
				return -1;
	}
	return -1;
}

static unsigned int Compute_size_for_var( M_FTT_MANAGEMENT_L_MSG_TYPE type, unsigned int period, unsigned short MTU, double load )
{

	switch (type){
		case M_FTT_MANAGEMENT_AM:
		case M_FTT_MANAGEMENT_AM_DUMMY:
			return M_FTT_CORE_L_get_corresponding_size_bytes( ASYNC_MESSAGE, period, MTU, load );
				break;
		case M_FTT_MANAGEMENT_SM:
		case M_FTT_MANAGEMENT_SM_DUMMY:
			return M_FTT_CORE_L_get_corresponding_size_bytes( SYNC_MESSAGE, period, MTU, load );
				break;
		default:
				return -1;
	}
	return -1;
}

static signed short M_manag_common_nodeID_links( unsigned short idx )
{
	unsigned short i;
	M_MANAG_DB_TABTYPE **back_table;
	M_MANAG_DB_ITEM_TYPE *temp_p;
	QOS_BW_PAIR highest_BW;

	/* Get the backtime table */
	if ( M_MANAG_DB_Get_backtime_table( &back_table ) < 0 )
		return -1;

	temp_p = &(((*back_table)->m_message)[idx]);

	/* Get the most demanding BW for this */
	M_FTT_MANAGEMENT_L_QOS_REQUIREMENTS_get_highest_BW_pair( &(temp_p->qos.req), &(temp_p->load.req), &highest_BW );

//	PRINT_MSG("The highest BW pair: %d %d", highest_BW.size, highest_BW.period);

	/* Decrement this BW from the available loads struct */
	{
		double the_load;
		/* the_load =  highest_BW - minimum_BW */
		the_load = Compute_load_for_var(
				temp_p->type_and_id.type, 
				highest_BW.size,
				highest_BW.period, 
				temp_p->load.req.m_MTU );
		the_load = the_load - Compute_load_for_var(
				temp_p->type_and_id.type,
				temp_p->load.req.m_size,
				temp_p->load.req.m_period,
				temp_p->load.req.m_MTU );

//		PRINT_MSG("The load with the pair: %g", the_load);

		/* For the producer */
		nodes_load[ (temp_p->con.req.Producer_nodeID)[0] ].aux_up_no ++; //one more var assign

		/* For the consumers */
//		PRINT_MSG("Hey man---- check the broadcast vars - 2");
//#warning check the broadcasts
		for ( i=0; i<temp_p->con.req.Consumers_no; i++){
			nodes_load[(temp_p->con.req.Consumer_nodeID)[i]].aux_down_no ++;
		}

		/* Assign this load as given to the var */
		temp_p->qos.req.qos_given_load = the_load;//Once > 0 means that load was already assigned. Flag
//PRINT_MSG("--->Assigned qos_given_load: %d Mbps ", (int)((temp_p->qos.req.qos_given_load)/1000000) );
	}

	return 0;
}

static signed short M_manag_distribute_BW( unsigned short *idxs, unsigned short idxs_no )
{
	unsigned short i,j,k;
	M_MANAG_DB_TABTYPE **back_table;
	M_MANAG_DB_ITEM_TYPE *temp_p;
	double excess, normalized_excess;

	/* Get the backtime table */
	if ( M_MANAG_DB_Get_backtime_table( &back_table ) < 0 )
		return -1;

//PRINT_MSG(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>In M_manag_distribute_BW ");
//PRINT_MSG("--->Before");
//for ( i=0; i<idxs_no; i++ ){
//	temp_p = &(((*back_table)->m_message)[idxs[i]]);
//	PRINT_MSG("idx:%d  qos_given:%d Mbps",idxs[i] , (int)(temp_p->qos.req.qos_given_load/1000000));
//}

	/* For each nodeID */
	for ( i=0; i<MAX_NODES_NO; i++ )
		if (nodes_load[i].in_use){
			/* uplink - check if BW exceeded */
			//if (nodes_load[i].up < 0)
			{
				/* quantify the excess */
				excess = nodes_load[i].up_link_load;

//				PRINT_MSG("The excess is :%gMbps", excess*1e-6);

				/* Guess how much we will remove back from each 
				 * of the vars assigned in this nodeID link */
				normalized_excess = excess/(nodes_load[i].aux_up_no);
//PRINT_MSG("Normalized excess is :%gMbps", normalized_excess*1e-6);

				/* track the vars that produce here */
				for (j=0; j<idxs_no; j++){
					temp_p = &(((*back_table)->m_message)[idxs[j]]);
					if ( (temp_p->con.req.Producer_nodeID)[0] == i){
						/* var indexed by j produces here */
						if (temp_p->qos.req.qos_given_load > normalized_excess)
							temp_p->qos.req.qos_given_load = normalized_excess;
					}
				}

			}
			/* downlink - check if BW exceeded */
			//if (nodes_load[i].down < 0)
			{
				/* quantify the excess */
				excess = nodes_load[i].down_link_load;

//				PRINT_MSG("The excess is :%gMbps", excess*1e-6);

				/* Guess how much we will remove back from each 
				 * of the vars assigned in this nodeID link */
				normalized_excess = excess/(nodes_load[i].aux_down_no);
//PRINT_MSG("Normalized excess is :%gMbps", normalized_excess*1e-6);

				/* track the vars that consume here */
				for (j=0; j<idxs_no; j++){
					temp_p = &(((*back_table)->m_message)[idxs[j]]);
//					PRINT_MSG("Hey man---- check the broadcast vars - 4");
//#warning check the broadcasts
					for (k=0; k<(temp_p->con.req.Consumers_no); k++)
						if ((temp_p->con.req.Consumer_nodeID)[k] == i){
							/* var indexed by j produces here */
							if (temp_p->qos.req.qos_given_load > normalized_excess)
								temp_p->qos.req.qos_given_load = normalized_excess;
							break;
						}
				}

			}


		}
//PRINT_MSG("-->After");
//for ( i=0; i<idxs_no; i++ ){
//	temp_p = &(((*back_table)->m_message)[idxs[i]]);
//	PRINT_MSG("idx:%d  qos_given:%d Mbps",idxs[i] , (int)(temp_p->qos.req.qos_given_load * 1e-6));
//}

	return 0;
}

static signed short M_manag_compute_CT( unsigned short idx )
{
	M_MANAG_DB_TABTYPE **back_table;
	M_MANAG_DB_ITEM_TYPE *temp_p;
	unsigned int highest_C;
	unsigned int corresponding_period;
	unsigned int period_from_points;
	unsigned int corresponding_size;
	unsigned int size_from_points;
	int i;

	double overall_load; //bps
	double minimum_load;

//TODO -Implementar o Cmin_desejado que vem da aplicação
	/* Let's for now admit as the application wished C the maximum registered in the pairs
	 * -If the application does not want so much, it has to update the parameters */

	/* Get the backtime table */
	if ( M_MANAG_DB_Get_backtime_table( &back_table ) < 0 )
		return -1;

	temp_p = &(((*back_table)->m_message)[idx]);

//	PRINT_MSG(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>In M_manag_compute_CT for idx:%d", idx);
//	PRINT_MSG("the given %gMbps + minimum %gMbps", temp_p->qos.req.qos_given_load*1e-6 , Compute_load_for_var( 
//			temp_p->type_and_id.type,
//			temp_p->load.req.m_size,
//			temp_p->load.req.m_period,
//			temp_p->load.req.m_MTU)*1e-6);
	
	/* Get the wishing size */
	M_FTT_MANAGEMENT_L_QOS_REQUIREMENTS_get_highest_size( &(temp_p->qos.req), &(temp_p->load.req), &highest_C);

	/* Let's now seek the highest period meeting:
	 * - the qos_given_load+minimum_load
	 * - the requested C */
	minimum_load = Compute_load_for_var( 
			temp_p->type_and_id.type,
			temp_p->load.req.m_size,
			temp_p->load.req.m_period,
			temp_p->load.req.m_MTU);

	overall_load = temp_p->qos.req.qos_given_load + minimum_load;

//PRINT_MSG("The obtained BW:%g", overall_load*1e-6);
//PRINT_MSG("The Highest C:%d", highest_C);
	corresponding_period = Compute_period_for_var(
				temp_p->type_and_id.type,
				highest_C,
				temp_p->load.req.m_MTU,
				overall_load );
//PRINT_MSG("The corresponding period:%d", corresponding_period);
//PRINT_MSG("which gives us a load of %g", Compute_load_for_var( 
//			temp_p->type_and_id.type,
//			highest_C,corresponding_period, temp_p->load.req.m_MTU)*1e-6);


	if (M_FTT_MANAGEMENT_L_QOS_REQUIREMENTS_get_highest_period_until( &(temp_p->qos.req), &(temp_p->load.req), corresponding_period, &period_from_points ) < 0){
		/*means that the highest period exceeds the BW allocated */
//		PRINT_MSG("the highest period exceeds the BW allocated, this is:T=%d",period_from_points);

		/* re-check a lower value for C */
		corresponding_size = Compute_size_for_var(
				temp_p->type_and_id.type,
				temp_p->load.req.m_period,
				temp_p->load.req.m_MTU,
				overall_load );
//		PRINT_MSG("Going to M_FTT_MANAGEMENT_L_QOS_REQUIREMENTS_get_highest_size_until: size:%d",corresponding_size );
		if ( M_FTT_MANAGEMENT_L_QOS_REQUIREMENTS_get_highest_size_until( &(temp_p->qos.req), &(temp_p->load.req),corresponding_size, &size_from_points ) < 0 ){
			/* Well... now we have a problem. Most probably the minimum BW was badly guaranteed */
			ERROR_MSG("There is no C and T that mach the BW");
			return -1;
		}
	}else{
//		PRINT_MSG("perfect C mach");
		size_from_points = highest_C;
	}

//	PRINT_MSG("period_from_points %d",period_from_points);
//	PRINT_MSG("size_from_points %d",size_from_points);

	/* Store the values */
	temp_p->qos.req.qos_given_pair.size = size_from_points;
	temp_p->qos.req.qos_given_pair.period = period_from_points;

	/* Update the available load for dstribution on each link */
	overall_load = Compute_load_for_var(
			temp_p->type_and_id.type,
			size_from_points,
			period_from_points,
			temp_p->load.req.m_MTU)-minimum_load;

//PRINT_MSG("overall_load: %g", overall_load/1000000);
//PRINT_MSG("minimum_load: %g", minimum_load/1000000);

	/* For the producer */
	nodes_load[(temp_p->con.req.Producer_nodeID)[0]].up_link_load -= overall_load;

	/* For the consumers */
//	PRINT_MSG("Hey man---- check the broadcast vars - 2");
	for ( i=0; i<temp_p->con.req.Consumers_no; i++){
		nodes_load[(temp_p->con.req.Consumer_nodeID)[i]].down_link_load -= overall_load;
	}

	return 0;
}




signed char M_manag_Admission_test( void )
{
	int ret;
	double traffic_slot_load;
	double available_load;
	//unsigned short vars_no;
	int i;
	RET_SCHED_POLICIES policy;
	double max_pkt_load;
//	double val1,val2;
#ifdef POLITEE
	PRINT_MSG("Performing the Management Admission control test");
#endif

	/* Check the overall load for the SMs */
	if ( (ret = M_manag_populate_links_min_load( M_FTT_MANAGEMENT_SM, &max_pkt_load ) ) < 0 ){
		ERROR_MSG("M_manag_populate_links_min_load(M_FTT_MANAGEMENT_SM) ret: %d", ret);
		return -1;
	}

//	nodes_load_print();
	
	//vars_no = (unsigned short)ret;

	traffic_slot_load = M_FTT_CORE_L_get_SM_slot_load( );
	
//	val1 = traffic_slot_load/1000000;
//	PRINT_MSG("The traffic slot load is: %d.%dMbps", (int)(val1), (int)(10*(val1-(int)val1)) );

//	PRINT_MSG("max_pkt_load %d.%d Mbps",(int)(max_pkt_load), (int)(10*(max_pkt_load-(int)max_pkt_load)) );	
	/* max_pkt_load is 0 if no messages! */

	/* Compare with the scheduling policy */
	policy = M_FTT_CORE_L_ret_sm_sched_policy();
	switch(policy){
		case RET_SCHEDD_EDF:
			available_load = traffic_slot_load - max_pkt_load;
			break;
		case RET_SCHEDD_RM:
			//vars_no
			available_load = (traffic_slot_load - max_pkt_load) * 0.69;
//#warning actualizar com o limite rm correcto
			break;
		default:
			ERROR_MSG("Unespected scheduling policy");
			return -2;
	}
	
//	val2 = available_load/1000000;
//	PRINT_MSG("Feasability load : %d.%dMbps", (int)(val2), (int)(10*(val2-(int)val2)) );
	
	/* For all links check if they met */
	for ( i=0; i<MAX_NODES_NO; i++ )
		if (nodes_load[i].in_use){
			if (nodes_load[i].up_link_load > available_load)
				return -3;
			if ( (nodes_load[i].down_link_load + nodes_load[i].down_link_indirect_load) > available_load )
				return -4;
		}

	return 0;
}

signed char M_manag_QoS_distribution( void )
{
	int ret,i;
	unsigned short prio_idxs[MAX_NO_OF_VARS_SAME_PRIO];
	unsigned short prio_idxs_no;

	/* For the SM traffic */
#ifdef POLITEE
	PRINT_MSG("Performing the Management QoS distribution");
#endif

	/* Get the load available in each link after assigning the minimums */
	M_manag_links_available_load( M_FTT_MANAGEMENT_SM );

	/*The available load on each link is fully represented in the double vars:
	 *  - nodes_load[i].up_link_load
	 *  - nodes_load[i].down_link_load */

	/* Now the nodes_load structure is holding the EXTRA BW available to be
	 * distributed according to the QoS policy */

	/* Before distributing the BW let's clear the BW given to each var (qos_given_load)
	 *  - This is used to check the var as being already analysed and computed */
	M_manag_prepare_for_distribution( M_FTT_MANAGEMENT_SM ); //reset all the vars to as "no load assigned yet".
	
//	PRINT_MSG("After the preparation: (this is the available load for distrinution)");
//	nodes_load_print();

	/* GREEDY implementation */

	/* From the highest prio level to the lowest */
	while ( (ret = M_manag_get_highest_prio( M_FTT_MANAGEMENT_SM, prio_idxs, &prio_idxs_no )) >= 0 ){
		/* We are now in the priority level given by ret */
		
		/* We have prio_idxs_no idxs in prio_idxs maching the ret priority */
		if ((prio_idxs_no==0)||(prio_idxs_no>30)){
			ERROR_MSG("M_manag_get_highest_prio is not good - programming error");
			return -2;
		}
//PRINT_MSG("For the prio: %d     got:%d var idxs",ret, prio_idxs_no);

		/* Clear the temp vars that track the number of vars sharing a determined link */
		M_manag_clear_nodeID_vars_no();
//		PRINT_MSG("After M_manag_clear_nodeID_vars_no:");
//		nodes_load_print();


///////////7
		/*for each var got - remove the BW from available */
		for (i=0; i<prio_idxs_no; i++){
//			PRINT_MSG("M_manag_common_nodeID_links for idx: %d", i);
			/* Set in the qos_given_load the extra load that the var may use, i.e. MaximumBW-MinimumBW */
			M_manag_common_nodeID_links( prio_idxs[i] );
		}
//		nodes_load_print();

		/*So,
		 * for each var whithin this priority level, 
		 *  the 'qos_given_load' on each represents the maximum extra possible BW
		 * and for each node link (up&down), 
		 *  the 'up/down_link_load' holds the BW to be distributed. */

		M_manag_distribute_BW( prio_idxs, prio_idxs_no );

///////////7
#if 0
		/*for each var got - remove the BW from available */
		for (i=0; i<prio_idxs_no; i++){
			PRINT_MSG("Steal BW for idx: %d", i);
			M_manag_steal_BW( prio_idxs[i] );
		}
		
		PRINT_MSG("After BW steal:");
		nodes_load_print();
	

		/* If in any link of any nodeID , the available load was exceeded, reput */
		M_manag_reput_BW( prio_idxs, prio_idxs_no );

		PRINT_MSG("After M_manag_reput_BW:");
		nodes_load_print();
return -1;
#endif
		/* for each var got - evaluate the C,T given the BW */
		for (i=0; i<prio_idxs_no; i++){
			if ( M_manag_compute_CT( prio_idxs[i] ) < 0 ){
				ERROR_MSG("Couldn't get (C,T) pair dor idx:%d", i);
				return -3;
			}
		}

//		PRINT_MSG("After CT calc:");
//		nodes_load_print();
	}

	return 0;

}

void M_MANAG_DB_TABTYPE_Print(M_MANAG_DB_TABTYPE *p, unsigned char detailed_flag)
{
	int i,j;
	//PRINT_MSG("Locker : %d", p->locker_flag);
	PRINT_MSG("No. of items: %d", p->m_nvars);
	double val1, val2;

	for (i=0; i < (MESSAGE_IDS_MAX*2); i++){
//		PRINT_MSG("&(p->m_message)[i]): %p", &(p->m_message)[i]);
		if ( (p->m_message)[i].type_and_id.type != M_FTT_MANAGEMENT_NOT_INUSE ){
			PRINT_MSG("[%d]>typ%d id:0x%4x new_flag:%d fttid:0x%2x load:%d con:%d qos:%d",
					i,
					(p->m_message)[i].type_and_id.type,
					(p->m_message)[i].type_and_id.id,
					(p->m_message)[i].new_flag,
					(p->m_message)[i].ftt_id_idx,
					(p->m_message)[i].load.status,
					(p->m_message)[i].con.status,
					(p->m_message)[i].qos.status
				 );
			PRINTF("(");
			if ((p->m_message)[i].con.req.Producers_no){
				PRINTF(" Tx[0]:%d", (p->m_message)[i].con.req.Producer_nodeID[0] );
			}else{
				PRINTF(" noTx" );
			}
			if ((p->m_message)[i].con.req.Consumers_no){
				PRINTF(" Rx[0]:%d", (p->m_message)[i].con.req.Consumer_nodeID[0] );
			}else {
				PRINTF(" noRx" );
			}
			PRINTF(")");

//			PRINTF(" Prod_no %d   Cons_no %d   ptod_type %d", (p->m_message)[i].con.req.Producers_no, (p->m_message)[i].con.req.Consumers_no, (p->m_message)[i].con.req.Production_type );
			if (detailed_flag){
				if ((p->m_message)[i].load.status){
					val1 = Compute_load_for_var( (p->m_message)[i].type_and_id.type, (p->m_message)[i].load.req.m_size, (p->m_message)[i].load.req.m_period, (p->m_message)[i].load.req.m_MTU )/1000000;
 					PRINT_MSG("                Load:  C:%d[%d]  T:%d  C/T: %d.%dMbps", (p->m_message)[i].load.req.m_size, (p->m_message)[i].load.req.m_max_size, (p->m_message)[i].load.req.m_period, (int)(val1), (int)(10*(val1-(int)val1)) );
				}
				if ((p->m_message)[i].qos.status){
					PRINT_MSG("                QOS : prio/share %d   Pairs_no %d", (p->m_message)[i].qos.req.qos_share, (p->m_message)[i].qos.req.qos_pairs_no );
					/* Go for each pair */
					for (j=0; j<(p->m_message)[i].qos.req.qos_pairs_no; j++ )
						PRINTF(" [%d / %d]", ((p->m_message)[i].qos.req.qos_pairs)[j].size, ((p->m_message)[i].qos.req.qos_pairs)[j].period );
					val1 = Compute_load_for_var( (p->m_message)[i].type_and_id.type, (p->m_message)[i].qos.req.qos_given_pair.size, (p->m_message)[i].qos.req.qos_given_pair.period, (p->m_message)[i].load.req.m_MTU )/1000000;
					val2 = ((p->m_message)[i].qos.req.qos_given_load)/1000000;
					PRINT_MSG("                got : C:%d   T:%d  C/T: %d.%dMbps   (extra bandwidth %d.%dMbps)", (p->m_message)[i].qos.req.qos_given_pair.size, (p->m_message)[i].qos.req.qos_given_pair.period, (int)(val1), (int)(10*(val1-(int)val1)), (int)(val1), (int)(10*(val2-(int)val2)) );
				}
		
			}
		}
	}
}

#ifdef OLDCODE

#if 0
static signed short M_manag_steal_BW( unsigned short idx )
{
	unsigned short i;
	M_MANAG_DB_TABTYPE **back_table;
	M_MANAG_DB_ITEM_TYPE *temp_p;
	QOS_BW_PAIR highest_BW;

	/* Get the backtime table */
	if ( M_MANAG_DB_Get_backtime_table( &back_table ) < 0 )
		return -1;

	temp_p = &(((*back_table)->m_message)[idx]);

	/* Get the most demanding BW for this */
	M_FTT_MANAGEMENT_L_QOS_REQUIREMENTS_get_highest_BW_pair( &(temp_p->qos.req), &(temp_p->load.req), &highest_BW );

	PRINT_MSG("The highest BW pair: %d %d", highest_BW.size, highest_BW.period);

	/* Decrement this BW from the available loads struct */
	{
		double the_load;
		/* the_load =  highest_BW - minimum_BW */
		the_load = Compute_load_for_var(
				temp_p->type_and_id.type, 
				highest_BW.size,
				highest_BW.period, 
				temp_p->load.req.m_MTU );
		the_load = the_load - Compute_load_for_var(
				temp_p->type_and_id.type,
				temp_p->load.req.m_size,
				temp_p->load.req.m_period,
				temp_p->load.req.m_MTU );

		PRINT_MSG("The load with the pair: %g", the_load);

		/* For the producer */
		nodes_load[ (temp_p->con.req.Producer_nodeID)[0] ].up -= the_load;
		nodes_load[ (temp_p->con.req.Producer_nodeID)[0] ].up_no ++; //one more var assign

		/* For the consumers */
		PRINT_MSG("Hey man---- check the broadcast vars - 2");
		for ( i=0; i<temp_p->con.req.Consumers_no; i++){
			nodes_load[(temp_p->con.req.Consumer_nodeID)[i]].down -= the_load;
			nodes_load[(temp_p->con.req.Consumer_nodeID)[i]].down_no ++;
		}

		/* Assign this load as given to the var */
		temp_p->qos.req.qos_given_load = the_load;//Once > 0 means that load was already assigned. Flag
	}

	return 0;
}
#endif

#if 0
static signed short M_manag_give_BW( unsigned short idx, double to_give )
{
	unsigned short i;
	M_MANAG_DB_TABTYPE **back_table;
	M_MANAG_DB_ITEM_TYPE *temp_p;

	/* Get the backtime table */
	if ( M_MANAG_DB_Get_backtime_table( &back_table ) < 0 )
		return -1;

	temp_p = &(((*back_table)->m_message)[idx]);

	/* Increment this BW in the available loads struct */
	{
		/* For the producer */
		nodes_load[ (temp_p->con.req.Producer_nodeID)[0] ].up += to_give;

		/* For the consumers */
		PRINT_MSG("Hey man---- check the broadcast vars - 3");
		PRINT_MSG("M_manag_give_BW idx:%d",idx);
		for ( i=0; i<temp_p->con.req.Consumers_no; i++){
			PRINT_MSG("Giving back in down link nodeID:%d",(temp_p->con.req.Consumer_nodeID)[i]);
			nodes_load[(temp_p->con.req.Consumer_nodeID)[i]].down += to_give;
		}

		/* Assign this load as given to the var */
		temp_p->qos.req.qos_given_load -= to_give; //remove it

		/* It might happen that when normalizing a var is given more load to reput than the one 
		 * it got. There will be a BW wast but I don't care for now */
		if (temp_p->qos.req.qos_given_load<0) temp_p->qos.req.qos_given_load = 0;
	}

	return 0;
}
static signed short M_manag_reput_BW( unsigned short *idxs, unsigned short idxs_no )
{
	unsigned short i,j,k;
	M_MANAG_DB_TABTYPE **back_table;
	M_MANAG_DB_ITEM_TYPE *temp_p;
	double excess, normalized_excess;

	/* Get the backtime table */
	if ( M_MANAG_DB_Get_backtime_table( &back_table ) < 0 )
		return -1;
PRINT_MSG(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>In M_manag_reput_BW ");
PRINT_MSG("Before");
for ( i=0; i<idxs_no; i++ ){
	temp_p = &(((*back_table)->m_message)[idxs[i]]);
	PRINT_MSG("idx:%d  qos_given:%gMbps",idxs[i] , (temp_p->qos.req.qos_given_load * 1e-6));
}

	/* For each nodeID */
	for ( i=0; i<MAX_NODES_NO; i++ )
		if (nodes_load[i].in_use){
			/* uplink - check if BW exceeded */
			if (nodes_load[i].up < 0){
				/* quantify the excess */
				excess = -nodes_load[i].up;

				PRINT_MSG("The excess is :%gMbps", excess*1e-6);

				/* Guess how much we will remove back from each 
				 * of the vars assigned in this nodeID link */
				normalized_excess = excess/(nodes_load[i].up_no);
PRINT_MSG("Normalized excess is :%gMbps", normalized_excess*1e-6);

				/* track the vars that produce here */
				for (j=0; j<idxs_no; j++){
					temp_p = &(((*back_table)->m_message)[idxs[j]]);
					if ( (temp_p->con.req.Producer_nodeID)[0] == i){
						/* var indexed by j produces here
						 * Let's remove the excess */
						M_manag_give_BW( idxs[j], normalized_excess );
					}
				}
			}
			/* downlink - check if BW exceeded */
			if (nodes_load[i].down < 0){
				PRINT_MSG("hey-Reputting BW");
				/* quantify the excess */
				excess = -nodes_load[i].down;

				/* Guess how much we will remove back from each 
				 * of the vars assigned in this nodeID link */
				normalized_excess = excess/(nodes_load[i].down_no);

				/* track the vars that consume here */
				for (j=0; j<idxs_no; j++){
					temp_p = &(((*back_table)->m_message)[idxs[j]]);
					PRINT_MSG("Hey man---- check the broadcast vars - 4");
					for (k=0; k<(temp_p->con.req.Consumers_no); k++)
						if ((temp_p->con.req.Consumer_nodeID)[k] == i){
							/* var indexed by j produces here
							 * Let's remove the excess */
							M_manag_give_BW( idxs[j], normalized_excess );
							break;
						}
				}

			}
		}
PRINT_MSG("After");
for ( i=0; i<idxs_no; i++ ){
	temp_p = &(((*back_table)->m_message)[idxs[i]]);
	PRINT_MSG("idx:%d  qos_given:%gMbps",idxs[i] , (temp_p->qos.req.qos_given_load * 1e-6));
}

	return 0;
}

#endif
#endif
