/*****************************************************************************
 * M_manag_update.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/



#include "Ports.h"
#include "M_ftt-core.layer.h"
#include "ftt-management.layer_types.h"

#include "M_manag_db.h"


inline static void M_manag_Copy_Load_req_from_core( M_FTT_MANAGEMENT_L_LOAD_REQUIREMENTS *manag_req, MESSAGE_REQUESITS *core_req )
{
	manag_req->m_size = core_req->m_size;
	manag_req->m_max_size = core_req->m_max_size;
	manag_req->m_max_prod_no = core_req->sw_producer_max_no;
	manag_req->m_period = core_req->m_period;
	manag_req->m_deadline = core_req->m_deadline;
	manag_req->m_init = core_req->m_init;
	manag_req->m_relative_id = core_req->m_relative_id;
	manag_req->m_MTU = core_req->m_MTU;
	switch (core_req->m_timeliness){
		case TMLN_HARD: manag_req->m_timeliness=M_FTT_MANAGEMENT_HARD_RT; break;
		case TMLN_SOFT: manag_req->m_timeliness=M_FTT_MANAGEMENT_SOFT_RT; break;
		case TMLN_UNCONST: default: manag_req->m_timeliness=M_FTT_MANAGEMENT_UNCONSTRAINED; break;
	}
	return;
}

inline static void M_manag_Copy_Con_req_from_core( M_FTT_MANAGEMENT_L_CONNECTION_SET *manag_req, MESSAGE_REQUESITS *core_req )
{
	unsigned short i;
	/* If core's consumer==0 then it's broadcast */
	manag_req->Producers_no = core_req->sw_producer_no;
	manag_req->Consumers_no = core_req->sw_consumer_no;

	for ( i=0;i<manag_req->Producers_no; i++){
		manag_req->Producer_nodeID[i] = (core_req->sw_producer_idx)[i];
	}
	
	for ( i=0;i<manag_req->Consumers_no; i++){
		manag_req->Consumer_nodeID[i] = core_req->sw_consumer_idx[i];
	}

	switch (manag_req->Consumers_no){
		case 0: manag_req->Production_type = M_FTT_MANAGEMENT_P_BROADCAST; break;
		case 1: manag_req->Production_type = M_FTT_MANAGEMENT_P_UNICAST; break;
		default: manag_req->Production_type = M_FTT_MANAGEMENT_P_MULTICAST;
	}
}

inline static void M_manag_Copy_Message_req_to_core( 
		MESSAGE_REQUESITS *core_req, 
		M_FTT_MANAGEMENT_L_LOAD_REQUIREMENTS *load_req,
		M_FTT_MANAGEMENT_L_CONNECTION_SET *con_req,
		unsigned short ftt_id, 
		unsigned int m_size,
		unsigned int m_max_size,
	       	unsigned int m_period )
{
	unsigned short i;

	/*This memset is required to zero the prod/cons arrays.
	 * This was leading to a problem whel later memcmparing the whole structure */
	memset(core_req, 0, sizeof(MESSAGE_REQUESITS));
	
	core_req->m_id = ftt_id;
	core_req->m_size = m_size;
	core_req->m_max_size = m_max_size;
	core_req->m_period = m_period;
	core_req->m_deadline = m_period; //D=T
	core_req->m_init = load_req->m_init;
	if (load_req->m_init < 0) core_req->m_relative_id= 0;
	else core_req->m_relative_id = load_req->m_relative_id;
	
	core_req->m_MTU = load_req->m_MTU;
	switch (load_req->m_timeliness){
		case M_FTT_MANAGEMENT_HARD_RT: core_req->m_timeliness = TMLN_HARD; break;
		case M_FTT_MANAGEMENT_SOFT_RT: core_req->m_timeliness = TMLN_SOFT; break;
		case M_FTT_MANAGEMENT_UNCONSTRAINED: default: core_req->m_timeliness = TMLN_UNCONST; break;
	}

	core_req->sw_consumer_no = con_req->Consumers_no;
	for (i=0; i<(con_req->Consumers_no); i++)
		core_req->sw_consumer_idx[i] = con_req->Consumer_nodeID[i];

	core_req->sw_producer_no = con_req->Producers_no;
	core_req->sw_producer_max_no = load_req->m_max_prod_no;

	for (i=0; i<(con_req->Producers_no); i++)
		core_req->sw_producer_idx[i] = con_req->Producer_nodeID[i];	
	
	return;
}


signed char M_manag_Import_Core_Table(MESSAGES_TABLE *ftt_core_static_table)
{
	int ret,i;
	M_FTT_MANAGEMENT_L_LOAD_REQUIREMENTS temp_load_req;
	M_FTT_MANAGEMENT_L_CONNECTION_SET temp_con_req;


	/* Now we have to decode the ftt-core static table 
	 * For now will assume that this table is set only during the core setup phase 
	 * So, we only need to read it once, at this module boot phase */

	M_MANAG_DB_Changes_lock();

	/* Lock the ftt-core static table */
	MESSAGES_TABLE_EnterLock(ftt_core_static_table);

	for (i=0; i<(ftt_core_static_table->items_no); i++){
		M_manag_Copy_Load_req_from_core( &temp_load_req, &((ftt_core_static_table->data)[i].req) );
		M_manag_Copy_Con_req_from_core( &temp_con_req, &((ftt_core_static_table->data)[i].req) );
		
		ret = M_MANAG_DB_Add_blacklisted_ftt_id(
				(ftt_core_static_table->data)[i].message_id,
				( ((ftt_core_static_table->data)[i].message_type == SYNC_MESSAGE)?M_FTT_MANAGEMENT_SM:M_FTT_MANAGEMENT_AM ),
				&temp_load_req,
				&temp_con_req);
		if (ret<0){
			ERROR_MSG("M_MANAG_DB_Add_blacklisted_ftt_id ret: %d", ret);
			return -2;
		}
	}
	
	/* Unlock the ftt-core static table */
	MESSAGES_TABLE_LeaveLock(ftt_core_static_table);

	M_MANAG_DB_Changes_submit_unlock();
	
	return 0;
}

signed char M_manag_Export_Core_Table(MESSAGES_TABLE *table)
{
	int i;
	unsigned short counter;

	M_MANAG_DB_TABTYPE **run_table;
	M_MANAG_DB_ITEM_TYPE *temp_p;


	/* Get the runtime table and lock it */
	M_MANAG_DB_Get_runtime_table( &run_table );
	M_MANAG_DB_TABTYPE_EnterLock( *run_table );

//M_MANAG_DB_TABTYPE_Print(*run_table);	

	/* Lock the table where to export */
	MESSAGES_TABLE_EnterLock(table);
	MESSAGES_TABLE_Clear(table);

	/* Now we can fill the table with the runtable contents according to the QoS policy */
	
	counter = 0;
	for (i=0; i< (*run_table)->m_nvars; i++ ){
		temp_p = &(((*run_table)->m_message)[i]);
		counter = table->items_no;

		switch(temp_p->type_and_id.type){
			case M_FTT_MANAGEMENT_SM: (table->data)[counter].message_type = SYNC_MESSAGE; break;
			case M_FTT_MANAGEMENT_AM: (table->data)[counter].message_type = ASYNC_MESSAGE; break;

			case M_FTT_MANAGEMENT_SM_DUMMY:
			case M_FTT_MANAGEMENT_AM_DUMMY:
			default:
				continue;			
		}

		/* Let's check if this is a valid message to be added into the QoS stuff */
		if ( (temp_p->load.status != 1) || (temp_p->con.status != 1) ){
			continue;
		}

		(table->data)[counter].message_id = temp_p->ftt_id_idx;

		if (temp_p->new_flag){
			(table->data)[counter].add_type = 1; //Overwrite
			temp_p->new_flag = 0;
		}else{
			(table->data)[counter].add_type = 0; //No try update or preserve
		}

		/* Check if we have the QoS module working for this var */
		if ( temp_p->qos.status ){
			M_manag_Copy_Message_req_to_core(
					&( (table->data)[counter].req ), //destination
					&(temp_p->load.req),
					&(temp_p->con.req),
					temp_p->ftt_id_idx,
				    temp_p->qos.req.qos_given_pair.size, /* these are the QoS result */
			       	temp_p->load.req.m_max_size,
					temp_p->qos.req.qos_given_pair.period
			       		);
		}else{
			M_manag_Copy_Message_req_to_core(
					&( (table->data)[counter].req ), //destination
					&(temp_p->load.req),
					&(temp_p->con.req),
					temp_p->ftt_id_idx,
					temp_p->load.req.m_size, /* these are the QoS result */
			       	temp_p->load.req.m_max_size,
					temp_p->load.req.m_period
			       	);
		}

		(table->items_no) ++;
	}
	
	MESSAGES_TABLE_LeaveLock(table);
	M_MANAG_DB_TABTYPE_LeaveLock( *run_table );

	return 0;
}

