/*****************************************************************************
 * ftt-master.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#include "Ports.h"

#include "ethernet.layer.h"
#include "M_ftt-core.layer.h"
#include "M_ftt-management.layer.h"
#include "M_ftt-interface.layer.h"
#include "M_debug.module.h"

//layer.cfg
//#define ENABLE_ETHERNET
//#define ENABLE_FTT_CORE
//#define ENABLE_FTT_MANAGEMENT
//#define ENABLE_FTT_INTERFACE
//#define ENABLE_DEBUG_MODULE

signed char ftt_master_start_up(unsigned char *mac, unsigned char dev_num, char *dev_name, unsigned long ec_time)
{
	signed char temp;
	unsigned char local_mac[6];


#ifdef POLITE
	PRINT_MSG("Starting the Master FTT modules");
#endif

#ifdef ENABLE_ETHERNET
	if ( (temp=ETH_L_init(dev_num, dev_name,local_mac))<0 ){
		ERROR_MSG("Error starting the ETH_L_init module... ret=%d",temp);
		return -1;
	}
	if (mac != NULL)
		ETH_L_Str2Addr(mac, local_mac);
#endif

#ifdef ENABLE_FTT_CORE
	if ( (temp=M_FTT_CORE_L_init(local_mac, ec_time))<0 ){
		ERROR_MSG("Error starting the M_FTT_CORE_L_init module... ret=%d",temp);
		return -2;
	}
#endif

#ifdef ENABLE_FTT_MANAGEMENT
	if ( (temp=M_FTT_MANAGEMENT_L_init())<0 ){
		ERROR_MSG("Error starting the M_FTT_MANAGEMENT_L_init module... ret=%d",temp);
		return -3;
	}
#endif

#ifdef ENABLE_FTT_INTERFACE
	if ( (temp=M_FTT_INTERFACE_L_init())<0 ){
		ERROR_MSG("Error starting the M_FTT_INTERFACE_L_init module... ret=%d",temp);
		return -3;
	}
#endif

#ifdef ENABLE_DEBUG_MODULE
	if ( (temp=M_DEBUG_MODULE_init())<0 ){
		ERROR_MSG("Error starting the M_DEBUG_MODULE_init module... ret=%d",temp);
		return -4;
	}
#endif

	PRINT_MSG("Finished ftt_master_start_up\n");
	PRINTF("*****************************************************\n");
	return 0;
}

void ftt_master_close_up(void)
{

#ifdef POLITE
	PRINT_MSG("Closing the Master FTT modules");
#endif

#ifdef ENABLE_DEBUG_MODULE
	M_DEBUG_MODULE_close();
#endif

#ifdef ENABLE_FTT_INTERFACE
	M_FTT_INTERFACE_L_close();
#endif
#ifdef ENABLE_FTT_MANAGEMENT
	M_FTT_MANAGEMENT_L_close();
#endif
#ifdef ENABLE_FTT_CORE
	M_FTT_CORE_L_close();
#endif
#ifdef ENABLE_ETHERNET
	ETH_L_close();
#endif
	return ;
}


