/*****************************************************************************
 * ftt-slave.c:
 *****************************************************************************
 * Copyright (C) 2006-2012 the FTT-SE team.
 *
 * Author: Ricardo Marau <marau at fe.up.pt>
 *
 * FTT-SE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  FTT-SE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with FTT-SE.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#include "Ports.h"

#include "ethernet.layer.h"
#include "S_ftt-core.layer.h"

#include "S_ftt-interface.layer.h"
#include "S_debug.module.h"

//layer.cfg
//#define ENABLE_ETHERNET
//#define ENABLE_FTT_CORE
//#define ENABLE_FTT_MANAGEMENT
//#define ENABLE_FTT_INTERFACE
//#define ENABLE_DEBUG_MODULE

signed char ftt_slave_start_up(unsigned char *mac, unsigned char dev_num, char *dev_name, unsigned char app_nodeID)
{
	signed char temp;
	unsigned char local_mac[6];

#ifdef POLITE
	PRINT_MSG("Starting the Slave FTT modules");
#endif

#ifdef ENABLE_ETHERNET
	if ( (temp=ETH_L_init(dev_num, dev_name, local_mac))<0 ){
		ERROR_MSG("Error starting the ETH_L_init module... ret=%d",temp);
		return -1;
	}
	if (mac != NULL)
		ETH_L_Str2Addr(mac, local_mac);
#endif

#ifdef ENABLE_FTT_CORE
	if ( (temp=S_FTT_CORE_L_init(local_mac))<0 ){
		ERROR_MSG("Error starting the S_FTT_CORE_L_init module... ret=%d",temp);
		return -2;
	}
#endif

#ifdef ENABLE_FTT_INTERFACE	
	if ( (temp=S_FTT_INTERFACE_L_init(app_nodeID))<0 ){
		ERROR_MSG("Error starting the S_FTT_INTERFACE_L_init module... ret=%d",temp);
		return -3;
	}
#endif
#ifdef ENABLE_DEBUG_MODULE
	if ( (temp=S_DEBUG_MODULE_init())<0 ){
		ERROR_MSG("Error starting the S_DEBUG_MODULE_init module... ret=%d",temp);
		return -3;
	}
#endif
	PRINT_MSG("Finished ftt_slave_start_up\n");
	PRINTF("*****************************************************\n");

	return 0;

}

void ftt_slave_close_up(void)
{
	
#ifdef POLITE
	PRINT_MSG("Closing the Slave FTT modules");
#endif

#ifdef ENABLE_DEBUG_MODULE
	S_DEBUG_MODULE_close();
#endif

#ifdef ENABLE_FTT_INTERFACE
	S_FTT_INTERFACE_L_close();
#endif

#ifdef ENABLE_FTT_CORE
	S_FTT_CORE_L_close();
#endif

#ifdef ENABLE_ETHERNET
	ETH_L_close();
#endif

return ;
}


